@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/backends/midi/coreaudio.cpp $
- * $Id: coreaudio.cpp 46315 2009-12-09 18:05:20Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/backends/midi/coreaudio.cpp $
+ * $Id: coreaudio.cpp 50128 2010-06-21 21:36:36Z fingolfin $
  */
 
 #ifdef MACOSX
@@ -142,7 +142,7 @@ int MidiDriver_CORE::open() {
 		}
 
 		if (err != noErr)
-			warning("Failed loading custom sound font '%s' (error %ld)\n", soundfont, err);
+			warning("Failed loading custom sound font '%s' (error %ld)\n", soundfont, (long)err);
 	}
 
 #ifdef COREAUDIO_DISABLE_REVERB
@@ -218,7 +218,7 @@ public:
 	}
 
 	MusicDevices getDevices() const;
-	Common::Error createInstance(MidiDriver **mididriver) const;
+	Common::Error createInstance(MidiDriver **mididriver, MidiDriver::DeviceHandle = 0) const;
 };
 
 MusicDevices CoreAudioMusicPlugin::getDevices() const {
@@ -229,21 +229,12 @@ MusicDevices CoreAudioMusicPlugin::getDevices() const {
 	return devices;
 }
 
-Common::Error CoreAudioMusicPlugin::createInstance(MidiDriver **mididriver) const {
+Common::Error CoreAudioMusicPlugin::createInstance(MidiDriver **mididriver, MidiDriver::DeviceHandle) const {
 	*mididriver = new MidiDriver_CORE();
 
 	return Common::kNoError;
 }
 
-MidiDriver *MidiDriver_CORE_create() {
-	MidiDriver *mididriver;
-
-	CoreAudioMusicPlugin p;
-	p.createInstance(&mididriver);
-
-	return mididriver;
-}
-
 //#if PLUGIN_ENABLED_DYNAMIC(COREAUDIO)
 	//REGISTER_PLUGIN_DYNAMIC(COREAUDIO, PLUGIN_TYPE_MUSIC, CoreAudioMusicPlugin);
 //#else
