@@ -18,15 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/backends/platform/dc/display.cpp $
- * $Id: display.cpp 47628 2010-01-28 05:20:52Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/backends/platform/dc/display.cpp $
+ * $Id: display.cpp 49838 2010-06-15 12:21:08Z fingolfin $
  *
  */
 
 #define RONIN_TIMER_ACCESS
 
 #include <common/scummsys.h>
-#include <graphics/scaler/intern.h>
 #include <graphics/surface.h>
 #include "dc.h"
 
@@ -199,7 +198,7 @@ Graphics::PixelFormat OSystem_Dreamcast::getScreenFormat() const
   return screenFormats[_screenFormat];
 }
 
-Common::List<Graphics::PixelFormat> OSystem_Dreamcast::getSupportedFormats()
+Common::List<Graphics::PixelFormat> OSystem_Dreamcast::getSupportedFormats() const
 {
   Common::List<Graphics::PixelFormat> list;
   unsigned i;
