@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/backends/platform/n64/osys_n64_base.cpp $
- * $Id: osys_n64_base.cpp 48170 2010-03-06 13:49:20Z Hkz $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/backends/platform/n64/osys_n64_base.cpp $
+ * $Id: osys_n64_base.cpp 52644 2010-09-08 21:18:22Z hkz $
  *
  */
 
@@ -183,9 +183,6 @@ void OSystem_N64::initBackend() {
 		_savefile = new PAKSaveManager();
 	}
 
-	_mixer = new Audio::MixerImpl(this);
-	_mixer->setReady(false);
-
 	_timer = new DefaultTimerManager();
 
 	setTimerCallback(&timer_handler, 10);
@@ -349,8 +346,7 @@ int16 OSystem_N64::getWidth() {
 }
 
 void OSystem_N64::setPalette(const byte *colors, uint start, uint num) {
-	for (int i = 0; i < num; ++i) {
-		uint8 c[4];
+	for (uint i = 0; i < num; ++i) {
 		_screenPalette[start + i] = colRGB888toBGR555(colors[2], colors[1], colors[0]);
 		colors += 4;
 	}
@@ -416,7 +412,7 @@ void OSystem_N64::grabPalette(byte *colors, uint start, uint num) {
 }
 
 void OSystem_N64::setCursorPalette(const byte *colors, uint start, uint num) {
-	for (int i = 0; i < num; ++i) {
+	for (uint i = 0; i < num; ++i) {
 		_cursorPalette[start + i] = colRGB888toBGR555(colors[2], colors[1], colors[0]);
 		colors += 4;
 	}
@@ -871,7 +867,7 @@ void OSystem_N64::getTimeAndDate(TimeDate &t) const {
 	t.tm_hour = (((now / 1000) / 60) / 60) % 24;
 	t.tm_mday = 1;
 	t.tm_mon  = 0;
-	t.tm_year = 1900;
+	t.tm_year = 110;
 
 	return;
 }
@@ -881,6 +877,8 @@ FilesystemFactory *OSystem_N64::getFilesystemFactory() {
 }
 
 void OSystem_N64::setTimerCallback(TimerProc callback, int interval) {
+	assert (interval > 0);
+
 	if (callback != NULL) {
 		_timerCallbackTimer = interval;
 		_timerCallbackNext = getMillis() + interval;
@@ -890,6 +888,9 @@ void OSystem_N64::setTimerCallback(TimerProc callback, int interval) {
 }
 
 void OSystem_N64::setupMixer(void) {
+	_mixer = new Audio::MixerImpl(this, DEFAULT_SOUND_SAMPLE_RATE);
+	_mixer->setReady(false);
+
 	enableAudioPlayback();
 }
 
