@@ -18,12 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/backends/platform/sdl/sdl.cpp $
- * $Id: sdl.cpp 48101 2010-02-21 04:04:13Z bluddy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/backends/platform/sdl/sdl.cpp $
+ * $Id: sdl.cpp 52210 2010-08-19 12:43:37Z sev $
  *
  */
 
 #if defined(WIN32)
+#define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 // winnt.h defines ARRAYSIZE, but we want our own one... - this is needed before including util.h
 #undef ARRAYSIZE
@@ -94,10 +95,12 @@ AspectRatio::AspectRatio(int w, int h) {
 	_kh = h;
 }
 
-static const size_t AR_COUNT = 4;
-static const char*       desiredAspectRatioAsStrings[AR_COUNT] = {            "auto",            "4/3",            "16/9",            "16/10" };
-static const AspectRatio desiredAspectRatios[AR_COUNT]         = { AspectRatio(0, 0), AspectRatio(4,3), AspectRatio(16,9), AspectRatio(16,10) };
+#if !defined(_WIN32_WCE) && !defined(__SYMBIAN32__) && defined(USE_SCALERS)
 static AspectRatio getDesiredAspectRatio() {
+	const size_t AR_COUNT = 4;
+	const char*       desiredAspectRatioAsStrings[AR_COUNT] = {            "auto",            "4/3",            "16/9",            "16/10" };
+	const AspectRatio desiredAspectRatios[AR_COUNT]         = { AspectRatio(0, 0), AspectRatio(4,3), AspectRatio(16,9), AspectRatio(16,10) };
+
 	//TODO : We could parse an arbitrary string, if we code enough proper validation
 	Common::String desiredAspectRatio = ConfMan.get("desired_screen_aspect_ratio");
 
@@ -111,6 +114,7 @@ static AspectRatio getDesiredAspectRatio() {
 	// TODO : Report a warning
 	return AspectRatio(0, 0);
 }
+#endif
 
 void OSystem_SDL::initBackend() {
 	assert(!_inited);
@@ -146,8 +150,7 @@ void OSystem_SDL::initBackend() {
 	memset(&_videoMode, 0, sizeof(_videoMode));
 	memset(&_transactionDetails, 0, sizeof(_transactionDetails));
 
-	_cksumValid = false;
-#if !defined(_WIN32_WCE) && !defined(__SYMBIAN32__) && !defined(DISABLE_SCALERS)
+#if !defined(_WIN32_WCE) && !defined(DINGUX) && !defined(__SYMBIAN32__) && defined(USE_SCALERS)
 	_videoMode.mode = GFX_DOUBLESIZE;
 	_videoMode.scaleFactor = 2;
 	_videoMode.aspectRatioCorrection = ConfMan.getBool("aspect_ratio");
@@ -160,7 +163,6 @@ void OSystem_SDL::initBackend() {
 	_scalerProc = Normal1x;
 #endif
 	_scalerType = 0;
-	_modeFlags = 0;
 
 #if !defined(_WIN32_WCE) && !defined(__SYMBIAN32__)
 	_videoMode.fullscreen = ConfMan.getBool("fullscreen");
@@ -230,8 +232,7 @@ OSystem_SDL::OSystem_SDL()
 #endif
 	_overlayVisible(false),
 	_overlayscreen(0), _tmpscreen2(0),
-	_samplesPerSec(0),
-	_cdrom(0), _scalerProc(0), _modeChanged(false), _screenChangeCount(0), _dirtyChecksums(0),
+	_cdrom(0), _scalerProc(0), _modeChanged(false), _screenChangeCount(0),
 	_scrollLock(false),
 	_mouseVisible(false), _mouseNeedsRedraw(false), _mouseData(0), _mouseSurface(0),
 	_mouseOrigSurface(0), _cursorTargetScale(1), _cursorPaletteDisabled(true),
@@ -249,9 +250,9 @@ OSystem_SDL::OSystem_SDL()
 	_screenIsLocked(false),
 	_graphicsMutex(0), _transactionMode(kTransactionNone) {
 
-	// allocate palette storage
-	_currentPalette = (SDL_Color *)calloc(sizeof(SDL_Color), 256);
-	_cursorPalette = (SDL_Color *)calloc(sizeof(SDL_Color), 256);
+	// clear palette storage
+	memset(_currentPalette, 0, sizeof(_currentPalette));
+	memset(_cursorPalette, 0, sizeof(_cursorPalette));
 
 	_mouseBackup.x = _mouseBackup.y = _mouseBackup.w = _mouseBackup.h = 0;
 
@@ -279,9 +280,6 @@ OSystem_SDL::~OSystem_SDL() {
 	SDL_RemoveTimer(_timerID);
 	closeMixer();
 
-	free(_dirtyChecksums);
-	free(_currentPalette);
-	free(_cursorPalette);
 	free(_mouseData);
 
 	delete _savefile;
@@ -451,7 +449,6 @@ bool OSystem_SDL::hasFeature(Feature f) {
 	return
 		(f == kFeatureFullscreenMode) ||
 		(f == kFeatureAspectRatioCorrection) ||
-		(f == kFeatureAutoComputeDirtyRects) ||
 		(f == kFeatureCursorHasPalette) ||
 		(f == kFeatureIconifyWindow);
 }
@@ -464,12 +461,6 @@ void OSystem_SDL::setFeatureState(Feature f, bool enable) {
 	case kFeatureAspectRatioCorrection:
 		setAspectRatioCorrection(enable);
 		break;
-	case kFeatureAutoComputeDirtyRects:
-		if (enable)
-			_modeFlags |= DF_WANT_RECT_OPTIM;
-		else
-			_modeFlags &= ~DF_WANT_RECT_OPTIM;
-		break;
 	case kFeatureIconifyWindow:
 		if (enable)
 			SDL_WM_IconifyWindow();
@@ -487,14 +478,12 @@ bool OSystem_SDL::getFeatureState(Feature f) {
 		return _videoMode.fullscreen;
 	case kFeatureAspectRatioCorrection:
 		return _videoMode.aspectRatioCorrection;
-	case kFeatureAutoComputeDirtyRects:
-		return _modeFlags & DF_WANT_RECT_OPTIM;
 	default:
 		return false;
 	}
 }
 
-void OSystem_SDL::quit() {
+void OSystem_SDL::deinit() {
 	if (_cdrom) {
 		SDL_CDStop(_cdrom);
 		SDL_CDClose(_cdrom);
@@ -510,19 +499,20 @@ void OSystem_SDL::quit() {
 	SDL_RemoveTimer(_timerID);
 	closeMixer();
 
-	free(_dirtyChecksums);
-	free(_currentPalette);
-	free(_cursorPalette);
 	free(_mouseData);
 
 	delete _timer;
 
 	SDL_Quit();
 
-	// Even Manager requires save manager for storing
+	// Event Manager requires save manager for storing
 	// recorded events
 	delete getEventManager();
 	delete _savefile;
+}
+
+void OSystem_SDL::quit() {
+	deinit();
 
 #if !defined(SAMSUNGTV)
 	exit(0);
@@ -709,46 +699,44 @@ void OSystem_SDL::setupMixer() {
 	SDL_AudioSpec desired;
 
 	// Determine the desired output sampling frequency.
-	_samplesPerSec = 0;
+	uint32 samplesPerSec = 0;
 	if (ConfMan.hasKey("output_rate"))
-		_samplesPerSec = ConfMan.getInt("output_rate");
-	if (_samplesPerSec <= 0)
-		_samplesPerSec = SAMPLES_PER_SEC;
+		samplesPerSec = ConfMan.getInt("output_rate");
+	if (samplesPerSec <= 0)
+		samplesPerSec = SAMPLES_PER_SEC;
 
 	// Determine the sample buffer size. We want it to store enough data for
-	// at least 1/16th of a second (though at maximum 8192 samples). Note
+	// at least 1/16th of a second (though at most 8192 samples). Note
 	// that it must be a power of two. So e.g. at 22050 Hz, we request a
 	// sample buffer size of 2048.
-	int samples = 8192;
-	while (samples * 16 > _samplesPerSec * 2)
+	uint32 samples = 8192;
+	while (samples * 16 > samplesPerSec * 2)
 		samples >>= 1;
 
 	memset(&desired, 0, sizeof(desired));
-	desired.freq = _samplesPerSec;
+	desired.freq = samplesPerSec;
 	desired.format = AUDIO_S16SYS;
 	desired.channels = 2;
 	desired.samples = (uint16)samples;
 	desired.callback = mixCallback;
 	desired.userdata = this;
 
-	// Create the mixer instance
 	assert(!_mixer);
-	_mixer = new Audio::MixerImpl(this);
-	assert(_mixer);
-
 	if (SDL_OpenAudio(&desired, &_obtainedRate) != 0) {
 		warning("Could not open audio device: %s", SDL_GetError());
-		_samplesPerSec = 0;
+		_mixer = new Audio::MixerImpl(this, samplesPerSec);
+		assert(_mixer);
 		_mixer->setReady(false);
 	} else {
 		// Note: This should be the obtained output rate, but it seems that at
 		// least on some platforms SDL will lie and claim it did get the rate
 		// even if it didn't. Probably only happens for "weird" rates, though.
-		_samplesPerSec = _obtainedRate.freq;
-		debug(1, "Output sample rate: %d Hz", _samplesPerSec);
+		samplesPerSec = _obtainedRate.freq;
+		debug(1, "Output sample rate: %d Hz", samplesPerSec);
 
-		// Tell the mixer that we are ready and start the sound processing
-		_mixer->setOutputRate(_samplesPerSec);
+		// Create the mixer instance and start the sound processing
+		_mixer = new Audio::MixerImpl(this, samplesPerSec);
+		assert(_mixer);
 		_mixer->setReady(true);
 
 #if MIXER_DOUBLE_BUFFERING
