@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/base/commandLine.cpp $
- * $Id: commandLine.cpp 47205 2010-01-09 19:32:39Z wjpalenstijn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/base/commandLine.cpp $
+ * $Id: commandLine.cpp 53023 2010-10-04 22:42:28Z agent-q $
  *
  */
 
@@ -32,8 +32,6 @@
 #include "common/system.h"
 #include "common/fs.h"
 
-#include "sound/mididrv.h"
-
 #include "gui/ThemeEngine.h"
 
 #define DETECTOR_TESTING_HACK
@@ -51,7 +49,7 @@ static const char USAGE_STRING[] =
 ;
 
 // DONT FIXME: DO NOT ORDER ALPHABETICALLY, THIS IS ORDERED BY IMPORTANCE/CATEGORY! :)
-#if defined(PALMOS_MODE) || defined(__SYMBIAN32__) || defined(__GP32__)
+#if defined(__SYMBIAN32__) || defined(__GP32__) || defined(ANDROID) || defined(__DS__)
 static const char HELP_STRING[] = "NoUsageString"; // save more data segment space
 #else
 static const char HELP_STRING[] =
@@ -61,7 +59,7 @@ static const char HELP_STRING[] =
 	"  -h, --help               Display a brief help text and exit\n"
 	"  -z, --list-games         Display list of supported games and exit\n"
 	"  -t, --list-targets       Display list of configured targets and exit\n"
-	"  --list-saves=TARGET	    Display a list of savegames for the game (TARGET) specified\n"
+	"  --list-saves=TARGET      Display a list of savegames for the game (TARGET) specified\n"
 	"\n"
 	"  -c, --config=CONFIG      Use alternate configuration file\n"
 	"  -p, --path=PATH          Path to where the game is installed\n"
@@ -145,7 +143,7 @@ static void usage(const char *s, ...) {
 	vsnprintf(buf, STRINGBUFLEN, s, va);
 	va_end(va);
 
-#if !(defined(__GP32__) || defined (__SYMBIAN32__))
+#if !(defined(__GP32__) || defined (__SYMBIAN32__) || defined(__DS__))
 	printf(USAGE_STRING, s_appName, buf, s_appName, s_appName);
 #endif
 	exit(1);
@@ -364,8 +362,6 @@ Common::String parseCommandLine(Common::StringMap &settings, int argc, const cha
 			END_OPTION
 
 			DO_OPTION('e', "music-driver")
-				if (MidiDriver::findMusicDriver(option) == 0)
-					usage("Unrecognized music driver '%s'", option);
 			END_OPTION
 
 			DO_LONG_OPTION_INT("output-rate")
@@ -584,7 +580,7 @@ static void listTargets() {
 	ConfigManager::DomainMap::const_iterator iter;
 	for (iter = domains.begin(); iter != domains.end(); ++iter) {
 		Common::String name(iter->_key);
-		Common::String description(iter->_value.get("description"));
+		Common::String description(iter->_value.getVal("description"));
 
 		if (description.empty()) {
 			// FIXME: At this point, we should check for a "gameid" override
@@ -602,7 +598,9 @@ static void listTargets() {
 }
 
 /** List all saves states for the given target. */
-static void listSaves(const char *target) {
+static Common::Error listSaves(const char *target) {
+	Common::Error result = Common::kNoError;
+
 	// FIXME HACK
 	g_system->initBackend();
 
@@ -617,7 +615,7 @@ static void listSaves(const char *target) {
 	// Grab the gameid from the domain resp. use the target as gameid
 	Common::String gameid;
 	if (domain)
-		gameid = domain->get("gameid");
+		gameid = domain->getVal("gameid");
 	if (gameid.empty())
 		gameid = target;
 	gameid.toLowercase();	// Normalize it to lower case
@@ -627,25 +625,36 @@ static void listSaves(const char *target) {
 	GameDescriptor game = EngineMan.findGame(gameid, &plugin);
 
 	if (!plugin) {
-		error("Could not find any plugin to handle gameid '%s' (target '%s')", gameid.c_str(), target);
-		return;
+		warning("Could not find any plugin to handle target '%s' (gameid '%s')", target, gameid.c_str());
+		return Common::kPluginNotFound;
 	}
 
-	// Query the plugin for a list of savegames
-	SaveStateList saveList = (*plugin)->listSaves(target);
+	if (!(*plugin)->hasFeature(MetaEngine::kSupportsListSaves)) {
+		// TODO: Include more info about the target (desc, engine name, ...) ???
+		printf("ScummVM does not support listing save states for target '%s' (gameid '%s') .\n", target, gameid.c_str());
+		result = Common::kPluginNotSupportSaves;
+	} else {
+		// Query the plugin for a list of savegames
+		SaveStateList saveList = (*plugin)->listSaves(target);
 
-	// TODO: Include more info about the target (desc, engine name, ...) ???
-	printf("Saves for target '%s':\n", target);
-	printf("  Slot Description                                           \n"
-	       "  ---- ------------------------------------------------------\n");
+		// TODO: Include more info about the target (desc, engine name, ...) ???
+		printf("Saves for target '%s' (gameid '%s'):\n", target, gameid.c_str());
+		printf("  Slot Description                                           \n"
+		       "  ---- ------------------------------------------------------\n");
 
-	for (SaveStateList::const_iterator x = saveList.begin(); x != saveList.end(); ++x) {
-		printf("  %-4s %s\n", x->save_slot().c_str(), x->description().c_str());
-		// TODO: Could also iterate over the full hashmap, printing all key-value pairs
+		if (saveList.size() == 0)
+			result = Common::kNoSavesError;
+
+		for (SaveStateList::const_iterator x = saveList.begin(); x != saveList.end(); ++x) {
+			printf("  %-4s %s\n", x->save_slot().c_str(), x->description().c_str());
+			// TODO: Could also iterate over the full hashmap, printing all key-value pairs
+		}
 	}
 
 	// Revert to the old active domain
 	ConfMan.setActiveDomain(oldDomain);
+
+	return result;
 }
 
 /** Lists all usable themes */
@@ -674,8 +683,8 @@ static void runDetectorTest() {
 	int success = 0, failure = 0;
 	for (iter = domains.begin(); iter != domains.end(); ++iter) {
 		Common::String name(iter->_key);
-		Common::String gameid(iter->_value.get("gameid"));
-		Common::String path(iter->_value.get("path"));
+		Common::String gameid(iter->_value.getVal("gameid"));
+		Common::String path(iter->_value.getVal("path"));
 		printf("Looking at target '%s', gameid '%s', path '%s' ...\n",
 				name.c_str(), gameid.c_str(), path.c_str());
 		if (path.empty()) {
@@ -748,8 +757,8 @@ void upgradeTargets() {
 	for (iter = domains.begin(); iter != domains.end(); ++iter) {
 		Common::ConfigManager::Domain &dom = iter->_value;
 		Common::String name(iter->_key);
-		Common::String gameid(dom.get("gameid"));
-		Common::String path(dom.get("path"));
+		Common::String gameid(dom.getVal("gameid"));
+		Common::String path(dom.getVal("path"));
 		printf("Looking at target '%s', gameid '%s' ...\n",
 				name.c_str(), gameid.c_str());
 		if (path.empty()) {
@@ -768,9 +777,9 @@ void upgradeTargets() {
 			continue;
 		}
 
-		Common::Language lang = Common::parseLanguage(dom.get("language"));
-		Common::Platform plat = Common::parsePlatform(dom.get("platform"));
-		Common::String desc(dom.get("description"));
+		Common::Language lang = Common::parseLanguage(dom.getVal("language"));
+		Common::Platform plat = Common::parsePlatform(dom.getVal("platform"));
+		Common::String desc(dom.getVal("description"));
 
 		GameList candidates(EngineMan.detectGames(files));
 		GameDescriptor *g = 0;
@@ -858,7 +867,7 @@ Common::String parseCommandLine(Common::StringMap &settings, int argc, const cha
 #endif // DISABLE_COMMAND_LINE
 
 
-bool processSettings(Common::String &command, Common::StringMap &settings) {
+Common::Error processSettings(Common::String &command, Common::StringMap &settings) {
 
 #ifndef DISABLE_COMMAND_LINE
 
@@ -867,34 +876,33 @@ bool processSettings(Common::String &command, Common::StringMap &settings) {
 	// have been loaded.
 	if (command == "list-targets") {
 		listTargets();
-		return false;
+		return Common::kNoError;
 	} else if (command == "list-games") {
 		listGames();
-		return false;
+		return Common::kNoError;
 	} else if (command == "list-saves") {
-		listSaves(settings["list-saves"].c_str());
-		return false;
+		return listSaves(settings["list-saves"].c_str());
 	} else if (command == "list-themes") {
 		listThemes();
-		return false;
+		return Common::kNoError;
 	} else if (command == "version") {
 		printf("%s\n", gScummVMFullVersion);
 		printf("Features compiled in: %s\n", gScummVMFeatures);
-		return false;
+		return Common::kNoError;
 	} else if (command == "help") {
 		printf(HELP_STRING, s_appName);
-		return false;
+		return Common::kNoError;
 	}
 #ifdef DETECTOR_TESTING_HACK
 	else if (command == "test-detector") {
 		runDetectorTest();
-		return false;
+		return Common::kNoError;
 	}
 #endif
 #ifdef UPGRADE_ALL_TARGETS_HACK
 	else if (command == "upgrade-targets") {
 		upgradeTargets();
-		return false;
+		return Common::kNoError;
 	}
 #endif
 
@@ -936,7 +944,7 @@ bool processSettings(Common::String &command, Common::StringMap &settings) {
 	// environment variable. This is weaker than a --savepath on the
 	// command line, but overrides the default savepath, hence it is
 	// handled here, just before the command line gets parsed.
-#if !defined(MACOS_CARBON) && !defined(_WIN32_WCE) && !defined(PALMOS_MODE) && !defined(__GP32__)
+#if !defined(_WIN32_WCE) && !defined(__GP32__) && !defined(ANDROID) && !defined(__DS__)
 	if (!settings.contains("savepath")) {
 		const char *dir = getenv("SCUMMVM_SAVEPATH");
 		if (dir && *dir && strlen(dir) < MAXPATHLEN) {
@@ -966,7 +974,7 @@ bool processSettings(Common::String &command, Common::StringMap &settings) {
 		ConfMan.set(key, value, Common::ConfigManager::kTransientDomain);
 	}
 
-	return true;
+	return Common::kArgumentNotProcessed;
 }
 
 } // End of namespace Base
