@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/common/config-file.cpp $
- * $Id: config-file.cpp 46746 2009-12-30 14:55:33Z mthreepwood $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/common/config-file.cpp $
+ * $Id: config-file.cpp 49843 2010-06-15 12:33:20Z fingolfin $
  *
  */
 
@@ -125,7 +125,7 @@ bool ConfigFile::loadFromStream(SeekableReadStream &stream) {
 			if (*p == '\0')
 				error("ConfigFile::loadFromStream: missing ] in line %d", lineno);
 			else if (*p != ']')
-				error("ConfigFile::loadFromStream: Invalid character '%c' occured in section name in line %d", *p, lineno);
+				error("ConfigFile::loadFromStream: Invalid character '%c' occurred in section name in line %d", *p, lineno);
 
 			// Previous section is finished now, store it.
 			if (!section.name.empty())
