@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/common/config-manager.cpp $
- * $Id: config-manager.cpp 46781 2009-12-30 23:01:36Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/common/config-manager.cpp $
+ * $Id: config-manager.cpp 52296 2010-08-23 10:00:40Z fingolfin $
  *
  */
 
@@ -29,7 +29,7 @@
 #include "common/util.h"
 #include "common/system.h"
 
-DECLARE_SINGLETON(Common::ConfigManager);
+DECLARE_SINGLETON(Common::ConfigManager)
 
 static bool isValidDomainName(const Common::String &domName) {
 	const char *p = domName.c_str();
@@ -61,13 +61,18 @@ void ConfigManager::loadDefaultConfigFile() {
 	_filename.clear();	// clear the filename to indicate that we are using the default config file
 
 	// ... load it, if available ...
-	if (stream)
+	if (stream) {
 		loadFromStream(*stream);
 
-	// ... and close it again.
-	delete stream;
+		// ... and close it again.
+		delete stream;
 
-	flushToDisk();
+	} else {
+		// No config file -> create new one!
+		printf("Default configuration file missing, creating a new one\n");
+
+		flushToDisk();
+	}
 }
 
 void ConfigManager::loadConfigFile(const String &filename) {
@@ -214,32 +219,27 @@ void ConfigManager::flushToDisk() {
 		stream = dump;
 	}
 
+	// Write the application domain
+	writeDomain(*stream, kApplicationDomain, _appDomain);
+
+#ifdef ENABLE_KEYMAPPER
+	// Write the keymapper domain
+	writeDomain(*stream, kKeymapperDomain, _keymapperDomain);
+#endif
+
 	// First write the domains in _domainSaveOrder, in that order.
 	// Note: It's possible for _domainSaveOrder to list domains which
-	// are not present anymore.
-	StringList::const_iterator i;
+	// are not present anymore, so we validate each name.
+	Array<String>::const_iterator i;
 	for (i = _domainSaveOrder.begin(); i != _domainSaveOrder.end(); ++i) {
-		if (kApplicationDomain == *i) {
-			writeDomain(*stream, *i, _appDomain);
-#ifdef ENABLE_KEYMAPPER
-		} else if (kKeymapperDomain == *i) {
-			writeDomain(*stream, *i, _keymapperDomain);
-#endif
-		} else if (_gameDomains.contains(*i)) {
+		if (_gameDomains.contains(*i)) {
 			writeDomain(*stream, *i, _gameDomains[*i]);
 		}
 	}
 
 	DomainMap::const_iterator d;
 
-
 	// Now write the domains which haven't been written yet
-	if (find(_domainSaveOrder.begin(), _domainSaveOrder.end(), kApplicationDomain) == _domainSaveOrder.end())
-		writeDomain(*stream, kApplicationDomain, _appDomain);
-#ifdef ENABLE_KEYMAPPER
-	if (find(_domainSaveOrder.begin(), _domainSaveOrder.end(), kKeymapperDomain) == _domainSaveOrder.end())
-		writeDomain(*stream, kKeymapperDomain, _keymapperDomain);
-#endif
 	for (d = _gameDomains.begin(); d != _gameDomains.end(); ++d) {
 		if (find(_domainSaveOrder.begin(), _domainSaveOrder.end(), d->_key) == _domainSaveOrder.end())
 			writeDomain(*stream, d->_key, d->_value);
@@ -390,10 +390,8 @@ const String & ConfigManager::get(const String &key) const {
 		return (*_activeDomain)[key];
 	else if (_appDomain.contains(key))
 		return _appDomain[key];
-	else if (_defaultsDomain.contains(key))
-		return _defaultsDomain[key];
 
-	return _emptyString;
+	return _defaultsDomain.getVal(key);
 }
 
 const String & ConfigManager::get(const String &key, const String &domName) const {
@@ -412,18 +410,7 @@ const String & ConfigManager::get(const String &key, const String &domName) cons
 	if (domain->contains(key))
 		return (*domain)[key];
 
-	return _defaultsDomain.get(key);
-
-	if (!domain->contains(key)) {
-#if 1
-		return _emptyString;
-#else
-		error("ConfigManager::get(%s,%s) called on non-existent key",
-					key.c_str(), domName.c_str());
-#endif
-	}
-
-	return (*domain)[key];
+	return _defaultsDomain.getVal(key);
 }
 
 int ConfigManager::getInt(const String &key, const String &domName) const {
@@ -614,14 +601,6 @@ bool ConfigManager::hasGameDomain(const String &domName) const {
 #pragma mark -
 
 
-const String &ConfigManager::Domain::get(const String &key) const {
-	const_iterator iter(find(key));
-	if (iter != end())
-		return iter->_value;
-
-	return ConfMan._emptyString;
-}
-
 void ConfigManager::Domain::setDomainComment(const String &comment) {
 	_domainComment = comment;
 }
