@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/common/savefile.h $
- * $Id: savefile.h 41000 2009-05-29 14:38:22Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/common/savefile.h $
+ * $Id: savefile.h 49843 2010-06-15 12:33:20Z fingolfin $
  *
  */
 
@@ -29,11 +29,12 @@
 #include "common/noncopyable.h"
 #include "common/scummsys.h"
 #include "common/stream.h"
-#include "common/str.h"
+#include "common/str-array.h"
 #include "common/error.h"
 
 namespace Common {
 
+
 /**
  * A class which allows game engines to load game state data.
  * That typically means "save games", but also includes things like the
@@ -108,14 +109,14 @@ public:
 	/**
 	 * Open the savefile with the specified name in the given directory for saving.
 	 * @param name	the name of the savefile
-	 * @return pointer to an OutSaveFile, or NULL if an error occured.
+	 * @return pointer to an OutSaveFile, or NULL if an error occurred.
 	 */
 	virtual OutSaveFile *openForSaving(const String &name) = 0;
 
 	/**
 	 * Open the file with the specified name in the given directory for loading.
 	 * @param name	the name of the savefile
-	 * @return pointer to an InSaveFile, or NULL if an error occured.
+	 * @return pointer to an InSaveFile, or NULL if an error occurred.
 	 */
 	virtual InSaveFile *openForLoading(const String &name) = 0;
 
@@ -142,7 +143,7 @@ public:
 	 * @return list of strings for all present file names.
 	 * @see Common::matchString()
 	 */
-	virtual StringList listSavefiles(const String &pattern) = 0;
+	virtual StringArray listSavefiles(const String &pattern) = 0;
 };
 
 } // End of namespace Common
