@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/common/str.cpp $
- * $Id: str.cpp 47744 2010-01-31 10:22:09Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/common/str.cpp $
+ * $Id: str.cpp 52618 2010-09-07 11:40:44Z wjpalenstijn $
  */
 
 #include "common/str.h"
@@ -30,11 +30,6 @@
 
 #include <stdarg.h>
 
-#if !defined(__SYMBIAN32__)
-#include <new>
-#endif
-
-
 namespace Common {
 
 MemoryPool *g_refCountPool = 0; // FIXME: This is never freed right now
@@ -105,8 +100,6 @@ String::String(char c)
 	_storage[0] = c;
 	_storage[1] = 0;
 
-	// TODO/FIXME: There is no reason for the following check -- we *do*
-	// allow strings to contain 0 bytes!
 	_size = (c == 0) ? 0 : 1;
 }
 
@@ -151,7 +144,11 @@ void String::ensureCapacity(uint32 new_size, bool keep_old) {
 		// We need to allocate storage on the heap!
 
 		// Compute a suitable new capacity limit
-		newCapacity = MAX(curCapacity * 2, computeCapacity(new_size+1));
+		// If the current capacity is sufficient we use the same capacity
+		if (new_size < curCapacity)
+			newCapacity = curCapacity;
+		else
+			newCapacity = MAX(curCapacity * 2, computeCapacity(new_size+1));
 
 		// Allocate new storage
 		newStorage = new char[newCapacity];
@@ -252,9 +249,11 @@ String &String::operator=(const String &str) {
 String &String::operator=(char c) {
 	decRefCount(_extern._refCount);
 	_str = _storage;
-	_size = 1;
+
 	_str[0] = c;
 	_str[1] = 0;
+
+	_size = (c == 0) ? 0 : 1;
 	return *this;
 }
 
@@ -417,7 +416,7 @@ void String::trim() {
 
 	// Trim leading whitespace
 	char *t = _str;
-	while (isspace(*t))
+	while (isspace((unsigned char)*t))
 		t++;
 
 	if (t != _str) {
@@ -622,7 +621,7 @@ Common::String lastPathComponent(const Common::String &path, const char sep) {
 
 	// Now scan the whole component
 	const char *first = last - 1;
-	while (first >= str && *first != sep)
+	while (first > str && *first != sep)
 		--first;
 
 	if (*first == sep)
@@ -692,9 +691,18 @@ bool matchString(const char *str, const char *pat, bool ignoreCase, bool pathMod
 
 		switch (*pat) {
 		case '*':
-			// Record pattern / string position for backtracking
-			p = ++pat;
-			q = str;
+			if (*str) {
+				// Record pattern / string position for backtracking
+				p = ++pat;
+				q = str;
+			} else {
+				// If we've reached the end of str, we can't backtrack further
+				// NB: We can't simply check if pat also ended here, because
+				// the pattern might end with any number of *s.
+				++pat;
+				p = 0;
+				q = 0;
+			}
 			// If pattern ended with * -> match
 			if (!*pat)
 				return true;
@@ -724,4 +732,99 @@ bool matchString(const char *str, const char *pat, bool ignoreCase, bool pathMod
 	}
 }
 
+String tag2string(uint32 tag) {
+	char str[5];
+	str[0] = (char)(tag >> 24);
+	str[1] = (char)(tag >> 16);
+	str[2] = (char)(tag >> 8);
+	str[3] = (char)tag;
+	str[4] = '\0';
+	// Replace non-printable chars by dot
+	for (int i = 0; i < 4; ++i) {
+		if (!isprint((unsigned char)str[i]))
+			str[i] = '.';
+	}
+	return Common::String(str);
+}
+
+size_t strlcpy(char *dst, const char *src, size_t size) {
+	// Our backup of the source's start, we need this
+	// to calculate the source's length.
+	const char * const srcStart = src;
+
+	// In case a non-empty size was specified we
+	// copy over (size - 1) bytes at max.
+	if (size != 0) {
+		// Copy over (size - 1) bytes at max.
+		while (--size != 0) {
+			if ((*dst++ = *src) == 0)
+				break;
+			++src;
+		}
+
+		// In case the source string was longer than the
+		// destination, we need to add a terminating
+		// zero.
+		if (size == 0)
+			*dst = 0;
+	}
+
+	// Move to the terminating zero of the source
+	// string, we need this to determin the length
+	// of the source string.
+	while (*src)
+		++src;
+
+	// Return the source string's length.
+	return src - srcStart;
+}
+
+size_t strlcat(char *dst, const char *src, size_t size) {
+	// In case the destination buffer does not contain
+	// space for at least 1 character, we will just
+	// return the source string's length.
+	if (size == 0)
+		return strlen(src);
+
+	// Our backup of the source's start, we need this
+	// to calculate the source's length.
+	const char * const srcStart = src;
+
+	// Our backup of the destination's start, we need
+	// this to calculate the destination's length.
+	const char * const dstStart = dst;
+
+	// Search the end of the destination, but do not
+	// move past the terminating zero.
+	while (size-- != 0 && *dst != 0)
+		++dst;
+
+	// Calculate the destination's length;
+	const size_t dstLength = dst - dstStart;
+
+	// In case we reached the end of the destination
+	// buffer before we had a chance to append any
+	// characters we will just return the destination
+	// length plus the source string's length.
+	if (size == 0)
+		return dstLength + strlen(srcStart);
+
+	// Copy over all of the source that fits
+	// the destination buffer. We also need
+	// to take the terminating zero we will
+	// add into consideration.
+	while (size-- != 0 && *src != 0)
+		*dst++ = *src++;
+	*dst = 0;
+
+	// Move to the terminating zero of the source
+	// string, we need this to determin the length
+	// of the source string.
+	while (*src)
+		++src;
+
+	// Return the total length of the result string
+	return dstLength + (src - srcStart);
+}
+
 }	// End of namespace Common
