@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/common/stream.h $
- * $Id: stream.h 47180 2010-01-08 22:07:35Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/common/stream.h $
+ * $Id: stream.h 52724 2010-09-14 14:57:30Z Bluddy $
  *
  */
 
@@ -149,14 +149,13 @@ public:
 	void writeString(const String &str);
 };
 
-
 /**
  * Generic interface for a readable data stream.
  */
 class ReadStream : virtual public Stream {
 public:
 	/**
-	 * Returns true if a read failed because the stream has been reached.
+	 * Returns true if a read failed because the stream end has been reached.
 	 * This flag is cleared by clearErr().
 	 * For a SeekableReadStream, it is also cleared by a successful seek.
 	 */
@@ -495,16 +494,17 @@ protected:
 	DisposeAfterUse::Flag _disposeParentStream;
 	byte *_buf;
 	uint32 _pos;
+	bool _eos; // end of stream
 	uint32 _bufSize;
 	uint32 _realBufSize;
 
 public:
 	BufferedReadStream(ReadStream *parentStream, uint32 bufSize, DisposeAfterUse::Flag disposeParentStream = DisposeAfterUse::NO);
-	~BufferedReadStream();
+	virtual ~BufferedReadStream();
 
-	virtual bool eos() const { return (_pos == _bufSize) && _parentStream->eos(); }
+	virtual bool eos() const { return _eos; }
 	virtual bool err() const { return _parentStream->err(); }
-	virtual void clearErr() { _parentStream->clearErr(); }
+	virtual void clearErr() { _eos = false; _parentStream->clearErr(); }
 
 	virtual uint32 read(void *dataPtr, uint32 dataSize);
 };
@@ -525,7 +525,25 @@ public:
 	virtual bool seek(int32 offset, int whence = SEEK_SET);
 };
 
+/**
+ * Wrapper class which adds buffering to any WriteStream.
+ */
+class BufferedWriteStream : public WriteStream {
+protected:
+	WriteStream *_parentStream;
+	DisposeAfterUse::Flag _disposeParentStream;
+	byte *_buf;
+	uint32 _pos;
+	uint32 _bufSize;
+	bool flushBuffer();						// write out the data in the buffer
 
+public:
+	BufferedWriteStream(WriteStream *parentStream, uint32 bufSize, DisposeAfterUse::Flag disposeParentStream = DisposeAfterUse::NO);
+	virtual ~BufferedWriteStream();
+	
+	virtual uint32 write(const void *dataPtr, uint32 dataSize);
+	virtual bool flush();
+};
 
 /**
  * Simple memory based 'stream', which implements the ReadStream interface for
@@ -687,6 +705,8 @@ public:
 	uint32 size() const { return _size; }
 
 	byte *getData() { return _data; }
+
+	bool seek(int32 offset, int whence = SEEK_SET);
 };
 
 }	// End of namespace Common
