@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/common/unzip.cpp $
- * $Id: unzip.cpp 48155 2010-02-28 14:40:40Z Hkz $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/common/unzip.cpp $
+ * $Id: unzip.cpp 52204 2010-08-19 10:15:12Z sev $
  */
 
 /* unzip.c -- IO on .zip files using zlib
@@ -79,10 +79,37 @@
 #include <zlib.h>
 #endif
 
+#else  // !USE_ZLIB
+
+// Even when zlib is not linked in, we can still open ZIP archives and read
+// uncompressed files from them.  Attempted decompression of compressed files
+// will result in an error.
+//
+// Define the constants and types used by zlib.
+#define Z_ERRNO -1
+#define Z_OK 0
+#define Z_DEFLATED 8
+typedef void *voidp;
+typedef unsigned int uInt;
+typedef unsigned long uLong;
+typedef long z_off_t;
+typedef unsigned char Byte;
+typedef Byte Bytef;
+typedef struct {
+    Bytef    *next_in, *next_out;
+    uInt     avail_in, avail_out;
+    uLong    total_out;
+} z_stream;
+
+#endif  // !USE_ZLIB
+
 #include "common/fs.h"
 #include "common/unzip.h"
 #include "common/file.h"
 
+#include "common/hashmap.h"
+#include "common/hash-str.h"
+
 #if defined(STRICTUNZIP) || defined(STRICTZIPUNZIP)
 /* like the STRICT of WIN32, we define a pointer that cannot be converted
     from (void*) without cast */
@@ -338,6 +365,16 @@ typedef struct {
 	uLong byte_before_the_zipfile;/* byte before the zipfile, (>0 for sfx)*/
 } file_in_zip_read_info_s;
 
+typedef struct {
+	uLong num_file;					/* number of the current file in the zipfile*/
+	uLong pos_in_central_dir;		/* pos of the current file in the central dir*/
+	uLong current_file_ok;			/* flag about the usability of the current file*/
+	unz_file_info cur_file_info;					/* public info about the current file in zip*/
+	unz_file_info_internal cur_file_info_internal;	/* private info about it*/
+} cached_file_in_zip;
+
+typedef Common::HashMap<Common::String, cached_file_in_zip, Common::IgnoreCase_Hash, 
+	Common::IgnoreCase_EqualTo> ZipHash;
 
 /* unz_s contain internal information about the zipfile
 */
@@ -358,6 +395,7 @@ typedef struct {
 	unz_file_info_internal cur_file_info_internal;	/* private info about it*/
 	file_in_zip_read_info_s* pfile_in_zip_read;		/* structure about the current
 													file if we are decompressing it */
+	ZipHash _hash;
 } unz_s;
 
 /* ===========================================================================
@@ -565,7 +603,27 @@ unzFile unzOpen(Common::SeekableReadStream *stream) {
 	us->central_pos = central_pos;
 	us->pfile_in_zip_read = NULL;
 
-	unzGoToFirstFile((unzFile)us);
+	err = unzGoToFirstFile((unzFile)us);
+
+	while (err == UNZ_OK) {
+		// Get the file details
+		char szCurrentFileName[UNZ_MAXFILENAMEINZIP+1];
+		unzGetCurrentFileInfo(us, NULL, szCurrentFileName, sizeof(szCurrentFileName) - 1,
+							NULL, 0, NULL, 0);
+
+		// Save details into the hash
+		cached_file_in_zip fe;
+		fe.num_file = us->num_file;
+		fe.pos_in_central_dir = us->pos_in_central_dir;
+		fe.current_file_ok = us->current_file_ok;
+		fe.cur_file_info = us->cur_file_info;
+		fe.cur_file_info_internal = us->cur_file_info_internal;
+
+		us->_hash[Common::String(szCurrentFileName)] = fe;
+
+		// Move to the next file
+		err = unzGoToNextFile((unzFile)us);
+	}
 	return (unzFile)us;
 }
 
@@ -846,7 +904,6 @@ int unzGoToNextFile(unzFile file) {
 	return err;
 }
 
-
 /*
   Try locate the file szFileName in the zipfile.
   For the iCaseSensitivity signification, see unzipStringFileNameCompare
@@ -857,12 +914,6 @@ int unzGoToNextFile(unzFile file) {
 */
 int unzLocateFile(unzFile file, const char *szFileName, int iCaseSensitivity) {
 	unz_s* s;
-	int err;
-
-
-	uLong num_fileSaved;
-	uLong pos_in_central_dirSaved;
-
 
 	if (file==NULL)
 		return UNZ_PARAMERROR;
@@ -874,25 +925,20 @@ int unzLocateFile(unzFile file, const char *szFileName, int iCaseSensitivity) {
 	if (!s->current_file_ok)
 		return UNZ_END_OF_LIST_OF_FILE;
 
-	num_fileSaved = s->num_file;
-	pos_in_central_dirSaved = s->pos_in_central_dir;
+	// Check to see if the entry exists
+	ZipHash::iterator i = s->_hash.find(Common::String(szFileName));
+	if (i == s->_hash.end())
+		return UNZ_END_OF_LIST_OF_FILE;
 
-	err = unzGoToFirstFile(file);
+	// Found it, so reset the details in the main structure
+	cached_file_in_zip &fe = i->_value;
+	s->num_file = fe.num_file;
+	s->pos_in_central_dir = fe.pos_in_central_dir;
+	s->current_file_ok = fe.current_file_ok;
+	s->cur_file_info = fe.cur_file_info;
+	s->cur_file_info_internal = fe.cur_file_info_internal;
 
-	while (err == UNZ_OK) {
-		char szCurrentFileName[UNZ_MAXFILENAMEINZIP+1];
-		unzGetCurrentFileInfo(file,NULL,
-								szCurrentFileName,sizeof(szCurrentFileName)-1,
-								NULL,0,NULL,0);
-		if (unzStringFileNameCompare(szCurrentFileName,
-										szFileName,iCaseSensitivity)==0)
-			return UNZ_OK;
-		err = unzGoToNextFile(file);
-	}
-
-	s->num_file = num_fileSaved ;
-	s->pos_in_central_dir = pos_in_central_dirSaved ;
-	return err;
+	return UNZ_OK;
 }
 
 
@@ -1044,6 +1090,7 @@ int unzOpenCurrentFile (unzFile file) {
 	pfile_in_zip_read_info->stream.total_out = 0;
 
 	if (!Store) {
+#ifdef USE_ZLIB
 		pfile_in_zip_read_info->stream.zalloc = (alloc_func)0;
 		pfile_in_zip_read_info->stream.zfree = (free_func)0;
 		pfile_in_zip_read_info->stream.opaque = (voidpf)0;
@@ -1058,6 +1105,9 @@ int unzOpenCurrentFile (unzFile file) {
 	 * In unzip, i don't wait absolutely Z_STREAM_END because I known the
 	 * size of both compressed and uncompressed data
 	 */
+#else
+		err=UNZ_BADZIPFILE;
+#endif
 	}
 	pfile_in_zip_read_info->rest_read_compressed = s->cur_file_info.compressed_size;
 	pfile_in_zip_read_info->rest_read_uncompressed = s->cur_file_info.uncompressed_size;
@@ -1068,9 +1118,8 @@ int unzOpenCurrentFile (unzFile file) {
 
 	pfile_in_zip_read_info->stream.avail_in = (uInt)0;
 
-
 	s->pfile_in_zip_read = pfile_in_zip_read_info;
-	return UNZ_OK;
+	return err;
 }
 
 
@@ -1143,9 +1192,11 @@ int unzReadCurrentFile(unzFile file, voidp buf, unsigned len) {
 			for (i=0;i<uDoCopy;i++)
 				*(pfile_in_zip_read_info->stream.next_out+i) = *(pfile_in_zip_read_info->stream.next_in+i);
 
+#ifdef USE_ZLIB
 			pfile_in_zip_read_info->crc32_data = crc32(pfile_in_zip_read_info->crc32_data,
 								pfile_in_zip_read_info->stream.next_out,
 								uDoCopy);
+#endif  // otherwise leave crc32_data as is and it won't be verified at the end
 			pfile_in_zip_read_info->rest_read_uncompressed-=uDoCopy;
 			pfile_in_zip_read_info->stream.avail_in -= uDoCopy;
 			pfile_in_zip_read_info->stream.avail_out -= uDoCopy;
@@ -1154,6 +1205,7 @@ int unzReadCurrentFile(unzFile file, voidp buf, unsigned len) {
 			pfile_in_zip_read_info->stream.total_out += uDoCopy;
 			iRead += uDoCopy;
 		} else {
+#ifdef USE_ZLIB
 			uLong uTotalOutBefore,uTotalOutAfter;
 			const Bytef *bufBefore;
 			uLong uOutThis;
@@ -1184,6 +1236,11 @@ int unzReadCurrentFile(unzFile file, voidp buf, unsigned len) {
 				return (iRead==0) ? UNZ_EOF : iRead;
 			if (err!=Z_OK)
 				break;
+#else
+			// Cannot decompress the file without zlib.
+			err = UNZ_BADZIPFILE;
+			break;
+#endif
 		}
 	}
 
@@ -1302,16 +1359,20 @@ int unzCloseCurrentFile(unzFile file) {
 		return UNZ_PARAMERROR;
 
 
+#ifdef USE_ZLIB
+	// Only verify crc32_data when zlib is linked in, because otherwise crc32() is
+	// not defined.
 	if (pfile_in_zip_read_info->rest_read_uncompressed == 0) {
 		if (pfile_in_zip_read_info->crc32_data != pfile_in_zip_read_info->crc32_wait)
 			err=UNZ_CRCERROR;
 	}
+	if (pfile_in_zip_read_info->stream_initialised)
+		inflateEnd(&pfile_in_zip_read_info->stream);
+#endif
 
 
 	free(pfile_in_zip_read_info->read_buffer);
 	pfile_in_zip_read_info->read_buffer = NULL;
-	if (pfile_in_zip_read_info->stream_initialised)
-		inflateEnd(&pfile_in_zip_read_info->stream);
 
 	pfile_in_zip_read_info->stream_initialised = 0;
 	free(pfile_in_zip_read_info);
@@ -1433,11 +1494,11 @@ Common::SeekableReadStream *ZipArchive::createReadStreamForMember(const Common::
 	unz_file_info fileInfo;
 	unzOpenCurrentFile(_zipFile);
 	unzGetCurrentFileInfo(_zipFile, &fileInfo, NULL, 0, NULL, 0, NULL, 0);
-	byte *buffer = (byte *)calloc(fileInfo.uncompressed_size+1, 1);
+	byte *buffer = (byte *)malloc(fileInfo.uncompressed_size);
 	assert(buffer);
 	unzReadCurrentFile(_zipFile, buffer, fileInfo.uncompressed_size);
 	unzCloseCurrentFile(_zipFile);
-	return new Common::MemoryReadStream(buffer, fileInfo.uncompressed_size+1, DisposeAfterUse::YES);
+	return new Common::MemoryReadStream(buffer, fileInfo.uncompressed_size, DisposeAfterUse::YES);
 
 	// FIXME: instead of reading all into a memory stream, we could
 	// instead create a new ZipStream class. But then we have to be
@@ -1466,5 +1527,3 @@ Archive *makeZipArchive(SeekableReadStream *stream) {
 }
 
 }	// End of namespace Common
-
-#endif
