@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/advancedDetector.cpp $
- * $Id: advancedDetector.cpp 48750 2010-04-20 17:24:13Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/advancedDetector.cpp $
+ * $Id: advancedDetector.cpp 52386 2010-08-25 12:03:43Z lordhoto $
  *
  */
 
@@ -29,6 +29,7 @@
 #include "common/util.h"
 #include "common/hash-str.h"
 #include "common/file.h"
+#include "common/macresman.h"
 #include "common/md5.h"
 #include "common/config-manager.h"
 
@@ -207,12 +208,42 @@ static void updateGameDescriptor(GameDescriptor &desc, const ADGameDescription *
 		desc["extra"] = realDesc->extra;
 
 	desc.setGUIOptions(realDesc->guioptions | params.guioptions);
+	desc.appendGUIOptions(getGameGUIOptionsDescriptionLanguage(realDesc->language));
+
+	if (realDesc->flags & ADGF_ADDENGLISH)
+		desc.appendGUIOptions(getGameGUIOptionsDescriptionLanguage(Common::EN_ANY));
 }
 
+bool cleanupPirated(ADGameDescList &matched) {
+	// OKay, now let's sense presense of pirated games
+	if (!matched.empty()) {
+		for (uint j = 0; j < matched.size();) {
+			if (matched[j]->flags & ADGF_PIRATED)
+				matched.remove_at(j);
+			else
+				++j;
+		}
+
+		// We ruled out all variants and now have nothing
+		if (matched.empty()) {
+			
+			warning("Illegitimate copy of the game detected. We give no support in such cases %d", matched.size());
+
+			return true;
+		}
+	}
+
+	return false;
+}
+
+
 GameList AdvancedMetaEngine::detectGames(const Common::FSList &fslist) const {
 	ADGameDescList matches = detectGame(fslist, params, Common::UNK_LANG, Common::kPlatformUnknown, "");
 	GameList detectedGames;
 
+	if (cleanupPirated(matches))
+		return detectedGames;
+
 	// Use fallback detector if there were no matches by other means
 	if (matches.empty()) {
 		const ADGameDescription *fallbackDesc = fallbackDetect(fslist);
@@ -264,7 +295,7 @@ Common::Error AdvancedMetaEngine::createInstance(OSystem *syst, Engine **engine)
 		// config file.
 		//
 		// Fixes bug #1544799
-		ConfMan.set("autoadded", "true");
+		ConfMan.setBool("autoadded", true);
 
 		warning("No path was provided. Assuming the data files are in the current directory");
 	}
@@ -277,6 +308,9 @@ Common::Error AdvancedMetaEngine::createInstance(OSystem *syst, Engine **engine)
 
 	ADGameDescList matches = detectGame(files, params, language, platform, extra);
 
+	if (cleanupPirated(matches))
+		return Common::kNoGameDataFoundError;
+
 	if (params.singleid == NULL) {
 		for (uint i = 0; i < matches.size(); i++) {
 			if (matches[i]->gameid == gameid) {
@@ -304,7 +338,12 @@ Common::Error AdvancedMetaEngine::createInstance(OSystem *syst, Engine **engine)
 
 	// If the GUI options were updated, we catch this here and update them in the users config
 	// file transparently.
-	Common::updateGameGUIOptions(agdDesc->guioptions | params.guioptions);
+	Common::String lang = getGameGUIOptionsDescriptionLanguage(agdDesc->language);
+	if (agdDesc->flags & ADGF_ADDENGLISH)
+		lang += " " + getGameGUIOptionsDescriptionLanguage(Common::EN_ANY);
+
+	Common::updateGameGUIOptions(agdDesc->guioptions | params.guioptions, lang);
+
 
 	debug(2, "Running %s", toGameDescriptor(*agdDesc, params.list).description().c_str());
 	if (!createInstance(syst, engine, agdDesc))
@@ -332,13 +371,55 @@ static void reportUnknown(const Common::FSNode &path, const SizeMD5Map &filesSiz
 	printf("of the game you tried to add and its version/language/etc.:\n");
 
 	for (SizeMD5Map::const_iterator file = filesSizeMD5.begin(); file != filesSizeMD5.end(); ++file)
-		printf("  \"%s\", \"%s\", %d\n", file->_key.c_str(), file->_value.md5, file->_value.size);
+		printf("  {\"%s\", 0, \"%s\", %d},\n", file->_key.c_str(), file->_value.md5, file->_value.size);
 
 	printf("\n");
 }
 
 static ADGameDescList detectGameFilebased(const FileMap &allFiles, const ADParams &params);
 
+static void composeFileHashMap(const Common::FSList &fslist, FileMap &allFiles, int depth, const char * const *directoryGlobs) {
+	if (depth <= 0)
+		return;
+
+	if (fslist.empty())
+		return;
+
+	// First we compose a hashmap of all files in fslist.
+	// Includes nifty stuff like removing trailing dots and ignoring case.
+	for (Common::FSList::const_iterator file = fslist.begin(); file != fslist.end(); ++file) {
+		if (file->isDirectory()) {
+			Common::FSList files;
+
+			if (!directoryGlobs)
+				continue;
+
+			bool matched = false;
+			for (const char * const *glob = directoryGlobs; *glob; glob++)
+				if (file->getName().matchString(*glob, true)) {
+					matched = true;
+					break;
+				}
+					
+			if (!matched)
+				continue;
+
+			if (!file->getChildren(files, Common::FSNode::kListAll))
+				continue;
+
+			composeFileHashMap(files, allFiles, depth - 1, directoryGlobs);
+		}
+
+		Common::String tstr = file->getName();
+
+		// Strip any trailing dot
+		if (tstr.lastChar() == '.')
+			tstr.deleteLastChar();
+
+		allFiles[tstr] = *file;	// Record the presence of this file
+	}
+}
+
 static ADGameDescList detectGame(const Common::FSList &fslist, const ADParams &params, Common::Language language, Common::Platform platform, const Common::String &extra) {
 	FileMap allFiles;
 	SizeMD5Map filesSizeMD5;
@@ -354,18 +435,7 @@ static ADGameDescList detectGame(const Common::FSList &fslist, const ADParams &p
 
 	// First we compose a hashmap of all files in fslist.
 	// Includes nifty stuff like removing trailing dots and ignoring case.
-	for (Common::FSList::const_iterator file = fslist.begin(); file != fslist.end(); ++file) {
-		if (file->isDirectory())
-			continue;
-
-		Common::String tstr = file->getName();
-
-		// Strip any trailing dot
-		if (tstr.lastChar() == '.')
-			tstr.deleteLastChar();
-
-		allFiles[tstr] = *file;	// Record the presence of this file
-	}
+	composeFileHashMap(fslist, allFiles, (params.depth == 0 ? 1 : params.depth), params.directoryGlobs);
 
 	// Check which files are included in some ADGameDescription *and* present
 	// in fslist. Compute MD5s and file sizes for these files.
@@ -374,24 +444,38 @@ static ADGameDescList detectGame(const Common::FSList &fslist, const ADParams &p
 
 		for (fileDesc = g->filesDescriptions; fileDesc->fileName; fileDesc++) {
 			Common::String fname = fileDesc->fileName;
-			if (allFiles.contains(fname) && !filesSizeMD5.contains(fname)) {
-				debug(3, "+ %s", fname.c_str());
+			SizeMD5 tmp;
 
-				SizeMD5 tmp;
-				Common::File testFile;
-
-				if (testFile.open(allFiles[fname])) {
-					tmp.size = (int32)testFile.size();
-					if (!md5_file_string(testFile, tmp.md5, params.md5Bytes))
+			if (g->flags & ADGF_MACRESFORK) {
+				Common::MacResManager *macResMan = new Common::MacResManager();
+				
+				if (macResMan->open(parent, fname)) {
+					if (!macResMan->getResForkMD5(tmp.md5, params.md5Bytes))
 						tmp.md5[0] = 0;
-				} else {
-					tmp.size = -1;
-					tmp.md5[0] = 0;
+					tmp.size = macResMan->getResForkSize();
+					debug(3, "> '%s': '%s'", fname.c_str(), tmp.md5);
+					filesSizeMD5[fname] = tmp;
 				}
 
-				debug(3, "> '%s': '%s'", fname.c_str(), tmp.md5);
+				delete macResMan;
+			} else {
+				if (allFiles.contains(fname) && !filesSizeMD5.contains(fname)) {
+					debug(3, "+ %s", fname.c_str());
 
-				filesSizeMD5[fname] = tmp;
+					Common::File testFile;
+
+					if (testFile.open(allFiles[fname])) {
+						tmp.size = (int32)testFile.size();
+						if (!md5_file_string(testFile, tmp.md5, params.md5Bytes))
+							tmp.md5[0] = 0;
+					} else {
+						tmp.size = -1;
+						tmp.md5[0] = 0;
+					}
+				
+					debug(3, "> '%s': '%s'", fname.c_str(), tmp.md5);
+					filesSizeMD5[fname] = tmp;
+				}
 			}
 		}
 	}
@@ -408,7 +492,8 @@ static ADGameDescList detectGame(const Common::FSList &fslist, const ADParams &p
 
 		// Do not even bother to look at entries which do not have matching
 		// language and platform (if specified).
-		if ((language != Common::UNK_LANG && g->language != Common::UNK_LANG && g->language != language) ||
+		if ((language != Common::UNK_LANG && g->language != Common::UNK_LANG && g->language != language
+			 && !(language == Common::EN_ANY && (g->flags & ADGF_ADDENGLISH))) ||
 			(platform != Common::kPlatformUnknown && g->platform != Common::kPlatformUnknown && g->platform != platform)) {
 			continue;
 		}
