@@ -18,21 +18,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/agi/agi.cpp $
- * $Id: agi.cpp 48430 2010-03-30 22:13:29Z agent-q $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/agi/agi.cpp $
+ * $Id: agi.cpp 52839 2010-09-21 06:06:37Z eriktorbjorn $
  *
  */
 
-#ifdef __DS__
-#include "dsmain.h"
-#endif
-
 #include "common/md5.h"
 #include "common/events.h"
 #include "common/EventRecorder.h"
 #include "common/file.h"
 #include "common/savefile.h"
 #include "common/config-manager.h"
+#include "common/debug-channels.h"
+#include "common/random.h"
+
+#include "engines/util.h"
 
 #include "base/plugins.h"
 #include "base/version.h"
@@ -48,7 +48,6 @@
 #include "agi/keyboard.h"
 #include "agi/menu.h"
 
-
 namespace Agi {
 
 void AgiEngine::allowSynthetic(bool allow) {
@@ -273,20 +272,18 @@ void AgiEngine::processEvents() {
 }
 
 void AgiEngine::pollTimer() {
-	static uint32 m = 0;
 	uint32 dm;
 
-	if (_tickTimer < m)
-		m = 0;
+	if (_tickTimer < _lastTickTimer)
+		_lastTickTimer = 0;
 
-	while ((dm = _tickTimer - m) < 5) {
+	while ((dm = _tickTimer - _lastTickTimer) < 5) {
 		processEvents();
-		if (_console->isAttached())
-			_console->onFrame();
+		_console->onFrame();
 		_system->delayMillis(10);
 		_system->updateScreen();
 	}
-	m = _tickTimer;
+	_lastTickTimer = _tickTimer;
 }
 
 void AgiEngine::agiTimerFunctionLow(void *refCon) {
@@ -346,7 +343,7 @@ int AgiEngine::agiInit() {
 
 	// clear view table
 	for (i = 0; i < MAX_VIEWTABLE; i++)
-		memset(&_game.viewTable[i], 0, sizeof(VtEntry));
+		memset(&_game.viewTable[i], 0, sizeof(struct VtEntry));
 
 	initWords();
 
@@ -410,7 +407,6 @@ int AgiEngine::agiInit() {
 	// Normally, the engine loads the predictive text dictionary when the predictive dialog
 	// is shown.  On the DS version, the word completion feature needs the dictionary too.
 	loadDict();
-	DS::setControls("agi");
 #endif
 
 	_egoHoldKey = false;
@@ -505,31 +501,23 @@ AgiBase::AgiBase(OSystem *syst, const AGIGameDescription *gameDesc) : Engine(sys
 AgiEngine::AgiEngine(OSystem *syst, const AGIGameDescription *gameDesc) : AgiBase(syst, gameDesc) {
 
 	// Setup mixer
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, ConfMan.getInt("sfx_volume"));
-	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, ConfMan.getInt("music_volume"));
-
-	const GameSettings *g;
-
-	const char *gameid = ConfMan.get("gameid").c_str();
-	for (g = agiSettings; g->gameid; ++g)
-		if (!scumm_stricmp(g->gameid, gameid))
-			_gameId = g->id;
+	syncSoundSettings();
 
 	parseFeatures();
 
 	_rnd = new Common::RandomSource();
 	g_eventRec.registerRandomSource(*_rnd, "agi");
 
-	Common::addDebugChannel(kDebugLevelMain, "Main", "Generic debug level");
-	Common::addDebugChannel(kDebugLevelResources, "Resources", "Resources debugging");
-	Common::addDebugChannel(kDebugLevelSprites, "Sprites", "Sprites debugging");
-	Common::addDebugChannel(kDebugLevelInventory, "Inventory", "Inventory debugging");
-	Common::addDebugChannel(kDebugLevelInput, "Input", "Input events debugging");
-	Common::addDebugChannel(kDebugLevelMenu, "Menu", "Menu debugging");
-	Common::addDebugChannel(kDebugLevelScripts, "Scripts", "Scripts debugging");
-	Common::addDebugChannel(kDebugLevelSound, "Sound", "Sound debugging");
-	Common::addDebugChannel(kDebugLevelText, "Text", "Text output debugging");
-	Common::addDebugChannel(kDebugLevelSavegame, "Savegame", "Saving & restoring game debugging");
+	DebugMan.addDebugChannel(kDebugLevelMain, "Main", "Generic debug level");
+	DebugMan.addDebugChannel(kDebugLevelResources, "Resources", "Resources debugging");
+	DebugMan.addDebugChannel(kDebugLevelSprites, "Sprites", "Sprites debugging");
+	DebugMan.addDebugChannel(kDebugLevelInventory, "Inventory", "Inventory debugging");
+	DebugMan.addDebugChannel(kDebugLevelInput, "Input", "Input events debugging");
+	DebugMan.addDebugChannel(kDebugLevelMenu, "Menu", "Menu debugging");
+	DebugMan.addDebugChannel(kDebugLevelScripts, "Scripts", "Scripts debugging");
+	DebugMan.addDebugChannel(kDebugLevelSound, "Sound", "Sound debugging");
+	DebugMan.addDebugChannel(kDebugLevelText, "Text", "Text output debugging");
+	DebugMan.addDebugChannel(kDebugLevelSavegame, "Savegame", "Saving & restoring game debugging");
 
 
 	memset(&_game, 0, sizeof(struct AgiGame));
@@ -545,6 +533,7 @@ AgiEngine::AgiEngine(OSystem *syst, const AGIGameDescription *gameDesc) : AgiBas
 	_allowSynthetic = false;
 
 	_tickTimer = 0;
+	_lastTickTimer = 0;
 
 	_intobj = NULL;
 
@@ -558,7 +547,7 @@ AgiEngine::AgiEngine(OSystem *syst, const AGIGameDescription *gameDesc) : AgiBas
 
 	_restartGame = false;
 
-	_oldMode = -1;
+	_oldMode = INPUT_NONE;
 
 	_predictiveDialogRunning = false;
 	_predictiveDictText = NULL;
@@ -571,6 +560,10 @@ AgiEngine::AgiEngine(OSystem *syst, const AGIGameDescription *gameDesc) : AgiBas
 	_game.lastController = 0;
 	for (int i = 0; i < MAX_DIRS; i++)
 		_game.controllerOccured[i] = false;
+
+	setupOpcodes();
+	_curLogic = NULL;
+	_timerHack = 0;
 }
 
 void AgiEngine::initialize() {
@@ -584,14 +577,27 @@ void AgiEngine::initialize() {
 		_soundemu = SOUND_EMU_APPLE2GS;
 	} else if (getPlatform() == Common::kPlatformCoCo3) {
 		_soundemu = SOUND_EMU_COCO3;
+	} else if (ConfMan.get("music_driver") == "auto") {
+		// Default sound is the proper PCJr emulation
+		_soundemu = SOUND_EMU_PCJR;
 	} else {
-		switch (MidiDriver::detectMusicDriver(MDT_PCSPK)) {
-		case MD_PCSPK:
+		switch (MidiDriver::getMusicType(MidiDriver::detectDevice(MDT_PCSPK|MDT_AMIGA|MDT_ADLIB|MDT_PCJR|MDT_MIDI))) {
+		case MT_PCSPK:
 			_soundemu = SOUND_EMU_PC;
 			break;
-		default:
+		case MT_ADLIB:
 			_soundemu = SOUND_EMU_NONE;
 			break;
+		case MT_PCJR:
+			_soundemu = SOUND_EMU_PCJR;
+			break;
+		case MT_AMIGA:
+			_soundemu = SOUND_EMU_AMIGA;
+			break;
+		default:
+			debug(0, "DEF");
+			_soundemu = SOUND_EMU_MIDI;
+			break;
 		}
 	}
 
@@ -609,6 +615,8 @@ void AgiEngine::initialize() {
 			_renderMode = Common::kRenderEGA;
 			break;
 		}
+	} else {
+		_renderMode = Common::kRenderDefault;
 	}
 
 	_buttonStyle = AgiButtonStyle(_renderMode);
@@ -707,18 +715,6 @@ Common::Error AgiEngine::go() {
 	return Common::kNoError;
 }
 
-void AgiEngine::syncSoundSettings() {
-	// FIXME/TODO: Please explain why we are using "music_volume" for all
-	// three different entries here.
-	int soundVolumeMusic = ConfMan.getInt("music_volume");
-	int soundVolumeSFX = ConfMan.getInt("music_volume");
-	int soundVolumeSpeech = ConfMan.getInt("music_volume");
-
-	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, soundVolumeMusic);
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, soundVolumeSFX);
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSpeechSoundType, soundVolumeSpeech);
-}
-
 void AgiEngine::parseFeatures() {
 	if (!ConfMan.hasKey("features"))
 		return;
