@@ -18,16 +18,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/agos/agos.cpp $
- * $Id: agos.cpp 48366 2010-03-23 00:42:42Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/agos/agos.cpp $
+ * $Id: agos.cpp 52905 2010-09-26 12:08:16Z eriktorbjorn $
  *
  */
 
 #include "common/config-manager.h"
-#include "common/file.h"
-#include "common/system.h"
-#include "common/events.h"
 #include "common/EventRecorder.h"
+#include "common/events.h"
+#include "common/file.h"
+#include "common/fs.h"
+#include "common/system.h"
+
+#include "engines/util.h"
 
 #include "agos/debugger.h"
 #include "agos/intern.h"
@@ -199,19 +202,19 @@ AGOSEngine::AGOSEngine(OSystem *syst)
 	_scanFlag = false;
 
 	_scriptVar2 = 0;
-	_runScriptReturn1 = 0;
-	_skipVgaWait = 0;
-	_noParentNotify = 0;
-	_beardLoaded = 0;
-	_litBoxFlag = 0;
-	_mortalFlag = 0;
+	_runScriptReturn1 = false;
+	_skipVgaWait = false;
+	_noParentNotify = false;
+	_beardLoaded = false;
+	_litBoxFlag = false;
+	_mortalFlag = false;
 	_displayFlag = 0;
-	_syncFlag2 = 0;
-	_inCallBack = 0;
-	_cepeFlag = 0;
-	_fastMode = 0;
+	_syncFlag2 = false;
+	_inCallBack = false;
+	_cepeFlag = false;
+	_fastMode = false;
 
-	_backFlag = 0;
+	_backFlag = false;
 
 	_debugMode = 0;
 	_dumpScripts = false;
@@ -514,20 +517,22 @@ AGOSEngine::AGOSEngine(OSystem *syst)
 		"\x5\x5\x4\x6\x5\x3\x4\x5\x6\x3\x5\x5\x4\x6\x5\x3\x4\x6\x5\x6\x6\x6\x5\x5\x5\x6\x5\x6\x6\x6\x6\x6", 32);
 
 
+	const Common::FSNode gameDataDir(ConfMan.get("path"));
+
 	// Add default file directories for Acorn version of
 	// Simon the Sorcerer 1
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "execute");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "execute");
 
 	// Add default file directories for Amiga/Macintosh
 	// verisons of Simon the Sorcerer 2
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "voices");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "voices");
 
 	// Add default file directories for Amiga & Macintosh
 	// versions of The Feeble Files
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "gfx");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "movies");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "sfx");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "speech");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "gfx");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "movies");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "sfx");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "speech");
 
 	g_eventRec.registerRandomSource(_rnd, "agos");
 }
@@ -552,16 +557,16 @@ Common::Error AGOSEngine::init() {
 		(getPlatform() == Common::kPlatformPC)) {
 
 		// Setup midi driver
-		MidiDriverType midiDriver = MidiDriver::detectMusicDriver(MDT_ADLIB | MDT_MIDI);
-		_nativeMT32 = ((midiDriver == MD_MT32) || ConfMan.getBool("native_mt32"));
+		MidiDriver::DeviceHandle dev = MidiDriver::detectDevice(MDT_ADLIB | MDT_MIDI | (getGameType() == GType_SIMON1 ? MDT_PREFER_MT32 : MDT_PREFER_GM));
+		_nativeMT32 = ((MidiDriver::getMusicType(dev) == MT_MT32) || ConfMan.getBool("native_mt32"));
 
-		_driver = MidiDriver::createMidi(midiDriver);
+		_driver = MidiDriver::createMidi(dev);
 
-		if (_nativeMT32) {
+		if (_nativeMT32)
 			_driver->property(MidiDriver::PROP_CHANNEL_MASK, 0x03FE);
-		}
 
-		_midi.mapMT32toGM (getGameType() != GType_SIMON2 && !_nativeMT32);
+		_midi.setNativeMT32(_nativeMT32);
+		_midi.mapMT32toGM(getGameType() != GType_SIMON2 && !_nativeMT32);
 
 		_midi.setDriver(_driver);
 
@@ -626,9 +631,11 @@ Common::Error AGOSEngine::init() {
 
 	if (ConfMan.hasKey("sfx_mute") && ConfMan.getBool("sfx_mute") == 1) {
 		if (getGameId() == GID_SIMON1DOS)
-			_midi._enable_sfx ^= 1;
-		else
-			_sound->effectsPause(_effectsPaused ^= 1);
+			_midi._enable_sfx = !_midi._enable_sfx;
+		else {
+			_effectsPaused = !_effectsPaused;
+			_sound->effectsPause(_effectsPaused);
+		}
 	}
 
 	_copyProtection = ConfMan.getBool("copy_protection");
@@ -643,7 +650,7 @@ Common::Error AGOSEngine::init() {
 
 		if (getGameType() == GType_SIMON1) {
 			// English and German versions don't have full subtitles
-			 if (_language == Common::EN_ANY || _language == Common::DE_DEU)
+			if (_language == Common::EN_ANY || _language == Common::DE_DEU)
 				_subtitles = false;
 			// Other versions require speech to be enabled
 			else
@@ -919,10 +926,10 @@ AGOSEngine::~AGOSEngine() {
 	free(_textMem);
 	free(_xtblList);
 
-	free(_backGroundBuf);
-	free(_backBuf);
+	delete _backGroundBuf;
+	delete _backBuf;
 	free(_planarBuf);
-	free(_scaleBuf);
+	delete _scaleBuf;
 	free(_zoneBuffers);
 
 	free(_window4BackScn);
@@ -1032,12 +1039,21 @@ uint32 AGOSEngine::getTime() const {
 }
 
 void AGOSEngine::syncSoundSettings() {
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, ConfMan.getInt("sfx_volume"));
-	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, ConfMan.getInt("music_volume"));
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSpeechSoundType, ConfMan.getInt("speech_volume"));
+	// Sync the engine with the config manager
+	int soundVolumeMusic = ConfMan.getInt("music_volume");
+	int soundVolumeSFX = ConfMan.getInt("sfx_volume");
+	int soundVolumeSpeech = ConfMan.getInt("speech_volume");
+
+	bool mute = false;
+	if (ConfMan.hasKey("mute"))
+		mute = ConfMan.getBool("mute");
+
+	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, (mute ? 0 : (_musicPaused ? 0 : soundVolumeMusic)));
+	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, (mute ? 0 : soundVolumeSFX));
+	_mixer->setVolumeForSoundType(Audio::Mixer::kSpeechSoundType, (mute ? 0 : soundVolumeSpeech));
 
 	if (_midiEnabled)
-		_midi.setVolume(ConfMan.getInt("music_volume"), ConfMan.getInt("sfx_volume"));
+		_midi.setVolume((mute ? 0 : soundVolumeMusic), (mute ? 0 : soundVolumeSFX));
 }
 
 } // End of namespace AGOS
