@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/agos/agos.h $
- * $Id: agos.h 48366 2010-03-23 00:42:42Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/agos/agos.h $
+ * $Id: agos.h 50643 2010-07-04 07:32:12Z eriktorbjorn $
  *
  */
 
@@ -30,6 +30,7 @@
 
 #include "common/array.h"
 #include "common/keyboard.h"
+#include "common/random.h"
 #include "common/rect.h"
 #include "common/stack.h"
 #include "common/util.h"
@@ -392,7 +393,7 @@ protected:
 	Common::Point _mouseOld;
 
 	byte *_mouseData;
-	byte _animatePointer;
+	bool _animatePointer;
 	byte _maxCursorWidth, _maxCursorHeight;
 	byte _mouseAnim, _mouseAnimMax, _mouseCursor;
 	byte _currentMouseAnim, _currentMouseCursor;
