@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/agos/detection_tables.h $
- * $Id: detection_tables.h 46815 2010-01-01 11:44:33Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/agos/detection_tables.h $
+ * $Id: detection_tables.h 52464 2010-08-31 05:18:38Z Kirben $
  *
  */
 
@@ -1672,7 +1672,7 @@ static const AGOSGameDescription gameDescriptions[] = {
 	{ "tbllist",		GAME_TBLFILE,	"d198a80de2c59e4a0cd24b98814849e8", 711},
 	{ NULL, 0, NULL, 0}
 			},
-			Common::HB_ISR,
+			Common::HE_ISR,
 			Common::kPlatformPC,
 			ADGF_NO_FLAGS,
 			GUIO_NONE
@@ -2148,7 +2148,7 @@ static const AGOSGameDescription gameDescriptions[] = {
 	{ "tbllist",		GAME_TBLFILE,	"2082f8d02075e590300478853a91ffd9", 513},
 	{ NULL, 0, NULL, 0}
 			},
-			Common::HB_ISR,
+			Common::HE_ISR,
 			Common::kPlatformPC,
 			ADGF_NO_FLAGS,
 			GUIO_NONE
@@ -2811,6 +2811,27 @@ static const AGOSGameDescription gameDescriptions[] = {
 		GF_OLD_BUNDLE | GF_TALKIE
 	},
 
+	// Simon the Sorcerer's Puzzle Pack - Swampy Adventures - Polish
+	{
+		{
+			"swampy",
+			"CD",
+
+			{
+	{ "Gswampy",		GAME_BASEFILE,	"31bfb5169b47ccc19177e61bd31d4391", -1},
+	{ NULL, 0, NULL, 0}
+			},
+			Common::PL_POL,
+			Common::kPlatformWindows,
+			ADGF_NO_FLAGS,
+			GUIO_NOSUBTITLES
+		},
+
+		GType_PP,
+		GID_SWAMPY,
+		GF_OLD_BUNDLE | GF_TALKIE
+	},
+
 	// Simon the Sorcerer's Puzzle Pack - Swampy Adventures - Spanish
 	{
 		{
