@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/agos/gfx.cpp $
- * $Id: gfx.cpp 43304 2009-08-12 02:24:03Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/agos/gfx.cpp $
+ * $Id: gfx.cpp 50643 2010-07-04 07:32:12Z eriktorbjorn $
  *
  */
 
@@ -730,7 +730,7 @@ void AGOSEngine_Simon1::drawImage(VC10_state *state) {
 		state->paletteMod = 208;
 	}
 
-	if (_backFlag == 1) {
+	if (_backFlag) {
 		drawBackGroundImage(state);
 	} else if (state->flags & kDFMasked) {
 		drawMaskedImage(state);
@@ -947,7 +947,7 @@ void AGOSEngine::drawImage(VC10_state *state) {
 	if (getGameType() == GType_ELVIRA2 && getPlatform() == Common::kPlatformAtariST && yoffs > 133)
 		state->palette = 208;
 
-	if (_backFlag == 1) {
+	if (_backFlag) {
 		drawBackGroundImage(state);
 	} else {
 		drawVertImage(state);
@@ -1351,7 +1351,7 @@ void AGOSEngine::setWindowImage(uint16 mode, uint16 vgaSpriteId, bool specialCas
 
 	if (getGameType() == GType_FF || getGameType() == GType_PP) {
 		fillBackGroundFromBack();
-		_syncFlag2 = 1;
+		_syncFlag2 = true;
 	} else {
 		_copyScnFlag = 2;
 		_vgaSpriteChanged++;
