@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/agos/icons.cpp $
- * $Id: icons.cpp 47541 2010-01-25 01:39:44Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/agos/icons.cpp $
+ * $Id: icons.cpp 49843 2010-06-15 12:33:20Z fingolfin $
  *
  */
 
@@ -448,7 +448,7 @@ void AGOSEngine_Feeble::drawIconArray(uint num, Item *itemRef, int line, int cla
 				setupIconHitArea(window, k++, xp, yp, itemRef);
 		} else {
 /*
- *	Just remember the overflow has occured
+ *	Just remember the overflow has occurred
  */
 			window->iconPtr->iconArray[icount].item = NULL;	/* END MARKINGS */
 			_iOverflow = 1;
