@@ -18,14 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/agos/saveload.cpp $
- * $Id: saveload.cpp 46941 2010-01-03 19:37:43Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/agos/saveload.cpp $
+ * $Id: saveload.cpp 52758 2010-09-17 12:54:00Z lordhoto $
  *
  */
 
 #include "common/file.h"
 #include "common/savefile.h"
 #include "common/system.h"
+#include "common/translation.h"
 
 #include "gui/about.h"
 #include "gui/message.h"
@@ -37,7 +38,7 @@ namespace AGOS {
 
 int AGOSEngine::countSaveGames() {
 	Common::InSaveFile *f;
-	Common::StringList filenames;
+	Common::StringArray filenames;
 	uint i = 1;
 	char slot[4];
 	int slotNum;
@@ -49,7 +50,7 @@ int AGOSEngine::countSaveGames() {
 	memset(marks, false, 256 * sizeof(bool));	//assume no savegames for this title
 	filenames = _saveFileMan->listSavefiles(prefix);
 
-	for (Common::StringList::const_iterator file = filenames.begin(); file != filenames.end(); ++file){
+	for (Common::StringArray::const_iterator file = filenames.begin(); file != filenames.end(); ++file){
 		//Obtain the last 3 digits of the filename, since they correspond to the save slot
 		slot[0] = file->c_str()[file->size()-3];
 		slot[1] = file->c_str()[file->size()-2];
@@ -146,14 +147,14 @@ void AGOSEngine::quickLoadOrSave() {
 	}
 
 	bool success;
-	char buf[60];
+	Common::String buf;
 
 	char *filename = genSaveName(_saveLoadSlot);
 	if (_saveLoadType == 2) {
 		Subroutine *sub;
 		success = loadGame(genSaveName(_saveLoadSlot));
 		if (!success) {
-			sprintf(buf, "Failed to load game state to file:\n\n%s", filename);
+			buf = Common::String::printf(_("Failed to load game state from file:\n\n%s"), filename);
 		} else if (getGameType() == GType_SIMON1 || getGameType() == GType_SIMON2) {
 			drawIconArray(2, me(), 0, 0);
 			setBitFlag(97, true);
@@ -188,7 +189,7 @@ void AGOSEngine::quickLoadOrSave() {
 	} else {
 		success = saveGame(_saveLoadSlot, _saveLoadName);
 		if (!success)
-			sprintf(buf, "Failed to save game state to file:\n\n%s", filename);
+			buf = Common::String::printf(_("Failed to save game state to file:\n\n%s"), filename);
 	}
 
 	if (!success) {
@@ -196,7 +197,7 @@ void AGOSEngine::quickLoadOrSave() {
 		dialog.runModal();
 
 	} else if (_saveLoadType == 1) {
-		sprintf(buf, "Successfully saved game state in file:\n\n%s", filename);
+		buf = Common::String::printf(_("Successfully saved game state in file:\n\n%s"), filename);
 		GUI::TimedMessageDialog dialog(buf, 1500);
 		dialog.runModal();
 
@@ -608,13 +609,13 @@ void AGOSEngine_Simon1::listSaveGames(char *dst) {
 		lastSlot = slot;
 		if (slot < 10) {
 			showMessageFormat(" ");
-		} else if (_language == Common::HB_ISR) {
+		} else if (_language == Common::HE_ISR) {
 			lastSlot = (slot % 10) * 10;
 			lastSlot += slot / 10;
 		}
 
 		showMessageFormat("%d", lastSlot);
-		if (_language == Common::HB_ISR && !(slot % 10))
+		if (_language == Common::HE_ISR && !(slot % 10))
 			showMessageFormat("0");
 		showMessageFormat(".%s\n", dst);
 		dst += 18;
@@ -672,7 +673,7 @@ void AGOSEngine_Simon1::userGame(bool load) {
 	char *name;
 	bool b;
 	char buf[108];
-	int maxChar = (_language == Common::HB_ISR) ? 155: 128;
+	int maxChar = (_language == Common::HE_ISR) ? 155: 128;
 
 	_saveOrLoad = load;
 
@@ -711,7 +712,7 @@ restart:;
 		window->textRow = result;
 
 		// init x offset with a 2 character savegame number + a period (18 pix)
-		if (_language == Common::HB_ISR) {
+		if (_language == Common::HE_ISR) {
 			window->textColumn = 3;
 			window->textColumnOffset = 6;
 		} else {
@@ -725,7 +726,7 @@ restart:;
 		// now process entire savegame name to get correct x offset for cursor
 		_saveGameNameLen = 0;
 		while (name[_saveGameNameLen]) {
-			if (_language == Common::HB_ISR) {
+			if (_language == Common::HE_ISR) {
 				byte width = 6;
 				if (name[_saveGameNameLen] >= 64 && name[_saveGameNameLen] < 91)
 					width = _hebrewCharWidths [name[_saveGameNameLen] - 64];
@@ -770,7 +771,7 @@ restart:;
 				goto restart;
 			}
 
-			if (_language == Common::HB_ISR) {
+			if (_language == Common::HE_ISR) {
 				if (i >= 128)
 					i -= 64;
 				else if (i >= 32)
@@ -788,7 +789,7 @@ restart:;
 					_saveGameNameLen--;
 					m = name[_saveGameNameLen];
 
-					if (_language == Common::HB_ISR)
+					if (_language == Common::HE_ISR)
 						x = 8;
 					else
 						x = (name[_saveGameNameLen] == 'i' || name[_saveGameNameLen] == 'l') ? 1 : 8;
@@ -889,7 +890,7 @@ void AGOSEngine::userGameBackSpace(WindowBlock *window, int x, byte b) {
 	oldTextColor = window->textColor;
 	window->textColor = window->fillColor;
 
-	if (_language == Common::HB_ISR) {
+	if (_language == Common::HE_ISR) {
 		x = 128;
 	} else {
 		x += 120;
