@@ -18,22 +18,20 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/cine/msg.h $
- * $Id: msg.h 31651 2008-04-21 20:51:17Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/cine/msg.h $
+ * $Id: msg.h 51937 2010-08-09 11:38:01Z sev $
  *
  */
 
 #ifndef CINE_MSG_H
 #define CINE_MSG_H
 
-#include "common/str.h"
+#include "common/str-array.h"
 
 namespace Cine {
 
 #define NUM_MAX_MESSAGE 255
 
-extern Common::StringList messageTable;
-
 void loadMsg(char *pMsgName);
 
 } // End of namespace Cine
