@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/cine/prc.h $
- * $Id: prc.h 33068 2008-07-14 23:10:51Z buddha_ $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/cine/prc.h $
+ * $Id: prc.h 51937 2010-08-09 11:38:01Z sev $
  *
  */
 
@@ -28,9 +28,6 @@
 
 namespace Cine {
 
-extern ScriptList globalScripts;
-extern ScriptList objectScripts;
-
 bool loadPrc(const char *pPrcName);
 
 } // End of namespace Cine
