@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/cruise/cruise_main.cpp $
- * $Id: cruise_main.cpp 48101 2010-02-21 04:04:13Z bluddy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/cruise/cruise_main.cpp $
+ * $Id: cruise_main.cpp 54667 2010-11-30 08:27:15Z dreammaster $
  *
  */
 
@@ -42,13 +42,11 @@ unsigned int timer = 0;
 
 gfxEntryStruct* linkedMsgList = NULL;
 
-Common::List<byte *> memList;
-
 void MemoryList() {
-	if (!memList.empty()) {
+	if (!_vm->_memList.empty()) {
 		printf("Current list of un-freed memory blocks:\n");
 		Common::List<byte *>::iterator i;
-		for (i = memList.begin(); i != memList.end(); ++i) {
+		for (i = _vm->_memList.begin(); i != _vm->_memList.end(); ++i) {
 			byte *v = *i;
 			printf("%s - %d\n", (const char *)(v - 68), *((int32 *)(v - 72)));
 		}
@@ -73,7 +71,7 @@ void *MemoryAlloc(uint32 size, bool clearFlag, int32 lineNum, const char *fname)
 
 		// Add the block to the memory list
 		result = v + 64 + 8;
-		memList.push_back(result);
+		_vm->_memList.push_back(result);
 	} else
 		result = (byte *)malloc(size);
 
@@ -91,7 +89,7 @@ void MemoryFree(void *v) {
 		byte *p = (byte *)v;
 		assert(*((uint32 *) (p - 4)) == 0x41424344);
 
-		memList.remove(p);
+		_vm->_memList.remove(p);
 		free(p - 8 - 64);
 	} else
 		free(v);
@@ -105,8 +103,8 @@ void drawBlackSolidBoxSmall() {
 void loadPackedFileToMem(int fileIdx, uint8 *buffer) {
 	changeCursor(CURSOR_DISK);
 
-	currentVolumeFile.seek(volumePtrToFileDescriptor[fileIdx].offset, SEEK_SET);
-	currentVolumeFile.read(buffer, volumePtrToFileDescriptor[fileIdx].size);
+	_vm->_currentVolumeFile.seek(volumePtrToFileDescriptor[fileIdx].offset, SEEK_SET);
+	_vm->_currentVolumeFile.read(buffer, volumePtrToFileDescriptor[fileIdx].size);
 }
 
 int getNumObjectsByClass(int scriptIdx, int param) {
@@ -1833,19 +1831,17 @@ void CruiseEngine::mainLoop() {
 				if (!skipEvents)
 					skipEvents = manageEvents();
 
-				if (playerDontAskQuit) break;
+				if (playerDontAskQuit)
+					break;
 
-				if (_vm->getDebugger()->isAttached())
-					_vm->getDebugger()->onFrame();
+				_vm->getDebugger()->onFrame();
 			} while (currentTick < lastTick + _gameSpeed);
 		} else {
 			manageEvents();
 
 			if (currentTick >= (lastTickDebug + 10)) {
 				lastTickDebug = currentTick;
-
-				if (_vm->getDebugger()->isAttached())
-					_vm->getDebugger()->onFrame();
+				_vm->getDebugger()->onFrame();
 			}
 		}
 		if (playerDontAskQuit)
@@ -1890,11 +1886,27 @@ void CruiseEngine::mainLoop() {
 			currentMouseButton = 0;
 		}
 
-		manageScripts(&relHead);
-		manageScripts(&procHead);
+		// FIXME: I suspect that the original game does multiple script executions between game frames; the bug with
+		// Raoul appearing when looking at the book is being there are 3 script iterations separation between the
+		// scene being changed to the book, and the Raoul actor being frozen/disabled. This loop is a hack to ensure
+		// that when a background changes, a few extra script executions are done
+		bool bgChanged;
+		int numIterations = 1;
 
-		removeFinishedScripts(&relHead);
-		removeFinishedScripts(&procHead);
+		while (numIterations-- > 0) {
+			bgChanged = backgroundChanged[masterScreen];
+		
+			manageScripts(&relHead);
+			manageScripts(&procHead);
+
+			removeFinishedScripts(&relHead);
+			removeFinishedScripts(&procHead);
+
+			if (!bgChanged && backgroundChanged[masterScreen]) {
+				bgChanged = true;
+				numIterations += 2;
+			}
+		}
 
 		processAnimation();
 
