@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/cruise/saveload.cpp $
- * $Id: saveload.cpp 48095 2010-02-20 10:37:08Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/cruise/saveload.cpp $
+ * $Id: saveload.cpp 51936 2010-08-09 10:30:40Z sev $
  *
  */
 
@@ -579,10 +579,10 @@ static void syncPerso(Common::Serializer &s, persoStruct &p) {
 }
 
 static void syncCT(Common::Serializer &s) {
-	int v = (polyStruct) ? 1 : 0;
+	int v = (_vm->_polyStruct) ? 1 : 0;
 	s.syncAsSint32LE(v);
 	if (s.isLoading())
-		polyStruct = (v != 0) ? &polyStructNorm : NULL;
+		_vm->_polyStruct = (v != 0) ? &_vm->_polyStructNorm : NULL;
 
 	if (v == 0)
 		// There is no further data to load or save
