@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/draci/animation.cpp $
- * $Id: animation.cpp 48574 2010-04-06 19:17:55Z spalek $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/draci/animation.cpp $
+ * $Id: animation.cpp 50618 2010-07-03 05:05:28Z spalek $
  *
  */
 
@@ -166,10 +166,10 @@ void Animation::drawFrame(Surface *surface) {
 
 	const SoundSample *sample = _samples[_currentFrame];
 	if (_hasChangedFrame && sample) {
+		uint duration = _vm->_sound->playSound(sample, Audio::Mixer::kMaxChannelVolume, false);
 		debugC(3, kDraciSoundDebugLevel,
-			"Playing sample on animation %d, frame %d: %d+%d at %dHz",
-			_id, _currentFrame, sample->_offset, sample->_length, sample->_frequency);
-		_vm->_sound->playSound(sample, Audio::Mixer::kMaxChannelVolume, false);
+			"Playing sample on animation %d, frame %d: %d+%d at %dHz: %dms",
+			_id, _currentFrame, sample->_offset, sample->_length, sample->_frequency, duration);
 	}
 	_hasChangedFrame = false;
 }
