@@ -18,11 +18,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/draci/game.cpp $
- * $Id: game.cpp 48723 2010-04-19 16:40:22Z spalek $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/draci/game.cpp $
+ * $Id: game.cpp 50519 2010-06-30 08:27:09Z spalek $
  *
  */
 
+#include "common/keyboard.h"
 #include "common/serializer.h"
 #include "common/stream.h"
 #include "common/system.h"
@@ -174,8 +175,20 @@ void Game::start() {
 
 		// Call the outer loop doing all the hard job.
 		loop(kOuterLoop, false);
-	}
 
+		// Fade out the palette after leaving the location.
+		fadePalette(true);
+
+		if (!isReloaded()) {
+			// We are changing location.  Run the hero's LOOK
+			// program to trigger a possible cut-scene.  This is
+			// the behavior of the original game player, whose
+			// intention was to run the cut sequences after the
+			// certain location change.
+			const GameObject *dragon = getObject(kDragonObject);
+			_vm->_script->run(dragon->_program, dragon->_look);
+		}
+	}
 }
 
 void Game::init() {
@@ -193,7 +206,8 @@ void Game::init() {
 	_animUnderCursor = NULL;
 
 	_currentItem = _itemUnderCursor = NULL;
-
+	_previousItemPosition = -1;
+  
 	_vm->_mouse->setCursorType(kHighlightedCursor);	// anything different from kNormalCursor
 
 	_objUnderCursor = NULL;
@@ -263,8 +277,8 @@ void Game::handleOrdinaryLoop(int x, int y) {
 	if (_vm->_mouse->lButtonPressed()) {
 		_vm->_mouse->lButtonSet(false);
 
-		if (_currentItem) {
-			putItem(_currentItem, 0);
+		if (getCurrentItem()) {
+			putItem(getCurrentItem(), getPreviousItemPosition());
 			updateOrdinaryCursor();
 		} else {
 			if (_objUnderCursor) {
@@ -318,6 +332,16 @@ void Game::handleOrdinaryLoop(int x, int y) {
 	}
 }
 
+int Game::inventoryPositionFromMouse() const {
+	const int column = CLIP(scummvm_lround(
+		(_vm->_mouse->getPosX() - kInventoryX + kInventoryItemWidth / 2.) /
+		kInventoryItemWidth) - 1, 0L, (long) kInventoryColumns - 1);
+	const int line = CLIP(scummvm_lround(
+		(_vm->_mouse->getPosY() - kInventoryY + kInventoryItemHeight / 2.) /
+		kInventoryItemHeight) - 1, 0L, (long) kInventoryLines - 1);
+	return line * kInventoryColumns + column;
+}
+
 void Game::handleInventoryLoop() {
 	if (_loopSubstatus != kOuterLoop) {
 		return;
@@ -344,19 +368,12 @@ void Game::handleInventoryLoop() {
 
 		// If there is an inventory item under the cursor and we aren't
 		// holding any item, run its look GPL program
-		if (_itemUnderCursor && !_currentItem) {
+		if (_itemUnderCursor && !getCurrentItem()) {
 			_vm->_script->runWrapper(_itemUnderCursor->_program, _itemUnderCursor->_look, true, false);
 		// Otherwise, if we are holding an item, try to place it inside the
 		// inventory
-		} else if (_currentItem) {
-			const int column = CLIP(scummvm_lround(
-				(_vm->_mouse->getPosX() - kInventoryX + kInventoryItemWidth / 2.) /
-				kInventoryItemWidth) - 1, 0L, (long) kInventoryColumns - 1);
-			const int line = CLIP(scummvm_lround(
-				(_vm->_mouse->getPosY() - kInventoryY + kInventoryItemHeight / 2.) /
-				kInventoryItemHeight) - 1, 0L, (long) kInventoryLines - 1);
-			const int index = line * kInventoryColumns + column;
-			putItem(_currentItem, index);
+		} else if (getCurrentItem()) {
+			putItem(getCurrentItem(), inventoryPositionFromMouse());
 			updateInventoryCursor();
 		}
 	} else if (_vm->_mouse->rButtonPressed()) {
@@ -372,8 +389,9 @@ void Game::handleInventoryLoop() {
 
 			// The first is that there is no item in our hands.
 			// In that case, just take the inventory item from the inventory.
-			if (!_currentItem) {
-				_currentItem = _itemUnderCursor;
+			if (!getCurrentItem()) {
+				setCurrentItem(_itemUnderCursor);
+				setPreviousItemPosition(inventoryPositionFromMouse());
 				removeItem(_itemUnderCursor);
 
 			// The second is that there *is* an item in our hands.
@@ -413,6 +431,22 @@ void Game::handleDialogueLoop() {
 	}
 }
 
+void Game::fadePalette(bool fading_out) {
+	const byte *startPal = NULL;
+	const byte *endPal = _currentRoom._palette >= 0
+		? _vm->_paletteArchive->getFile(_currentRoom._palette)->_data
+		: NULL;
+	if (fading_out) {
+		startPal = endPal;
+		endPal = NULL;
+	}
+	for (int i = 1; i <= kBlackFadingIterations; ++i) {
+		_vm->_system->delayMillis(kBlackFadingTimeUnit);
+		_vm->_screen->interpolatePalettes(startPal, endPal, 0, kNumColours, i, kBlackFadingIterations);
+		_vm->_screen->copyToScreen();
+	}
+}
+
 void Game::advanceAnimationsAndTestLoopExit() {
 	// Fade the palette if requested
 	if (_fadePhase > 0 && (_vm->_system->getMillis() - _fadeTick) >= kFadingTimeUnit) {
@@ -470,7 +504,7 @@ void Game::advanceAnimationsAndTestLoopExit() {
 	// callbacks) and redraw screen
 	_vm->_anims->drawScene(_vm->_screen->getSurface());
 	_vm->_screen->copyToScreen();
-	_vm->_system->delayMillis(20);
+	_vm->_system->delayMillis(kTimeUnit);
 
 	// If the hero has arrived at his destination, after even the last
 	// phase was correctly animated, run the callback.
@@ -614,10 +648,10 @@ void Game::updateOrdinaryCursor() {
 	// If there is no game object under the cursor, try using the room itself
 	if (!_objUnderCursor) {
 		if (_vm->_script->testExpression(_currentRoom._program, _currentRoom._canUse)) {
-			if (!_currentItem) {
+			if (!getCurrentItem()) {
 				_vm->_mouse->setCursorType(kHighlightedCursor);
 			} else {
-				_vm->_mouse->loadItemCursor(_currentItem, true);
+				_vm->_mouse->loadItemCursor(getCurrentItem(), true);
 			}
 			mouseChanged = true;
 		}
@@ -628,10 +662,10 @@ void Game::updateOrdinaryCursor() {
 		// update the cursor image (highlight it).
 		if (_objUnderCursor->_walkDir == 0) {
 			if (_vm->_script->testExpression(_objUnderCursor->_program, _objUnderCursor->_canUse)) {
-				if (!_currentItem) {
+				if (!getCurrentItem()) {
 					_vm->_mouse->setCursorType(kHighlightedCursor);
 				} else {
-					_vm->_mouse->loadItemCursor(_currentItem, true);
+					_vm->_mouse->loadItemCursor(getCurrentItem(), true);
 				}
 				mouseChanged = true;
 			}
@@ -645,10 +679,10 @@ void Game::updateOrdinaryCursor() {
 	// Load the appropriate cursor (item image if an item is held or ordinary cursor
 	// if not)
 	if (!mouseChanged) {
-		if (!_currentItem) {
+		if (!getCurrentItem()) {
 			_vm->_mouse->setCursorType(kNormalCursor);
 		} else {
-			_vm->_mouse->loadItemCursor(_currentItem, false);
+			_vm->_mouse->loadItemCursor(getCurrentItem(), false);
 		}
 	}
 }
@@ -659,19 +693,19 @@ void Game::updateInventoryCursor() {
 
 	if (_itemUnderCursor) {
 		if (_vm->_script->testExpression(_itemUnderCursor->_program, _itemUnderCursor->_canUse)) {
-			if (!_currentItem) {
+			if (!getCurrentItem()) {
 				_vm->_mouse->setCursorType(kHighlightedCursor);
 			} else {
-				_vm->_mouse->loadItemCursor(_currentItem, true);
+				_vm->_mouse->loadItemCursor(getCurrentItem(), true);
 			}
 			mouseChanged = true;
 		}
 	}
 	if (!mouseChanged) {
-		if (!_currentItem) {
+		if (!getCurrentItem()) {
 			_vm->_mouse->setCursorType(kNormalCursor);
 		} else {
-			_vm->_mouse->loadItemCursor(_currentItem, false);
+			_vm->_mouse->loadItemCursor(getCurrentItem(), false);
 		}
 	}
 }
@@ -723,6 +757,8 @@ const GameObject *Game::getObjectWithAnimation(const Animation *anim) const {
 }
 
 void Game::removeItem(GameItem *item) {
+	if (!item)
+		return;
 	for (uint i = 0; i < kInventorySlots; ++i) {
 		if (_inventory[i] == item) {
 			_inventory[i] = NULL;
@@ -744,7 +780,7 @@ void Game::loadItemAnimation(GameItem *item) {
 
 void Game::putItem(GameItem *item, int position) {
 	// Empty our hands
-	_currentItem = NULL;
+	setCurrentItem(NULL);
 
 	if (!item)
 		return;
@@ -758,6 +794,7 @@ void Game::putItem(GameItem *item, int position) {
 			break;
 		}
 	}
+	setPreviousItemPosition(position);
 
 	const int line = position / kInventoryColumns + 1;
 	const int column = position % kInventoryColumns + 1;
@@ -845,6 +882,55 @@ void Game::inventoryReload() {
 	for (uint i = 0; i < kInventorySlots; ++i) {
 		putItem(_inventory[i], i);
 	}
+	setPreviousItemPosition(0);
+}
+
+void Game::inventorySwitch(int keycode) {
+	switch (keycode) {
+	case Common::KEYCODE_SLASH:
+		// Switch between holding an item and the ordinary mouse cursor.
+		if (!getCurrentItem()) {
+			if (getPreviousItemPosition() >= 0) {
+				GameItem* last_item = _inventory[getPreviousItemPosition()];
+				setCurrentItem(last_item);
+				removeItem(last_item);
+			}
+		} else {
+			putItem(getCurrentItem(), getPreviousItemPosition());
+		}
+		break;
+	case Common::KEYCODE_COMMA:
+	case Common::KEYCODE_PERIOD:
+		// Iterate between the items in the inventory.
+		if (getCurrentItem()) {
+			assert(getPreviousItemPosition() >= 0);
+			int direction = keycode == Common::KEYCODE_PERIOD ? +1 : -1;
+			// Find the next available item.
+			int pos = getPreviousItemPosition() + direction;
+			while (true) {
+			      if (pos < 0)
+				      pos += kInventorySlots;
+			      else if (pos >= kInventorySlots)
+				      pos -= kInventorySlots;
+			      if (pos == getPreviousItemPosition() || _inventory[pos]) {
+				      break;
+			      }
+			      pos += direction;
+			}
+			// Swap it with the current item.
+			putItem(getCurrentItem(), getPreviousItemPosition());
+			GameItem* new_item = _inventory[pos];
+			setCurrentItem(new_item);
+			setPreviousItemPosition(pos);
+			removeItem(new_item);
+		}
+		break;
+	}
+	if (getLoopStatus() == kStatusOrdinary) {
+		updateOrdinaryCursor();
+	} else {
+		updateInventoryCursor();
+	}
 }
 
 void Game::dialogueMenu(int dialogueID) {
@@ -1297,10 +1383,11 @@ void Game::enterNewRoom() {
 	loadRoomObjects();
 	loadOverlays();
 
-	// Set room palette
-	const BAFile *f;
-	f = _vm->_paletteArchive->getFile(_currentRoom._palette);
-	_vm->_screen->setPalette(f->_data, 0, kNumColours);
+	// Draw the scene with the black palette and slowly fade into the right palette.
+	_vm->_screen->setPalette(NULL, 0, kNumColours);
+	_vm->_anims->drawScene(_vm->_screen->getSurface());
+	_vm->_screen->copyToScreen();
+	fadePalette(false);
 
 	// Run the program for the gate the dragon came through
 	debugC(6, kDraciLogicDebugLevel, "Running program for gate %d", _newGate);
