@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/draci/walking.cpp $
- * $Id: walking.cpp 46320 2009-12-09 21:03:22Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/draci/walking.cpp $
+ * $Id: walking.cpp 50890 2010-07-14 17:02:14Z eriktorbjorn $
  *
  */
 
@@ -324,7 +324,7 @@ void WalkingMap::drawOverlayRectangle(const Common::Point &p, byte colour, byte
 }
 
 int WalkingMap::pointsBetween(const Common::Point &p1, const Common::Point &p2) {
-	return MAX(abs(p2.x - p1.x), abs(p2.y - p1.y));
+	return MAX(ABS(p2.x - p1.x), ABS(p2.y - p1.y));
 }
 
 Common::Point WalkingMap::interpolate(const Common::Point &p1, const Common::Point &p2, int i, int n) {
@@ -636,7 +636,7 @@ bool WalkingState::walkOnNextEdge() {
 Movement WalkingState::animationForDirection(const Common::Point &here, const Common::Point &there) {
 	const int dx = there.x - here.x;
 	const int dy = there.y - here.y;
-	if (abs(dx) >= abs(dy)) {
+	if (ABS(dx) >= ABS(dy)) {
 		return dx >= 0 ? kMoveRight : kMoveLeft;
 	} else {
 		return dy >= 0 ? kMoveDown : kMoveUp;
