@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/drascula/drascula.cpp $
- * $Id: drascula.cpp 48013 2010-02-09 01:22:24Z peres001 $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/drascula/drascula.cpp $
+ * $Id: drascula.cpp 49722 2010-06-15 10:24:38Z sev $
  *
  */
 
@@ -33,9 +33,12 @@
 #include "base/plugins.h"
 #include "base/version.h"
 
+#include "engines/util.h"
+
 #include "sound/mixer.h"
 
 #include "drascula/drascula.h"
+#include "drascula/console.h"
 
 namespace Drascula {
 
@@ -85,6 +88,7 @@ DrasculaEngine::DrasculaEngine(OSystem *syst, const DrasculaGameDescription *gam
 	_textverbs = 0;
 	_textmisc = 0;
 	_textd1 = 0;
+	_talkSequences = 0;
 
 	_color = 0;
 	blinking = 0;
@@ -102,12 +106,18 @@ DrasculaEngine::DrasculaEngine(OSystem *syst, const DrasculaGameDescription *gam
 	_lang = kEnglish;
 
 	_keyBufferHead = _keyBufferTail = 0;
+
+	_roomHandlers = 0;
 }
 
 DrasculaEngine::~DrasculaEngine() {
 	delete _rnd;
 	stopSound();
 
+	freeRoomsTable();
+
+	delete _console;
+
 	free(_charMap);
 	free(_itemLocations);
 	free(_polX);
@@ -166,6 +176,8 @@ Common::Error DrasculaEngine::run() {
 		_lang = kEnglish;
 	}
 
+	_console = new Console(this);
+
 	if (!loadDrasculaDat())
 		return Common::kUnknownError;
 
@@ -217,7 +229,6 @@ Common::Error DrasculaEngine::run() {
 		term_int = 0;
 		musicStopped = 0;
 		selectionMade = 0;
-		_useMemForArj = false;
 		globalSpeed = 0;
 		curExcuseLook = 0;
 		curExcuseAction = 0;
@@ -235,6 +246,8 @@ Common::Error DrasculaEngine::run() {
 		if (currentChapter != 3)
 			loadPic(96, frontSurface, COMPLETE_PAL);
 
+		loadPic(99, cursorSurface);
+
 		if (currentChapter == 1) {
 		} else if (currentChapter == 2) {
 			loadPic("pts.alg", drawSurface2);
@@ -248,8 +261,12 @@ Common::Error DrasculaEngine::run() {
 			clearRoom();
 		} else if (currentChapter == 5) {
 		} else if (currentChapter == 6) {
-			igorX = 105, igorY = 85, trackIgor = 1;
-			drasculaX = 62, drasculaY = 99, trackDrascula = 1;
+			igorX = 105;
+			igorY = 85;
+			trackIgor = 1;
+			drasculaX = 62;
+			drasculaY = 99;
+			trackDrascula = 1;
 			actorFrames[kFramePendulum] = 0;
 			flag_tv = 0;
 		}
@@ -262,6 +279,7 @@ Common::Error DrasculaEngine::run() {
 			loadPic(974, tableSurface);
 
 		if (currentChapter != 2) {
+			loadPic(99, cursorSurface);
 			loadPic(99, backSurface);
 			loadPic(97, extraSurface);
 		}
@@ -310,7 +328,7 @@ bool DrasculaEngine::runCurrentChapter() {
 		}
 	}
 
-	for (n = 1; n < 43; n++)
+	for (n = 1; n < ARRAYSIZE(inventoryObjects); n++)
 		inventoryObjects[n] = 0;
 
 	for (n = 0; n < NUM_FLAGS; n++)
@@ -489,11 +507,15 @@ bool DrasculaEngine::runCurrentChapter() {
 #else
 		if (rightMouseButton == 1 && _menuScreen) {
 #endif
+			rightMouseButton = 0;
 			delay(100);
-			if (currentChapter == 2)
+			if (currentChapter == 2) {
+				loadPic(menuBackground, cursorSurface);
 				loadPic(menuBackground, backSurface);
-			else
+			} else {
+				loadPic(99, cursorSurface);
 				loadPic(99, backSurface);
+			}
 			setPalette((byte *)&gamePalette);
 			_menuScreen = false;
 #ifndef _WIN32_WCE
@@ -514,18 +536,24 @@ bool DrasculaEngine::runCurrentChapter() {
 		if (rightMouseButton == 1 && !_menuScreen &&
 			!(currentChapter == 5 && pickedObject == 16)) {
 #endif
+			rightMouseButton = 0;
 			delay(100);
 			characterMoved = 0;
 			if (trackProtagonist == 2)
 				trackProtagonist = 1;
-			if (currentChapter == 4)
+			if (currentChapter == 4) {
 				loadPic("icons2.alg", backSurface);
-			else if (currentChapter == 5)
+				loadPic("icons2.alg", cursorSurface);
+			} else if (currentChapter == 5) {
 				loadPic("icons3.alg", backSurface);
-			else if (currentChapter == 6)
+				loadPic("icons3.alg", cursorSurface);
+			} else if (currentChapter == 6) {
 				loadPic("iconsp.alg", backSurface);
-			else
+				loadPic("iconsp.alg", cursorSurface);
+			} else {
 				loadPic("icons.alg", backSurface);
+				loadPic("icons.alg", cursorSurface);
+			}
 			_menuScreen = true;
 #ifndef _WIN32_WCE
 			updateEvents();
@@ -584,6 +612,9 @@ bool DrasculaEngine::runCurrentChapter() {
 		} else if (key == Common::KEYCODE_ESCAPE) {
 			if (!confirmExit())
 				return false;
+		} else if (key == Common::KEYCODE_TILDE || key == Common::KEYCODE_BACKQUOTE) {
+			_console->attach();
+			_console->onFrame();
 		} else if (currentChapter == 6 && key == Common::KEYCODE_0 && roomNumber == 61) {
 			loadPic("alcbar.alg", bgSurface, 255);
 		}
@@ -731,10 +762,10 @@ void DrasculaEngine::updateEvents() {
 			leftMouseButton = 0;
 			break;
 		case Common::EVENT_RBUTTONDOWN:
-			rightMouseButton = 1;
+			// We changed semantic and react only on button up event
 			break;
 		case Common::EVENT_RBUTTONUP:
-			rightMouseButton = 0;
+			rightMouseButton = 1;
 			break;
 		case Common::EVENT_QUIT:
 			// TODO
@@ -748,15 +779,21 @@ void DrasculaEngine::updateEvents() {
 }
 
 void DrasculaEngine::delay(int ms) {
-	_system->delayMillis(ms * 2); // originaly was 1
+	uint32 end = _system->getMillis() + ms * 2; // originally was 1
+
+	do {
+		_system->delayMillis(10);
+		updateEvents();
+		_system->updateScreen();
+	} while (_system->getMillis() < end);
 }
 
 void DrasculaEngine::pause(int duration) {
-	_system->delayMillis(duration * 30); // was originaly 2
+	delay(duration * 15);
 }
 
 int DrasculaEngine::getTime() {
-	return _system->getMillis() / 20; // originaly was 1
+	return _system->getMillis() / 20; // originally was 1
 }
 
 void DrasculaEngine::reduce_hare_chico(int xx1, int yy1, int xx2, int yy2, int width, int height, int factor, byte *dir_inicio, byte *dir_fin) {
@@ -1029,7 +1066,7 @@ void DrasculaEngine::freeTexts(char **ptr) {
 	if (!ptr)
 		return;
 
-	free(ptr[0]);
+	free(*ptr);
 	free(ptr);
 }
 
