@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/drascula/drascula.h $
- * $Id: drascula.h 48834 2010-04-28 20:55:52Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/drascula/drascula.h $
+ * $Id: drascula.h 49713 2010-06-15 10:21:42Z sev $
  *
  */
 
@@ -27,15 +27,16 @@
 #define DRASCULA_H
 
 #include "common/scummsys.h"
+#include "common/archive.h"
 #include "common/endian.h"
-#include "common/util.h"
+#include "common/events.h"
 #include "common/file.h"
+#include "common/hash-str.h"
+#include "common/keyboard.h"
+#include "common/random.h"
 #include "common/savefile.h"
 #include "common/system.h"
-#include "common/hash-str.h"
-#include "common/events.h"
-#include "common/keyboard.h"
-#include "common/archive.h"
+#include "common/util.h"
 
 #include "sound/mixer.h"
 
@@ -311,8 +312,12 @@ public:
 
 #define KEYBUFSIZE		16
 
-static const int interf_x[] ={ 1, 65, 129, 193, 1, 65, 129 };
-static const int interf_y[] ={ 51, 51, 51, 51, 83, 83, 83 };
+static const int interf_x[] = { 1, 65, 129, 193, 1, 65, 129 };
+static const int interf_y[] = { 51, 51, 51, 51, 83, 83, 83 };
+
+struct RoomHandlers;
+
+class Console;
 
 class DrasculaEngine : public ::Engine {
 protected:
@@ -386,14 +391,13 @@ public:
 	// Graphics buffers/pointers
 	byte *bgSurface;
 	byte *backSurface;
+	byte *cursorSurface;
 	byte *drawSurface3;
 	byte *drawSurface2;
 	byte *tableSurface;
 	byte *extraSurface;	// not sure about this one, was "dir_hare_dch"
 	byte *screenSurface;
 	byte *frontSurface;
-	byte *memPtr;
-	byte *mSession;
 
 	byte cPal[768];
 
@@ -573,12 +577,8 @@ public:
 	void mixVideo(byte *OldScreen, byte *NewScreen);
 	void decodeRLE(byte *BufferRLE, byte *MiVideoRLE);
 	void decodeOffset(byte *BufferOFF, byte *MiVideoOFF, int length);
-	byte *TryInMem(Common::SeekableReadStream *stream);
 	int playFrameSSN(Common::SeekableReadStream *stream);
 
-	bool _useMemForArj;
-	byte CHUNK;
-	byte CMP, dacSSN[768];
 	int FrameSSN;
 	int globalSpeed;
 	uint32 LastFrame;
@@ -597,6 +597,7 @@ public:
 	int whichObject();
 	bool checkMenuFlags();
 	void setupRoomsTable();
+	void freeRoomsTable();
 	bool roomParse(int, int);
 	void cleanupString(char *string);
 	void playTalkSequence(int sequence);
@@ -735,6 +736,8 @@ public:
 private:
 	int _lang;
 
+	Console *_console;
+
 	CharInfo *_charMap;
 	int _charMapSize;
 
@@ -780,6 +783,9 @@ private:
 
 	char **loadTexts(Common::File &in);
 	void freeTexts(char **ptr);
+
+protected:
+	RoomHandlers	*_roomHandlers;
 };
 
 
