@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/gob/expression.cpp $
- * $Id: expression.cpp 44299 2009-09-24 10:02:13Z thebluegr $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/gob/expression.cpp $
+ * $Id: expression.cpp 52106 2010-08-15 23:37:56Z strangerke $
  *
  */
 
@@ -77,10 +77,10 @@ int32 Expression::encodePtr(byte *ptr, int type) {
 		offset = _vm->_game->_script->getOffset(ptr);
 		break;
 	case kInterVar:
-		offset = ptr - ((byte *) _vm->_inter->_variables->getAddressOff8(0));
+		offset = ptr - ((byte *)_vm->_inter->_variables->getAddressOff8(0));
 		break;
 	case kResStr:
-		offset = ptr - ((byte *) _resultStr);
+		offset = ptr - ((byte *)_resultStr);
 		break;
 	default:
 		error("Expression::encodePtr(): Unknown pointer type");
@@ -96,10 +96,10 @@ byte *Expression::decodePtr(int32 n) {
 	case kExecPtr:
 		return _vm->_game->_script->getData((n & 0x0FFFFFFF));
 	case kInterVar:
-		ptr = (byte *) _vm->_inter->_variables->getAddressOff8(0);
+		ptr = (byte *)_vm->_inter->_variables->getAddressOff8(0);
 		break;
 	case kResStr:
-		ptr = (byte *) _resultStr;
+		ptr = (byte *)_resultStr;
 		break;
 	default:
 		error("Expression::decodePtr(): Unknown pointer type");
@@ -495,7 +495,7 @@ int Expression::cmpHelper(const StackFrame &stackFrame) {
 	} else if (type == OP_LOAD_IMM_STR) {
 		if ((char *)decodePtr(stackFrame.values[-3]) != _resultStr) {
 			strcpy(_resultStr, (char *)decodePtr(stackFrame.values[-3]));
-			stackFrame.values[-3] = encodePtr((byte *) _resultStr, kResStr);
+			stackFrame.values[-3] = encodePtr((byte *)_resultStr, kResStr);
 		}
 		cmpTemp = strcmp(_resultStr, (char *)decodePtr(stackFrame.values[-1]));
 	}
@@ -727,7 +727,7 @@ void Expression::loadValue(byte operation, uint32 varBase, const StackFrame &sta
 
 	case OP_LOAD_IMM_STR:
 		*stackFrame.opers = OP_LOAD_IMM_STR;
-		*stackFrame.values = encodePtr((byte *) _vm->_game->_script->readString(), kExecPtr);
+		*stackFrame.values = encodePtr((byte *)_vm->_game->_script->readString(), kExecPtr);
 		break;
 
 	case OP_LOAD_VAR_INT32:
@@ -797,11 +797,11 @@ void Expression::simpleArithmetic1(StackFrame &stackFrame) {
 	switch (stackFrame.opers[-1]) {
 	case OP_ADD:
 		if (stackFrame.opers[-2] == OP_LOAD_IMM_STR) {
-			if ((char *) decodePtr(stackFrame.values[-2]) != _resultStr) {
-				strcpy(_resultStr, (char *) decodePtr(stackFrame.values[-2]));
-				stackFrame.values[-2] = encodePtr((byte *) _resultStr, kResStr);
+			if ((char *)decodePtr(stackFrame.values[-2]) != _resultStr) {
+				strcpy(_resultStr, (char *)decodePtr(stackFrame.values[-2]));
+				stackFrame.values[-2] = encodePtr((byte *)_resultStr, kResStr);
 			}
-			strcat(_resultStr, (char *) decodePtr(stackFrame.values[0]));
+			strcat(_resultStr, (char *)decodePtr(stackFrame.values[0]));
 			stackFrame.pop(2);
 		}
 		break;
@@ -873,12 +873,12 @@ bool Expression::complexArithmetic(Stack &stack, StackFrame &stackFrame, int16 b
 		if (stack.opers[brackStart] == OP_LOAD_IMM_INT16) {
 			stack.values[brackStart] += stackFrame.values[-1];
 		} else if (stack.opers[brackStart] == OP_LOAD_IMM_STR) {
-			if ((char *) decodePtr(stack.values[brackStart]) != _resultStr) {
-				strcpy(_resultStr, (char *) decodePtr(stack.values[brackStart]));
+			if ((char *)decodePtr(stack.values[brackStart]) != _resultStr) {
+				strcpy(_resultStr, (char *)decodePtr(stack.values[brackStart]));
 				stack.values[brackStart] =
-					encodePtr((byte *) _resultStr, kResStr);
+					encodePtr((byte *)_resultStr, kResStr);
 			}
-			strcat(_resultStr, (char *) decodePtr(stackFrame.values[-1]));
+			strcat(_resultStr, (char *)decodePtr(stackFrame.values[-1]));
 		}
 		stackFrame.pop(2);
 		break;
@@ -982,8 +982,8 @@ void Expression::getResult(byte operation, int32 value, byte *type) {
 		break;
 
 	case OP_LOAD_IMM_STR:
-		if ((char *) decodePtr(value) != _resultStr)
-			strcpy(_resultStr, (char *) decodePtr(value));
+		if ((char *)decodePtr(value) != _resultStr)
+			strcpy(_resultStr, (char *)decodePtr(value));
 		break;
 
 	case OP_LOAD_VAR_INT32:
@@ -1002,7 +1002,6 @@ int16 Expression::parseExpr(byte stopToken, byte *type) {
 	Stack stack;
 	StackFrame stackFrame(stack);
 	byte operation;
-	bool escape;
 	int16 brackStart;
 	uint32 varBase;
 
@@ -1037,7 +1036,6 @@ int16 Expression::parseExpr(byte stopToken, byte *type) {
 		if ((operation == stopToken) || (operation == OP_OR) ||
 				(operation == OP_AND) || (operation == OP_END_EXPR)) {
 			while (stackFrame.pos >= 2) {
-				escape = false;
 				if ((stackFrame.opers[-2] == OP_BEGIN_EXPR) &&
 						((operation == OP_END_EXPR) || (operation == stopToken))) {
 					stackFrame.opers[-2] = stackFrame.opers[-1];
@@ -1113,11 +1111,11 @@ int16 Expression::parseExpr(byte stopToken, byte *type) {
 					if (stackFrame.opers[-3] == OP_LOAD_IMM_INT16) {
 						stackFrame.values[-3] += stackFrame.values[-1];
 					} else if (stackFrame.opers[-3] == OP_LOAD_IMM_STR) {
-						if ((char *) decodePtr(stackFrame.values[-3]) != _resultStr) {
-							strcpy(_resultStr, (char *) decodePtr(stackFrame.values[-3]));
-							stackFrame.values[-3] = encodePtr((byte *) _resultStr, kResStr);
+						if ((char *)decodePtr(stackFrame.values[-3]) != _resultStr) {
+							strcpy(_resultStr, (char *)decodePtr(stackFrame.values[-3]));
+							stackFrame.values[-3] = encodePtr((byte *)_resultStr, kResStr);
 						}
-						strcat(_resultStr, (char *) decodePtr(stackFrame.values[-1]));
+						strcat(_resultStr, (char *)decodePtr(stackFrame.values[-1]));
 					}
 					stackFrame.pop(2);
 
