@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/gob/goblin_v1.cpp $
- * $Id: goblin_v1.cpp 46784 2009-12-30 23:21:33Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/gob/goblin_v1.cpp $
+ * $Id: goblin_v1.cpp 53795 2010-10-25 03:47:53Z drmccoy $
  *
  */
 
@@ -173,10 +173,10 @@ void Goblin_v1::movePathFind(Mult::Mult_Object *obj,
 			_pathExistence = 0;
 		}
 
-		nextAct = _vm->_map->getDirection(_vm->_map->_curGoblinX,
+		nextAct = (int16) _vm->_map->getDirection(_vm->_map->_curGoblinX,
 				_vm->_map->_curGoblinY, _vm->_map->_destX, _vm->_map->_destY);
 
-		if (nextAct == 0)
+		if (nextAct == Map::kDirNone)
 			_pathExistence = 0;
 	} else if (_pathExistence == 3) {
 		_vm->_map->_curGoblinX = _gobPositions[_currentGoblin].x;
@@ -229,7 +229,7 @@ void Goblin_v1::movePathFind(Mult::Mult_Object *obj,
 					}
 				}
 			}
-			nextAct = _vm->_map->getDirection(_vm->_map->_curGoblinX,
+			nextAct = (int16) _vm->_map->getDirection(_vm->_map->_curGoblinX,
 					_vm->_map->_curGoblinY, _vm->_map->_destX, _vm->_map->_destY);
 		}
 	}
