@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/gob/inter.h $
- * $Id: inter.h 48061 2010-02-14 18:30:12Z strangerke $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/gob/inter.h $
+ * $Id: inter.h 52316 2010-08-24 06:25:08Z strangerke $
  *
  */
 
@@ -432,6 +432,8 @@ protected:
 
 	void oFascin_playProtracker(OpGobParams &params);
 
+	bool oFascin_repeatUntil(OpFuncParams &params);
+	bool oFascin_assign(OpFuncParams &params);
 	bool oFascin_copySprite(OpFuncParams &params);
 	bool oFascin_keyFunc(OpFuncParams &params);
 
@@ -452,6 +454,7 @@ protected:
 	void oFascin_closeWin();
 	void oFascin_activeWin();
 	void oFascin_openWin();
+	void oFascin_initCursorAnim();
 	void oFascin_setRenderFlags();
 	void oFascin_setWinFlags();
 };
