@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/gob/inter_v1.cpp $
- * $Id: inter_v1.cpp 48011 2010-02-08 23:12:42Z strangerke $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/gob/inter_v1.cpp $
+ * $Id: inter_v1.cpp 54074 2010-11-04 20:42:19Z drmccoy $
  *
  */
 
@@ -328,7 +328,7 @@ void Inter_v1::o1_initCursor() {
 			(height != _vm->_draw->_cursorHeight) ||
 	    (_vm->_draw->_cursorSprites->getWidth() != (width * count))) {
 
-		_vm->_draw->freeSprite(23);
+		_vm->_draw->freeSprite(Draw::kCursorSurface);
 		_vm->_draw->_cursorSprites.reset();
 		_vm->_draw->_cursorSpritesBack.reset();
 		_vm->_draw->_scummvmCursor.reset();
@@ -344,9 +344,9 @@ void Inter_v1::o1_initCursor() {
 		if (count > 0x80)
 			count -= 0x80;
 
-		_vm->_draw->initSpriteSurf(23, _vm->_draw->_cursorWidth * count,
+		_vm->_draw->initSpriteSurf(Draw::kCursorSurface, _vm->_draw->_cursorWidth * count,
 				_vm->_draw->_cursorHeight, 2);
-		_vm->_draw->_cursorSpritesBack = _vm->_draw->_spritesArray[23];
+		_vm->_draw->_cursorSpritesBack = _vm->_draw->_spritesArray[Draw::kCursorSurface];
 		_vm->_draw->_cursorSprites = _vm->_draw->_cursorSpritesBack;
 
 		_vm->_draw->_scummvmCursor =
@@ -468,7 +468,7 @@ void Inter_v1::o1_initMult() {
 			_vm->_mult->_objects[i].pPosY = new VariableReference(*_variables, offPosY);
 
 			_vm->_mult->_objects[i].pAnimData =
-				(Mult::Mult_AnimData *) _variables->getAddressOff8(offAnim);
+				(Mult::Mult_AnimData *)_variables->getAddressOff8(offAnim);
 
 			_vm->_mult->_objects[i].pAnimData->isStatic = 1;
 			_vm->_mult->_objects[i].tick = 0;
@@ -482,14 +482,14 @@ void Inter_v1::o1_initMult() {
 	if (_vm->_mult->_animSurf &&
 	    ((oldAnimWidth != _vm->_mult->_animWidth) ||
 			 (oldAnimHeight != _vm->_mult->_animHeight))) {
-		_vm->_draw->freeSprite(22);
+		_vm->_draw->freeSprite(Draw::kAnimSurface);
 		_vm->_mult->_animSurf.reset();
 	}
 
 	if (!_vm->_mult->_animSurf) {
-		_vm->_draw->initSpriteSurf(22, _vm->_mult->_animWidth,
+		_vm->_draw->initSpriteSurf(Draw::kAnimSurface, _vm->_mult->_animWidth,
 				_vm->_mult->_animHeight, 0);
-		_vm->_mult->_animSurf = _vm->_draw->_spritesArray[22];
+		_vm->_mult->_animSurf = _vm->_draw->_spritesArray[Draw::kAnimSurface];
 	}
 
 	_vm->_video->drawSprite(*_vm->_draw->_backSurface, *_vm->_mult->_animSurf,
@@ -526,7 +526,7 @@ void Inter_v1::o1_loadMultObject() {
 
 	debugC(4, kDebugGameFlow, "Loading mult object %d", objIndex);
 
-	multData = (byte *) _vm->_mult->_objects[objIndex].pAnimData;
+	multData = (byte *)_vm->_mult->_objects[objIndex].pAnimData;
 	for (int i = 0; i < 11; i++) {
 		if (_vm->_game->_script->peekUint16() != 99) {
 			_vm->_game->_script->evalExpr(&val);
@@ -813,6 +813,14 @@ bool Inter_v1::o1_if(OpFuncParams &params) {
 	byte cmd;
 	bool boolRes;
 
+	// WORKAROUND: Gob1 goblin stuck on reload bugs present in original - bugs #3018918 and 3065914
+	if ((_vm->getGameType() == kGameTypeGob1) && (_vm->_game->_script->pos() == 2933) &&
+			!scumm_stricmp(_vm->_game->_curTotFile, "inter.tot") && VAR(285) != 0) {
+		warning("Workaround for Gob1 Goblin Stuck On Reload Bug applied...");
+		// VAR(59) actually locks goblin movement, but these variables trigger this in the script.
+		WRITE_VAR(285, 0);
+	}
+
 	boolRes = _vm->_game->_script->evalBoolResult();
 	if (boolRes) {
 		if ((params.counter == params.cmdCount) && (params.retFlag == 2))
@@ -914,7 +922,7 @@ bool Inter_v1::o1_printText(OpFuncParams &params) {
 	_vm->_draw->_backColor = _vm->_game->_script->readValExpr();
 	_vm->_draw->_frontColor = _vm->_game->_script->readValExpr();
 	_vm->_draw->_fontIndex = _vm->_game->_script->readValExpr();
-	_vm->_draw->_destSurface = 21;
+	_vm->_draw->_destSurface = Draw::kBackSurface;
 	_vm->_draw->_textToPrint = buf;
 	_vm->_draw->_transparency = 0;
 
@@ -1119,12 +1127,12 @@ bool Inter_v1::o1_palLoad(OpFuncParams &params) {
 		if (!resource)
 			break;
 
-		memcpy((char *) _vm->_draw->_vgaPalette, resource->getData(), MIN<int>(768, resource->getSize()));
+		memcpy((char *)_vm->_draw->_vgaPalette, resource->getData(), MIN<int>(768, resource->getSize()));
 		delete resource;
 		break;
 
 	case 54:
-		memset((char *) _vm->_draw->_vgaPalette, 0, 768);
+		memset((char *)_vm->_draw->_vgaPalette, 0, 768);
 		break;
 
 	case 61:
@@ -1135,7 +1143,7 @@ bool Inter_v1::o1_palLoad(OpFuncParams &params) {
 		if (!resource)
 			break;
 
-		memcpy((char *) _vm->_draw->_vgaPalette + index1 * 3,
+		memcpy((char *)_vm->_draw->_vgaPalette + index1 * 3,
 		       resource->getData() + index1 * 3, index2);
 		delete resource;
 
@@ -1331,7 +1339,8 @@ bool Inter_v1::o1_goblinFunc(OpFuncParams &params) {
 	gobParams.retVarPtr.set(*_variables, 236);
 
 	cmd = _vm->_game->_script->readInt16();
-	_vm->_game->_script->skip(2);
+	gobParams.paramCount = _vm->_game->_script->readInt16();
+
 	if ((cmd > 0) && (cmd < 17)) {
 		objDescSet = true;
 		gobParams.extraData = _vm->_game->_script->readInt16();
@@ -1778,7 +1787,7 @@ bool Inter_v1::o1_readData(OpFuncParams &params) {
 		if (((dataVar >> 2) == 59) && (size == 4))
 			WRITE_VAR(59, stream->readUint32LE());
 		else
-			retSize = stream->read((byte *) _variables->getAddressOff8(dataVar), size);
+			retSize = stream->read((byte *)_variables->getAddressOff8(dataVar), size);
 
 		if (retSize == size)
 			WRITE_VAR(1, 0);
