@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/gob/inter_v2.cpp $
- * $Id: inter_v2.cpp 43713 2009-08-24 22:37:09Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/gob/inter_v2.cpp $
+ * $Id: inter_v2.cpp 51895 2010-08-08 00:55:27Z drmccoy $
  *
  */
 
@@ -344,7 +344,7 @@ void Inter_v2::o2_initMult() {
 			_vm->_mult->_objects[i].pPosY = new VariableReference(*_vm->_inter->_variables, offPosY);
 
 			_vm->_mult->_objects[i].pAnimData =
-				(Mult::Mult_AnimData *) _variables->getAddressOff8(offAnim);
+				(Mult::Mult_AnimData *)_variables->getAddressOff8(offAnim);
 
 			_vm->_mult->_objects[i].pAnimData->isStatic = 1;
 			_vm->_mult->_objects[i].tick = 0;
@@ -360,24 +360,24 @@ void Inter_v2::o2_initMult() {
 	if (_vm->_mult->_animSurf &&
 	    ((oldAnimWidth != _vm->_mult->_animWidth) ||
 			 (oldAnimHeight != _vm->_mult->_animHeight))) {
-		_vm->_draw->freeSprite(22);
+		_vm->_draw->freeSprite(Draw::kAnimSurface);
 		_vm->_mult->_animSurf.reset();
 	}
 
 	_vm->_draw->adjustCoords(0,
 			&_vm->_mult->_animWidth, &_vm->_mult->_animHeight);
 	if (!_vm->_mult->_animSurf) {
-		_vm->_draw->initSpriteSurf(22, _vm->_mult->_animWidth,
+		_vm->_draw->initSpriteSurf(Draw::kAnimSurface, _vm->_mult->_animWidth,
 				_vm->_mult->_animHeight, 0);
-		_vm->_mult->_animSurf = _vm->_draw->_spritesArray[22];
+		_vm->_mult->_animSurf = _vm->_draw->_spritesArray[Draw::kAnimSurface];
 		if (_terminate)
 			return;
 	}
 
 	_vm->_draw->adjustCoords(1,
 			&_vm->_mult->_animWidth, &_vm->_mult->_animHeight);
-	_vm->_draw->_sourceSurface = 21;
-	_vm->_draw->_destSurface = 22;
+	_vm->_draw->_sourceSurface = Draw::kBackSurface;
+	_vm->_draw->_destSurface = Draw::kAnimSurface;
 	_vm->_draw->_spriteLeft = _vm->_mult->_animLeft;
 	_vm->_draw->_spriteTop = _vm->_mult->_animTop;
 	_vm->_draw->_spriteRight = _vm->_mult->_animWidth;
@@ -409,7 +409,7 @@ void Inter_v2::o2_loadMultObject() {
 
 	debugC(4, kDebugGameFlow, "Loading mult object %d", objIndex);
 
-	multData = (byte *) _vm->_mult->_objects[objIndex].pAnimData;
+	multData = (byte *)_vm->_mult->_objects[objIndex].pAnimData;
 	for (int i = 0; i < 11; i++) {
 		if (_vm->_game->_script->peekByte() != 99)
 			multData[i] = _vm->_game->_script->readValExpr();
@@ -481,7 +481,7 @@ void Inter_v2::o2_loadMultObject() {
 		if ((((int32) *(obj.pPosX)) == -1234) && (((int32) *(obj.pPosY)) == -4321)) {
 
 			if (obj.videoSlot > 0)
-				_vm->_vidPlayer->slotClose(obj.videoSlot - 1);
+				_vm->_vidPlayer->closeVideo(obj.videoSlot - 1);
 
 			obj.videoSlot = 0;
 			obj.lastLeft = -1;
@@ -959,50 +959,50 @@ void Inter_v2::o2_setScrollOffset() {
 
 void Inter_v2::o2_playImd() {
 	char imd[128];
-	int16 x, y;
-	int16 startFrame;
-	int16 lastFrame;
-	int16 breakKey;
-	int16 flags;
-	int16 palStart;
-	int16 palEnd;
-	uint16 palCmd;
 	bool close;
 
 	_vm->_game->_script->evalExpr(0);
 	_vm->_game->_script->getResultStr()[8] = 0;
 	strncpy0(imd, _vm->_game->_script->getResultStr(), 127);
 
-	x = _vm->_game->_script->readValExpr();
-	y = _vm->_game->_script->readValExpr();
-	startFrame = _vm->_game->_script->readValExpr();
-	lastFrame = _vm->_game->_script->readValExpr();
-	breakKey = _vm->_game->_script->readValExpr();
-	flags = _vm->_game->_script->readValExpr();
-	palStart = _vm->_game->_script->readValExpr();
-	palEnd = _vm->_game->_script->readValExpr();
-	palCmd = 1 << (flags & 0x3F);
+	VideoPlayer::Properties props;
+
+	props.x          = _vm->_game->_script->readValExpr();
+	props.y          = _vm->_game->_script->readValExpr();
+	props.startFrame = _vm->_game->_script->readValExpr();
+	props.lastFrame  = _vm->_game->_script->readValExpr();
+	props.breakKey   = _vm->_game->_script->readValExpr();
+	props.flags      = _vm->_game->_script->readValExpr();
+	props.palStart   = _vm->_game->_script->readValExpr();
+	props.palEnd     = _vm->_game->_script->readValExpr();
+	props.palCmd     = 1 << (props.flags & 0x3F);
 
 	debugC(1, kDebugVideo, "Playing video \"%s\" @ %d+%d, frames %d - %d, "
-			"paletteCmd %d (%d - %d), flags %X", _vm->_game->_script->getResultStr(), x, y,
-			startFrame, lastFrame, palCmd, palStart, palEnd, flags);
+			"paletteCmd %d (%d - %d), flags %X", imd,
+			props.x, props.y, props.startFrame, props.lastFrame,
+			props.palCmd, props.palStart, props.palEnd, props.flags);
 
-	if ((imd[0] != 0) && !_vm->_vidPlayer->primaryOpen(imd, x, y, flags)) {
-		WRITE_VAR(11, (uint32) -1);
-		return;
+	int slot = 0;
+	if (imd[0] != 0) {
+		_vm->_vidPlayer->evaluateFlags(props);
+		if ((slot = _vm->_vidPlayer->openVideo(true, imd, props)) < 0) {
+			WRITE_VAR(11, (uint32) -1);
+			return;
+		}
 	}
 
-	close = (lastFrame == -1);
-	if (startFrame == -2) {
-		startFrame = lastFrame = 0;
+	close = (props.lastFrame == -1);
+	if (props.startFrame == -2) {
+		props.startFrame = 0;
+		props.lastFrame  = 0;
 		close = false;
 	}
 
-	if (startFrame >= 0)
-		_vm->_vidPlayer->primaryPlay(startFrame, lastFrame, breakKey, palCmd, palStart, palEnd, 0);
+	if (props.startFrame >= 0)
+		_vm->_vidPlayer->play(slot, props);
 
 	if (close)
-		_vm->_vidPlayer->primaryClose();
+		_vm->_vidPlayer->closeVideo(slot);
 }
 
 void Inter_v2::o2_getImdInfo() {
@@ -1011,10 +1011,10 @@ void Inter_v2::o2_getImdInfo() {
 	int16 varWidth, varHeight;
 
 	_vm->_game->_script->evalExpr(0);
-	varX = _vm->_game->_script->readVarIndex();
-	varY = _vm->_game->_script->readVarIndex();
+	varX      = _vm->_game->_script->readVarIndex();
+	varY      = _vm->_game->_script->readVarIndex();
 	varFrames = _vm->_game->_script->readVarIndex();
-	varWidth = _vm->_game->_script->readVarIndex();
+	varWidth  = _vm->_game->_script->readVarIndex();
 	varHeight = _vm->_game->_script->readVarIndex();
 
 	// WORKAROUND: The nut rolling animation in the administration center
@@ -1106,7 +1106,7 @@ bool Inter_v2::o2_printText(OpFuncParams &params) {
 	_vm->_draw->_backColor = _vm->_game->_script->readValExpr();
 	_vm->_draw->_frontColor = _vm->_game->_script->readValExpr();
 	_vm->_draw->_fontIndex = _vm->_game->_script->readValExpr();
-	_vm->_draw->_destSurface = 21;
+	_vm->_draw->_destSurface = Draw::kBackSurface;
 	_vm->_draw->_textToPrint = buf;
 	_vm->_draw->_transparency = 0;
 
