@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/gob/mult_v1.cpp $
- * $Id: mult_v1.cpp 41839 2009-06-24 21:49:37Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/gob/mult_v1.cpp $
+ * $Id: mult_v1.cpp 51852 2010-08-08 00:35:18Z drmccoy $
  *
  */
 
@@ -236,7 +236,7 @@ void Mult_v1::freeMultKeys() {
 		_animArrayData = 0;
 
 		_animSurf.reset();
-		_vm->_draw->freeSprite(22);
+		_vm->_draw->freeSprite(Draw::kAnimSurface);
 
 		_animDataAllocated = false;
 	}
@@ -318,7 +318,7 @@ void Mult_v1::playMultInit() {
 
 		_animSurf = _vm->_video->initSurfDesc(_vm->_global->_videoMode,
 				320, 200, 0);
-		_vm->_draw->_spritesArray[22] = _animSurf;
+		_vm->_draw->_spritesArray[Draw::kAnimSurface] = _animSurf;
 
 		_vm->_video->drawSprite(*_vm->_draw->_backSurface,
 			*_animSurf, 0, 0, 319, 199, 0, 0, 0);
@@ -579,8 +579,8 @@ void Mult_v1::animate() {
 		if ((pNeedRedraw[i] == 0) || (_objects[i].lastLeft == -1))
 			continue;
 
-		_vm->_draw->_sourceSurface = 22;
-		_vm->_draw->_destSurface = 21;
+		_vm->_draw->_sourceSurface = Draw::kAnimSurface;
+		_vm->_draw->_destSurface = Draw::kBackSurface;
 		_vm->_draw->_spriteLeft = pDirtyLefts[i] - _animLeft;
 		_vm->_draw->_spriteTop = pDirtyTops[i] - _animTop;
 		_vm->_draw->_spriteRight = pDirtyRights[i] - pDirtyLefts[i] + 1;
