@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/gob/sound/adlib.cpp $
- * $Id: adlib.cpp 47839 2010-02-03 03:25:50Z drmccoy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/gob/sound/adlib.cpp $
+ * $Id: adlib.cpp 48637 2010-04-12 09:14:17Z fingolfin $
  *
  */
 
@@ -74,7 +74,7 @@ void AdLib::init() {
 		_pollNotes[i] = 0;
 	setFreqs();
 
-	_mixer->playInputStream(Audio::Mixer::kMusicSoundType, &_handle,
+	_mixer->playStream(Audio::Mixer::kMusicSoundType, &_handle,
 			this, -1, Audio::Mixer::kMaxChannelVolume, 0, DisposeAfterUse::NO, true);
 }
 
