@@ -18,16 +18,19 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/groovie/music.cpp $
- * $Id: music.cpp 47333 2010-01-16 21:34:36Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/groovie/music.cpp $
+ * $Id: music.cpp 52736 2010-09-15 22:00:20Z lordhoto $
  *
  */
 
 #include "groovie/music.h"
+#include "groovie/groovie.h"
 #include "groovie/resource.h"
 
 #include "common/config-manager.h"
+#include "common/macresman.h"
 #include "sound/audiocd.h"
+#include "sound/midiparser.h"
 
 namespace Groovie {
 
@@ -108,6 +111,20 @@ void MusicPlayer::playCD(uint8 track) {
 
 	// Play the track starting at the requested offset (1000ms = 75 frames)
 	AudioCD.play(track - 1, 1, startms * 75 / 1000, 0);
+
+	// If the audio is not playing from the CD, play the "fallback" MIDI.
+	// The Mac version has no CD tracks, so it will always use the MIDI.
+	if (!AudioCD.isPlaying()) {
+		if (track == 2) {
+			// Intro MIDI fallback
+			if (_vm->getPlatform() == Common::kPlatformMacintosh)
+				playSong(70);
+			else
+				playSong((19 << 10) | 36); // XMI.GJD, file 36
+		} else if (track == 3) {
+			// TODO: Credits MIDI fallback
+		}
+	}
 }
 
 void MusicPlayer::startBackground() {
@@ -383,8 +400,8 @@ MusicPlayerXMI::MusicPlayerXMI(GroovieEngine *vm, const Common::String &gtlName)
 	_midiParser = MidiParser::createParser_XMIDI();
 
 	// Create the driver
-	MidiDriverType driver = detectMusicDriver(MDT_MIDI | MDT_ADLIB | MDT_PREFER_MIDI);
-	_driver = createMidi(driver);
+	MidiDriver::DeviceHandle dev = MidiDriver::detectDevice(MDT_MIDI | MDT_ADLIB | MDT_PREFER_GM);
+	_driver = createMidi(dev);
 	this->open();
 
 	// Set the parser's driver
@@ -399,9 +416,9 @@ MusicPlayerXMI::MusicPlayerXMI(GroovieEngine *vm, const Common::String &gtlName)
 	}
 
 	// Load the Global Timbre Library
-	if (driver == MD_ADLIB) {
+	if (MidiDriver::getMusicType(dev) == MT_ADLIB) {
 		// MIDI through AdLib
-		_musicType = MD_ADLIB;
+		_musicType = MT_ADLIB;
 		loadTimbres(gtlName + ".ad");
 
 		// Setup the percussion channel
@@ -409,11 +426,15 @@ MusicPlayerXMI::MusicPlayerXMI(GroovieEngine *vm, const Common::String &gtlName)
 			if (_timbres[i].bank == 0x7F)
 				setTimbreAD(9, _timbres[i]);
 		}
-	} else if ((driver == MD_MT32) || ConfMan.getBool("native_mt32")) {
+	} else if ((MidiDriver::getMusicType(dev) == MT_MT32) || ConfMan.getBool("native_mt32")) {
+		_driver->sendMT32Reset();
+
 		// MT-32
-		_musicType = MD_MT32;
+		_musicType = MT_MT32;
 		loadTimbres(gtlName + ".mt");
 	} else {
+		_driver->sendGMReset();
+
 		// GM
 		_musicType = 0;
 	}
@@ -452,9 +473,9 @@ void MusicPlayerXMI::send(uint32 b) {
 			for (int i = 0; i < numTimbres; i++) {
 				if ((_timbres[i].bank == _chanBanks[chan]) &&
 					(_timbres[i].patch == patch)) {
-					if (_musicType == MD_ADLIB) {
+					if (_musicType == MT_ADLIB) {
 						setTimbreAD(chan, _timbres[i]);
-					} else if (_musicType == MD_MT32) {
+					} else if (_musicType == MT_MT32) {
 						setTimbreMT(chan, _timbres[i]);
 					}
 					return;
@@ -609,42 +630,155 @@ void MusicPlayerXMI::setTimbreAD(byte channel, const Timbre &timbre) {
 	}
 }
 
-void MusicPlayerXMI::setTimbreMT(byte channel, const Timbre &timbre) {
-	// Verify the timbre size
-	if (timbre.size != 0xF6) {
-		error("Groovie::Music: Invalid size for an MT-32 timbre: %d", timbre.size);
-	}
+
+#include "common/pack-start.h"	// START STRUCT PACKING
+
+struct RolandInstrumentSysex {
+	byte roland_id;
+	byte device_id;
+	byte model_id;
+	byte command;
+	byte address[3];
+	byte instrument[0xF6];
+	byte checksum;
+} PACKED_STRUCT;
+
+#include "common/pack-end.h"	// END STRUCT PACKING
+
+void setRolandInstrument(MidiDriver *drv, byte channel, byte *instrument) {
+	RolandInstrumentSysex sysex;
+	memcpy(&sysex.instrument, instrument, 0xF6);
 
 	// Show the timbre name as extra debug information
-	Common::String name((char*)timbre.data, 10);
-	debugC(5, kGroovieDebugMIDI | kGroovieDebugAll, "Groovie::Music: Using MT32 timbre: %s", name.c_str());
+	Common::String name((char *)instrument, 10);
+	debugC(5, kGroovieDebugMIDI | kGroovieDebugAll, "Groovie::Music: Setting MT32 timbre '%s' to channel %d", name.c_str(), channel);
 
-	// TODO: Support MT-32 custom instruments
-	warning("Groovie::Music: Setting MT32 custom instruments isn't supported yet");
+	sysex.roland_id = 0x41;
+	sysex.device_id = channel; // Unit#
+	sysex.model_id = 0x16; // MT32
+	sysex.command = 0x12; // Data set
+
+	// Remap instrument to appropriate address space.
+	int address = 0x008000;
+	sysex.address[0] = (address >> 14) & 0x7F;
+	sysex.address[1] = (address >>  7) & 0x7F;
+	sysex.address[2] = (address      ) & 0x7F;
+
+	// Compute the checksum.
+	byte checksum = 0;
+	byte *ptr = sysex.address;
+	for (int i = 4; i < (int)sizeof(RolandInstrumentSysex) - 1; ++i)
+		checksum -= *ptr++;
+	sysex.checksum = checksum & 0x7F;
+
+	// Send sysex
+	drv->sysEx((byte *)&sysex, sizeof(RolandInstrumentSysex));
+
+
+	// Wait the time it takes to send the SysEx data
+	uint32 delay = (sizeof(RolandInstrumentSysex) + 2) * 1000 / 3125;
+
+	// Plus an additional delay for the MT-32 rev00
+	delay += 40;
+
+	g_system->delayMillis(delay);
+}
+
+void MusicPlayerXMI::setTimbreMT(byte channel, const Timbre &timbre) {
+	// Verify the timbre size
+	if (timbre.size != 0xF6)
+		error("Groovie::Music: Invalid size for an MT-32 timbre: %d", timbre.size);
+
+	setRolandInstrument(_driver, channel, timbre.data);
 }
 
 
 // MusicPlayerMac
 
-MusicPlayerMac::MusicPlayerMac(GroovieEngine *vm) :
-	MusicPlayerMidi(vm) {
+MusicPlayerMac::MusicPlayerMac(GroovieEngine *vm) : MusicPlayerMidi(vm) {
+	// Create the parser
+	_midiParser = MidiParser::createParser_SMF();
+
+	// Create the driver
+	MidiDriver::DeviceHandle dev = MidiDriver::detectDevice(MDT_MIDI | MDT_ADLIB | MDT_PREFER_GM);
+	_driver = createMidi(dev);
+	this->open();
+
+	// Set the parser's driver
+	_midiParser->setMidiDriver(this);
+
+	// Set the timer rate
+	_midiParser->setTimerRate(_driver->getBaseTempo());
+
+	// Sanity check
+	assert(_vm->_macResFork);
 }
 
 bool MusicPlayerMac::load(uint32 fileref, bool loop) {
 	debugC(1, kGroovieDebugMIDI | kGroovieDebugAll, "Groovie::Music: Starting the playback of song: %04X", fileref);
-	debug("Groovie::Music: Starting the playback of song: %04X %d", fileref, fileref);
 
-	// Open the song resource
-	/*
-	Common::SeekableReadStream *file = _vm->_resMan->open(fileref);
-	if (!file) {
-		error("Groovie::Music: Couldn't find resource 0x%04X", fileref);
-		return false;
+	// First try for compressed MIDI
+	Common::SeekableReadStream *file = _vm->_macResFork->getResource(MKID_BE('cmid'), fileref & 0x3FF);
+
+	if (file) {
+		// Found the resource, decompress it
+		Common::SeekableReadStream *tmp = decompressMidi(file);
+		delete file;
+		file = tmp;
+	} else {
+		// Otherwise, it's uncompressed
+		file = _vm->_macResFork->getResource(MKID_BE('Midi'), fileref & 0x3FF);
+		if (!file)
+			error("Groovie::Music: Couldn't find resource 0x%04X", fileref);
 	}
-	*/
 
-	//return loadParser(file, loop);
-	return false;
+	return loadParser(file, loop);
+}
+
+Common::SeekableReadStream *MusicPlayerMac::decompressMidi(Common::SeekableReadStream *stream) {
+	// Initialize an output buffer of the given size
+	uint32 size = stream->readUint32BE();
+	byte *output = (byte *)malloc(size);
+
+	byte *current = output;
+	uint32 decompBytes = 0;
+	while ((decompBytes < size) && !stream->eos()) {
+		// 8 flags
+		byte flags = stream->readByte();
+
+		for (byte i = 0; (i < 8) && !stream->eos(); i++) {
+			if (flags & 1) {
+				// 1: Next byte is a literal
+				*(current++) = stream->readByte();
+				if (stream->eos())
+					continue;
+				decompBytes++;
+			} else {
+				// 0: It's a reference to part of the history
+				uint16 args = stream->readUint16BE();
+				if (stream->eos())
+					continue;
+
+				// Length = 4bit unsigned (3 minimal)
+				uint8 length = (args >> 12) + 3;
+
+				// Offset = 12bit signed (all values are negative)
+				int16 offset = (args & 0xFFF) | 0xF000;
+
+				// Copy from the past decompressed bytes
+				decompBytes += length;
+				while (length > 0) {
+					*(current) = *(current + offset);
+					current++;
+					length--;
+				}
+			}
+			flags = flags >> 1;
+		}
+	}
+
+	// Return the output buffer wrapped in a MemoryReadStream
+	return new Common::MemoryReadStream(output, size, DisposeAfterUse::YES);
 }
 
 } // End of Groovie namespace
