@@ -18,22 +18,26 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/groovie/script.cpp $
- * $Id: script.cpp 46524 2009-12-24 08:35:11Z scott_t $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/groovie/script.cpp $
+ * $Id: script.cpp 50288 2010-06-25 20:51:57Z athrxx $
  *
  */
 
-#include "groovie/debug.h"
-#include "groovie/music.h"
 #include "groovie/script.h"
-#include "groovie/groovie.h"
 #include "groovie/cell.h"
+#include "groovie/cursor.h"
+#include "groovie/graphics.h"
+#include "groovie/groovie.h"
+#include "groovie/music.h"
+#include "groovie/player.h"
+#include "groovie/resource.h"
 #include "groovie/saveload.h"
 
+#include "common/archive.h"
 #include "common/config-manager.h"
-#include "common/endian.h"
-#include "common/events.h"
+#include "common/debug-channels.h"
 #include "common/EventRecorder.h"
+#include "common/macresman.h"
 
 #define NUM_OPCODES 90
 
@@ -45,8 +49,8 @@ static void debugScript(int level, bool nl, const char *s, ...) {
 	char buf[STRINGBUFLEN];
 	va_list va;
 
-	if (!Common::isDebugChannelEnabled(kGroovieDebugScript) &&
-	    !Common::isDebugChannelEnabled(kGroovieDebugAll))
+	if (!DebugMan.isDebugChannelEnabled(kGroovieDebugScript) &&
+	    !DebugMan.isDebugChannelEnabled(kGroovieDebugAll))
 		return;
 
 	va_start(va, s);
@@ -60,9 +64,8 @@ static void debugScript(int level, bool nl, const char *s, ...) {
 }
 
 Script::Script(GroovieEngine *vm, EngineVersion version) :
-	_code(NULL), _savedCode(NULL), _stacktop(0),
-	_debugger(NULL), _vm(vm),
-	_videoFile(NULL), _videoRef(0), _font(NULL), _staufsMove(NULL) {
+	_code(NULL), _savedCode(NULL), _stacktop(0), _debugger(NULL), _vm(vm),
+	_videoFile(NULL), _videoRef(0), _staufsMove(NULL) {
 	// Initialize the opcode set depending on the engine version
 	switch (version) {
 	case kGroovieT7G:
@@ -83,11 +86,11 @@ Script::Script(GroovieEngine *vm, EngineVersion version) :
 	}
 
 	// Initialize the music type variable
-	int midiDriver = MidiDriver::detectMusicDriver(MDT_MIDI | MDT_ADLIB | MDT_PREFER_MIDI);
-	if (midiDriver == MD_ADLIB) {
+	MidiDriver::DeviceHandle dev = MidiDriver::detectDevice(MDT_MIDI | MDT_ADLIB | MDT_PREFER_GM);
+	if (MidiDriver::getMusicType(dev) == MT_ADLIB) {
 		// MIDI through AdLib
 		setVariable(0x100, 0);
-	} else if ((midiDriver == MD_MT32) || ConfMan.getBool("native_mt32")) {
+	} else if ((MidiDriver::getMusicType(dev) == MT_MT32) || ConfMan.getBool("native_mt32")) {
 		// MT-32
 		setVariable(0x100, 2);
 	} else {
@@ -109,7 +112,6 @@ Script::~Script() {
 	delete[] _code;
 	delete[] _savedCode;
 
-	delete _font;
 	delete _videoFile;
 }
 
@@ -127,22 +129,29 @@ void Script::timerTick() {
 }
 
 bool Script::loadScript(Common::String filename) {
-	// Try to open the script file
-	Common::File scriptfile;
-	if (!scriptfile.open(filename)) {
-		return false;
+	Common::SeekableReadStream *scriptfile = 0;
+
+	if (_vm->_macResFork) {
+		// Try to open the script file from the resource fork
+		scriptfile = _vm->_macResFork->getResource(filename);
+	} else {
+		// Try to open the script file
+		scriptfile = SearchMan.createReadStreamForMember(filename);
 	}
 
+	if (!scriptfile)
+		return false;
+
 	// Save the script filename
 	_scriptFile = filename;
 
 	// Load the code
-	_codeSize = scriptfile.size();
+	_codeSize = scriptfile->size();
 	_code = new byte[_codeSize];
 	if (!_code)
 		return false;
-	scriptfile.read(_code, _codeSize);
-	scriptfile.close();
+	scriptfile->read(_code, _codeSize);
+	delete scriptfile;
 
 	// Patch the loaded code for known script bugs
 	if (filename.equals("dr.grv")) {
@@ -357,8 +366,8 @@ bool Script::hotspot(Common::Rect rect, uint16 address, uint8 cursor) {
 	bool contained = rect.contains(mousepos);
 
 	// Show hotspots when debugging
-	if (Common::isDebugChannelEnabled(kGroovieDebugHotspots) ||
-	    Common::isDebugChannelEnabled(kGroovieDebugAll)) {
+	if (DebugMan.isDebugChannelEnabled(kGroovieDebugHotspots) ||
+	    DebugMan.isDebugChannelEnabled(kGroovieDebugAll)) {
 		rect.translate(0, -80);
 		_vm->_graphicsMan->_foreground.frameRect(rect, 250);
 		_vm->_system->copyRectToScreen((byte*)_vm->_graphicsMan->_foreground.getBasePtr(0, 0), _vm->_graphicsMan->_foreground.pitch, 0, 80, 640, 320);
@@ -419,6 +428,22 @@ void Script::savegame(uint slot) {
 	_saveNames[slot] = save;
 }
 
+void Script::printString(Graphics::Surface *surface, const char *str) {
+	char message[15];
+	memset(message, 0, 15);
+
+	// Preprocess the string
+	for (int i = 0; i < 14; i++) {
+		if (str[i] <= 0x00 || str[i] == 0x24)
+			break;
+		message[i] = str[i];
+	}
+	Common::rtrim(message);
+
+	// Draw the string
+	_vm->_font->drawString(surface, message, 0, 16, 640, 0xE2, Graphics::kTextAlignCenter);
+}
+
 // OPCODES
 
 void Script::o_invalid() {
@@ -1239,11 +1264,16 @@ void Script::o_printstring() {
 
 	stringstorage[counter] = 0;
 
-	// Load the font if required
-	if (!_font) {
-		_font = new Font(_vm->_system);
-	}
-	_font->printstring(stringstorage);
+	Common::Rect topbar(640, 80);
+	Graphics::Surface *gamescreen = _vm->_system->lockScreen();
+
+	// Clear the top bar
+	gamescreen->fillRect(topbar, 0);
+
+	// Draw the string
+	printString(gamescreen, stringstorage);
+
+	_vm->_system->unlockScreen();
 }
 
 void Script::o_hotspot_slot() {
@@ -1263,11 +1293,15 @@ void Script::o_hotspot_slot() {
 			return;
 		}
 
-		// Load the font if required
-		if (!_font) {
-			_font = new Font(_vm->_system);
-		}
-		_font->printstring(_saveNames[slot].c_str());
+		Common::Rect topbar(640, 80);
+		Graphics::Surface *gamescreen = _vm->_system->lockScreen();
+
+		// Clear the top bar
+		gamescreen->fillRect(topbar, 0);
+
+		printString(gamescreen, _saveNames[slot].c_str());
+
+		_vm->_system->unlockScreen();
 
 		// Save the currently highlighted slot
 		_hotspotSlot = slot;
