@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/gui_lok.cpp $
- * $Id: gui_lok.cpp 47685 2010-01-29 22:43:23Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/gui_lok.cpp $
+ * $Id: gui_lok.cpp 50952 2010-07-16 23:30:29Z lordhoto $
  *
  */
 
@@ -579,11 +579,10 @@ void GUI_LoK::setupSavegames(Menu &menu, int num) {
 	for (int i = startSlot; i < num; ++i)
 		menu.item[i].enabled = 0;
 
-	KyraEngine_v1::SaveHeader header;
+	KyraEngine_LoK::SaveHeader header;
 	for (int i = startSlot; i < num && uint(_savegameOffset + i) < _saveSlots.size(); i++) {
 		if ((in = _vm->openSaveForReading(_vm->getSavegameFilename(_saveSlots[i + _savegameOffset]), header))) {
-			strncpy(savenames[i], header.description.c_str(), ARRAYSIZE(savenames[0]));
-			savenames[i][34] = 0;
+			Common::strlcpy(savenames[i], header.description.c_str(), ARRAYSIZE(savenames[0]));
 
 			Util::convertISOToDOS(savenames[i]);
 
@@ -748,8 +747,7 @@ int GUI_LoK::saveGame(Button *button) {
 	} else {
 		for (int i = 0; i < 5; i++) {
 			if (_menu[2].item[i].saveSlot == _vm->_gameToLoad) {
-				strncpy(_savegameName, _menu[2].item[i].itemString, 31);
-				_savegameName[30] = 0;
+				Common::strlcpy(_savegameName, _menu[2].item[i].itemString, 31);
 				break;
 			}
 		}
