@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/gui_lol.cpp $
- * $Id: gui_lol.cpp 48078 2010-02-17 16:08:47Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/gui_lol.cpp $
+ * $Id: gui_lol.cpp 50952 2010-07-16 23:30:29Z lordhoto $
  *
  */
 
@@ -2312,39 +2312,32 @@ int GUI_LoL::runMenu(Menu &menu) {
 
 		if (_currentMenu == &_gameOptions) {
 			char *s = (char *)_vm->_tempBuffer5120;
-			strncpy(s, _vm->getLangString(0x406f + _vm->_monsterDifficulty), 30);
-			s[29] = 0;
+			Common::strlcpy(s, _vm->getLangString(0x406f + _vm->_monsterDifficulty), 30);
 			_currentMenu->item[_vm->gameFlags().isTalkie ? 0 : 2].itemString = s;
 			s += (strlen(s) + 1);
 
-			strncpy(s, _vm->getLangString(_vm->_smoothScrollingEnabled ? 0x4068 : 0x4069), 30);
-			s[29] = 0;
+			Common::strlcpy(s, _vm->getLangString(_vm->_smoothScrollingEnabled ? 0x4068 : 0x4069), 30);
 			_currentMenu->item[_vm->gameFlags().isTalkie ? 1 : 3].itemString = s;
 			s += (strlen(s) + 1);
 
-			strncpy(s, _vm->getLangString(_vm->_floatingCursorsEnabled ? 0x4068 : 0x4069), 30);
-			s[29] = 0;
+			Common::strlcpy(s, _vm->getLangString(_vm->_floatingCursorsEnabled ? 0x4068 : 0x4069), 30);
 			_currentMenu->item[_vm->gameFlags().isTalkie ? 2 : 4].itemString = s;
 			s += (strlen(s) + 1);
 
 			if (_vm->gameFlags().isTalkie) {
-				strncpy(s, _vm->getLangString(0x42d6 + _vm->_lang), 30);
-				s[29] = 0;
+				Common::strlcpy(s, _vm->getLangString(0x42d6 + _vm->_lang), 30);
 				_currentMenu->item[3].itemString = s;
 				s += (strlen(s) + 1);
 
-				strncpy(s, _vm->getLangString(_vm->textEnabled() ? 0x4068 : 0x4069), 30);
-				s[29] = 0;
+				Common::strlcpy(s, _vm->getLangString(_vm->textEnabled() ? 0x4068 : 0x4069), 30);
 				_currentMenu->item[4].itemString = s;
 				s += (strlen(s) + 1);
 			} else {
-				strncpy(s, _vm->getLangString(_vm->_configMusic ? 0x4068 : 0x4069), 30);
-				s[29] = 0;
+				Common::strlcpy(s, _vm->getLangString(_vm->_configMusic ? 0x4068 : 0x4069), 30);
 				_currentMenu->item[0].itemString = s;
 				s += (strlen(s) + 1);
 
-				strncpy(s, _vm->getLangString(_vm->_configSounds ? 0x4068 : 0x4069), 30);
-				s[29] = 0;
+				Common::strlcpy(s, _vm->getLangString(_vm->_configSounds ? 0x4068 : 0x4069), 30);
 				_currentMenu->item[1].itemString = s;
 				s += (strlen(s) + 1);
 			}
@@ -2582,8 +2575,7 @@ void GUI_LoL::setupSaveMenuSlots(Menu &menu, int num) {
 
 	for (int i = startSlot; i < num && _savegameOffset + i - slotOffs < _savegameListSize; ++i) {
 		if (_savegameList[_saveSlots[i + _savegameOffset - slotOffs]]) {
-			strncpy(s, _savegameList[_saveSlots[i + _savegameOffset - slotOffs]], 80);
-			s[79] = 0;
+			Common::strlcpy(s, _savegameList[_saveSlots[i + _savegameOffset - slotOffs]], 80);
 			menu.item[i].itemString = s;
 			s += (strlen(s) + 1);
 			menu.item[i].saveSlot = _saveSlots[i + _savegameOffset - slotOffs];
@@ -2619,16 +2611,16 @@ void GUI_LoL::updateSavegameList() {
 	if (_savegameListSize) {
 		Common::sort(_saveSlots.begin(), _saveSlots.end(), Common::Greater<int>());
 
-		KyraEngine_v1::SaveHeader header;
+		LoLEngine::SaveHeader header;
 		Common::InSaveFile *in;
 
-		_savegameList = new char*[_savegameListSize];
+		_savegameList = new char *[_savegameListSize];
 
 		for (int i = 0; i < _savegameListSize; i++) {
 			in = _vm->openSaveForReading(_vm->getSavegameFilename(i), header);
 			if (in) {
 				_savegameList[i] = new char[header.description.size() + 1];
-				strncpy(_savegameList[i], header.description.c_str(), header.description.size() + 1);
+				Common::strlcpy(_savegameList[i], header.description.c_str(), header.description.size() + 1);
 				Util::convertISOToDOS(_savegameList[i]);
 				delete in;
 			} else {
@@ -2666,7 +2658,7 @@ int GUI_LoL::getInput() {
 	if (_currentMenu == &_savenameMenu) {
 		_vm->updateInput();
 
-		for (Common::List<KyraEngine_v1::Event>::const_iterator evt = _vm->_eventList.begin(); evt != _vm->_eventList.end(); ++evt) {
+		for (Common::List<LoLEngine::Event>::const_iterator evt = _vm->_eventList.begin(); evt != _vm->_eventList.end(); ++evt) {
 			if (evt->event.type == Common::EVENT_KEYDOWN)
 				_keyPressed = evt->event.kbd;
 		}
@@ -2759,7 +2751,7 @@ int GUI_LoL::clickedSaveMenu(Button *button) {
 	_newMenu = &_savenameMenu;
 	int16 s = (int16)button->arg;
 	_menuResult = _saveMenu.item[-s - 2].saveSlot + 1;
-	_saveDescription = (char*)_vm->_tempBuffer5120 + 1000;
+	_saveDescription = (char *)_vm->_tempBuffer5120 + 1000;
 	_saveDescription[0] = 0;
 	if (_saveMenu.item[-s - 2].saveSlot != -3)
 		strcpy(_saveDescription, _saveMenu.item[-s - 2].itemString);
