@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/gui_v2.cpp $
- * $Id: gui_v2.cpp 47541 2010-01-25 01:39:44Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/gui_v2.cpp $
+ * $Id: gui_v2.cpp 50952 2010-07-16 23:30:29Z lordhoto $
  *
  */
 
@@ -452,13 +452,12 @@ void GUI_v2::setupSavegameNames(Menu &menu, int num) {
 	if (_isSaveMenu && _savegameOffset == 0)
 		startSlot = 1;
 
-	KyraEngine_v1::SaveHeader header;
+	KyraEngine_v2::SaveHeader header;
 	Common::InSaveFile *in;
 	for (int i = startSlot; i < num && uint(_savegameOffset + i) < _saveSlots.size(); ++i) {
 		if ((in = _vm->openSaveForReading(_vm->getSavegameFilename(_saveSlots[i + _savegameOffset]), header)) != 0) {
 			char *s = getTableString(menu.item[i].itemId);
-			strncpy(s, header.description.c_str(), 80);
-			s[79] = 0;
+			Common::strlcpy(s, header.description.c_str(), 80);
 			Util::convertISOToDOS(s);
 
 			menu.item[i].saveSlot = _saveSlots[i + _savegameOffset];
