@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/kyra_lok.cpp $
- * $Id: kyra_lok.cpp 47541 2010-01-25 01:39:44Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/kyra_lok.cpp $
+ * $Id: kyra_lok.cpp 51179 2010-07-22 23:16:33Z lordhoto $
  *
  */
 
@@ -29,6 +29,7 @@
 #include "common/system.h"
 #include "common/savefile.h"
 #include "common/config-manager.h"
+#include "common/debug-channels.h"
 
 #include "gui/message.h"
 
@@ -111,7 +112,7 @@ KyraEngine_LoK::~KyraEngine_LoK() {
 		_emc->unload(&_scriptClickData);
 	}
 
-	Common::clearAllDebugChannels();
+	DebugMan.clearAllDebugChannels();
 
 	delete _screen;
 	delete _sprites;
@@ -233,6 +234,7 @@ Common::Error KyraEngine_LoK::init() {
 
 	_talkingCharNum = -1;
 	_charSayUnk3 = -1;
+	_disabledTalkAnimObject = _enabledTalkAnimObject = 0;
 	memset(_currSentenceColor, 0, 3);
 	_startSentencePalIndex = -1;
 	_fadeText = false;
