@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/kyra_lok.h $
- * $Id: kyra_lok.h 45906 2009-11-14 22:51:35Z athrxx $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/kyra_lok.h $
+ * $Id: kyra_lok.h 51757 2010-08-05 14:41:00Z lordhoto $
  *
  */
 
@@ -132,16 +132,16 @@ public:
 	int _paletteChanged;
 	int16 _northExitHeight;
 
-	typedef void (KyraEngine_LoK::*IntroProc)();
+	typedef bool (KyraEngine_LoK::*IntroProc)();
 
 	// static data access
-	const char * const*seqWSATable() { return _seq_WSATable; }
-	const char * const*seqCPSTable() { return _seq_CPSTable; }
-	const char * const*seqCOLTable() { return _seq_COLTable; }
-	const char * const*seqTextsTable() { return _seq_textsTable; }
+	const char * const *seqWSATable() { return _seq_WSATable; }
+	const char * const *seqCPSTable() { return _seq_CPSTable; }
+	const char * const *seqCOLTable() { return _seq_COLTable; }
+	const char * const *seqTextsTable() { return _seq_textsTable; }
 
-	const uint8 * const*palTable1() { return &_specialPalettes[0]; }
-	const uint8 * const*palTable2() { return &_specialPalettes[29]; }
+	const uint8 * const *palTable1() { return &_specialPalettes[0]; }
+	const uint8 * const *palTable2() { return &_specialPalettes[29]; }
 
 protected:
 	virtual Common::Error go();
@@ -157,11 +157,12 @@ protected:
 
 	// -> intro
 	void seq_intro();
-	void seq_introLogos();
-	void seq_introStory();
-	void seq_introMalcolmTree();
-	void seq_introKallakWriting();
-	void seq_introKallakMalcolm();
+	bool seq_introPublisherLogos();
+	bool seq_introLogos();
+	bool seq_introStory();
+	bool seq_introMalcolmTree();
+	bool seq_introKallakWriting();
+	bool seq_introKallakMalcolm();
 
 	// -> ingame animations
 	void seq_createAmuletJewel(int jewel, int page, int noSound, int drawOnly);
@@ -318,7 +319,7 @@ protected:
 	// chat
 	// -> process
 	void characterSays(int vocFile, const char *chatStr, int8 charNum, int8 chatDuration);
-	void waitForChatToFinish(int vocFile, int16 chatDuration, const char *str, uint8 charNum);
+	void waitForChatToFinish(int vocFile, int16 chatDuration, const char *str, uint8 charNum, const bool printText);
 
 	// -> initialization
 	int initCharacterChat(int8 charNum);
@@ -373,20 +374,24 @@ protected:
 	//void setTimer19();
 	void setupTimers();
 	void timerUpdateHeadAnims(int timerNum);
-	void timerSetFlags1(int timerNum);
-	void timerSetFlags2(int timerNum);
-	void timerSetFlags3(int timerNum);
-	void timerCheckAnimFlag1(int timerNum);
-	void timerCheckAnimFlag2(int timerNum);
+	void timerTulipCreator(int timerNum);
+	void timerRubyCreator(int timerNum);
+	void timerAsInvisibleTimeout(int timerNum);
+	void timerAsWillowispTimeout(int timerNum);
 	void checkAmuletAnimFlags();
 	void timerRedrawAmulet(int timerNum);
+	void timerLavenderRoseCreator(int timerNum);
+	void timerAcornCreator(int timerNum);
+	void timerBlueberryCreator(int timerNum);
 	void timerFadeText(int timerNum);
-	void updateAnimFlag1(int timerNum);
-	void updateAnimFlag2(int timerNum);
+	void timerWillowispFrameTimer(int timerNum);
+	void timerInvisibleFrameTimer(int timerNum);
 	void drawAmulet();
 	void setTextFadeTimerCountdown(int16 countdown);
 	void setWalkspeed(uint8 newSpeed);
 
+	void setItemCreationFlags(int offset, int count);
+
 	int buttonInventoryCallback(Button *caller);
 	int buttonAmuletCallback(Button *caller);
 
@@ -472,6 +477,8 @@ protected:
 	int8 _charSayUnk2;
 	int8 _charSayUnk3;
 	int8 _currHeadShape;
+	int8 _disabledTalkAnimObject;
+	int8 _enabledTalkAnimObject;
 	uint8 _currSentenceColor[3];
 	int8 _startSentencePalIndex;
 	bool _fadeText;
