@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/kyra_mr.cpp $
- * $Id: kyra_mr.cpp 47479 2010-01-23 16:15:18Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/kyra_mr.cpp $
+ * $Id: kyra_mr.cpp 52201 2010-08-19 02:39:07Z lordhoto $
  *
  */
 
@@ -185,7 +185,7 @@ KyraEngine_MR::~KyraEngine_MR() {
 	delete[] _sceneStrings;
 	delete[] _talkObjectList;
 
-	for (Common::Array<const Opcode*>::iterator i = _opcodesDialog.begin(); i != _opcodesDialog.end(); ++i)
+	for (Common::Array<const Opcode *>::iterator i = _opcodesDialog.begin(); i != _opcodesDialog.end(); ++i)
 		delete *i;
 	_opcodesDialog.clear();
 
@@ -369,9 +369,7 @@ void KyraEngine_MR::playVQA(const char *name) {
 	VQAMovie vqa(this, _system);
 
 	char filename[20];
-	int size = 0;		// TODO: Movie size is 0, 1 or 2.
-
-	snprintf(filename, sizeof(filename), "%s%d.VQA", name, size);
+	snprintf(filename, sizeof(filename), "%s%d.VQA", name, _configVQAQuality);
 
 	if (vqa.open(filename)) {
 		for (int i = 0; i < 4; ++i) {
@@ -656,8 +654,6 @@ void KyraEngine_MR::startup() {
 	_invWsa->open("MOODOMTR.WSA", 1, 0);
 	_invWsaFrame = 6;
 	saveGameState(0, "New Game", 0);
-	_soundDigital->beginFadeOut(_musicSoundChannel, 60);
-	delayWithTicks(60);
 	if (_gameToLoad == -1)
 		enterNewScene(_mainCharacter.sceneId, _mainCharacter.facing, 0, 0, 1);
 	else
@@ -1129,7 +1125,7 @@ void KyraEngine_MR::updateWithText() {
 
 	restorePage3();
 	drawAnimObjects();
-	if (textEnabled() && _chatText) {
+	if (_chatTextEnabled && _chatText) {
 		int curPage = _screen->_curPage;
 		_screen->_curPage = 2;
 		objectChatPrintText(_chatText, _chatObject);
@@ -1297,18 +1293,12 @@ void KyraEngine_MR::restoreGfxRect32x32(int x, int y) {
 char *KyraEngine_MR::appendLanguage(char *buf, int lang, int bufSize) {
 	assert(lang < _languageExtensionSize);
 
-	int size = strlen(buf) + strlen(_languageExtension[lang]);
-
-	if (size > bufSize) {
+	const int size = Common::strlcat(buf, _languageExtension[lang], bufSize);
+	if (size >= bufSize) {
 		warning("buffer too small to append language extension");
 		return 0;
 	}
 
-	char *temp = buf + strlen(buf);
-	bufSize -= strlen(buf);
-
-	strncat(temp, _languageExtension[lang], bufSize);
-
 	return buf;
 }
 
@@ -1318,8 +1308,7 @@ int KyraEngine_MR::loadLanguageFile(const char *file, uint8 *&buffer) {
 
 	uint32 size = 0;
 	char nBuf[32];
-	strncpy(nBuf, file, 32);
-	nBuf[31] = 0;
+	Common::strlcpy(nBuf, file, sizeof(nBuf));
 	buffer = _res->fileData(appendLanguage(nBuf, _lang, sizeof(nBuf)), &size);
 
 	return buffer ? size : 0 ;
@@ -1327,7 +1316,7 @@ int KyraEngine_MR::loadLanguageFile(const char *file, uint8 *&buffer) {
 
 uint8 *KyraEngine_MR::getTableEntry(uint8 *buffer, int id) {
 	uint16 tableEntries = READ_LE_UINT16(buffer);
-	const uint16 *indexTable = (const uint16*)(buffer + 2);
+	const uint16 *indexTable = (const uint16 *)(buffer + 2);
 	const uint16 *offsetTable = indexTable + tableEntries;
 
 	int num = 0;
@@ -1467,6 +1456,8 @@ void KyraEngine_MR::registerDefaultSettings() {
 	ConfMan.registerDefault("studio_audience", true);
 	ConfMan.registerDefault("skip_support", true);
 	ConfMan.registerDefault("helium_mode", false);
+	// 0 - best, 1 - mid, 2 - low
+	ConfMan.registerDefault("video_quality", 0);
 }
 
 void KyraEngine_MR::writeSettings() {
@@ -1497,11 +1488,12 @@ void KyraEngine_MR::writeSettings() {
 }
 
 void KyraEngine_MR::readSettings() {
-	KyraEngine_v1::readSettings();
+	KyraEngine_v2::readSettings();
 
 	_configStudio = ConfMan.getBool("studio_audience");
 	_configSkip = ConfMan.getBool("skip_support");
 	_configHelium = ConfMan.getBool("helium_mode");
+	_configVQAQuality = CLIP(ConfMan.getInt("video_quality"), 0, 2);
 }
 
 } // End of namespace Kyra
