@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/scene_mr.cpp $
- * $Id: scene_mr.cpp 44634 2009-10-04 21:26:33Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/scene_mr.cpp $
+ * $Id: scene_mr.cpp 50948 2010-07-16 23:04:59Z eriktorbjorn $
  *
  */
 
@@ -79,11 +79,9 @@ void KyraEngine_MR::enterNewScene(uint16 sceneId, int facing, int unk1, int unk2
 
 	musicUpdate(0);
 	uint32 waitUntilTimer = 0;
-	bool newSoundFile = false;
 	if (_lastMusicCommand != _sceneList[sceneId].sound) {
 		fadeOutMusic(60);
 		waitUntilTimer = _system->getMillis() + 60 * _tickLength;
-		newSoundFile = true;
 	}
 
 	_chatAltFlag = false;
