@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/screen.cpp $
- * $Id: screen.cpp 47541 2010-01-25 01:39:44Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/screen.cpp $
+ * $Id: screen.cpp 52340 2010-08-24 13:41:24Z lordhoto $
  *
  */
 
@@ -27,6 +27,8 @@
 #include "common/endian.h"
 #include "common/system.h"
 
+#include "engines/util.h"
+
 #include "graphics/cursorman.h"
 #include "graphics/sjis.h"
 
@@ -38,7 +40,7 @@ namespace Kyra {
 
 Screen::Screen(KyraEngine_v1 *vm, OSystem *system)
 	: _system(system), _vm(vm), _sjisInvisibleColor(0),
-	_cursorColorKey((vm->gameFlags().gameID == GI_KYRA1) ? 0xFF : 0x00) {
+	_cursorColorKey((vm->game() == GI_KYRA1) ? 0xFF : 0x00) {
 	_debugEnabled = false;
 	_maskMinY = _maskMaxY = -1;
 
@@ -69,8 +71,6 @@ Screen::~Screen() {
 
 	for (uint i = 0; i < _palettes.size(); ++i)
 		delete _palettes[i];
-
-	CursorMan.popAllCursors();
 }
 
 bool Screen::init() {
@@ -86,7 +86,7 @@ bool Screen::init() {
 	if (_vm->gameFlags().useHiResOverlay) {
 		_useOverlays = true;
 		_useSJIS = (_vm->gameFlags().lang == Common::JA_JPN);
-		_sjisInvisibleColor = (_vm->gameFlags().gameID == GI_KYRA1) ? 0x80 : 0xF6;
+		_sjisInvisibleColor = (_vm->game() == GI_KYRA1) ? 0x80 : 0xF6;
 
 		for (int i = 0; i < SCREEN_OVLS_NUM; ++i) {
 			if (!_sjisOverlayPtrs[i]) {
@@ -352,8 +352,8 @@ void Screen::scale2x(byte *dst, int dstPitch, const byte *src, int srcPitch, int
 		for (int x = 0; x < w; ++x, dstL1 += 2, dstL2 += 2) {
 			uint16 col = *src++;
 			col |= col << 8;
-			*(uint16*)(dstL1) = col;
-			*(uint16*)(dstL2) = col;
+			*(uint16 *)(dstL1) = col;
+			*(uint16 *)(dstL2) = col;
 		}
 		dstL1 += dstAdd; dstL2 += dstAdd;
 		src += srcAdd;
@@ -611,14 +611,6 @@ void Screen::fadePalette(const Palette &pal, int delay, const UpdateFunctor *upF
 		_vm->delay((delayAcc >> 8) * 1000 / 60);
 		delayAcc &= 0xFF;
 	}
-
-	if (_vm->shouldQuit()) {
-		setScreenPalette(pal);
-		if (upFunc && upFunc->isValid())
-			(*upFunc)();
-		else
-			_system->updateScreen();
-	}
 }
 
 void Screen::getFadeParams(const Palette &pal, int delay, int &delayInc, int &diff) {
@@ -1272,19 +1264,19 @@ void Screen::drawShape(uint8 pageNum, const uint8 *shapeData, int x, int y, int
 	_dsDrawLayer = 0;
 
 	if (flags & 0x8000) {
-		_dsTable2 = va_arg(args, uint8*);
+		_dsTable2 = va_arg(args, uint8 *);
 	}
 
 	if (flags & 0x100) {
-		_dsTable = va_arg(args, uint8*);
+		_dsTable = va_arg(args, uint8 *);
 		_dsTableLoopCount = va_arg(args, int);
 		if (!_dsTableLoopCount)
 			flags &= ~0x100;
 	}
 
 	if (flags & 0x1000) {
-		_dsTable3 = va_arg(args, uint8*);
-		_dsTable4 = va_arg(args, uint8*);
+		_dsTable3 = va_arg(args, uint8 *);
+		_dsTable4 = va_arg(args, uint8 *);
 	}
 
 	if (flags & 0x200) {
@@ -1308,8 +1300,8 @@ void Screen::drawShape(uint8 pageNum, const uint8 *shapeData, int x, int y, int
 		_dsScaleH = 0x100;
 	}
 
-	if ((flags & 0x2000) && _vm->gameFlags().gameID != GI_KYRA1)
-		_dsTable5 = va_arg(args,  uint8*);
+	if ((flags & 0x2000) && _vm->game() != GI_KYRA1)
+		_dsTable5 = va_arg(args, uint8 *);
 
 	static const DsMarginSkipFunc dsMarginFunc[] = {
 		&Screen::drawShapeMarginNoScaleUpwind,
@@ -1444,7 +1436,7 @@ void Screen::drawShape(uint8 pageNum, const uint8 *shapeData, int x, int y, int
 
 	uint16 frameSize = READ_LE_UINT16(src); src += 2;
 
-	int colorTableColors = ((_vm->gameFlags().gameID != GI_KYRA1) && (shapeFlags & 4)) ? *src++ : 16;
+	int colorTableColors = ((_vm->game() != GI_KYRA1) && (shapeFlags & 4)) ? *src++ : 16;
 
 	if (!(flags & 0x8000) && (shapeFlags & 1))
 		_dsTable2 = src;
@@ -3024,10 +3016,10 @@ byte *Screen::getOverlayPtr(int page) {
 	else if (page == 2 || page == 3)
 		return _sjisOverlayPtrs[2];
 
-	if (_vm->gameFlags().gameID == GI_KYRA2) {
+	if (_vm->game() == GI_KYRA2) {
 		if (page == 12 || page == 13)
 			return _sjisOverlayPtrs[3];
-	} else if (_vm->gameFlags().gameID == GI_LOL) {
+	} else if (_vm->game() == GI_LOL) {
 		if (page == 4 || page == 5)
 			return _sjisOverlayPtrs[3];
 		if (page == 6 || page == 7)
