@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/screen.h $
- * $Id: screen.h 46269 2009-12-06 00:54:24Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/screen.h $
+ * $Id: screen.h 48359 2010-03-22 20:28:08Z fingolfin $
  *
  */
 
@@ -532,7 +532,7 @@ protected:
 	int _mouseLockCount;
 	const uint8 _cursorColorKey;
 
-	virtual void postProcessCursor(uint8 *data, int w, int h, int pitch) {};
+	virtual void postProcessCursor(uint8 *data, int w, int h, int pitch) {}
 
 	enum {
 		kMaxDirtyRects = 50
