@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/script_tim.cpp $
- * $Id: script_tim.cpp 46601 2009-12-26 19:02:16Z athrxx $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/script_tim.cpp $
+ * $Id: script_tim.cpp 52340 2010-08-24 13:41:24Z lordhoto $
  *
  */
 
@@ -95,7 +95,7 @@ TIMInterpreter::TIMInterpreter(KyraEngine_v1 *engine, Screen_v2 *screen_v2, OSys
 	_textDisplayed = false;
 	_textAreaBuffer = new uint8[320*40];
 	assert(_textAreaBuffer);
-	if ((_vm->gameFlags().platform == Common::kPlatformPC98 || _vm->gameFlags().isDemo) && _vm->gameFlags().gameID == GI_LOL)
+	if ((_vm->gameFlags().platform == Common::kPlatformPC98 || _vm->gameFlags().isDemo) && _vm->game() == GI_LOL)
 		_drawPage2 = 0;
 	else
 		_drawPage2 = 8;
@@ -134,7 +134,7 @@ bool TIMInterpreter::callback(Common::IFFChunk &chunk) {
 		break;
 
 	default:
-		warning("Unexpected chunk '%s' of size %d found in file '%s'", Common::tag2string(chunk._type).c_str(), chunk._size, _filename);
+		warning("Unexpected chunk '%s' of size %d found in file '%s'", tag2str(chunk._type), chunk._size, _filename);
 	}
 
 	return false;
@@ -159,7 +159,7 @@ TIM *TIMInterpreter::load(const char *filename, const Common::Array<const TIMOpc
 	_tim->opcodes = opcodes;
 
 	IFFParser iff(*stream);
-	Common::Functor1Mem< Common::IFFChunk &, bool, TIMInterpreter > c(this, &TIMInterpreter::callback);
+	Common::Functor1Mem<Common::IFFChunk &, bool, TIMInterpreter> c(this, &TIMInterpreter::callback);
 	iff.parse(c);
 
 	if (!_tim->avtl)
@@ -170,14 +170,13 @@ TIM *TIMInterpreter::load(const char *filename, const Common::Array<const TIMOpc
 
 	delete stream;
 
-	int num = (_avtlChunkSize < TIM::kCountFuncs) ? _avtlChunkSize : (int)TIM::kCountFuncs;
+	const int num = (_avtlChunkSize < TIM::kCountFuncs) ? _avtlChunkSize : (int)TIM::kCountFuncs;
 	for (int i = 0; i < num; ++i)
 		_tim->func[i].avtl = _tim->avtl + _tim->avtl[i];
 
-	strncpy(_tim->filename, filename, 13);
-	_tim->filename[12] = 0;
+	Common::strlcpy(_tim->filename, filename, 13);
 
-	_tim->isLoLOutro = (_vm->gameFlags().gameID == GI_LOL) && !scumm_stricmp(filename, "LOLFINAL.TIM");
+	_tim->isLoLOutro = (_vm->game() == GI_LOL) && !scumm_stricmp(filename, "LOLFINAL.TIM");
 	_tim->lolCharacter = 0;
 
 	TIM *r = _tim;
@@ -468,7 +467,7 @@ void TIMInterpreter::setupTextPalette(uint index, int fadePalette) {
 int TIMInterpreter::initAnimStruct(int index, const char *filename, int x, int y, int, int offscreenBuffer, uint16 wsaFlags) {
 	Movie *wsa = 0;
 
-	const bool isLoLDemo = _vm->gameFlags().isDemo && _vm->gameFlags().gameID == GI_LOL;
+	const bool isLoLDemo = _vm->gameFlags().isDemo && _vm->game() == GI_LOL;
 
 	if (isLoLDemo || _vm->gameFlags().platform == Common::kPlatformPC98 || _currentTim->isLoLOutro)
 		_drawPage2 = 0;
@@ -605,7 +604,7 @@ int TIMInterpreter::execCommand(int cmd, const uint16 *param) {
 		return 0;
 	}
 
-	debugC(5, kDebugLevelScript, "TIMInterpreter::%s(%p)", _commands[cmd].desc, (const void* )param);
+	debugC(5, kDebugLevelScript, "TIMInterpreter::%s(%p)", _commands[cmd].desc, (const void *)param);
 	return (this->*_commands[cmd].proc)(param);
 }
 
@@ -756,7 +755,7 @@ int TIMInterpreter::cmd_loadSoundFile(const uint16 *param) {
 	const char *file = (const char *)(_currentTim->text + READ_LE_UINT16(_currentTim->text + (param[0]<<1)));
 
 	_vm->sound()->loadSoundFile(file);
-	if (_vm->gameFlags().gameID == GI_LOL)
+	if (_vm->game() == GI_LOL)
 		_vm->sound()->loadSfxFile(file);
 
 	return 1;
@@ -1172,7 +1171,7 @@ int TIMInterpreter_LoL::execCommand(int cmd, const uint16 *param) {
 		return 0;
 	}
 
-	debugC(5, kDebugLevelScript, "TIMInterpreter::%s(%p)", _commands[cmd].desc, (const void* )param);
+	debugC(5, kDebugLevelScript, "TIMInterpreter::%s(%p)", _commands[cmd].desc, (const void *)param);
 	return (this->*_commands[cmd].proc)(param);
 }
 
