@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/seqplayer.cpp $
- * $Id: seqplayer.cpp 47541 2010-01-25 01:39:44Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/seqplayer.cpp $
+ * $Id: seqplayer.cpp 49069 2010-05-17 23:29:44Z lordhoto $
  *
  */
 
@@ -481,7 +481,7 @@ void SeqPlayer::s1_miscUnk3() {
 }
 
 void SeqPlayer::s1_prefetchVocFile() {
-	*_seqData++;
+	_seqData++;
 	// we do not have to prefetch the vocfiles on modern systems
 }
 
@@ -579,7 +579,7 @@ bool SeqPlayer::playSequence(const uint8 *seqData, bool skipSeq) {
 		SEQOP(2, s1_prefetchVocFile)
 	};
 
-	const SeqEntry* commands;
+	const SeqEntry *commands;
 	int numCommands;
 
 	if (_vm->gameFlags().isTalkie) {
