@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/sequences_hof.cpp $
- * $Id: sequences_hof.cpp 47685 2010-01-29 22:43:23Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/sequences_hof.cpp $
+ * $Id: sequences_hof.cpp 49069 2010-05-17 23:29:44Z lordhoto $
  *
  */
 
@@ -133,8 +133,6 @@ void KyraEngine_HoF::seq_playSequences(int startSeq, int endSeq) {
 		}
 
 		if (cseq.flags & 1) {
-			int w2 = _seqWsa->width();
-			int h2 = _seqWsa->height();
 			int x = cseq.xPos;
 			int y = cseq.yPos;
 
@@ -142,13 +140,13 @@ void KyraEngine_HoF::seq_playSequences(int startSeq, int endSeq) {
 
 			if (_seqWsa) {
 				if (x < 0) {
+					_seqWsa->setWidth(_seqWsa->width() + x);
 					x = 0;
-					w2 = 0;
 				}
 
 				if (y < 0) {
+					_seqWsa->setHeight(_seqWsa->height() + y);
 					y = 0;
-					h2 = 0;
 				}
 
 				if (cseq.xPos + _seqWsa->width() > 319)
@@ -2558,9 +2556,9 @@ void KyraEngine_HoF::seq_displayScrollText(uint8 *data, const ScreenDim *d, int
 
 			char *str = (char *)ptr;
 
-			ptr = (uint8*)strpbrk(str, mark);
+			ptr = (uint8 *)strpbrk(str, mark);
 			if (!ptr)
-				ptr = (uint8*)strchr(str, 0);
+				ptr = (uint8 *)strchr(str, 0);
 
 			textData[cnt + 1].unk1 = *ptr;
 			*ptr = 0;
@@ -2598,7 +2596,7 @@ void KyraEngine_HoF::seq_displayScrollText(uint8 *data, const ScreenDim *d, int
 				cH -= (textData[cnt].height + (textData[cnt].height >> 3));
 
 			textData[cnt + 1].y = cH;
-			textData[cnt + 1].text = (uint8*)str;
+			textData[cnt + 1].text = (uint8 *)str;
 			cnt++;
 		}
 
