@@ -34,8 +34,8 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/sound_adlib.cpp $
- * $Id: sound_adlib.cpp 48618 2010-04-11 15:08:09Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/sound_adlib.cpp $
+ * $Id: sound_adlib.cpp 52340 2010-08-24 13:41:24Z lordhoto $
  *
  */
 
@@ -451,7 +451,7 @@ AdLibDriver::AdLibDriver(Audio::Mixer *mixer, bool v2) {
 	// is used by SFX or music, and then adjust the volume accordingly. Since Kyrandia 2 supports
 	// different volumes for SFX and music, looking at the disasm and checking how the original does it
 	// would be a good idea.
-	_mixer->playInputStream(Audio::Mixer::kMusicSoundType, &_soundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, DisposeAfterUse::NO, true);
+	_mixer->playStream(Audio::Mixer::kMusicSoundType, &_soundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, DisposeAfterUse::NO, true);
 
 	_samplesPerCallback = getRate() / CALLBACKS_PER_SECOND;
 	_samplesPerCallbackRemainder = getRate() % CALLBACKS_PER_SECOND;
@@ -509,7 +509,7 @@ int AdLibDriver::snd_setSoundData(va_list &list) {
 		delete[] _soundData;
 		_soundData = 0;
 	}
-	_soundData = va_arg(list, uint8*);
+	_soundData = va_arg(list, uint8 *);
 	return 0;
 }
 
@@ -2241,7 +2241,7 @@ const int SoundAdLibPC::_kyra1NumSoundTriggers = ARRAYSIZE(SoundAdLibPC::_kyra1S
 SoundAdLibPC::SoundAdLibPC(KyraEngine_v1 *vm, Audio::Mixer *mixer)
 	: Sound(vm, mixer), _driver(0), _trackEntries(), _soundDataPtr(0) {
 	memset(_trackEntries, 0, sizeof(_trackEntries));
-	_v2 = (_vm->gameFlags().gameID == GI_KYRA2) || (_vm->gameFlags().gameID == GI_LOL && !_vm->gameFlags().isDemo);
+	_v2 = (_vm->game() == GI_KYRA2) || (_vm->game() == GI_LOL && !_vm->gameFlags().isDemo);
 	_driver = new AdLibDriver(mixer, _v2);
 	assert(_driver);
 
