@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/text_lol.cpp $
- * $Id: text_lol.cpp 47541 2010-01-25 01:39:44Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/text_lol.cpp $
+ * $Id: text_lol.cpp 52578 2010-09-05 22:56:03Z lordhoto $
  *
  */
 
@@ -204,10 +204,12 @@ void TextDisplayer_LoL::printDialogueText(int dim, char *str, EMCState *script,
 }
 
 void TextDisplayer_LoL::printMessage(uint16 type, const char *str, ...) {
-	static uint8 textColors256[] = { 0xfe, 0xa2, 0x84, 0x97, 0x9F };
-	static uint8 textColors16[] = { 0x33, 0xaa, 0x88, 0x55, 0x99 };
-	static uint8 soundEffect[] = { 0x0B, 0x00, 0x2B, 0x1B, 0x00 };
+	static const uint8 textColors256[] = { 0xfe, 0xa2, 0x84, 0x97, 0x9F };
+	static const uint8 textColors16[] = { 0x33, 0xaa, 0x88, 0x55, 0x99 };
+	static const uint8 soundEffect[] = { 0x0B, 0x00, 0x2B, 0x1B, 0x00 };
+
 	const uint8 *textColors = _vm->gameFlags().use16ColorMode ? textColors16 : textColors256;
+
 	if (type & 4)
 		type ^= 4;
 	else
