@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/text_mr.cpp $
- * $Id: text_mr.cpp 47479 2010-01-23 16:15:18Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/text_mr.cpp $
+ * $Id: text_mr.cpp 51757 2010-08-05 14:41:00Z lordhoto $
  *
  */
 
@@ -265,7 +265,8 @@ void KyraEngine_MR::objectChatInit(const char *str, int object, int vocHigh, int
 
 	_screen->hideMouse();
 
-	if (textEnabled()) {
+	_chatTextEnabled = textEnabled();
+	if (_chatTextEnabled) {
 		objectChatPrintText(str, object);
 		_chatEndTime = _system->getMillis() + chatCalcDuration(str) * _tickLength;
 	} else {
