@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/kyra/vqa.cpp $
- * $Id: vqa.cpp 47579 2010-01-26 22:48:45Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/kyra/vqa.cpp $
+ * $Id: vqa.cpp 51187 2010-07-23 00:39:23Z lordhoto $
  *
  */
 
@@ -260,8 +260,8 @@ bool VQAMovie::open(const char *filename) {
 					_header.bits = 8;
 			}
 
-			setX((Screen::SCREEN_W - _header.width) / 2);
-			setY((Screen::SCREEN_H - _header.height) / 2);
+			_x = (Screen::SCREEN_W - _header.width) / 2;
+			_y = (Screen::SCREEN_H - _header.height) / 2;
 
 			_frameInfo = new uint32[_header.numFrames];
 			_frame = new byte[_header.width * _header.height];
@@ -649,7 +649,7 @@ void VQAMovie::play() {
 		}
 	}
 
-	_vm->_mixer->playInputStream(Audio::Mixer::kSFXSoundType, &_sound, _stream);
+	_vm->_mixer->playStream(Audio::Mixer::kSFXSoundType, &_sound, _stream);
 	Common::EventManager *eventMan = _vm->getEventManager();
 
 	for (uint i = 0; i < _header.numFrames; i++) {
