@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/lure/debugger.cpp $
- * $Id: debugger.cpp 36095 2009-01-27 10:27:07Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/lure/debugger.cpp $
+ * $Id: debugger.cpp 50963 2010-07-17 18:38:42Z fingolfin $
  *
  */
 
@@ -105,7 +105,7 @@ bool Debugger::cmd_enterRoom(int argc, const char **argv) {
 		if (!remoteFlag)
 			res.getActiveHotspot(PLAYER_ID)->setRoomNumber(roomNumber);
 
-		_detach_now = true;
+		detach();
 		return false;
 	}
 
