@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/lure/game.cpp $
- * $Id: game.cpp 48662 2010-04-16 10:04:45Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/lure/game.cpp $
+ * $Id: game.cpp 51101 2010-07-21 20:12:35Z lordhoto $
  *
  */
 
@@ -56,8 +56,6 @@ Game::Game() {
 	_debugFlag = gDebugLevel >= ERROR_BASIC;
 
 	_soundFlag = true;
-	_musicVolume = ConfMan.getBool("music_mute") ? 0 : MIN(255, ConfMan.getInt("music_volume"));
-	_sfxVolume = ConfMan.getBool("sfx_mute") ? 0 : MIN(255, ConfMan.getInt("sfx_volume"));
 }
 
 Game::~Game() {
@@ -281,8 +279,7 @@ void Game::execute() {
 			system.updateScreen();
 			system.delayMillis(10);
 
-			if (_debugger->isAttached())
-				_debugger->onFrame();
+			_debugger->onFrame();
 		}
 
 		room.leaveRoom();
@@ -421,6 +418,7 @@ void Game::displayChuteAnimation() {
 void Game::displayBarrelAnimation() {
 	Mouse &mouse = Mouse::getReference();
 	Resources &res = Resources::getReference();
+
 	debugC(ERROR_INTERMEDIATE, kLureDebugAnimations, "Starting barrel animation");
 	Palette palette(BARREL_PALETTE_ID);
 	AnimationSequence *anim = new AnimationSequence(BARREL_ANIM_ID, palette, false);
