@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/lure/menu.cpp $
- * $Id: menu.cpp 48413 2010-03-27 22:02:05Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/lure/menu.cpp $
+ * $Id: menu.cpp 54748 2010-12-03 09:33:51Z anotherguest $
  *
  */
 
@@ -584,12 +584,11 @@ uint16 PopupMenu::Show(int numEntries, const char *actions[]) {
 			} else if (e.type() == Common::EVENT_LBUTTONDOWN || e.type() == Common::EVENT_MOUSEMOVE) {
 				int16 x = mouse.x();
 				int16 y = mouse.y() - yMiddle + (s->height() / 2);
+				refreshFlag = true;
 				if (r.contains(x, y)) {
 					selectedIndex = (y - r.top) / FONT_HEIGHT;
 					if (e.type() == Common::EVENT_LBUTTONDOWN)
 						goto bail_out;
-					else
-						refreshFlag = true;
 				}
 #else
 			} else if ((e.type() == Common::EVENT_LBUTTONDOWN) ||
@@ -629,6 +628,8 @@ uint16 PopupMenu::Show(int numEntries, const char *actions[]) {
 	}
 
 bail_out:
+	delete s;
+
 #ifndef LURE_CLICKABLE_MENUS
 	mouse.setPosition(oldX, oldY);
 	mouse.cursorOn();
