@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/lure/res.cpp $
- * $Id: res.cpp 48814 2010-04-27 11:09:11Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/lure/res.cpp $
+ * $Id: res.cpp 54668 2010-11-30 09:00:20Z dreammaster $
  *
  */
 
@@ -33,8 +33,6 @@
 
 namespace Lure {
 
-using namespace Common;
-
 static Resources *int_resources = NULL;
 
 Resources &Resources::getReference() {
@@ -625,12 +623,16 @@ Hotspot *Resources::activateHotspot(uint16 hotspotId) {
 				CharacterScheduleEntry *entry = resources.charSchedules().getEntry(res->npcScheduleId);
 				res->npcSchedule.addFront(DISPATCH_ACTION, entry, res->roomNumber);
 			}
-			if ((hotspotId == GOEWIN_ID) && (hotspot->roomNumber() == 39))
+			if ((hotspotId == GOEWIN_ID) && (hotspot->roomNumber() == 39)) {
 				// WORKAROUND: When you re-join Goewin in the caves, clear her schedule. This may prevent a
 				// situation where you could close the left door, and she'd be permanently stuck trying to go
 				// the next room on the left, since her old schedule still had her following your old path
 				hotspot->currentActions().clear();
 
+				// Since she's no longer a follower, clear her start room field
+				hotspot->setStartRoomNumber(0);
+			}
+
 			// TODO: Figure out why there's a room set in the animation decode for a range of characters,
 			// particularly since it doesn't seem to match what happens in-game
 			/*
