@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/lure/res_struct.cpp $
- * $Id: res_struct.cpp 47443 2010-01-22 19:05:02Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/lure/res_struct.cpp $
+ * $Id: res_struct.cpp 54668 2010-11-30 09:00:20Z dreammaster $
  *
  */
 
@@ -32,7 +32,7 @@
 
 namespace Lure {
 
-extern const int actionNumParams[NPC_JUMP_ADDRESS+1] = {0,
+const int actionNumParams[NPC_JUMP_ADDRESS+1] = {0,
 	1, 0, 1, 1, 1, 1, 1, 1, 1, 2, 2, 1, 1, 1, 1, 1, 1, 2, 0, 1,
 	0, 1, 1, 1, 1, 0, 0, 2, 1, 1, 0, 0, 1, 1, 2, 2, 5, 2, 2, 1};
 
@@ -68,7 +68,7 @@ static const BarEntry default_barList[3] = {
 		&ewanExtraGraphic1[0], &ewanExtraGraphic2[0]}, 16, NULL}
 };
 
-extern const RoomTranslationRecord roomTranslations[] = {
+const RoomTranslationRecord roomTranslations[] = {
 	{0x1E, 0x13}, {0x07, 0x08}, {0x1C, 0x12}, {0x26, 0x0F},
 	{0x27, 0x0F}, {0x28, 0x0F}, {0x29, 0x0F}, {0x22, 0x0A},
 	{0x23, 0x13}, {0x24, 0x14}, {0x31, 0x2C}, {0x2F, 0x2C},
@@ -875,7 +875,7 @@ CharacterScheduleEntry::CharacterScheduleEntry(CharacterScheduleEntry *src) {
 	_parent = src->_parent;
 	_action = src->_action;
 	_numParams = src->_numParams;
-	Common::copy(src->_params, src->_params + MAX_TELL_COMMANDS * 3 * sizeof(uint16), _params);
+	Common::copy(src->_params, src->_params + MAX_TELL_COMMANDS * 3, _params);
 }
 
 uint16 CharacterScheduleEntry::param(int index) {
