@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/lure/res_struct.h $
- * $Id: res_struct.h 48713 2010-04-19 09:55:54Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/lure/res_struct.h $
+ * $Id: res_struct.h 54668 2010-11-30 09:00:20Z dreammaster $
  *
  */
 
@@ -445,6 +445,10 @@ public:
 	void setRoomNumber(uint16 roomNum) { _roomNumber = roomNum; }
 	void setSupportData(CharacterScheduleEntry *newRec) {
 		assert((newRec == NULL) || (newRec->parent() != NULL));
+		if (_dynamicSupportData) {
+			delete _supportData;
+			_dynamicSupportData = false;
+		}
 		_supportData = newRec;
 	}
 	void setSupportData(uint16 entryId);
@@ -704,7 +708,7 @@ public:
 
 class SequenceDelayData {
 private:
-	SequenceDelayData() {};
+	SequenceDelayData() {}
 public:
 	SequenceDelayData(uint16 delay, uint16 seqOffset, bool canClearFlag);
 	static SequenceDelayData *load(uint32 delay, uint16 seqOffset, bool canClearFlag);
