@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/lure/scripts.cpp $
- * $Id: scripts.cpp 44634 2009-10-04 21:26:33Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/lure/scripts.cpp $
+ * $Id: scripts.cpp 52972 2010-10-02 00:10:21Z dreammaster $
  *
  */
 
@@ -901,6 +901,16 @@ uint16 Script::execute(uint16 startOffset) {
 	uint16 offset = startOffset;
 	bool breakFlag = false;
 
+	// WORKAROUND: Prevents the Weregate door closing prematurely
+	if (startOffset == 3941) {
+		Hotspot *goewinHotspot = r.getActiveHotspot(GOEWIN_ID);
+		if (!goewinHotspot->doorCloseCheck(10025)) {
+			// Goewin is still blocking the door, so reschedule the closing
+			r.delayList().add(1, startOffset, false);
+			return 0;
+		}
+	}
+
 	param = 0;
 	fields.setField(SEQUENCE_RESULT, 0);
 
@@ -1297,6 +1307,11 @@ bool HotspotScript::execute(Hotspot *h) {
 
 		default:
 			// Set the animation frame number
+
+			// WORKAROUND: In Lure English EGA, the apparatus in room #30 can be set with an invalid frame number
+			if ((h->hotspotId() == 1059) && (opcode >= h->numFrames()))
+				opcode = h->numFrames() - 1;
+
 			debugC(ERROR_DETAILED, kLureDebugScripts, "SET FRAME NUMBER = %d", opcode);
 
 			h->setFrameNumber(opcode);
