@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/m4/converse.cpp $
- * $Id: converse.cpp 48172 2010-03-07 05:06:58Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/m4/converse.cpp $
+ * $Id: converse.cpp 50872 2010-07-14 04:42:27Z eriktorbjorn $
  *
  */
 
@@ -96,7 +96,7 @@ void ConversationView::setNode(int32 nodeIndex) {
 	_vm->_font->setFont(FONT_CONVERSATION);
 
 	// TODO: Conversation styles and colors
-	_vm->_font->setColors(2, 1, 3);
+	_vm->_font->current()->setColours(2, 1, 3);
 
 	_currentNodeIndex = nodeIndex;
 
@@ -124,7 +124,7 @@ void ConversationView::setNode(int32 nodeIndex) {
 			}
 
 			// Figure out the longest string to determine where option highlighting ends
-			int tempX = _vm->_font->getWidth(node->entries[i]->text, 0) +
+			int tempX = _vm->_font->current()->getWidth(node->entries[i]->text, 0) +
 				CONV_ENTRIES_X_OFFSET + 10;
 			_xEnd = MAX(_xEnd, tempX);
 		}
@@ -163,10 +163,10 @@ void ConversationView::onRefresh(RectList *rects, M4Surface *destSurface) {
 			if (i > CONV_MAX_SHOWN_ENTRIES - 1)
 				break;
 
-			_vm->_font->setColor((_highlightedIndex == i) ? CONVERSATION_ENTRY_HIGHLIGHTED :
+			_vm->_font->current()->setColour((_highlightedIndex == i) ? CONVERSATION_ENTRY_HIGHLIGHTED :
 				CONVERSATION_ENTRY_NORMAL);
 
-			_vm->_font->writeString(this, _activeItems[i]->text, CONV_ENTRIES_X_OFFSET,
+			_vm->_font->current()->writeString(this, _activeItems[i]->text, CONV_ENTRIES_X_OFFSET,
 				CONV_ENTRIES_Y_OFFSET + CONV_ENTRIES_HEIGHT * i, 0, 0);
 		}
 	}
@@ -858,8 +858,8 @@ void Converse::loadConversationMads(const char *convName) {
 		if (buffer[curPos - 1] == '\0') {
 			// end of string
 			//printf("%s\n", buffer);
-			buf = new char[strlen(buffer)];
-			sprintf(buf, "%s", buffer);
+			buf = new char[strlen(buffer) + 1];
+			strcpy(buf, buffer);
 			_convStrings.push_back(buf);
 			curPos = 0;
 			*buffer = 0;
