@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/m4/m4.cpp $
- * $Id: m4.cpp 48172 2010-03-07 05:06:58Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/m4/m4.cpp $
+ * $Id: m4.cpp 50723 2010-07-06 11:33:09Z dreammaster $
  *
  */
 
@@ -52,12 +52,14 @@
 #include "m4/mads_menus.h"
 
 #include "common/file.h"
+#include "common/fs.h"
 #include "common/events.h"
 #include "common/EventRecorder.h"
 #include "common/endian.h"
 #include "common/system.h"
 #include "common/config-manager.h"
-#include "engines/engine.h"
+#include "common/debug-channels.h"
+#include "engines/util.h"
 #include "graphics/surface.h"
 #include "sound/mididrv.h"
 
@@ -111,11 +113,14 @@ MadsM4Engine::MadsM4Engine(OSystem *syst, const M4GameDescription *gameDesc) :
 	_vm = this;
 	_madsVm = NULL;
 
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "goodstuf");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "resource");
+	const Common::FSNode gameDataDir(ConfMan.get("path"));
 
-	Common::addDebugChannel(kDebugScript, "script", "Script debug level");
-	Common::addDebugChannel(kDebugConversations, "conversations", "Conversations debugging");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "goodstuf");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "resource");
+
+	DebugMan.addDebugChannel(kDebugScript, "script", "Script debug level");
+	DebugMan.addDebugChannel(kDebugGraphics, "graphics", "Graphics debug level");
+	DebugMan.addDebugChannel(kDebugConversations, "conversations", "Conversations debugging");
 
 	_resourceManager = NULL;
 	_globals = NULL;
@@ -140,22 +145,25 @@ MadsM4Engine::~MadsM4Engine() {
 	delete _script;
 	delete _ws;
 	delete _random;
-	delete _animation;
 	delete _palette;
+	delete _globals;
+	delete _sound;
+	delete _driver;
+	delete _resourceManager;
 }
 
 Common::Error MadsM4Engine::run() {
 	// Initialize backend
 	_screen = new M4Surface(true); // Special form for creating screen reference
 
-	MidiDriverType midiDriver = MidiDriver::detectMusicDriver(MDT_MIDI | MDT_ADLIB | MDT_PREFER_MIDI);
-	bool native_mt32 = ((midiDriver == MD_MT32) || ConfMan.getBool("native_mt32"));
+	MidiDriver::DeviceHandle dev = MidiDriver::detectDevice(MDT_MIDI | MDT_ADLIB | MDT_PREFER_GM);
+	bool native_mt32 = ((MidiDriver::getMusicType(dev) == MT_MT32) || ConfMan.getBool("native_mt32"));
 
-	MidiDriver *driver = MidiDriver::createMidi(midiDriver);
+	_driver = MidiDriver::createMidi(dev);
 	if (native_mt32)
-		driver->property(MidiDriver::PROP_CHANNEL_MASK, 0x03FE);
+		_driver->property(MidiDriver::PROP_CHANNEL_MASK, 0x03FE);
 
-	_midi = new MidiPlayer(this, driver);
+	_midi = new MidiPlayer(this, _driver);
 	_midi->setGM(true);
 	_midi->setNativeMT32(native_mt32);
 
@@ -165,7 +173,7 @@ Common::Error MadsM4Engine::run() {
 	_events = new Events(this);
 	_kernel = new Kernel(this);
 	_player = new Player(this);
-	_font = new Font(this);
+	_font = new FontManager(this);
 	if (getGameType() == GType_Burger) {
 		_actor = new Actor(this);
 		_conversationView = new ConversationView(this);
@@ -179,7 +187,6 @@ Common::Error MadsM4Engine::run() {
 	_sound = new Sound(this, _mixer, 255);
 	_script = new ScriptInterpreter(this);
 	_ws = new WoodScript(this);
-	_animation = new Animation(this);
 	//_callbacks = new Callbacks(this);
 	_random = new Common::RandomSource();
 	g_eventRec.registerRandomSource(*_random, "m4");
@@ -300,8 +307,6 @@ M4Engine::M4Engine(OSystem *syst, const M4GameDescription *gameDesc): MadsM4Engi
 }
 
 M4Engine::~M4Engine() {
-	delete _resourceManager;
-	delete _globals;
 	delete _converse;
 }
 
@@ -412,7 +417,7 @@ Common::Error M4Engine::run() {
 			_scene->loadScene(_kernel->currentRoom);
 
 			_ws->setBackgroundSurface(_scene->getBackgroundSurface());
-			_ws->setInverseColorTable(_scene->getInverseColorTable());
+			_ws->setInverseColourTable(scene()->getInverseColourTable());
 
 			_kernel->loadSectionScriptFunctions();
 			_kernel->loadRoomScriptFunctions();
@@ -493,11 +498,10 @@ MadsEngine::MadsEngine(OSystem *syst, const M4GameDescription *gameDesc): MadsM4
 	_madsVm = this;
 
 	_globals = new MadsGlobals(this);
+	_currentTimer = 0;
 }
 
 MadsEngine::~MadsEngine() {
-	delete _globals;
-	delete _resourceManager;
 }
 
 Common::Error MadsEngine::run() {
@@ -510,7 +514,6 @@ Common::Error MadsEngine::run() {
 	// Set up needed common functionality
 	MadsM4Engine::run();
 
-	_scene = new MadsScene(this);
 	_palette->setMadsSystemPalette();
 
 	_mouse->init("cursor.ss", NULL);
@@ -519,37 +522,42 @@ Common::Error MadsEngine::run() {
 	// Load MADS data files
 	MadsGlobals *globs = (MadsGlobals *)_globals;
 	globs->loadMadsVocab();			// vocab.dat
-	globs->loadMadsQuotes();			// quotes.dat
+	globs->loadQuotes();			// quotes.dat
 	globs->loadMadsMessagesInfo();	// messages.dat
 	globs->loadMadsObjects();
 
 	// Setup globals
-	globs->easyMouse = true;
-	globs->invObjectsStill = false;
-	globs->textWindowStill = false;
-	globs->storyMode = 0;
+	globs->_config.easyMouse = true;
+	globs->_config.invObjectsStill = false;
+	globs->_config.textWindowStill = false;
+	globs->_config.storyMode = 1;	// Naughty
+	globs->_config.screenFades = 0;
 
 	// Test code to dump all messages to the console
 	//for (int i = 0; i < _globals->getMessagesSize(); i++)
 	//printf("%s\n----------\n", _globals->loadMessage(i));
 
-	if ((getGameType() == GType_RexNebular) || (getGameType() == GType_DragonSphere)) {
-		loadMenu(MAIN_MENU);
+	if (getGameType() == GType_RexNebular) {
+		MadsGameLogic::initialiseGlobals();
 
+		_scene = NULL;
+		loadMenu(MAIN_MENU);
 	} else {
-		if (getGameType() == GType_DragonSphere) {
-			_scene->loadScene(FIRST_SCENE);
-		} else if (getGameType() == GType_Phantom) {
-			//_scene->loadScene(FIRST_SCENE);
-			_scene->loadScene(106);		// a more interesting scene
-		}
+		// Test code
+		_scene = new MadsScene(this);
+
+		startScene(FIRST_SCENE);
+		RGBList *_bgPalData;
+		_scene->loadBackground(FIRST_SCENE, &_bgPalData);
+		_palette->addRange(_bgPalData);
+		_scene->translate(_bgPalData);
 
 		_scene->show();
 
 		_font->setFont(FONT_MAIN_MADS);
-		_font->setColors(2, 1, 3);
-		_font->writeString(_scene->getBackgroundSurface(), "Testing the M4/MADS ScummVM engine", 5, 160, 310, 2);
-		_font->writeString(_scene->getBackgroundSurface(), "ABCDEFGHIJKLMNOPQRSTUVWXYZ", 5, 180, 310, 2);
+		_font->current()->setColours(2, 1, 3);
+		_font->current()->writeString(_scene->getBackgroundSurface(), "Testing the M4/MADS ScummVM engine", 5, 160, 310, 2);
+		_font->current()->writeString(_scene->getBackgroundSurface(), "ABCDEFGHIJKLMNOPQRSTUVWXYZ", 5, 180, 310, 2);
 
 		if (getGameType() == GType_DragonSphere) {
 			//_scene->showMADSV2TextBox("Test", 10, 10, NULL);
@@ -561,35 +569,46 @@ Common::Error MadsEngine::run() {
 	_viewManager->systemHotkeys().add(Common::KEYCODE_ESCAPE, &escapeHotkeyHandler);
 	_viewManager->systemHotkeys().add(Common::KEYCODE_KP_MULTIPLY, &textviewHotkeyHandler);
 
-	// Load the general game SFX/voices
-	if (getGameType() == GType_RexNebular) {
-		_sound->loadDSRFile("rex009.dsr");
-	} else if (getGameType() == GType_Phantom) {
-		_sound->loadDSRFile("phan009.dsr");
-	} else if (getGameType() == GType_DragonSphere) {
-		_sound->loadDSRFile("drag009.dsr");
-	}
-
 	uint32 nextFrame = g_system->getMillis();
 	while (!_events->quitFlag) {
 		eventHandler();
 
-		_animation->updateAnim();
-
-		// Call the updateState method of all views
-		_viewManager->updateState();
-
 		if (g_system->getMillis() >= nextFrame) {
+			nextFrame = g_system->getMillis() + GAME_FRAME_DELAY;
+			++_currentTimer;
 
+			// Call the updateState method of all views
+			_viewManager->updateState();
+
+			// Refresh the display
 			_viewManager->refreshAll();
-			nextFrame = g_system->getMillis();// + GAME_FRAME_DELAY;
 		}
 
 		g_system->delayMillis(10);
+
+		if (globals()->dialogType != DIALOG_NONE)
+			showDialog();
 	}
 
 	return Common::kNoError;
 }
 
+void MadsEngine::showDialog() {
+	// Switch to showing the given dialog
+	RexDialogView *dlg = NULL;
+	switch (globals()->dialogType) {
+	case DIALOG_GAME_MENU:
+		dlg = new RexGameMenuDialog();
+		break;
+	case DIALOG_OPTIONS:
+		dlg = new RexOptionsDialog();
+		break;
+	default:
+		error("Unknown dialog type");
+	};
+
+	globals()->dialogType = DIALOG_NONE;
+	_viewManager->addView(dlg);
+}
 
 } // End of namespace M4
