@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/m4/ws_sequence.cpp $
- * $Id: ws_sequence.cpp 46280 2009-12-07 18:22:18Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/m4/ws_sequence.cpp $
+ * $Id: ws_sequence.cpp 52148 2010-08-17 11:01:20Z sev $
  *
  */
 
@@ -226,7 +226,7 @@ bool Sequence::runProgram() {
 	while (!done) {
 		Instruction instruction;
 		_code->loadInstruction(instruction);
-		if (sequenceCommandsTable[instruction.instr] != NULL)
+		if (sequenceCommandsTable[instruction.instr] != 0)
 			done = !(this->*sequenceCommandsTable[instruction.instr])(instruction);
 		else { fflush(stdout); /*g_system->delayMillis(1000);*/ }
 	}
@@ -289,7 +289,7 @@ void Sequence::draw(M4Surface *surface, const Common::Rect &clipRect, Common::Re
 	info.hotX = _curFrame->xOffset;
 	info.hotY = _curFrame->yOffset;
 	info.encoding = _curFrame->encoding;
-	info.inverseColorTable = _vm->_scene->getInverseColorTable();
+	info.inverseColourTable = _m4Vm->scene()->getInverseColourTable();
 	info.palette = _ws->getMainPalette();
 	info.width = _curFrame->width();
 	info.height = _curFrame->height();
