@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/mohawk/console.cpp $
- * $Id: console.cpp 48080 2010-02-17 19:59:08Z tdhs $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/mohawk/console.cpp $
+ * $Id: console.cpp 52631 2010-09-07 20:18:30Z mthreepwood $
  *
  */
 
@@ -28,9 +28,10 @@
 #include "mohawk/myst_scripts.h"
 #include "mohawk/graphics.h"
 #include "mohawk/riven.h"
+#include "mohawk/riven_external.h"
 #include "mohawk/livingbooks.h"
 #include "mohawk/sound.h"
-#include "mohawk/video/video.h"
+#include "mohawk/video.h"
 
 namespace Mohawk {
 
@@ -307,6 +308,8 @@ RivenConsole::RivenConsole(MohawkEngine_Riven *vm) : GUI::Debugger(), _vm(vm) {
 	DCmd_Register("dumpScript",     WRAP_METHOD(RivenConsole, Cmd_DumpScript));
 	DCmd_Register("listZipCards",   WRAP_METHOD(RivenConsole, Cmd_ListZipCards));
 	DCmd_Register("getRMAP",		WRAP_METHOD(RivenConsole, Cmd_GetRMAP));
+	DCmd_Register("combos",         WRAP_METHOD(RivenConsole, Cmd_Combos));
+	DCmd_Register("sliderState",    WRAP_METHOD(RivenConsole, Cmd_SliderState));
 }
 
 RivenConsole::~RivenConsole() {
@@ -347,7 +350,7 @@ bool RivenConsole::Cmd_Var(int argc, const char **argv) {
 		return true;
 	}
 
-	uint32 *globalVar = _vm->matchVarToString(argv[1]);
+	uint32 *globalVar = _vm->getVar(argv[1]);
 
 	if (!globalVar) {
 		DebugPrintf("Unknown variable \'%s\'\n", argv[1]);
@@ -364,19 +367,13 @@ bool RivenConsole::Cmd_Var(int argc, const char **argv) {
 
 bool RivenConsole::Cmd_PlaySound(int argc, const char **argv) {
 	if (argc < 2) {
-		DebugPrintf("Usage: playSound <value> (<use main sound file, default = true>)\n");
-		DebugPrintf("The main sound file is default, but you can use the word \'false\' to make it use the current stack file.\n");
-
+		DebugPrintf("Usage: playSound <value>\n");
 		return true;
 	}
 
 	_vm->_sound->stopSound();
 	_vm->_sound->stopAllSLST();
-
-	bool mainSoundFile = (argc < 3) || (scumm_stricmp(argv[2], "false") != 0);
-
-	_vm->_sound->playSound((uint16)atoi(argv[1]), mainSoundFile);
-
+	_vm->_sound->playSound((uint16)atoi(argv[1]));
 	return false;
 }
 
@@ -390,13 +387,9 @@ bool RivenConsole::Cmd_PlaySLST(int argc, const char **argv) {
 	_vm->_sound->stopSound();
 	_vm->_sound->stopAllSLST();
 
-	uint16 card = _vm->getCurCard();
-
-	if (argc == 3)
-		card = (uint16)atoi(argv[2]);
+	uint16 card = (argc == 3) ? (uint16)atoi(argv[2]) : _vm->getCurCard();
 
 	_vm->_sound->playSLST((uint16)atoi(argv[1]), card);
-
 	return false;
 }
 
@@ -405,7 +398,6 @@ bool RivenConsole::Cmd_StopSound(int argc, const char **argv) {
 
 	_vm->_sound->stopSound();
 	_vm->_sound->stopAllSLST();
-
 	return true;
 }
 
@@ -464,10 +456,11 @@ bool RivenConsole::Cmd_Hotspots(int argc, const char **argv) {
 		DebugPrintf("Hotspot %d, index %d, BLST ID %d (", i, _vm->_hotspots[i].index, _vm->_hotspots[i].blstID);
 
 		if (_vm->_hotspots[i].enabled)
-			DebugPrintf("enabled)\n");
+			DebugPrintf("enabled");
 		else
-			DebugPrintf("disabled)\n");
+			DebugPrintf("disabled");
 
+		DebugPrintf(") - (%d, %d, %d, %d)\n", _vm->_hotspots[i].rect.left, _vm->_hotspots[i].rect.top, _vm->_hotspots[i].rect.right, _vm->_hotspots[i].rect.bottom);
 		DebugPrintf("    Name = %s\n", _vm->getHotspotName(i).c_str());
 	}
 
@@ -475,7 +468,7 @@ bool RivenConsole::Cmd_Hotspots(int argc, const char **argv) {
 }
 
 bool RivenConsole::Cmd_ZipMode(int argc, const char **argv) {
-	uint32 *zipModeActive = _vm->matchVarToString("azip");
+	uint32 *zipModeActive = _vm->getVar("azip");
 	*zipModeActive = !(*zipModeActive);
 
 	DebugPrintf("Zip Mode is ");
@@ -510,7 +503,7 @@ bool RivenConsole::Cmd_DumpScript(int argc, const char **argv) {
 
 	// Load in Variable Names
 	Common::SeekableReadStream *nameStream = _vm->getRawData(ID_NAME, VariableNames);
-	Common::StringList varNames;
+	Common::StringArray varNames;
 
 	uint16 namesCount = nameStream->readUint16BE();
 	uint16 *stringOffsets = new uint16[namesCount];
@@ -531,7 +524,7 @@ bool RivenConsole::Cmd_DumpScript(int argc, const char **argv) {
 
 	// Load in External Command Names
 	nameStream = _vm->getRawData(ID_NAME, ExternalCommandNames);
-	Common::StringList xNames;
+	Common::StringArray xNames;
 
 	namesCount = nameStream->readUint16BE();
 	stringOffsets = new uint16[namesCount];
@@ -556,9 +549,11 @@ bool RivenConsole::Cmd_DumpScript(int argc, const char **argv) {
 		printf ("==================================\n\n");
 		Common::SeekableReadStream *cardStream = _vm->getRawData(MKID_BE('CARD'), (uint16)atoi(argv[3]));
 		cardStream->seek(4);
-		RivenScriptList scriptList = RivenScript::readScripts(_vm, cardStream);
-		for (uint32 i = 0; i < scriptList.size(); i++)
+		RivenScriptList scriptList = _vm->_scriptMan->readScripts(cardStream, false);
+		for (uint32 i = 0; i < scriptList.size(); i++) {
 			scriptList[i]->dumpScript(varNames, xNames, 0);
+			delete scriptList[i];
+		}
 		delete cardStream;
 	} else if (!scumm_stricmp(argv[2], "HSPT")) {
 		printf ("\n\nDumping scripts for %s\'s card %d hotspots!\n", argv[1], (uint16)atoi(argv[3]));
@@ -571,9 +566,11 @@ bool RivenConsole::Cmd_DumpScript(int argc, const char **argv) {
 		for (uint16 i = 0; i < hotspotCount; i++) {
 			printf ("Hotspot %d:\n", i);
 			hsptStream->seek(22, SEEK_CUR);	// Skip non-script related stuff
-			RivenScriptList scriptList = RivenScript::readScripts(_vm, hsptStream);
-			for (uint32 j = 0; j < scriptList.size(); j++)
+			RivenScriptList scriptList = _vm->_scriptMan->readScripts(hsptStream, false);
+			for (uint32 j = 0; j < scriptList.size(); j++) {
 				scriptList[j]->dumpScript(varNames, xNames, 1);
+				delete scriptList[j];
+			}
 		}
 
 		delete hsptStream;
@@ -608,6 +605,41 @@ bool RivenConsole::Cmd_GetRMAP(int argc, const char **argv) {
 	return true;
 }
 
+bool RivenConsole::Cmd_Combos(int argc, const char **argv) {
+	// In the vain of SCUMM's 'drafts' command, this command will list
+	// out all combinations needed in Riven, decoded from the variables.
+	// You'll need to look up the Rebel Tunnel puzzle on your own; the
+	// solution is constant.
+
+	uint32 teleCombo = *_vm->getVar("tcorrectorder");
+	uint32 prisonCombo = *_vm->getVar("pcorrectorder");
+	uint32 domeCombo = *_vm->getVar("adomecombo");
+	
+	DebugPrintf("Telescope Combo:\n  ");
+	for (int i = 0; i < 5; i++)
+		DebugPrintf("%d ", _vm->_externalScriptHandler->getComboDigit(teleCombo, i));
+
+	DebugPrintf("\nPrison Combo:\n  ");
+	for (int i = 0; i < 5; i++)
+		DebugPrintf("%d ", _vm->_externalScriptHandler->getComboDigit(prisonCombo, i));
+
+	DebugPrintf("\nDome Combo:\n  ");
+	for (int i = 1; i <= 25; i++)
+		if (domeCombo & (1 << (25 - i)))
+			DebugPrintf("%d ", i);
+
+	DebugPrintf("\n");
+	return true;
+}
+
+bool RivenConsole::Cmd_SliderState(int argc, const char **argv) {
+	if (argc > 1)
+		_vm->_externalScriptHandler->setDomeSliderState((uint32)atoi(argv[1]));
+
+	DebugPrintf("Dome Slider State = %08x\n", _vm->_externalScriptHandler->getDomeSliderState());
+	return true;
+}
+
 LivingBooksConsole::LivingBooksConsole(MohawkEngine_LivingBooks *vm) : GUI::Debugger(), _vm(vm) {
 	DCmd_Register("playSound",			WRAP_METHOD(LivingBooksConsole, Cmd_PlaySound));
 	DCmd_Register("stopSound",			WRAP_METHOD(LivingBooksConsole, Cmd_StopSound));
