@@ -18,12 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/mohawk/myst.cpp $
- * $Id: myst.cpp 48080 2010-02-17 19:59:08Z tdhs $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/mohawk/myst.cpp $
+ * $Id: myst.cpp 52692 2010-09-12 21:35:49Z criezy $
  *
  */
 
 #include "common/config-manager.h"
+#include "common/debug-channels.h"
+#include "common/translation.h"
 
 #include "mohawk/graphics.h"
 #include "mohawk/myst.h"
@@ -32,21 +34,21 @@
 #include "mohawk/dialogs.h"
 #include "mohawk/resource.h"
 #include "mohawk/resource_cache.h"
-#include "mohawk/video/video.h"
+#include "mohawk/video.h"
 
 namespace Mohawk {
 
 MohawkEngine_Myst::MohawkEngine_Myst(OSystem *syst, const MohawkGameDescription *gamedesc) : MohawkEngine(syst, gamedesc) {
-	Common::addDebugChannel(kDebugVariable, "Variable", "Track Variable Accesses");
-	Common::addDebugChannel(kDebugSaveLoad, "SaveLoad", "Track Save/Load Function");
-	Common::addDebugChannel(kDebugView, "View", "Track Card File (VIEW) Parsing");
-	Common::addDebugChannel(kDebugHint, "Hint", "Track Cursor Hints (HINT) Parsing");
-	Common::addDebugChannel(kDebugResource, "Resource", "Track Resource (RLST) Parsing");
-	Common::addDebugChannel(kDebugINIT, "Init", "Track Card Init Script (INIT) Parsing");
-	Common::addDebugChannel(kDebugEXIT, "Exit", "Track Card Exit Script (EXIT) Parsing");
-	Common::addDebugChannel(kDebugScript, "Script", "Track Script Execution");
-	Common::addDebugChannel(kDebugHelp, "Help", "Track Help File (HELP) Parsing");
-	Common::addDebugChannel(kDebugCache, "Cache", "Track Resource Cache Accesses");
+	DebugMan.addDebugChannel(kDebugVariable, "Variable", "Track Variable Accesses");
+	DebugMan.addDebugChannel(kDebugSaveLoad, "SaveLoad", "Track Save/Load Function");
+	DebugMan.addDebugChannel(kDebugView, "View", "Track Card File (VIEW) Parsing");
+	DebugMan.addDebugChannel(kDebugHint, "Hint", "Track Cursor Hints (HINT) Parsing");
+	DebugMan.addDebugChannel(kDebugResource, "Resource", "Track Resource (RLST) Parsing");
+	DebugMan.addDebugChannel(kDebugINIT, "Init", "Track Card Init Script (INIT) Parsing");
+	DebugMan.addDebugChannel(kDebugEXIT, "Exit", "Track Card Exit Script (EXIT) Parsing");
+	DebugMan.addDebugChannel(kDebugScript, "Script", "Track Script Execution");
+	DebugMan.addDebugChannel(kDebugHelp, "Help", "Track Help File (HELP) Parsing");
+	DebugMan.addDebugChannel(kDebugCache, "Cache", "Track Resource Cache Accesses");
 
 	_zipMode = false;
 	_transitionsEnabled = false;
@@ -74,8 +76,10 @@ MohawkEngine_Myst::MohawkEngine_Myst(OSystem *syst, const MohawkGameDescription
 
 	_scriptParser->disableInitOpcodes();
 
-	if ((getFeatures() & GF_ME) && getPlatform() == Common::kPlatformMacintosh)
-		SearchMan.addSubDirectoryMatching(_gameDataDir, "CD Data");
+	if ((getFeatures() & GF_ME) && getPlatform() == Common::kPlatformMacintosh) {
+		const Common::FSNode gameDataDir(ConfMan.get("path"));
+		SearchMan.addSubDirectoryMatching(gameDataDir, "CD Data");
+	}
 }
 
 MohawkEngine_Myst::~MohawkEngine_Myst() {
@@ -94,10 +98,10 @@ MohawkEngine_Myst::~MohawkEngine_Myst() {
 
 // Uses cached data objects in preference to disk access
 Common::SeekableReadStream *MohawkEngine_Myst::getRawData(uint32 tag, uint16 id) {
-	Common::SeekableReadStream *ret;
+	Common::SeekableReadStream *ret = _cache.search(tag, id);
 
-	ret = _cache.search(tag, id);
-	if(ret != NULL) return ret;
+	if (ret)
+		return ret;
 
 	for (uint32 i = 0; i < _mhk.size(); i++)
 		if (_mhk[i]->hasResource(tag, id)) {
@@ -106,24 +110,37 @@ Common::SeekableReadStream *MohawkEngine_Myst::getRawData(uint32 tag, uint16 id)
 			return ret;
 		}
 
-	error ("Could not find a \'%s\' resource with ID %04x", tag2str(tag), id);
+	error("Could not find a \'%s\' resource with ID %04x", tag2str(tag), id);
 	return NULL;
 }
 
 void MohawkEngine_Myst::cachePreload(uint32 tag, uint16 id) {
-	Common::SeekableReadStream *tempData;
+	if (!_cache.enabled)
+		return;
 
-	if (!_cache.enabled) return;
+	for (uint32 i = 0; i < _mhk.size(); i++) {
+		// Check for MJMP in Myst ME
+		if ((getFeatures() & GF_ME) && tag == ID_MSND && _mhk[i]->hasResource(ID_MJMP, id)) {
+			Common::SeekableReadStream *tempData = _mhk[i]->getRawData(ID_MJMP, id);
+			uint16 msndId = tempData->readUint16LE();
+			delete tempData;
 
-	for (uint32 i = 0; i < _mhk.size(); i++)
-		if (_mhk[i]->hasResource(tag, id)) {
-			tempData = _mhk[i]->getRawData(tag, id);
+			// We've found where the real MSND data is, so go get that
+			tempData = _mhk[i]->getRawData(tag, msndId);
 			_cache.add(tag, id, tempData);
 			delete tempData;
 			return;
 		}
 
-	warning ("cachePreload : Could not find a \'%s\' resource with ID %04x", tag2str(tag), id);
+		if (_mhk[i]->hasResource(tag, id)) {
+			Common::SeekableReadStream *tempData = _mhk[i]->getRawData(tag, id);
+			_cache.add(tag, id, tempData);
+			delete tempData;
+			return;
+		}
+	}
+
+	warning("cachePreload: Could not find a \'%s\' resource with ID %04x", tag2str(tag), id);
 }
 
 static const char *mystFiles[] = {
@@ -202,7 +219,7 @@ Common::Error MohawkEngine_Myst::run() {
 	_varStore = new MystVar(this);
 	_saveLoad = new MystSaveLoad(this, _saveFileMan);
 	_scriptParser = new MystScriptParser(this);
-	_loadDialog = new GUI::SaveLoadChooser("Load Game:", "Load");
+	_loadDialog = new GUI::SaveLoadChooser(_("Load game:"), _("Load"));
 	_loadDialog->setSaveMode(false);
 	_optionsDialog = new MystOptionsDialog(this);
 
@@ -222,7 +239,7 @@ Common::Error MohawkEngine_Myst::run() {
 	// Load game from launcher/command line if requested
 	if (ConfMan.hasKey("save_slot") && !(getFeatures() & GF_DEMO)) {
 		uint32 gameToLoad = ConfMan.getInt("save_slot");
-		Common::StringList savedGamesList = _saveLoad->generateSaveGameList();
+		Common::StringArray savedGamesList = _saveLoad->generateSaveGameList();
 		if (gameToLoad > savedGamesList.size())
 			error ("Could not find saved game");
 		_saveLoad->loadGame(savedGamesList[gameToLoad]);
@@ -411,7 +428,7 @@ void MohawkEngine_Myst::changeToCard(uint16 card) {
 
 		// NOTE: All sounds are looped when played via the sound section of the
 		// VIEW resources.
-		_sound->playSound(soundAction, true, soundActionVolume, true);
+		_sound->playSound(soundAction, soundActionVolume, true);
 	} else {
 		error("Unknown sound action %d", soundAction);
 	}
@@ -1593,7 +1610,7 @@ Common::Error MohawkEngine_Myst::loadGameState(int slot) {
 }
 
 Common::Error MohawkEngine_Myst::saveGameState(int slot, const char *desc) {
-	Common::StringList saveList = _saveLoad->generateSaveGameList();
+	Common::StringArray saveList = _saveLoad->generateSaveGameList();
 
 	if ((uint)slot < saveList.size())
 		_saveLoad->deleteSave(saveList[slot]);
