@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/mohawk/myst_saveload.cpp $
- * $Id: myst_saveload.cpp 46965 2010-01-04 08:10:29Z tdhs $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/mohawk/myst_saveload.cpp $
+ * $Id: myst_saveload.cpp 48570 2010-04-06 15:40:13Z salty-horse $
  *
  */
 
@@ -39,7 +39,7 @@ MystSaveLoad::~MystSaveLoad() {
 	delete _v;
 }
 
-Common::StringList MystSaveLoad::generateSaveGameList() {
+Common::StringArray MystSaveLoad::generateSaveGameList() {
 	return _saveFileMan->listSavefiles("*.mys");
 }
 
@@ -495,7 +495,7 @@ static const char *game_globals_names[] = {
 	"Current Age / Stack",
 	"Page Being Held",
 	"Unknown - Fixed at 1",
-	"Slide Transistions",
+	"Slide Transitions",
 	"Zip Mode",
 	"Red Pages in Book",
 	"Blue Pages in Book"
