@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/mohawk/riven.cpp $
- * $Id: riven.cpp 48134 2010-02-26 08:14:33Z mthreepwood $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/mohawk/riven.cpp $
+ * $Id: riven.cpp 52692 2010-09-12 21:35:49Z criezy $
  *
  */
 
@@ -27,29 +27,55 @@
 #include "common/events.h"
 #include "common/EventRecorder.h"
 #include "common/keyboard.h"
+#include "common/translation.h"
 
 #include "mohawk/graphics.h"
+#include "mohawk/resource.h"
 #include "mohawk/riven.h"
 #include "mohawk/riven_external.h"
 #include "mohawk/riven_saveload.h"
 #include "mohawk/dialogs.h"
-#include "mohawk/video/video.h"
+#include "mohawk/video.h"
 
 namespace Mohawk {
 
+Common::Rect *g_atrusJournalRect1;
+Common::Rect *g_atrusJournalRect2;
+Common::Rect *g_cathJournalRect2;
+Common::Rect *g_atrusJournalRect3;
+Common::Rect *g_cathJournalRect3;
+Common::Rect *g_trapBookRect3;
+Common::Rect *g_demoExitRect;
+
 MohawkEngine_Riven::MohawkEngine_Riven(OSystem *syst, const MohawkGameDescription *gamedesc) : MohawkEngine(syst, gamedesc) {
 	_showHotspots = false;
 	_cardData.hasData = false;
 	_gameOver = false;
 	_activatedSLST = false;
+	_ignoreNextMouseUp = false;
 	_extrasFile = NULL;
+	_curStack = aspit;
+	_hotspots = NULL;
 
-	// Attempt to let game run from the CD's
-	// NOTE: assets2 contains higher quality audio than assets1
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "all");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "data");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "exe");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "assets2");
+	// NOTE: We can never really support CD swapping. All of the music files
+	// (*_Sounds.mhk) are stored on disc 1. They are copied to the hard drive
+	// during install and used from there. The same goes for the extras.mhk
+	// file. The following directories allow Riven to be played directly
+	// from the DVD.
+
+	const Common::FSNode gameDataDir(ConfMan.get("path"));
+	SearchMan.addSubDirectoryMatching(gameDataDir, "all");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "data");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "exe");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "assets1");
+
+	g_atrusJournalRect1 = new Common::Rect(295, 402, 313, 426);
+	g_atrusJournalRect2 = new Common::Rect(259, 402, 278, 426);
+	g_cathJournalRect2 = new Common::Rect(328, 408, 348, 419);
+	g_atrusJournalRect3 = new Common::Rect(222, 402, 240, 426);
+	g_cathJournalRect3 = new Common::Rect(291, 408, 311, 419);
+	g_trapBookRect3 = new Common::Rect(363, 396, 386, 432);
+	g_demoExitRect = new Common::Rect(291, 408, 317, 419);
 }
 
 MohawkEngine_Riven::~MohawkEngine_Riven() {
@@ -58,18 +84,24 @@ MohawkEngine_Riven::~MohawkEngine_Riven() {
 	delete _externalScriptHandler;
 	delete _extrasFile;
 	delete _saveLoad;
+	delete _scriptMan;
 	delete[] _vars;
-	delete _loadDialog;
 	delete _optionsDialog;
 	delete _rnd;
-	_cardData.scripts.clear();
+	delete[] _hotspots;
+	delete g_atrusJournalRect1;
+	delete g_atrusJournalRect2;
+	delete g_cathJournalRect2;
+	delete g_atrusJournalRect3;
+	delete g_cathJournalRect3;
+	delete g_trapBookRect3;
+	delete g_demoExitRect;
 }
 
 GUI::Debugger *MohawkEngine_Riven::getDebugger() {
 	return _console;
 }
 
-
 Common::Error MohawkEngine_Riven::run() {
 	MohawkEngine::run();
 
@@ -77,128 +109,141 @@ Common::Error MohawkEngine_Riven::run() {
 	_console = new RivenConsole(this);
 	_saveLoad = new RivenSaveLoad(this, _saveFileMan);
 	_externalScriptHandler = new RivenExternal(this);
-	_loadDialog = new GUI::SaveLoadChooser("Load Game:", "Load");
-	_loadDialog->setSaveMode(false);
 	_optionsDialog = new RivenOptionsDialog(this);
+	_scriptMan = new RivenScriptManager(this);
 
 	_rnd = new Common::RandomSource();
 	g_eventRec.registerRandomSource(*_rnd, "riven");
 
 	initVars();
 
-	// Open extras.mhk for common images (non-existant in the demo)
-	if (!(getFeatures() & GF_DEMO)) {
-		_extrasFile = new MohawkArchive();
-		_extrasFile->open("extras.mhk");
-	}
+	// Open extras.mhk for common images
+	_extrasFile = new MohawkArchive();
+	_extrasFile->open("extras.mhk");
 
 	// Start at main cursor
 	_gfx->changeCursor(kRivenMainCursor);
 
-	// Load game from launcher/command line if requested
-	if (ConfMan.hasKey("save_slot") && !(getFeatures() & GF_DEMO)) {
+	// Let's begin, shall we?
+	if (getFeatures() & GF_DEMO) {
+		// Start the demo off with the videos
+		changeToStack(aspit);
+		changeToCard(6);
+	} else if (ConfMan.hasKey("save_slot")) {
+		// Load game from launcher/command line if requested
 		uint32 gameToLoad = ConfMan.getInt("save_slot");
-		Common::StringList savedGamesList = _saveLoad->generateSaveGameList();
+		Common::StringArray savedGamesList = _saveLoad->generateSaveGameList();
 		if (gameToLoad > savedGamesList.size())
 			error ("Could not find saved game");
 		_saveLoad->loadGame(savedGamesList[gameToLoad]);
-	} else { // Otherwise, start us off at aspit's card 1
+	} else {
+		// Otherwise, start us off at aspit's card 1 (the main menu)
         changeToStack(aspit);
 		changeToCard(1);
 	}
 
+	
+	while (!_gameOver && !shouldQuit())
+		handleEvents();
+
+	return Common::kNoError;
+}
+
+void MohawkEngine_Riven::handleEvents() {
 	Common::Event event;
-	while (!_gameOver) {
-		bool needsUpdate = _gfx->runScheduledWaterEffects();
-		needsUpdate |= _video->updateBackgroundMovies();
 
-		while (_eventMan->pollEvent(event)) {
-			switch (event.type) {
-			case Common::EVENT_MOUSEMOVE:
-				_mousePos = event.mouse;
-				checkHotspotChange();
+	// Update background videos and the water effect
+	bool needsUpdate = _gfx->runScheduledWaterEffects();
+	needsUpdate |= _video->updateBackgroundMovies();
 
-				// Check to show the inventory
-				if (_mousePos.y >= 392)
+	while (_eventMan->pollEvent(event)) {
+		switch (event.type) {
+		case Common::EVENT_MOUSEMOVE:
+			checkHotspotChange();
+
+			if (!(getFeatures() & GF_DEMO)) {
+				// Check to show the inventory, but it is always "showing" in the demo
+				if (_eventMan->getMousePos().y >= 392)
 					_gfx->showInventory();
 				else
 					_gfx->hideInventory();
+			}
 
-				needsUpdate = true;
-				break;
-			case Common::EVENT_LBUTTONDOWN:
-				if (_curHotspot >= 0)
-					runHotspotScript(_curHotspot, kMouseDownScript);
-				break;
-			case Common::EVENT_LBUTTONUP:
+			needsUpdate = true;
+			break;
+		case Common::EVENT_LBUTTONDOWN:
+			if (_curHotspot >= 0)
+				runHotspotScript(_curHotspot, kMouseDownScript);
+			break;
+		case Common::EVENT_LBUTTONUP:
+			// See RivenScript::switchCard() for more information on why we sometimes
+			// disable the next up event.
+			if (!_ignoreNextMouseUp) {
 				if (_curHotspot >= 0)
 					runHotspotScript(_curHotspot, kMouseUpScript);
 				else
 					checkInventoryClick();
+			}
+			_ignoreNextMouseUp = false;
+			break;
+		case Common::EVENT_KEYDOWN:
+			switch (event.kbd.keycode) {
+			case Common::KEYCODE_d:
+				if (event.kbd.flags & Common::KBD_CTRL) {
+					_console->attach();
+					_console->onFrame();
+				}
 				break;
-			case Common::EVENT_KEYDOWN:
-				switch (event.kbd.keycode) {
-				case Common::KEYCODE_d:
-					if (event.kbd.flags & Common::KBD_CTRL) {
-						_console->attach();
-						_console->onFrame();
-					}
-					break;
-				case Common::KEYCODE_SPACE:
-					pauseGame();
-					break;
-				case Common::KEYCODE_F4:
-					_showHotspots = !_showHotspots;
-					if (_showHotspots) {
-						for (uint16 i = 0; i < _hotspotCount; i++)
-							_gfx->drawRect(_hotspots[i].rect, _hotspots[i].enabled);
-						needsUpdate = true;
-					} else
-						refreshCard();
-					break;
-				case Common::KEYCODE_F5:
-					runDialog(*_optionsDialog);
-					updateZipMode();
-					break;
-				case Common::KEYCODE_ESCAPE:
-					if (getFeatures() & GF_DEMO) {
-						if (_curStack != aspit)
-							changeToStack(aspit);
-						changeToCard(1);
-					}
-					break;
-				default:
-					break;
+			case Common::KEYCODE_SPACE:
+				pauseGame();
+				break;
+			case Common::KEYCODE_F4:
+				_showHotspots = !_showHotspots;
+				if (_showHotspots) {
+					for (uint16 i = 0; i < _hotspotCount; i++)
+						_gfx->drawRect(_hotspots[i].rect, _hotspots[i].enabled);
+					needsUpdate = true;
+				} else
+					refreshCard();
+				break;
+			case Common::KEYCODE_F5:
+				runDialog(*_optionsDialog);
+				updateZipMode();
+				break;
+			case Common::KEYCODE_r:
+				// Return to the main menu in the demo on ctrl+r
+				if (event.kbd.flags & Common::KBD_CTRL && getFeatures() & GF_DEMO) {
+					if (_curStack != aspit)
+						changeToStack(aspit);
+					changeToCard(1);
+				}
+				break;
+			case Common::KEYCODE_p:
+				// Play the intro videos in the demo on ctrl+p
+				if (event.kbd.flags & Common::KBD_CTRL && getFeatures() & GF_DEMO) {
+					if (_curStack != aspit)
+						changeToStack(aspit);
+					changeToCard(6);
 				}
 				break;
 			default:
 				break;
 			}
+			break;
+		default:
+			break;
 		}
-
-		if (_curHotspot >= 0)
-			runHotspotScript(_curHotspot, kMouseInsideScript);
-
-		if (shouldQuit()) {
-			if (_eventMan->shouldRTL() && (getFeatures() & GF_DEMO) && !(_curStack == aspit && _curCard == 12)) {
-				if (_curStack != aspit)
-					changeToStack(aspit);
-				changeToCard(12);
-				_eventMan->resetRTL();
-				continue;
-			}
-			return Common::kNoError;
-		}
-
-		// Update the screen if we need to
-		if (needsUpdate)
-			_system->updateScreen();
-
-		// Cut down on CPU usage
-		_system->delayMillis(10);
 	}
 
-	return Common::kNoError;
+	if (_curHotspot >= 0)
+		runHotspotScript(_curHotspot, kMouseInsideScript);
+
+	// Update the screen if we need to
+	if (needsUpdate)
+		_system->updateScreen();
+
+	// Cut down on CPU usage
+	_system->delayMillis(10);
 }
 
 // Stack/Card-Related Functions
@@ -207,7 +252,7 @@ void MohawkEngine_Riven::changeToStack(uint16 n) {
 	// The endings are in reverse order because of the way the 1.02 patch works.
 	// The only "Data3" file is j_Data3.mhk from that patch. Patch files have higher
 	// priorities over the regular files and are therefore loaded and checked first.
-	static const char *endings[] = { "_Data3.mhk", "_Data2.mhk", "_Data1.mhk", "_Data.mhk" };
+	static const char *endings[] = { "_Data3.mhk", "_Data2.mhk", "_Data1.mhk", "_Data.mhk", "_Sounds.mhk" };
 
 	// Don't change stack to the current stack (if the files are loaded)
 	if (_curStack == n && !_mhk.empty())
@@ -217,6 +262,7 @@ void MohawkEngine_Riven::changeToStack(uint16 n) {
 
 	// Stop any videos playing
 	_video->stopVideos();
+	_video->clearMLST();
 
 	// Clear the old stack files out
 	for (uint32 i = 0; i < _mhk.size(); i++)
@@ -240,9 +286,8 @@ void MohawkEngine_Riven::changeToStack(uint16 n) {
 	if (_mhk.empty())
 		error("Could not load stack %s", getStackName(_curStack).c_str());
 
-	// Stop any currently playing sounds and load the new sound file too
+	// Stop any currently playing sounds
 	_sound->stopAllSLST();
-	_sound->loadRivenSounds(_curStack);
 }
 
 // Riven uses some hacks to change stacks for linking books
@@ -294,7 +339,6 @@ void MohawkEngine_Riven::refreshCard() {
 	_gfx->clearWaterEffects();
 	_gfx->_activatedPLSTs.clear();
 	_video->stopVideos();
-	_video->_mlstRecords.clear();
 	_gfx->drawPLST(1);
 	_activatedSLST = false;
 
@@ -317,13 +361,13 @@ void MohawkEngine_Riven::refreshCard() {
 }
 
 void MohawkEngine_Riven::loadCard(uint16 id) {
-	// NOTE: Do not clear the card scripts because it may delete a currently running script!
+	// NOTE: The card scripts are cleared by the RivenScriptManager automatically.
 
 	Common::SeekableReadStream* inStream = getRawData(ID_CARD, id);
 
 	_cardData.name = inStream->readSint16BE();
 	_cardData.zipModePlace = inStream->readUint16BE();
-	_cardData.scripts = RivenScript::readScripts(this, inStream);
+	_cardData.scripts = _scriptMan->readScripts(inStream);
 	_cardData.hasData = true;
 
 	delete inStream;
@@ -341,14 +385,16 @@ void MohawkEngine_Riven::loadCard(uint16 id) {
 }
 
 void MohawkEngine_Riven::loadHotspots(uint16 id) {
-	// NOTE: Do not clear the hotspots because it may delete a currently running script!
+	// Clear old hotspots
+	delete[] _hotspots;
+	
+	// NOTE: The hotspot scripts are cleared by the RivenScriptManager automatically.
 
-	Common::SeekableReadStream* inStream = getRawData(ID_HSPT, id);
+	Common::SeekableReadStream *inStream = getRawData(ID_HSPT, id);
 
 	_hotspotCount = inStream->readUint16BE();
 	_hotspots = new RivenHotspot[_hotspotCount];
 
-
 	for (uint16 i = 0; i < _hotspotCount; i++) {
 		_hotspots[i].enabled = true;
 
@@ -360,30 +406,25 @@ void MohawkEngine_Riven::loadHotspots(uint16 id) {
 		int16 right = inStream->readSint16BE();
 		int16 bottom = inStream->readSint16BE();
 
-		// Riven has some weird hotspots, disable them here
+		// Riven has some invalid rects, disable them here
+		// Known weird hotspots:
+		// - tspit 371 (DVD: 377), hotspot 4
 		if (left >= right || top >= bottom) {
-			left = top = right = bottom = 0;
-			_hotspots[i].enabled = 0;
+			warning("%s %d hotspot %d is invalid: (%d, %d, %d, %d)", getStackName(_curStack).c_str(), _curCard, i, left, top, right, bottom);
+			left = top = right = bottom = 0; 	 
+			_hotspots[i].enabled = 0; 	 
 		}
 
 		_hotspots[i].rect = Common::Rect(left, top, right, bottom);
 
 		_hotspots[i].u0 = inStream->readUint16BE();
-
-		if (_hotspots[i].u0 != 0)
-			warning("Hotspot %d u0 non-zero", i);
-
 		_hotspots[i].mouse_cursor = inStream->readUint16BE();
 		_hotspots[i].index = inStream->readUint16BE();
 		_hotspots[i].u1 = inStream->readSint16BE();
-
-		if (_hotspots[i].u1 != -1)
-			warning("Hotspot %d u1 not -1", i);
-
 		_hotspots[i].zipModeHotspot = inStream->readUint16BE();
 
 		// Read in the scripts now
-		_hotspots[i].scripts = RivenScript::readScripts(this, inStream);
+		_hotspots[i].scripts = _scriptMan->readScripts(inStream);
 	}
 
 	delete inStream;
@@ -395,7 +436,7 @@ void MohawkEngine_Riven::updateZipMode() {
 
 	for (uint32 i = 0; i < _hotspotCount; i++) {
 		if (_hotspots[i].zipModeHotspot) {
-			if (*matchVarToString("azip") != 0) {
+			if (*getVar("azip") != 0) {
 				// Check if a zip mode hotspot is enabled by checking the name/id against the ZIPS records.
 				Common::String hotspotName = getName(HotspotNames, _hotspots[i].name_resource);
 
@@ -419,7 +460,7 @@ void MohawkEngine_Riven::checkHotspotChange() {
 	uint16 hotspotIndex = 0;
 	bool foundHotspot = false;
 	for (uint16 i = 0; i < _hotspotCount; i++)
-		if (_hotspots[i].enabled && _hotspots[i].rect.contains(_mousePos)) {
+		if (_hotspots[i].enabled && _hotspots[i].rect.contains(_eventMan->getMousePos())) {
 			foundHotspot = true;
 			hotspotIndex = i;
 		}
@@ -445,39 +486,71 @@ Common::String MohawkEngine_Riven::getHotspotName(uint16 hotspot) {
 }
 
 void MohawkEngine_Riven::checkInventoryClick() {
+	Common::Point mousePos = _eventMan->getMousePos();
+
 	// Don't even bother. We're not in the inventory portion of the screen.
-	if (_mousePos.y < 392)
+	if (mousePos.y < 392)
 		return;
 
-	// No inventory in the demo or opening screens.
-	if (getFeatures() & GF_DEMO || _curStack == aspit)
+	// In the demo, check if we've clicked the exit button
+	if (getFeatures() & GF_DEMO) {
+		if (g_demoExitRect->contains(mousePos)) {
+			if (_curStack == aspit && _curCard == 1) {
+				// From the main menu, go to the "quit" screen
+				changeToCard(12);
+			} else if (_curStack == aspit && _curCard == 12) {
+				// From the "quit" screen, just quit
+				_gameOver = true;
+			} else {
+				// Otherwise, return to the main menu
+				if (_curStack != aspit)
+					changeToStack(aspit);
+				changeToCard(1);
+			}
+		}
+		return;
+	}
+
+	// No inventory shown on aspit
+	if (_curStack == aspit)
 		return;
 
 	// Set the return stack/card id's.
-	*matchVarToString("returnstackid") = _curStack;
-	*matchVarToString("returncardid") = _curCard;
+	*getVar("returnstackid") = _curStack;
+	*getVar("returncardid") = _curCard;
 
 	// See RivenGraphics::showInventory() for an explanation
-	// of why only this variable is used.
-	bool hasCathBook = *matchVarToString("acathbook") != 0;
+	// of the variables' meanings.
+	bool hasCathBook = *getVar("acathbook") != 0;
+	bool hasTrapBook = *getVar("atrapbook") != 0;
 
 	// Go to the book if a hotspot contains the mouse
 	if (!hasCathBook) {
-		if (atrusJournalRectSolo.contains(_mousePos)) {
+		if (g_atrusJournalRect1->contains(mousePos)) {
 			_gfx->hideInventory();
 			changeToStack(aspit);
 			changeToCard(5);
 		}
-	} else {
-		if (atrusJournalRect.contains(_mousePos)) {
+	} else if (!hasTrapBook) {
+		if (g_atrusJournalRect2->contains(mousePos)) {
 			_gfx->hideInventory();
 			changeToStack(aspit);
 			changeToCard(5);
-		} else if (cathJournalRect.contains(_mousePos)) {
+		} else if (g_cathJournalRect2->contains(mousePos)) {
 			_gfx->hideInventory();
 			changeToStack(aspit);
 			changeToCard(6);
-		} else if (trapBookRect.contains(_mousePos)) {
+		}
+	} else {
+		if (g_atrusJournalRect3->contains(mousePos)) {
+			_gfx->hideInventory();
+			changeToStack(aspit);
+			changeToCard(5);
+		} else if (g_cathJournalRect3->contains(mousePos)) {
+			_gfx->hideInventory();
+			changeToStack(aspit);
+			changeToCard(6);
+		} else if (g_trapBookRect3->contains(mousePos)) {
 			_gfx->hideInventory();
 			changeToStack(aspit);
 			changeToCard(7);
@@ -560,8 +633,38 @@ void MohawkEngine_Riven::runHotspotScript(uint16 hotspot, uint16 scriptType) {
 		}
 }
 
+void MohawkEngine_Riven::delayAndUpdate(uint32 ms) {
+	uint32 startTime = _system->getMillis();
+
+	while (_system->getMillis() < startTime + ms && !shouldQuit()) {
+		bool needsUpdate = _gfx->runScheduledWaterEffects();
+		needsUpdate |= _video->updateBackgroundMovies();
+
+		Common::Event event;
+		while (_system->getEventManager()->pollEvent(event))
+			;
+
+		if (needsUpdate)
+			_system->updateScreen();
+
+		_system->delayMillis(10); // Ease off the CPU
+	}
+}
+
 void MohawkEngine_Riven::runLoadDialog() {
-	runDialog(*_loadDialog);
+	GUI::SaveLoadChooser slc(_("Load game:"), _("Load"));
+	slc.setSaveMode(false);
+
+	Common::String gameId = ConfMan.get("gameid");
+
+	const EnginePlugin *plugin = 0;
+	EngineMan.findGame(gameId, &plugin);
+
+	int slot = slc.runModal(plugin, ConfMan.getActiveDomainName());
+	if (slot >= 0)
+		loadGameState(slot);
+
+	slc.close();
 }
 
 Common::Error MohawkEngine_Riven::loadGameState(int slot) {
@@ -569,7 +672,7 @@ Common::Error MohawkEngine_Riven::loadGameState(int slot) {
 }
 
 Common::Error MohawkEngine_Riven::saveGameState(int slot, const char *desc) {
-	Common::StringList saveList = _saveLoad->generateSaveGameList();
+	Common::StringArray saveList = _saveLoad->generateSaveGameList();
 
 	if ((uint)slot < saveList.size())
 		_saveLoad->deleteSave(saveList[slot]);
@@ -577,23 +680,23 @@ Common::Error MohawkEngine_Riven::saveGameState(int slot, const char *desc) {
 	return _saveLoad->saveGame(Common::String(desc)) ? Common::kNoError : Common::kUnknownError;
 }
 
-static const char *rivenStackNames[] = {
-	"aspit",
-	"bspit",
-	"gspit",
-	"jspit",
-	"ospit",
-	"pspit",
-	"rspit",
-	"tspit"
-};
+Common::String MohawkEngine_Riven::getStackName(uint16 stack) const {
+	static const char *rivenStackNames[] = {
+		"aspit",
+		"bspit",
+		"gspit",
+		"jspit",
+		"ospit",
+		"pspit",
+		"rspit",
+		"tspit"
+	};
 
-Common::String MohawkEngine_Riven::getStackName(uint16 stack) {
-	return Common::String(rivenStackNames[stack]);
+	return rivenStackNames[stack];
 }
 
 bool ZipMode::operator== (const ZipMode &z) const {
 	return z.name == name && z.id == id;
 }
 
-}
+} // End of namespace Mohawk
