@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/mohawk/riven_scripts.cpp $
- * $Id: riven_scripts.cpp 48134 2010-02-26 08:14:33Z mthreepwood $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/mohawk/riven_scripts.cpp $
+ * $Id: riven_scripts.cpp 52643 2010-09-08 20:50:56Z mthreepwood $
  *
  */
 
@@ -28,40 +28,28 @@
 #include "mohawk/riven_external.h"
 #include "mohawk/riven_scripts.h"
 #include "mohawk/sound.h"
-#include "mohawk/video/video.h"
+#include "mohawk/video.h"
 
 #include "common/stream.h"
 #include "graphics/cursorman.h"
 
 namespace Mohawk {
 
-RivenScript::RivenScript(MohawkEngine_Riven *vm, Common::SeekableReadStream *stream, uint16 scriptType)
-	: _vm(vm), _stream(stream), _scriptType(scriptType) {
+RivenScript::RivenScript(MohawkEngine_Riven *vm, Common::SeekableReadStream *stream, uint16 scriptType, uint16 parentStack, uint16 parentCard)
+	: _vm(vm), _stream(stream), _scriptType(scriptType), _parentStack(parentStack), _parentCard(parentCard) {
 	setupOpcodes();
+	_isRunning = _continueRunning = false;
 }
 
 RivenScript::~RivenScript() {
 	delete _stream;
 }
 
-RivenScriptList RivenScript::readScripts(MohawkEngine_Riven *vm, Common::SeekableReadStream *stream) {
-	RivenScriptList scriptList;
-
-	uint16 scriptCount = stream->readUint16BE();
-	for (uint16 i = 0; i < scriptCount; i++) {
-		uint16 scriptType = stream->readUint16BE();
-		uint32 scriptSize = calculateScriptSize(stream);
-		scriptList.push_back(Common::SharedPtr<RivenScript>(new RivenScript(vm, stream->readStream(scriptSize), scriptType)));
-	}
-
-	return scriptList;
-}
-
 uint32 RivenScript::calculateCommandSize(Common::SeekableReadStream* script) {
 	uint16 command = script->readUint16BE();
 	uint32 commandSize = 2;
 	if (command == 8) {
-		if (script->readUint16BE() != 2)
+		if (script->readUint16BE() != 2) // Arg count?
 			warning ("if-then-else unknown value is not 2");
 		script->readUint16BE();								// variable to check against
 		uint16 logicBlockCount = script->readUint16BE();	// number of logic blocks
@@ -161,7 +149,7 @@ void RivenScript::setupOpcodes() {
 		OPCODE(activateFLST),
 		OPCODE(zipMode),
 		OPCODE(activateMLST),
-		OPCODE(activateSLSTWithVolume)
+		OPCODE(empty)                       // Activate an SLST with a volume parameter (not used)
 	};
 
 	_opcodes = riven_opcodes;
@@ -172,7 +160,7 @@ static void printTabs(byte tabs) {
 		printf ("\t");
 }
 
-void RivenScript::dumpScript(Common::StringList varNames, Common::StringList xNames, byte tabs) {
+void RivenScript::dumpScript(Common::StringArray varNames, Common::StringArray xNames, byte tabs) {
 	if (_stream->pos() != 0)
 		_stream->seek(0);
 
@@ -180,7 +168,7 @@ void RivenScript::dumpScript(Common::StringList varNames, Common::StringList xNa
 	dumpCommands(varNames, xNames, tabs + 1);
 }
 
-void RivenScript::dumpCommands(Common::StringList varNames, Common::StringList xNames, byte tabs) {
+void RivenScript::dumpCommands(Common::StringArray varNames, Common::StringArray xNames, byte tabs) {
 	uint16 commandCount = _stream->readUint16BE();
 
 	for (uint16 i = 0; i < commandCount; i++) {
@@ -239,10 +227,13 @@ void RivenScript::dumpCommands(Common::StringList varNames, Common::StringList x
 }
 
 void RivenScript::runScript() {
+	_isRunning = _continueRunning = true;
+
 	if (_stream->pos() != 0)
 		_stream->seek(0);
 
 	processCommands(true);
+	_isRunning = false;
 }
 
 void RivenScript::processCommands(bool runCommands) {
@@ -251,7 +242,7 @@ void RivenScript::processCommands(bool runCommands) {
 
 	uint16 commandCount = _stream->readUint16BE();
 
-	for (uint16 j = 0; j < commandCount && !_vm->shouldQuit() && _stream->pos() < _stream->size(); j++) {
+	for (uint16 j = 0; j < commandCount && !_vm->shouldQuit() && _stream->pos() < _stream->size() && _continueRunning; j++) {
 		uint16 command = _stream->readUint16BE();
 
 		if (command == 8) {
@@ -298,13 +289,10 @@ void RivenScript::processCommands(bool runCommands) {
 
 // Command 1: draw tBMP resource (tbmp_id, left, top, right, bottom, u0, u1, u2, u3)
 void RivenScript::drawBitmap(uint16 op, uint16 argc, uint16 *argv) {
-	if (argc < 5) {
-		// Copy the image to the whole screen, ignoring the rest of the parameters
+	if (argc < 5) // Copy the image to the whole screen, ignoring the rest of the parameters
 		_vm->_gfx->copyImageToScreen(argv[0], 0, 0, 608, 392);
-	} else {
-		// Copy the image to a certain part of the screen
+	else // Copy the image to a certain part of the screen
 		_vm->_gfx->copyImageToScreen(argv[0], argv[1], argv[2], argv[3], argv[4]);
-	}
 
 	// Now, update the screen
 	_vm->_gfx->updateScreen();
@@ -313,6 +301,12 @@ void RivenScript::drawBitmap(uint16 op, uint16 argc, uint16 *argv) {
 // Command 2: go to card (card id)
 void RivenScript::switchCard(uint16 op, uint16 argc, uint16 *argv) {
 	_vm->changeToCard(argv[0]);
+
+	// WORKAROUND: If we changed card on a mouse down event,
+	// we want to ignore the next mouse up event so we don't
+	// change card when lifting the mouse on the next card.
+	if (_scriptType == kMouseDownScript)
+		_vm->ignoreNextMouseUp();
 }
 
 // Command 3: play an SLST from the script
@@ -352,9 +346,14 @@ void RivenScript::playScriptSLST(uint16 op, uint16 argc, uint16 *argv) {
 	_vm->_activatedSLST = true;
 }
 
-// Command 4: play local tWAV resource (twav_id, volume, u1)
+// Command 4: play local tWAV resource (twav_id, volume, block)
 void RivenScript::playSound(uint16 op, uint16 argc, uint16 *argv) {
-	_vm->_sound->playSound(argv[0], false);
+	byte volume = Sound::convertRivenVolume(argv[1]);
+
+	if (argv[2] == 1)
+		_vm->_sound->playSoundBlocking(argv[0], volume);
+	else
+		_vm->_sound->playSound(argv[0], volume);
 }
 
 // Command 7: set variable value (variable, value)
@@ -404,7 +403,7 @@ void RivenScript::changeCursor(uint16 op, uint16 argc, uint16 *argv) {
 void RivenScript::delay(uint16 op, uint16 argc, uint16 *argv) {
 	debug(2, "Delay %dms", argv[0]);
 	if (argv[0] > 0)
-		_vm->_system->delayMillis(argv[0]);
+		_vm->delayAndUpdate(argv[0]);
 }
 
 // Command 17: call external command
@@ -547,9 +546,8 @@ void RivenScript::activateSLST(uint16 op, uint16 argc, uint16 *argv) {
 
 // Command 41: activate MLST record and play
 void RivenScript::activateMLSTAndPlay(uint16 op, uint16 argc, uint16 *argv) {
-	_vm->_video->enableMovie(argv[0] - 1);
 	_vm->_video->activateMLST(argv[0], _vm->getCurCard());
-	// TODO: Play movie (blocking?)
+	_vm->_video->playMovie(argv[0]);
 }
 
 // Command 43: activate BLST record (card hotspot enabling lists)
@@ -579,7 +577,8 @@ void RivenScript::activateFLST(uint16 op, uint16 argc, uint16 *argv) {
 	for (uint16 i = 0; i < recordCount; i++) {
 		uint16 index = flst->readUint16BE();
 		uint16 sfxeID = flst->readUint16BE();
-		if(flst->readUint16BE() != 0)
+
+		if (flst->readUint16BE() != 0)
 			warning("FLST u0 non-zero");
 
 		if (index == argv[0]) {
@@ -608,9 +607,51 @@ void RivenScript::activateMLST(uint16 op, uint16 argc, uint16 *argv) {
 	_vm->_video->activateMLST(argv[0], _vm->getCurCard());
 }
 
-// Command 47: activate SLST record with a volume argument
-void RivenScript::activateSLSTWithVolume(uint16 op, uint16 argc, uint16 *argv) {
-	warning("STUB: activateSLSTWithVolume()");
+RivenScriptManager::RivenScriptManager(MohawkEngine_Riven *vm) {
+	_vm = vm;
+}
+
+RivenScriptManager::~RivenScriptManager() {
+	for (uint32 i = 0; i < _currentScripts.size(); i++)
+		delete _currentScripts[i];
+}
+
+RivenScriptList RivenScriptManager::readScripts(Common::SeekableReadStream *stream, bool garbageCollect) {
+	if (garbageCollect)
+		unloadUnusedScripts(); // Garbage collect!
+
+	RivenScriptList scriptList;
+
+	uint16 scriptCount = stream->readUint16BE();
+	for (uint16 i = 0; i < scriptCount; i++) {
+		uint16 scriptType = stream->readUint16BE();
+		uint32 scriptSize = RivenScript::calculateScriptSize(stream);
+		RivenScript *script = new RivenScript(_vm, stream->readStream(scriptSize), scriptType, _vm->getCurStack(), _vm->getCurCard());
+		scriptList.push_back(script);
+
+		// Only add it to the scripts that we will free later if it is requested.
+		// (ie. we don't want to store scripts from the dumpScript console command)
+		if (garbageCollect)
+			_currentScripts.push_back(script);
+	}
+
+	return scriptList;
+}
+
+void RivenScriptManager::stopAllScripts() {
+	for (uint32 i = 0; i < _currentScripts.size(); i++)
+		_currentScripts[i]->stopRunning();
+}
+
+void RivenScriptManager::unloadUnusedScripts() {
+	// Free any scripts that aren't part of the current card and aren't running
+	for (uint32 i = 0; i < _currentScripts.size(); i++) {
+		if ((_vm->getCurStack() != _currentScripts[i]->getParentStack() || _vm->getCurCard() != _currentScripts[i]->getParentCard()) && !_currentScripts[i]->isRunning()) {
+			delete _currentScripts[i];
+			_currentScripts.remove_at(i);
+			i--;
+		}
+	}
 }
 
 } // End of namespace Mohawk
