@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/parallaction/callables_ns.cpp $
- * $Id: callables_ns.cpp 48828 2010-04-28 18:55:48Z wjpalenstijn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/parallaction/callables_ns.cpp $
+ * $Id: callables_ns.cpp 50038 2010-06-19 02:51:03Z peres001 $
  *
  */
 
@@ -409,14 +409,14 @@ void Parallaction_ns::_c_testResult(void *parm) {
 
 	parseLocation("common");
 
-	GfxObj *labels[2];
-	labels[0] = _gfx->createLabel(_menuFont, _location._slideText[0].c_str(), 1);
-	labels[1] = _gfx->createLabel(_menuFont, _location._slideText[1].c_str(), 1);
+	destroyTestResultLabels();
 
-	_gfx->showLabel(labels[0], CENTER_LABEL_HORIZONTAL, 38);
-	_gfx->showLabel(labels[1], CENTER_LABEL_HORIZONTAL, 58);
+	_testResultLabels[0] = _gfx->createLabel(_menuFont, _location._slideText[0].c_str(), 1);
+	_testResultLabels[1] = _gfx->createLabel(_menuFont, _location._slideText[1].c_str(), 1);
+
+	_gfx->showLabel(_testResultLabels[0], CENTER_LABEL_HORIZONTAL, 38);
+	_gfx->showLabel(_testResultLabels[1], CENTER_LABEL_HORIZONTAL, 58);
 
-	// FIXME: this leaks two labels
 	return;
 }
 
@@ -453,14 +453,15 @@ void Parallaction_ns::_c_startIntro(void *parm) {
 }
 
 void Parallaction_ns::_c_endIntro(void *parm) {
-	// NOTE: suspend command execution queue, to
-	// avoid running the QUIT command before
-	// credits are displayed. This solves bug
-	// #2619824.
-	// Execution of the command list will resume
-	// as soon as runGameFrame is run.
-	_cmdExec->suspend();
-
+	if (getFeatures() & GF_DEMO) {
+		// NOTE: suspend command execution queue, to
+		// avoid running the QUIT command before
+		// credits are displayed. This solves bug
+		// #2619824.
+		// Execution of the command list will resume
+		// as soon as runGameFrame is run.
+		_cmdExec->suspend();
+	}
 	startCreditSequence();
 	_intro = false;
 }
