@@ -18,14 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/parallaction/disk_br.cpp $
- * $Id: disk_br.cpp 44603 2009-10-04 11:58:52Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/parallaction/disk_br.cpp $
+ * $Id: disk_br.cpp 52610 2010-09-06 21:17:26Z lordhoto $
  *
  */
 
 #include "graphics/iff.h"
 
 #include "common/config-manager.h"
+#include "common/fs.h"
 #include "parallaction/parallaction.h"
 #include "parallaction/parser.h"
 
@@ -190,7 +191,7 @@ GfxObj* DosDisk_br::loadTalk(const char *name) {
 Script* DosDisk_br::loadLocation(const char *name) {
 	debugC(5, kDebugDisk, "DosDisk_br::loadLocation");
 
-	static const Common::String langs[4] = { "it/", "fr/", "en/", "ge/" };
+	static const char * const langs[4] = { "it/", "fr/", "en/", "ge/" };
 
 	Common::String fullName(name);
 	if (!fullName.hasSuffix(".slf")) {
