@@ -19,14 +19,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/parallaction/graphics.cpp $
- * $Id: graphics.cpp 48845 2010-04-28 22:37:44Z wjpalenstijn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/parallaction/graphics.cpp $
+ * $Id: graphics.cpp 49620 2010-06-13 06:55:58Z peres001 $
  *
  */
 
 #include "common/system.h"
 #include "common/file.h"
 #include "graphics/primitives.h"
+#include "engines/util.h"
 
 #include "parallaction/input.h"
 #include "parallaction/parallaction.h"
@@ -316,8 +317,10 @@ void Gfx::drawList(Graphics::Surface &surface, GfxObjArray &list) {
 
 void Gfx::copyRectToScreen(const byte *buf, int pitch, int x, int y, int w, int h) {
 	if (_doubleBuffering) {
-		if (_overlayMode)
+		if (_overlayMode) {
 			x += _scrollPosX;
+			y += _scrollPosY;
+		}
 
 		byte *dst = (byte*)_backBuffer.getBasePtr(x, y);
 		for (int i = 0; i < h; i++) {
@@ -357,7 +360,7 @@ void Gfx::unlockScreen() {
 
 void Gfx::updateScreenIntern() {
 	if (_doubleBuffering) {
-		byte *data = (byte*)_backBuffer.getBasePtr(_scrollPosX, 0);
+		byte *data = (byte*)_backBuffer.getBasePtr(_scrollPosX, _scrollPosY);
 		_vm->_system->copyRectToScreen(data, _backBuffer.pitch, 0, 0, _vm->_screenWidth, _vm->_screenHeight);
 	}
 
@@ -535,12 +538,12 @@ GfxObj *Gfx::renderFloatingLabel(Font *font, char *text) {
 
 		setupLabelSurface(*cnv, w, h);
 
-		font->setColor((_vm->getGameType() == GType_BRA) ? 0 : 7);
+		font->setColor((_gameType == GType_BRA) ? 0 : 7);
 		font->drawString((byte*)cnv->pixels + 1, cnv->w, text);
 		font->drawString((byte*)cnv->pixels + 1 + cnv->w * 2, cnv->w, text);
 		font->drawString((byte*)cnv->pixels + cnv->w, cnv->w, text);
 		font->drawString((byte*)cnv->pixels + 2 + cnv->w, cnv->w, text);
-		font->setColor((_vm->getGameType() == GType_BRA) ? 11 : 1);
+		font->setColor((_gameType == GType_BRA) ? 11 : 1);
 		font->drawString((byte*)cnv->pixels + 1 + cnv->w, cnv->w, text);
 	} else {
 		w = font->getStringWidth(text);
@@ -808,7 +811,7 @@ GfxObj* Gfx::registerBalloon(Frames *frames, const char *text) {
 void Gfx::freeDialogueObjects() {
 	_items.clear();
 
-    _vm->_balloonMan->reset();
+	_vm->_balloonMan->reset();
 
 	for (uint i = 0; i < _balloons.size(); i++) {
 		delete _balloons[i];
@@ -832,7 +835,7 @@ void Gfx::setBackground(uint type, BackgroundInfo *info) {
 		// The PC version of BRA needs the entries 20-31 of the palette to be constant, but
 		// the background resource files are screwed up. The right colors come from an unused
 		// bitmap (pointer.bmp). Nothing is known about the Amiga version so far.
-		if ((_vm->getGameType() == GType_BRA) && (_vm->getPlatform() == Common::kPlatformPC)) {
+		if ((_gameType == GType_BRA) && (_vm->getPlatform() == Common::kPlatformPC)) {
 			int r, g, b;
 			for (uint i = 16; i < 32; i++) {
 				_backupPal.getEntry(i, r, g, b);
@@ -862,6 +865,8 @@ void Gfx::setBackground(uint type, BackgroundInfo *info) {
 
 	_minScrollX = 0;
 	_maxScrollX = MAX<int>(0, _backgroundInfo->width - _vm->_screenWidth);
+	_minScrollY = 0;
+	_maxScrollY = MAX<int>(0, _backgroundInfo->height - _vm->_screenHeight);
 }
 
 
