@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/queen/command.cpp $
- * $Id: command.cpp 42010 2009-07-01 20:51:34Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/queen/command.cpp $
+ * $Id: command.cpp 48800 2010-04-25 21:53:09Z cyx $
  *
  */
 
@@ -140,7 +140,7 @@ public:
 
 CmdText *CmdText::makeCmdTextInstance(uint8 y, QueenEngine *vm) {
 	switch (vm->resource()->getLanguage()) {
-	case Common::HB_ISR:
+	case Common::HE_ISR:
 		return new CmdTextHebrew(y, vm);
 	case Common::GR_GRE:
 		return new CmdTextGreek(y, vm);
@@ -1131,7 +1131,7 @@ void Command::setObjects(uint16 command) {
 					if (image1 != 0 && image2 == 0 && objData->room == _vm->logic()->currentRoom()) {
 						uint16 bobNum = _vm->logic()->findBob(dstObj);
 						if (bobNum != 0) {
-							_vm->graphics()->bob(bobNum)->clear();
+							_vm->graphics()->clearBob(bobNum);
 						}
 					}
 				}
