@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/queen/cutaway.cpp $
- * $Id: cutaway.cpp 30238 2008-01-05 12:45:14Z dreammaster $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/queen/cutaway.cpp $
+ * $Id: cutaway.cpp 48800 2010-04-25 21:53:09Z cyx $
  *
  */
 
@@ -1048,7 +1048,7 @@ void Cutaway::stop() {
 
 					ObjectData *from = _vm->logic()->objectData(fromIndex);
 					if (object->image && !from->image && bobIndex && _vm->logic()->currentRoom() == object->room)
-						_vm->graphics()->bob(bobIndex)->clear();
+						_vm->graphics()->clearBob(bobIndex);
 				}
 
 				if (_vm->logic()->currentRoom() == room)
@@ -1076,7 +1076,7 @@ void Cutaway::stop() {
 					int objectFrame = _vm->logic()->findFrame(objectIndex);
 
 					if (objectFrame == 1000) {
-						_vm->graphics()->bob(bobIndex)->clear();
+						_vm->graphics()->clearBob(bobIndex);
 					} else if (objectFrame) {
 						_vm->bankMan()->unpack(ABS(frame), objectFrame, bank);
 						pbs->frameNum = objectFrame;
