@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/queen/journal.cpp $
- * $Id: journal.cpp 47541 2010-01-25 01:39:44Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/queen/journal.cpp $
+ * $Id: journal.cpp 48645 2010-04-12 21:21:06Z salty-horse $
  *
  */
 
@@ -425,7 +425,7 @@ void Journal::drawPanelText(int y, const char *text) {
 		_panelTextY[_panelTextCount++] = y;
 	} else {
 		*p++ = '\0';
-		if (_vm->resource()->getLanguage() == Common::HB_ISR) {
+		if (_vm->resource()->getLanguage() == Common::HE_ISR) {
 			drawPanelText(y - 5, p);
 			drawPanelText(y + 5, s);
 		} else {
