@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/saga/actor.cpp $
- * $Id: actor.cpp 46254 2009-12-04 17:52:42Z h00ligan $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/saga/actor.cpp $
+ * $Id: actor.cpp 49727 2010-06-15 10:26:10Z sev $
  *
  */
 
@@ -1173,7 +1173,9 @@ void Actor::drawActors() {
 		return;
 	}
 
-	if (_vm->_anim->hasCutaway()) {
+	// WORKAROUND
+	// Bug #2928923: 'ITE: Graphic Glitches during racoon death "Cut Scene"'
+	if (_vm->_anim->hasCutaway()  || _vm->_scene->currentSceneNumber() == 287 || _vm->_scene->currentSceneNumber() == 286) {
 		drawSpeech();
 		return;
 	}
