@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/saga/animation.cpp $
- * $Id: animation.cpp 45616 2009-11-02 21:54:57Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/saga/animation.cpp $
+ * $Id: animation.cpp 49047 2010-05-16 10:36:21Z thebluegr $
  *
  */
 
@@ -324,7 +324,7 @@ void Anim::clearCutaway() {
 		_vm->_interface->restoreMode();
 		_vm->_gfx->showCursor(true);
 
-		if (_vm->getFeatures() & GF_IHNM_DEMO) {
+		if (_vm->isIHNMDemo()) {
 			// Enable the save reminder state after each cutaway for the IHNM demo
 			_vm->_interface->setSaveReminderState(true);
 		}
@@ -771,7 +771,6 @@ void Anim::decodeFrame(AnimationData *anim, size_t frameOffset, byte *buf, size_
 			break;
 		case SAGA_FRAME_END: // End of frame marker
 			return;
-			break;
 		default:
 			break;
 		}
