@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/saga/events.cpp $
- * $Id: events.cpp 48226 2010-03-10 20:38:39Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/saga/events.cpp $
+ * $Id: events.cpp 49728 2010-06-15 10:26:29Z sev $
  *
  */
 
@@ -306,7 +306,6 @@ int Events::handleOneShot(Event *event) {
 		_vm->_sndRes->playVoice(event->param);
 		break;
 	case kMusicEvent:
-		_vm->_music->stop();
 		if (event->op == kEventPlay)
 			_vm->_music->play(event->param, (MusicFlags)event->param2);
 		break;
@@ -508,13 +507,13 @@ int Events::handleOneShot(Event *event) {
 		case kEventSetNormalCursor:
 			// in ITE and IHNM demo there is just one cursor
 			// ITE never makes this call
-			if (!(_vm->getFeatures() & GF_IHNM_DEMO))
+			if (!_vm->isIHNMDemo())
 				_vm->_gfx->setCursor(kCursorNormal);
 			break;
 		case kEventSetBusyCursor:
 			// in ITE and IHNM demo there is just one cursor
 			// ITE never makes this call
-			if (!(_vm->getFeatures() & GF_IHNM_DEMO))
+			if (!_vm->isIHNMDemo())
 				_vm->_gfx->setCursor(kCursorBusy);
 			break;
 		default:
