@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/saga/font.cpp $
- * $Id: font.cpp 46254 2009-12-04 17:52:42Z h00ligan $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/saga/font.cpp $
+ * $Id: font.cpp 52800 2010-09-19 08:18:52Z thebluegr $
  *
  */
 
@@ -123,7 +123,7 @@ void Font::loadFont(uint32 fontResourceId) {
 	}
 
 	if (readS.pos() != FONT_DESCSIZE) {
-		error("Invalid font resource size.");
+		error("Invalid font resource size");
 	}
 
 	font->normal.font = (byte*)malloc(fontResourceLength - FONT_DESCSIZE);
@@ -610,7 +610,7 @@ void Font::textDrawRect(FontId fontId, const char *text, const Common::Rect &rec
 			}
 			w_total = 0;
 			len_total = 0;
-			if (wc == 0) {
+			if (wc == 0 && measurePointer) {
 				searchPointer = measurePointer + 1;
 			}
 			wc = 0;
@@ -651,7 +651,7 @@ Font::FontId Font::knownFont2FontIdx(KnownFont font) {
 	FontId fontId = kSmallFont;
 
 	// The demo version of IHNM has 3 font types (like ITE), not 6 (like the full version of IHNM)
-	if (_vm->getGameId() == GID_ITE || _vm->getFeatures() & GF_IHNM_DEMO) {
+	if (_vm->getGameId() == GID_ITE || _vm->isIHNMDemo()) {
 		switch (font) {
 		case (kKnownFontSmall):
 			fontId = kSmallFont;
@@ -674,7 +674,7 @@ Font::FontId Font::knownFont2FontIdx(KnownFont font) {
 			break;
 		}
 #ifdef ENABLE_IHNM
-	} else if (_vm->getGameId() == GID_IHNM && !(_vm->getFeatures() & GF_IHNM_DEMO)) {
+	} else if (_vm->getGameId() == GID_IHNM && !_vm->isIHNMDemo()) {
 		switch (font) {
 		case (kKnownFontSmall):
 			fontId = kSmallFont;
