@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/saga/interface.cpp $
- * $Id: interface.cpp 48101 2010-02-21 04:04:13Z bluddy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/saga/interface.cpp $
+ * $Id: interface.cpp 49698 2010-06-15 10:16:53Z sev $
  *
  */
 
@@ -186,7 +186,7 @@ Interface::Interface(SagaEngine *vm) : _vm(vm) {
 	free(resource);
 
 	// Option panel
-	if (!(_vm->getFeatures() & GF_NON_INTERACTIVE)) {
+	if (!_vm->_script->isNonInteractiveDemo()) {
 		_optionPanel.buttons = _vm->getDisplayInfo().optionPanelButtons;
 		_optionPanel.buttonsCount = _vm->getDisplayInfo().optionPanelButtonsCount;
 
@@ -237,7 +237,7 @@ Interface::Interface(SagaEngine *vm) : _vm(vm) {
 
 	// Main panel sprites
 	_vm->_sprite->loadList(_vm->getResourceDescription()->mainPanelSpritesResourceId, _mainPanel.sprites);
-	if (!(_vm->getFeatures() & GF_NON_INTERACTIVE)) {
+	if (!_vm->_script->isNonInteractiveDemo()) {
 		// Option panel sprites
 		_vm->_sprite->loadList(_vm->getResourceDescription()->optionPanelSpritesResourceId, _optionPanel.sprites);
 		// Save panel sprites
@@ -387,7 +387,7 @@ int Interface::activate() {
 		unlockMode();
 		if (_panelMode == kPanelMain || _panelMode == kPanelChapterSelection) {
 			_saveReminderState = 1;
-		} else if (_panelMode == kPanelNull && _vm->getFeatures() & GF_IHNM_DEMO) {
+		} else if (_panelMode == kPanelNull && _vm->isIHNMDemo()) {
 			_saveReminderState = 1;
 		}
 		_vm->_gfx->showCursor(true);
@@ -438,7 +438,7 @@ void Interface::setMode(int mode) {
 	} else if (mode == kPanelChapterSelection) {
 		_saveReminderState = 1;
 	} else if (mode == kPanelNull) {
-		if (_vm->getFeatures() & GF_IHNM_DEMO) {
+		if (_vm->isIHNMDemo()) {
 			_inMainMode = true;
 			_saveReminderState = 1;
 		}
@@ -544,8 +544,10 @@ bool Interface::processAscii(Common::KeyState keystate) {
 			return true;
 		}
 
+#ifdef ENABLE_IHNM
 		if (_vm->_scene->isNonInteractiveIHNMDemoPart())
 			_vm->_scene->showIHNMDemoSpecialScreen();
+#endif
 		break;
 	case kPanelCutaway:
 		if (keystate.keycode == Common::KEYCODE_ESCAPE) {
@@ -555,8 +557,10 @@ bool Interface::processAscii(Common::KeyState keystate) {
 			return true;
 		}
 
+#ifdef ENABLE_INHM
 		if (_vm->_scene->isNonInteractiveIHNMDemoPart())
 			_vm->_scene->showIHNMDemoSpecialScreen();
+#endif
 		break;
 	case kPanelVideo:
 		if (keystate.keycode == Common::KEYCODE_ESCAPE) {
@@ -570,8 +574,10 @@ bool Interface::processAscii(Common::KeyState keystate) {
 			return true;
 		}
 
+#ifdef ENABLE_IHNM
 		if (_vm->_scene->isNonInteractiveIHNMDemoPart())
 			_vm->_scene->showIHNMDemoSpecialScreen();
+#endif
 		break;
 	case kPanelOption:
 		// TODO: check input dialog keys
@@ -728,7 +734,7 @@ bool Interface::processAscii(Common::KeyState keystate) {
 #ifdef ENABLE_IHNM
 		if (_vm->getGameId() == GID_IHNM) {
 			// Any keypress here returns the user back to the game
-			if (!(_vm->getFeatures() & GF_IHNM_DEMO)) {
+			if (!_vm->isIHNMDemo()) {
 				_vm->_scene->clearPsychicProfile();
 			} else {
 				setMode(kPanelConverse);
@@ -809,7 +815,7 @@ void Interface::draw() {
 	drawStatusBar();
 
 	if (_panelMode == kPanelMain || _panelMode == kPanelMap ||
-		(_panelMode == kPanelNull && _vm->getFeatures() & GF_IHNM_DEMO)) {
+		(_panelMode == kPanelNull && _vm->isIHNMDemo())) {
 		_mainPanel.getRect(rect);
 		_vm->_gfx->drawRegion(rect, _mainPanel.image);
 
@@ -826,7 +832,7 @@ void Interface::draw() {
 
 	if (_panelMode == kPanelMain || _panelMode == kPanelConverse ||
 		_lockedMode == kPanelMain || _lockedMode == kPanelConverse ||
-		(_panelMode == kPanelNull && _vm->getFeatures() & GF_IHNM_DEMO)) {
+		(_panelMode == kPanelNull && _vm->isIHNMDemo())) {
 		leftPortraitPoint.x = _mainPanel.x + _vm->getDisplayInfo().leftPortraitXOffset;
 		leftPortraitPoint.y = _mainPanel.y + _vm->getDisplayInfo().leftPortraitYOffset;
 		_vm->_sprite->draw(_defPortraits, _leftPortrait, leftPortraitPoint, 256);
@@ -1079,7 +1085,7 @@ void Interface::setQuit(PanelButton *panelButton) {
 			break;
 		case kTextQuit:
 #ifdef ENABLE_IHNM
-			if (_vm->getFeatures() & GF_IHNM_DEMO)
+			if (_vm->isIHNMDemo())
 				_vm->_scene->creditsScene();	// display sales info for IHNM demo
 			else
 #endif
@@ -1853,8 +1859,9 @@ void Interface::update(const Point& mousePoint, int updateFlag) {
 		if (_vm->getGameId() == GID_IHNM) {
 			// Any mouse click here returns the user back to the game
 			if (updateFlag & UPDATE_MOUSECLICK) {
-				if (!(_vm->getFeatures() & GF_IHNM_DEMO)) {
+				if (!_vm->isIHNMDemo()) {
 					_vm->_scene->clearPsychicProfile();
+					_vm->_script->wakeUpThreads(kWaitTypeDelay);
 				} else {
 					setMode(kPanelConverse);
 					_vm->_scene->_textList.clear();
@@ -1866,8 +1873,10 @@ void Interface::update(const Point& mousePoint, int updateFlag) {
 		break;
 
 	case kPanelNull:
+#ifdef ENABLE_IHNM
 		if (_vm->_scene->isNonInteractiveIHNMDemoPart() && (updateFlag & UPDATE_MOUSECLICK))
 			_vm->_scene->showIHNMDemoSpecialScreen();
+#endif
 		break;
 	}
 
