@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/saga/puzzle.cpp $
- * $Id: puzzle.cpp 45616 2009-11-02 21:54:57Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/saga/puzzle.cpp $
+ * $Id: puzzle.cpp 49771 2010-06-15 10:50:49Z sev $
  *
  */
 
@@ -411,12 +411,12 @@ void Puzzle::solicitHint() {
 	switch (_hintRqState) {
 	case kRQSpeaking:
 		if (_vm->_actor->isSpeaking()) {
-			_vm->getTimerManager()->installTimerProc(&hintTimerCallback, 50000, this);
+			_vm->getTimerManager()->installTimerProc(&hintTimerCallback, 50 * 1000000, this);
 			break;
 		}
 
 		_hintRqState = _hintNextRqState;
-		_vm->getTimerManager()->installTimerProc(&hintTimerCallback, 333333, this);
+		_vm->getTimerManager()->installTimerProc(&hintTimerCallback, 100*1000000/3, this);
 		break;
 
 	case kRQNoHint:
@@ -439,11 +439,11 @@ void Puzzle::solicitHint() {
 		//	Roll to see if Sakka scolds
 		if (_vm->_rnd.getRandomNumber(1)) {
 			_hintRqState = kRQSakkaDenies;
-			_vm->getTimerManager()->installTimerProc(&hintTimerCallback, 200000, this);
+			_vm->getTimerManager()->installTimerProc(&hintTimerCallback, 200*1000000, this);
 		} else {
 			_hintRqState = kRQSpeaking;
 			_hintNextRqState = kRQHintRequested;
-			_vm->getTimerManager()->installTimerProc(&hintTimerCallback, 50000, this);
+			_vm->getTimerManager()->installTimerProc(&hintTimerCallback, 50*1000000, this);
 		}
 
 		break;
@@ -456,7 +456,7 @@ void Puzzle::solicitHint() {
 
 		_hintRqState = kRQSpeaking;
 		_hintNextRqState = kRQHintRequestedStage2;
-		_vm->getTimerManager()->installTimerProc(&hintTimerCallback, 50000, this);
+		_vm->getTimerManager()->installTimerProc(&hintTimerCallback, 50*1000000, this);
 
 		_vm->_interface->converseClear();
 		_vm->_interface->converseAddText(optionsStr[_lang][kROAccept], 0, 1, 0, 0 );
@@ -556,8 +556,7 @@ void Puzzle::giveHint() {
 			sprintf(hintBuf, optionsStr[_lang][kROHint], pieceNames[_lang][piece]);
 
 			_vm->_actor->nonActorSpeech(_hintBox, &hintPtr, 1, PUZZLE_TOOL_SOUNDS + _hintSpeaker + piece * 3, 0);
-		}
-		else {
+		} else {
 				//	If no pieces are in the wrong place
 			_vm->_actor->nonActorSpeech(_hintBox, &hintStr[_lang][3], 1, PUZZLE_HINT_SOUNDS + 3 * 3 + _hintSpeaker, 0);
 		}
