@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/saga/script.h $
- * $Id: script.h 48237 2010-03-11 20:33:27Z h00ligan $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/saga/script.h $
+ * $Id: script.h 49697 2010-06-15 10:16:32Z sev $
  *
  */
 
@@ -246,16 +246,19 @@ public:
 	}
 
 	void waitWalk(void *threadObj) {
+		debug(3, "waitWalk()");
 		wait(kWaitTypeWalk);
 		_threadObj = threadObj;
 	}
 
 	void waitDelay(int sleepTime) {
+		debug(3, "waitDelay(%d)", sleepTime);
 		wait(kWaitTypeDelay);
 		_sleepTime = sleepTime;
 	}
 
 	void waitFrames(int frames) {
+		debug(3, "waitFrames(%d)", frames);
 		wait(kWaitTypeWaitFrames);
 		_frameWait = frames;
 	}
@@ -268,13 +271,13 @@ public:
 
 	// copy constructor
 	ScriptThread(const ScriptThread& s) {
-		memcpy(this, &s, sizeof(*this));
-
 		// Verify that s doesn't have a non-zero _stackBuf, for else
 		// we would have to clone  that buffer, too, which we currently
 		// don't do. This case should never occur anyway, though (at
 		// least as long as the thread handling code does not change).
-		assert(!_stackBuf);
+		assert(!s._stackBuf);
+
+		memcpy(this, &s, sizeof(*this));
 	}
 
 	// assignment operator
@@ -282,14 +285,16 @@ public:
 		if (this == &s)
 			return *this;
 
+		// Verify that s doesn't have a non-zero _stackBuf, for else
+		// we would have to clone that buffer, too, which we currently
+		// don't do. This case should never occur anyway, though (at
+		// least as long as the thread handling code does not change).
+		assert(!s._stackBuf);
+
 		free(_stackBuf);
 		memcpy(this, &s, sizeof(*this));
 
-		// Verify that s doesn't have a non-zero _stackBuf, for else
-		// we would have to clone  that buffer, too, which we currently
-		// don't do. This case should never occur anyway, though (at
-		// least as long as the thread handling code does not change).
-		assert(!_stackBuf);
+		return *this;
 	}
 
 	~ScriptThread() {
@@ -339,6 +344,8 @@ public:
 	int getVerbType(VerbTypes verbType);
 	TextListEntry *getPlacardTextEntry() { return _placardTextEntry; }
 
+	bool isNonInteractiveDemo();
+
 protected:
 	// When reading or writing data to the common buffer, we have to use a
 	// well-defined byte order since it's stored in savegames. Otherwise,
