@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/saga/sfuncs.cpp $
- * $Id: sfuncs.cpp 39944 2009-04-11 21:38:41Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/saga/sfuncs.cpp $
+ * $Id: sfuncs.cpp 49725 2010-06-15 10:25:34Z sev $
  *
  */
 
@@ -226,7 +226,7 @@ void Script::sfMainMode(SCRIPTFUNC_PARAMS) {
 	// exit the game. Therefore, once this opcode is called in the older ITE demos,
 	// exit the game. Known non-interactive demos are GID_ITE_MACDEMO1 and
 	// GID_ITE_WINDEMO1
-	if (_vm->getFeatures() & GF_NON_INTERACTIVE)
+	if (_vm->_script->isNonInteractiveDemo())
 		_vm->quitGame();
 }
 
@@ -370,12 +370,15 @@ void Script::sfStopBgdAnim(SCRIPTFUNC_PARAMS) {
 // reenabled.
 // Param1: boolean
 void Script::sfLockUser(SCRIPTFUNC_PARAMS) {
-	if (thread->pop()) {
+	int16 param = thread->pop();
+
+	if (param != 0) {
 		_vm->_interface->deactivate();
 	} else {
 		_vm->_interface->activate();
 	}
 
+	debug(1, "sfLockUser(%d)", param);
 }
 
 // Script function #12 (0x0C)
@@ -1153,18 +1156,6 @@ void Script::sfPlacardOff(SCRIPTFUNC_PARAMS) {
 	_vm->_scene->clearPlacard();
 }
 
-void Script::sfPsychicProfile(SCRIPTFUNC_PARAMS) {
-	thread->wait(kWaitTypePlacard);
-
-	_vm->_scene->showPsychicProfile(thread->_strings->getString(thread->pop()));
-}
-
-void Script::sfPsychicProfileOff(SCRIPTFUNC_PARAMS) {
-	// This is called a while after the psychic profile is
-	// opened, to close it automatically
-	_vm->_scene->clearPsychicProfile();
-}
-
 // Script function #50 (0x32)
 void Script::sfSetProtagState(SCRIPTFUNC_PARAMS) {
 	_vm->_actor->setProtagState(thread->pop());
@@ -1473,6 +1464,8 @@ void Script::sfPlayLoopedSound(SCRIPTFUNC_PARAMS) {
 	} else {
 		_vm->_sound->stopSound();
 	}
+
+	debug(1, "sfPlayLoopedSound(%d)", param);
 }
 
 // Script function #72 (0x48)
