@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sci/decompressor.cpp $
- * $Id: decompressor.cpp 48086 2010-02-17 23:38:43Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sci/decompressor.cpp $
+ * $Id: decompressor.cpp 50184 2010-06-23 16:49:03Z m_kiewitz $
  *
  */
 
@@ -28,6 +28,7 @@
 #include "common/util.h"
 #include "common/endian.h"
 #include "common/debug.h"
+#include "common/debug-channels.h"
 #include "common/stream.h"
 
 #include "sci/decompressor.h"
@@ -573,15 +574,17 @@ void DecompressorLZW::reorderView(byte *src, byte *dest) {
 	for (c = 0; c < cel_total; c++)
 		decodeRLE(&rle_ptr, &pix_ptr, cc_pos[c] + 8, cc_lengths[c]);
 
-	*writer++ = 'P';
-	*writer++ = 'A';
-	*writer++ = 'L';
+	if (pal_offset) {
+		*writer++ = 'P';
+		*writer++ = 'A';
+		*writer++ = 'L';
 
-	for (c = 0; c < 256; c++)
-		*writer++ = c;
+		for (c = 0; c < 256; c++)
+			*writer++ = c;
 
-	seeker -= 4; /* The missing four. Don't ask why. */
-	memcpy(writer, seeker, 4*256 + 4);
+		seeker -= 4; /* The missing four. Don't ask why. */
+		memcpy(writer, seeker, 4*256 + 4);
+	}
 
 	free(cc_pos);
 	free(cc_lengths);
@@ -870,7 +873,7 @@ int DecompressorDCL::unpackDCL(byte* dest) {
 				for (uint32 i = 0; i < copy_length; i++)
 					putByte(dest[pos + i]);
 
-				if (Common::isDebugChannelEnabled(kDebugLevelDclInflate)) {
+				if (DebugMan.isDebugChannelEnabled(kDebugLevelDclInflate)) {
 					for (uint32 i = 0; i < copy_length; i++)
 						debugC(kDebugLevelDclInflate, "\33[32;31m%02x\33[37;37m ", dest[pos + i]);
 					debugC(kDebugLevelDclInflate, "\n");
