@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sci/engine/kernel.h $
- * $Id: kernel.h 48097 2010-02-20 23:30:59Z thebluegr $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sci/engine/kernel.h $
+ * $Id: kernel.h 53821 2010-10-25 11:32:23Z thebluegr $
  *
  */
 
@@ -29,8 +29,9 @@
 #include "common/scummsys.h"
 #include "common/debug.h"
 #include "common/rect.h"
+#include "common/str-array.h"
 
-#include "sci/sci.h"	// for USE_OLD_MUSIC_FUNCTIONS
+#include "sci/engine/selector.h"
 #include "sci/engine/vm_types.h"	// for reg_t
 #include "sci/engine/vm.h"
 
@@ -38,6 +39,8 @@ namespace Sci {
 
 struct Node;	// from segment.h
 struct List;	// from segment.h
+struct SelectorCache;	// from selector.h
+struct SciWorkaroundEntry;	// from workarounds.h
 
 /**
  * @defgroup VocabularyResources	Vocabulary resources in SCI
@@ -77,13 +80,13 @@ struct List;	// from segment.h
  *
  * SCI0 parser vocabularies:
  * - vocab.901 / 901.voc - suffix vocabulary
- * - vocab.900 / 900.vo1 - parse tree branches
+ * - vocab.900 / 900.voc - parse tree branches
  * - vocab.0 / 0.voc - main vocabulary, containing words and their attributes
  *                     (e.g. "onto" - "position")
  *
  * SCI01 parser vocabularies:
  * - vocab.902 / 902.voc - suffix vocabulary
- * - vocab.901 / 901.vo1 - parse tree branches
+ * - vocab.901 / 901.voc - parse tree branches
  * - vocab.900 / 900.voc - main vocabulary, containing words and their attributes
  *                        (e.g. "onto" - "position")
  *
@@ -91,49 +94,70 @@ struct List;	// from segment.h
 //@{
 
 //#define DEBUG_PARSER	// enable for parser debugging
-//#define DISABLE_VALIDATIONS	// enable to stop validation checks
 
-/* Generic description: */
-typedef reg_t KernelFunc(EngineState *s, int argc, reg_t *argv);
+// ---- Kernel signatures -----------------------------------------------------
 
-struct KernelFuncWithSignature {
-	KernelFunc *fun; /**< The actual function */
-	const char *signature;  /**< KernelFunc signature */
-	Common::String orig_name; /**< Original name, in case we couldn't map it */
-	bool isDummy;
+// internal kernel signature data
+enum {
+	SIG_TYPE_NULL          =  0x01, // may be 0:0       [0]
+	SIG_TYPE_INTEGER       =  0x02, // may be 0:*       [i], automatically also allows null
+	SIG_TYPE_UNINITIALIZED =  0x04, // may be FFFF:*    -> not allowable, only used for comparison
+	SIG_TYPE_OBJECT        =  0x08, // may be object    [o]
+	SIG_TYPE_REFERENCE     =  0x10, // may be reference [r]
+	SIG_TYPE_LIST          =  0x20, // may be list      [l]
+	SIG_TYPE_NODE          =  0x40, // may be node      [n]
+	SIG_TYPE_ERROR         =  0x80, // happens, when there is a identification error - only used for comparison
+	SIG_IS_INVALID         = 0x100, // ptr is invalid   [!] -> invalid offset
+	SIG_IS_OPTIONAL        = 0x200, // is optional
+	SIG_NEEDS_MORE         = 0x400, // needs at least one additional parameter following
+	SIG_MORE_MAY_FOLLOW    = 0x800  // may have more parameters of the same type following
 };
 
-enum AutoDetectedFeatures {
-	kFeatureOldScriptHeader = 1 << 0
+// this does not include SIG_TYPE_UNINITIALIZED, because we can not allow uninitialized values anywhere
+#define SIG_MAYBE_ANY    (SIG_TYPE_NULL | SIG_TYPE_INTEGER | SIG_TYPE_OBJECT | SIG_TYPE_REFERENCE | SIG_TYPE_LIST | SIG_TYPE_NODE)
+
+// ----------------------------------------------------------------------------
+
+/* Generic description: */
+typedef reg_t KernelFunctionCall(EngineState *s, int argc, reg_t *argv);
+
+struct KernelSubFunction {
+	KernelFunctionCall *function;
+	const char *name;
+	uint16 *signature;
+	const SciWorkaroundEntry *workarounds;
+	bool debugLogging;
+	bool debugBreakpoint;
+};
+
+struct KernelFunction {
+	KernelFunctionCall *function;
+	const char *name;
+	uint16 *signature;
+	const SciWorkaroundEntry *workarounds;
+	KernelSubFunction *subFunctions;
+	uint16 subFunctionCount;
+	bool debugLogging;
+	bool debugBreakpoint;
 };
 
 class Kernel {
 public:
 	/**
-	 * Initializes the SCI kernel
+	 * Initializes the SCI kernel.
 	 */
-	Kernel(ResourceManager *resMan);
+	Kernel(ResourceManager *resMan, SegManager *segMan);
 	~Kernel();
 
 	uint getSelectorNamesSize() const;
-	const Common::String &getSelectorName(uint selector) const;
+	const Common::String &getSelectorName(uint selector);
+	int findKernelFuncPos(Common::String kernelFuncName);
 
 	uint getKernelNamesSize() const;
 	const Common::String &getKernelName(uint number) const;
 
 	/**
-	 * Loads the kernel function names.
-	 *
-	 * This function reads the kernel function name table from resource_map,
-	 * and fills the _kernelNames array with them.
-	 * The resulting list has the same format regardless of the format of the
-	 * name table of the resource (the format changed between version 0 and 1).
-	 * @return true on success, false on failure
-	 */
-	bool loadKernelNames(Common::String gameId, EngineState *s);
-
-	/**
-	 * Determines the selector ID of a selector by its name
+	 * Determines the selector ID of a selector by its name.
 	 * @param selectorName Name of the selector to look up
 	 * @return The appropriate selector ID, or -1 on error
 	 */
@@ -144,26 +168,79 @@ public:
 	void dumpScriptObject(char *data, int seeker, int objsize);
 	void dumpScriptClass(char *data, int seeker, int objsize);
 
-	SelectorCache _selectorCache; /**< Shortcut list for important selectors */
-	typedef Common::Array<KernelFuncWithSignature> KernelFuncsContainer;
-	KernelFuncsContainer _kernelFuncs; /**< Table of kernel functions */
+	SelectorCache _selectorCache; /**< Shortcut list for important selectors. */
+	typedef Common::Array<KernelFunction> KernelFunctionArray;
+	KernelFunctionArray _kernelFuncs; /**< Table of kernel functions. */
+
+	/**
+	 * Determines whether a list of registers matches a given signature.
+	 * If no signature is given (i.e., if sig is NULL), this is always
+	 * treated as a match.
+	 *
+	 * @param sig	 signature to test against
+	 * @param argc	 number of arguments to test
+	 * @param argv	 argument list
+	 * @return true if the signature was matched, false otherwise
+	 */
+	bool signatureMatch(const uint16 *sig, int argc, const reg_t *argv);
+
+	// Prints out debug information in case a signature check fails
+	void signatureDebug(const uint16 *sig, int argc, const reg_t *argv);
+
+	/**
+	 * Determines the type of the object indicated by reg.
+	 * @param reg				register to check
+	 * @return one of KSIG_* below KSIG_NULL.
+	 *	       KSIG_INVALID set if the type of reg can be determined, but is invalid.
+	 *	       0 on error.
+	 */
+	uint16 findRegType(reg_t reg);
+
+	/******************** Text functionality ********************/
+	/**
+	 * Looks up text referenced by scripts.
+	 * SCI uses two values to reference to text: An address, and an index. The
+	 * address determines whether the text should be read from a resource file,
+	 * or from the heap, while the index either refers to the number of the
+	 * string in the specified source, or to a relative position inside the text.
+	 *
+	 * @param address The address to look up
+	 * @param index The relative index
+	 * @return The referenced text, or empty string on error.
+	 */
+	Common::String lookupText(reg_t address, int index);
+
+	/**
+	 * Loads the kernel function names.
+	 *
+	 * This function reads the kernel function name table from resource_map,
+	 * and fills the _kernelNames array with them.
+	 * The resulting list has the same format regardless of the format of the
+	 * name table of the resource (the format changed between version 0 and 1).
+	 */
+	void loadKernelNames(GameFeatures *features);
+
+	/**
+	 * Sets debug flags for a kernel function
+	 */
+	bool debugSetFunction(const char *kernelName, int logging, int breakpoint);
 
 private:
 	/**
-	 * Sets the default kernel function names, based on the SCI version used
+	 * Sets the default kernel function names, based on the SCI version used.
 	 */
-	void setDefaultKernelNames(Common::String gameId);
+	void setDefaultKernelNames(GameFeatures *features);
 
 #ifdef ENABLE_SCI32
 	/**
-	 * Sets the default kernel function names to the SCI2 kernel functions
+	 * Sets the default kernel function names to the SCI2 kernel functions.
 	 */
 	void setKernelNamesSci2();
 
 	/**
-	 * Sets the default kernel function names to the SCI2.1 kernel functions
+	 * Sets the default kernel function names to the SCI2.1 kernel functions.
 	 */
-	void setKernelNamesSci21(EngineState *s);
+	void setKernelNamesSci21(GameFeatures *features);
 #endif
 
 	/**
@@ -175,59 +252,33 @@ private:
 	 * Check for any hardcoded selector table we might have that can be used
 	 * if a game is missing the selector names.
 	 */
-	Common::StringList checkStaticSelectorNames();
+	Common::StringArray checkStaticSelectorNames();
 
 	/**
-	 * Maps special selectors
+	 * Maps special selectors.
 	 */
 	void mapSelectors();
 
 	/**
-	 * Maps kernel functions
+	 * Maps kernel functions.
 	 */
 	void mapFunctions();
 
 	ResourceManager *_resMan;
-	uint32 features;
+	SegManager *_segMan;
 
 	// Kernel-related lists
-	Common::StringList _selectorNames;
-	Common::StringList _kernelNames;
+	Common::StringArray _selectorNames;
+	Common::StringArray _kernelNames;
+	
+	const Common::String _invalid;
 };
 
-/******************** Text functionality ********************/
-/**
- * Looks up text referenced by scripts
- * SCI uses two values to reference to text: An address, and an index. The address
- * determines whether the text should be read from a resource file, or from the heap,
- * while the index either refers to the number of the string in the specified source,
- * or to a relative position inside the text.
- *
- * @param s The current state
- * @param address The address to look up
- * @param index The relative index
- * @return The referenced text, or empty string on error.
- */
-Common::String kernel_lookup_text(EngineState *s, reg_t address, int index);
-
-
-#ifdef USE_OLD_MUSIC_FUNCTIONS
-/******************** Misc functions ********************/
-
-/**
- * Get all sound events, apply their changes to the heap
- */
-void process_sound_events(EngineState *s);
-
-/******************** Constants ********************/
-#endif
-
-/* Maximum length of a savegame name (including terminator character) */
+/* Maximum length of a savegame name (including terminator character). */
 #define SCI_MAX_SAVENAME_LENGTH 0x24
 
 /******************** Kernel functions ********************/
 
-// New kernel functions
 reg_t kStrLen(EngineState *s, int argc, reg_t *argv);
 reg_t kGetFarText(EngineState *s, int argc, reg_t *argv);
 reg_t kReadNumber(EngineState *s, int argc, reg_t *argv);
@@ -359,6 +410,7 @@ reg_t kDoAudio(EngineState *s, int argc, reg_t *argv);
 reg_t kDoSync(EngineState *s, int argc, reg_t *argv);
 reg_t kMemorySegment(EngineState *s, int argc, reg_t *argv);
 reg_t kIntersections(EngineState *s, int argc, reg_t *argv);
+reg_t kMergePoly(EngineState *s, int argc, reg_t *argv);
 reg_t kResCheck(EngineState *s, int argc, reg_t *argv);
 reg_t kSetQuitStr(EngineState *s, int argc, reg_t *argv);
 reg_t kShowMovie(EngineState *s, int argc, reg_t *argv);
@@ -367,6 +419,12 @@ reg_t kStrSplit(EngineState *s, int argc, reg_t *argv);
 reg_t kPlatform(EngineState *s, int argc, reg_t *argv);
 reg_t kTextColors(EngineState *s, int argc, reg_t *argv);
 reg_t kTextFonts(EngineState *s, int argc, reg_t *argv);
+reg_t kShow(EngineState *s, int argc, reg_t *argv);
+reg_t kRemapColors(EngineState *s, int argc, reg_t *argv);
+reg_t kDummy(EngineState *s, int argc, reg_t *argv);
+reg_t kEmpty(EngineState *s, int argc, reg_t *argv);
+reg_t kStub(EngineState *s, int argc, reg_t *argv);
+reg_t kStubNull(EngineState *s, int argc, reg_t *argv);
 
 #ifdef ENABLE_SCI32
 // SCI2 Kernel Functions
@@ -374,6 +432,8 @@ reg_t kIsHiRes(EngineState *s, int argc, reg_t *argv);
 reg_t kArray(EngineState *s, int argc, reg_t *argv);
 reg_t kListAt(EngineState *s, int argc, reg_t *argv);
 reg_t kString(EngineState *s, int argc, reg_t *argv);
+reg_t kMulDiv(EngineState *s, int argc, reg_t *argv);
+reg_t kCantBeHere32(EngineState *s, int argc, reg_t *argv);
 // "Screen items" in SCI32 are views
 reg_t kAddScreenItem(EngineState *s, int argc, reg_t *argv);
 reg_t kUpdateScreenItem(EngineState *s, int argc, reg_t *argv);
@@ -385,20 +445,104 @@ reg_t kAddPlane(EngineState *s, int argc, reg_t *argv);
 reg_t kDeletePlane(EngineState *s, int argc, reg_t *argv);
 reg_t kUpdatePlane(EngineState *s, int argc, reg_t *argv);
 reg_t kRepaintPlane(EngineState *s, int argc, reg_t *argv);
+reg_t kSetShowStyle(EngineState *s, int argc, reg_t *argv);
 reg_t kGetHighPlanePri(EngineState *s, int argc, reg_t *argv);
 reg_t kFrameOut(EngineState *s, int argc, reg_t *argv);
+reg_t kIsOnMe(EngineState *s, int argc, reg_t *argv); // kOnMe for SCI2, kIsOnMe for SCI2.1
 reg_t kListIndexOf(EngineState *s, int argc, reg_t *argv);
 reg_t kListEachElementDo(EngineState *s, int argc, reg_t *argv);
 reg_t kListFirstTrue(EngineState *s, int argc, reg_t *argv);
 reg_t kListAllTrue(EngineState *s, int argc, reg_t *argv);
-reg_t kOnMe(EngineState *s, int argc, reg_t *argv);
 reg_t kInPolygon(EngineState *s, int argc, reg_t *argv);
 
 // SCI2.1 Kernel Functions
+reg_t kText(EngineState *s, int argc, reg_t *argv);
 reg_t kSave(EngineState *s, int argc, reg_t *argv);
 reg_t kList(EngineState *s, int argc, reg_t *argv);
 reg_t kRobot(EngineState *s, int argc, reg_t *argv);
+reg_t kPlayVMD(EngineState *s, int argc, reg_t *argv);
+reg_t kCD(EngineState *s, int argc, reg_t *argv);
+reg_t kAddPicAt(EngineState *s, int argc, reg_t *argv);
+reg_t kAddBefore(EngineState *s, int argc, reg_t *argv);
+reg_t kMoveToFront(EngineState *s, int argc, reg_t *argv);
+reg_t kMoveToEnd(EngineState *s, int argc, reg_t *argv);
+reg_t kGetWindowsOption(EngineState *s, int argc, reg_t *argv);
+reg_t kWinHelp(EngineState *s, int argc, reg_t *argv);
+reg_t kWinDLL(EngineState *s, int argc, reg_t *argv);
+reg_t kPrintDebug(EngineState *s, int argc, reg_t *argv);
+#endif
 
+reg_t kDoSoundInit(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundPlay(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundRestore(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundDispose(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundMute(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundStop(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundStopAll(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundPause(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundResumeAfterRestore(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundMasterVolume(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundUpdate(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundFade(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundGetPolyphony(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundUpdateCues(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundSendMidi(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundReverb(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundSetHold(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundDummy(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundGetAudioCapability(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundSuspend(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundSetVolume(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundSetPriority(EngineState *s, int argc, reg_t *argv);
+reg_t kDoSoundSetLoop(EngineState *s, int argc, reg_t *argv);
+
+reg_t kGraphGetColorCount(EngineState *s, int argc, reg_t *argv);
+reg_t kGraphDrawLine(EngineState *s, int argc, reg_t *argv);
+reg_t kGraphSaveBox(EngineState *s, int argc, reg_t *argv);
+reg_t kGraphRestoreBox(EngineState *s, int argc, reg_t *argv);
+reg_t kGraphFillBoxBackground(EngineState *s, int argc, reg_t *argv);
+reg_t kGraphFillBoxForeground(EngineState *s, int argc, reg_t *argv);
+reg_t kGraphFillBoxAny(EngineState *s, int argc, reg_t *argv);
+reg_t kGraphUpdateBox(EngineState *s, int argc, reg_t *argv);
+reg_t kGraphRedrawBox(EngineState *s, int argc, reg_t *argv);
+reg_t kGraphAdjustPriority(EngineState *s, int argc, reg_t *argv);
+reg_t kGraphSaveUpscaledHiresBox(EngineState *s, int argc, reg_t *argv);
+
+reg_t kPalVaryInit(EngineState *s, int argc, reg_t *argv);
+reg_t kPalVaryReverse(EngineState *s, int argc, reg_t *argv);
+reg_t kPalVaryGetCurrentStep(EngineState *s, int argc, reg_t *argv);
+reg_t kPalVaryDeinit(EngineState *s, int argc, reg_t *argv);
+reg_t kPalVaryChangeTarget(EngineState *s, int argc, reg_t *argv);
+reg_t kPalVaryChangeTicks(EngineState *s, int argc, reg_t *argv);
+reg_t kPalVaryPauseResume(EngineState *s, int argc, reg_t *argv);
+reg_t kPalVaryUnknown(EngineState *s, int argc, reg_t *argv);
+
+reg_t kPaletteSetFromResource(EngineState *s, int argc, reg_t *argv);
+reg_t kPaletteSetFlag(EngineState *s, int argc, reg_t *argv);
+reg_t kPaletteUnsetFlag(EngineState *s, int argc, reg_t *argv);
+reg_t kPaletteSetIntensity(EngineState *s, int argc, reg_t *argv);
+reg_t kPaletteFindColor(EngineState *s, int argc, reg_t *argv);
+reg_t kPaletteAnimate(EngineState *s, int argc, reg_t *argv);
+reg_t kPaletteSave(EngineState *s, int argc, reg_t *argv);
+reg_t kPaletteRestore(EngineState *s, int argc, reg_t *argv);
+
+reg_t kFileIOOpen(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIOClose(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIOReadRaw(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIOWriteRaw(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIOUnlink(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIOReadString(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIOWriteString(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIOSeek(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIOFindFirst(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIOFindNext(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIOExists(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIORename(EngineState *s, int argc, reg_t *argv);
+#ifdef ENABLE_SCI32
+reg_t kFileIOReadByte(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIOWriteByte(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIOReadWord(EngineState *s, int argc, reg_t *argv);
+reg_t kFileIOWriteWord(EngineState *s, int argc, reg_t *argv);
 #endif
 
 //@}
