@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sci/engine/kstring.cpp $
- * $Id: kstring.cpp 48066 2010-02-15 17:10:40Z mthreepwood $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sci/engine/kstring.cpp $
+ * $Id: kstring.cpp 52676 2010-09-11 13:30:42Z lskovlun $
  *
  */
 
@@ -33,40 +33,6 @@
 
 namespace Sci {
 
-/* Returns the string the script intended to address */
-Common::String kernel_lookup_text(EngineState *s, reg_t address, int index) {
-	char *seeker;
-	Resource *textres;
-
-	if (address.segment)
-		return s->_segMan->getString(address);
-	else {
-		int textlen;
-		int _index = index;
-		textres = g_sci->getResMan()->findResource(ResourceId(kResourceTypeText, address.offset), 0);
-
-		if (!textres) {
-			error("text.%03d not found", address.offset);
-			return NULL; /* Will probably segfault */
-		}
-
-		textlen = textres->size;
-		seeker = (char *) textres->data;
-
-		while (index--)
-			while ((textlen--) && (*seeker++))
-				;
-
-		if (textlen)
-			return seeker;
-		else {
-			error("Index %d out of bounds in text.%03d", _index, address.offset);
-			return 0;
-		}
-
-	}
-}
-
 reg_t kStrEnd(EngineState *s, int argc, reg_t *argv) {
 	reg_t address = argv[0];
 	address.offset += s->_segMan->strlen(address);
@@ -149,12 +115,14 @@ reg_t kStrAt(EngineState *s, int argc, reg_t *argv) {
 			if (argc > 2) { /* Request to modify this char */
 				tmp.offset &= 0xff00;
 				tmp.offset |= newvalue;
+				tmp.segment = 0;
 			}
 		} else {
 			value = tmp.offset >> 8;
 			if (argc > 2)  { /* Request to modify this char */
 				tmp.offset &= 0x00ff;
 				tmp.offset |= newvalue << 8;
+				tmp.segment = 0;
 			}
 		}
 	}
@@ -172,10 +140,39 @@ reg_t kReadNumber(EngineState *s, int argc, reg_t *argv) {
 	while (isspace((unsigned char)*source))
 		source++; /* Skip whitespace */
 
-	if (*source == '$') /* SCI uses this for hex numbers */
-		return make_reg(0, (int16)strtol(source + 1, NULL, 16)); /* Hex */
-	else
-		return make_reg(0, (int16)strtol(source, NULL, 10)); /* Force decimal */
+	int16 result = 0;
+
+	if (*source == '$') {
+		// Hexadecimal input
+		result = (int16)strtol(source + 1, NULL, 16);
+	} else {
+		// Decimal input. We can not use strtol/atoi in here, because while
+		// Sierra used atoi, it was a non standard compliant atoi, that didn't
+		// do clipping. In SQ4 we get the door code in here and that's even
+		// larger than uint32!
+		if (*source == '-') {
+			result = -1;
+			source++;
+		}
+		while (*source) {
+			if ((*source < '0') || (*source > '9')) {
+				// Sierra's atoi stopped processing at anything which is not
+				// a digit. Sometimes the input has a trailing space, that's
+				// fine (example: lsl3)
+				if (*source != ' ') {
+					// TODO: this happens in lsl5 right in the intro -> we get '1' '3' 0xCD 0xCD 0xCD 0xCD 0xCD
+					//       find out why this happens and fix it
+					warning("Invalid character in kReadNumber input");
+				}
+				break;
+			}
+			result *= 10;
+			result += *source - 0x30;
+			source++;
+		}
+	}
+
+	return make_reg(0, result);
 }
 
 
@@ -211,7 +208,7 @@ reg_t kFormat(EngineState *s, int argc, reg_t *argv) {
 	else
 		startarg = 3; /* First parameter to use for formatting */
 
-	Common::String source_str = kernel_lookup_text(s, position, index);
+	Common::String source_str = g_sci->getKernel()->lookupText(position, index);
 	const char* source = source_str.c_str();
 
 	debugC(2, kDebugLevelStrings, "Formatting \"%s\"", source);
@@ -274,11 +271,11 @@ reg_t kFormat(EngineState *s, int argc, reg_t *argv) {
 
 #ifdef ENABLE_SCI32
 				// If the string is a string object, get to the actual string in the data selector
-				if (s->_segMan->isObject(reg)) 
-					reg = GET_SEL32(s->_segMan, reg, SELECTOR(data));
+				if (s->_segMan->isObject(reg))
+					reg = readSelector(s->_segMan, reg, SELECTOR(data));
 #endif
 
-				Common::String tempsource = (reg == NULL_REG) ? "" : kernel_lookup_text(s, reg,
+				Common::String tempsource = g_sci->getKernel()->lookupText(reg,
 				                                  arguments[paramindex + 1]);
 				int slen = strlen(tempsource.c_str());
 				int extralen = str_leng - slen;
@@ -400,7 +397,7 @@ reg_t kFormat(EngineState *s, int argc, reg_t *argv) {
 	free(arguments);
 
 	*target = 0; /* Terminate string */
-	
+
 #ifdef ENABLE_SCI32
 	// Resize SCI32 strings if necessary
 	if (getSciVersion() >= SCI_VERSION_2) {
@@ -431,15 +428,16 @@ reg_t kGetFarText(EngineState *s, int argc, reg_t *argv) {
 
 	seeker = (char *)textres->data;
 	
-	// The second parameter (counter) determines the number of the string inside the text
-	// resource.
+	// The second parameter (counter) determines the number of the string
+	// inside the text resource.
 	while (counter--) {
 		while (*seeker++)
 			;
 	}
 
-	// If the third argument is NULL, allocate memory for the destination. This occurs in
-	// SCI1 Mac games. The memory will later be freed by the game's scripts.
+	// If the third argument is NULL, allocate memory for the destination. This
+	// occurs in SCI1 Mac games. The memory will later be freed by the game's
+	// scripts.
 	if (argv[2] == NULL_REG)
 		s->_segMan->allocDynmem(strlen(seeker) + 1, "Mac FarText", &argv[2]);
 
@@ -488,10 +486,12 @@ reg_t kMessage(EngineState *s, int argc, reg_t *argv) {
 	}
 #endif
 
-	if ((func != K_MESSAGE_NEXT) && (argc < 2)) {
-		warning("Message: not enough arguments passed to subfunction %d", func);
-		return NULL_REG;
-	}
+//	TODO: Perhaps fix this check, currently doesn't work with PUSH and POP subfunctions
+//	Pepper uses them to to handle the glossary
+//	if ((func != K_MESSAGE_NEXT) && (argc < 2)) {
+//		warning("Message: not enough arguments passed to subfunction %d", func);
+//		return NULL_REG;
+//	}
 
 	MessageTuple tuple;
 
@@ -560,6 +560,12 @@ reg_t kMessage(EngineState *s, int argc, reg_t *argv) {
 
 		return NULL_REG;
 	}
+	case K_MESSAGE_PUSH:
+		s->_msgState->pushCursorStack();
+		break;
+	case K_MESSAGE_POP:
+		s->_msgState->popCursorStack();
+		break;
 	default:
 		warning("Message: subfunction %i invoked (not implemented)", func);
 	}
@@ -568,8 +574,8 @@ reg_t kMessage(EngineState *s, int argc, reg_t *argv) {
 }
 
 reg_t kSetQuitStr(EngineState *s, int argc, reg_t *argv) {
-	Common::String quitStr = s->_segMan->getString(argv[0]);
-	debug("Setting quit string to '%s'", quitStr.c_str());
+	//Common::String quitStr = s->_segMan->getString(argv[0]);
+	//debug("Setting quit string to '%s'", quitStr.c_str());
 	return s->r_acc;
 }
 
@@ -586,11 +592,215 @@ reg_t kStrSplit(EngineState *s, int argc, reg_t *argv) {
 	// Make sure target buffer is large enough
 	SegmentRef buf_r = s->_segMan->dereference(argv[0]);
 	if (!buf_r.isValid() || buf_r.maxSize < (int)str.size() + 1) {
-		warning("StrSplit: buffer %04x:%04x invalid or too small to hold the following text of %i bytes: '%s'", PRINT_REG(argv[0]), str.size() + 1, str.c_str());
+		warning("StrSplit: buffer %04x:%04x invalid or too small to hold the following text of %i bytes: '%s'",
+						PRINT_REG(argv[0]), str.size() + 1, str.c_str());
 		return NULL_REG;
 	}
 	s->_segMan->strcpy(argv[0], str.c_str());
 	return argv[0];
 }
 
+#ifdef ENABLE_SCI32
+
+reg_t kText(EngineState *s, int argc, reg_t *argv) {
+	switch (argv[0].toUint16()) {
+	case 0:
+		return kTextSize(s, argc - 1, argv + 1);
+	default:
+		// TODO: Other subops here too, perhaps kTextColors and kTextFonts
+		warning("kText(%d)", argv[0].toUint16());
+		break;
+	}
+
+	return s->r_acc;
+}
+
+reg_t kString(EngineState *s, int argc, reg_t *argv) {
+	switch (argv[0].toUint16()) {
+	case 0: { // New
+		reg_t stringHandle;
+		SciString *string = s->_segMan->allocateString(&stringHandle);
+		string->setSize(argv[1].toUint16());
+
+		// Make sure the first character is a null character
+		if (string->getSize() > 0)
+			string->setValue(0, 0);
+
+		return stringHandle;
+		}
+	case 1: // Size
+		return make_reg(0, s->_segMan->getString(argv[1]).size());
+	case 2: { // At (return value at an index)
+		if (argv[1].segment == s->_segMan->getStringSegmentId())
+			return make_reg(0, s->_segMan->lookupString(argv[1])->getRawData()[argv[2].toUint16()]);
+
+		return make_reg(0, s->_segMan->getString(argv[1])[argv[2].toUint16()]);
+	}
+	case 3: { // Atput (put value at an index)
+		SciString *string = s->_segMan->lookupString(argv[1]);
+
+		uint32 index = argv[2].toUint16();
+		uint32 count = argc - 3;
+
+		if (index + count > 65535)
+			break;
+
+		if (string->getSize() < index + count)
+			string->setSize(index + count);
+
+		for (uint16 i = 0; i < count; i++)
+			string->setValue(i + index, argv[i + 3].toUint16());
+
+		return argv[1]; // We also have to return the handle
+	}
+	case 4: // Free
+		// Freeing of strings is handled by the garbage collector
+		return s->r_acc;
+	case 5: { // Fill
+		SciString *string = s->_segMan->lookupString(argv[1]);
+		uint16 index = argv[2].toUint16();
+
+		// A count of -1 means fill the rest of the array
+		uint16 count = argv[3].toSint16() == -1 ? string->getSize() - index : argv[3].toUint16();
+		uint16 stringSize = string->getSize();
+
+		if (stringSize < index + count)
+			string->setSize(index + count);
+
+		for (uint16 i = 0; i < count; i++)
+			string->setValue(i + index, argv[4].toUint16());
+
+		return argv[1];
+	}
+	case 6: { // Cpy
+		const char *string2 = 0;
+		uint32 string2Size = 0;
+
+		if (argv[3].segment == s->_segMan->getStringSegmentId()) {
+			SciString *string = s->_segMan->lookupString(argv[3]);
+			string2 = string->getRawData();
+			string2Size = string->getSize();
+		} else {
+			Common::String string = s->_segMan->getString(argv[3]);
+			string2 = string.c_str();
+			string2Size = string.size() + 1;
+		}
+
+		uint32 index1 = argv[2].toUint16();
+		uint32 index2 = argv[4].toUint16();
+
+		// The original engine ignores bad copies too
+		if (index2 > string2Size)
+			break;
+
+		// A count of -1 means fill the rest of the array
+		uint32 count = argv[5].toSint16() == -1 ? string2Size - index2 + 1 : argv[5].toUint16();
+	
+		// We have a special case here for argv[1] being a system string
+		if (argv[1].segment == s->_segMan->getSysStringsSegment()) {
+			// Resize if necessary
+			const uint16 sysStringId = argv[1].toUint16();
+			SystemString *sysString = s->_segMan->getSystemString(sysStringId);
+			assert(sysString);
+			if ((uint32)sysString->_maxSize < index1 + count) {
+				free(sysString->_value);
+				sysString->_maxSize = index1 + count;
+				sysString->_value = (char *)calloc(index1 + count, sizeof(char));
+			}
+
+			strncpy(sysString->_value + index1, string2 + index2, count);
+		} else {
+			SciString *string1 = s->_segMan->lookupString(argv[1]);
+
+			if (string1->getSize() < index1 + count)
+				string1->setSize(index1 + count);
+
+			// Note: We're accessing from c_str() here because the
+			// string's size ignores the trailing 0 and therefore
+			// triggers an assert when doing string2[i + index2].
+			for (uint16 i = 0; i < count; i++)
+				string1->setValue(i + index1, string2[i + index2]);
+		}
+		
+	} return argv[1];
+	case 7: { // Cmp
+		Common::String string1 = argv[1].isNull() ? "" : s->_segMan->getString(argv[1]);
+		Common::String string2 = argv[2].isNull() ? "" : s->_segMan->getString(argv[2]);
+
+		if (argc == 4) // Strncmp
+			return make_reg(0, strncmp(string1.c_str(), string2.c_str(), argv[3].toUint16()));
+		else           // Strcmp
+			return make_reg(0, strcmp(string1.c_str(), string2.c_str()));
+	}
+	case 8: { // Dup
+		const char *rawString = 0;
+		uint32 size = 0;
+
+		if (argv[1].segment == s->_segMan->getStringSegmentId()) {
+			SciString *string = s->_segMan->lookupString(argv[1]);
+			rawString = string->getRawData();
+			size = string->getSize();
+		} else {
+			Common::String string = s->_segMan->getString(argv[1]);
+			rawString = string.c_str();
+			size = string.size() + 1;
+		}
+
+		reg_t stringHandle;
+		SciString *dupString = s->_segMan->allocateString(&stringHandle);
+		dupString->setSize(size);
+
+		for (uint32 i = 0; i < size; i++)
+			dupString->setValue(i, rawString[i]);
+
+		return stringHandle;
+	}
+	case 9: // Getdata
+		if (!s->_segMan->isHeapObject(argv[1]))
+			return argv[1];
+
+		return readSelector(s->_segMan, argv[1], SELECTOR(data));
+	case 10: // Stringlen
+		return make_reg(0, s->_segMan->strlen(argv[1]));
+	case 11: { // Printf
+		reg_t stringHandle;
+		s->_segMan->allocateString(&stringHandle);
+
+		reg_t *adjustedArgs = new reg_t[argc];
+		adjustedArgs[0] = stringHandle;
+		memcpy(&adjustedArgs[1], argv + 1, (argc - 1) * sizeof(reg_t));
+
+		kFormat(s, argc, adjustedArgs);
+		delete[] adjustedArgs;
+		return stringHandle;
+		}
+	case 12: // Printf Buf
+		return kFormat(s, argc - 1, argv + 1);
+	case 13: { // atoi
+		Common::String string = s->_segMan->getString(argv[1]);
+		return make_reg(0, (uint16)atoi(string.c_str()));
+	}
+	default:
+		error("Unknown kString subop %d", argv[0].toUint16());
+	}
+
+	return NULL_REG;
+}
+
+/**
+ * Debug function, used in the demo of Shivers. It's marked as a stub
+ * in the original interpreter, but it gets called by the game scripts.
+ */
+reg_t kPrintDebug(EngineState *s, int argc, reg_t *argv) {
+	Common::String debugTemplate = s->_segMan->getString(argv[0]);
+	char debugString[500];
+
+	sprintf(debugString, debugTemplate.c_str(), argv[1].toUint16());
+	debugC(2, "kPrintDebug: \"%s\"\n", debugString);
+
+	return s->r_acc;
+}
+
+#endif
+
 } // End of namespace Sci
