@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sci/graphics/menu.cpp $
- * $Id: menu.cpp 48119 2010-02-23 22:47:53Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sci/graphics/menu.cpp $
+ * $Id: menu.cpp 52914 2010-09-26 18:59:01Z wjpalenstijn $
  *
  */
 
@@ -29,9 +29,9 @@
 
 #include "sci/sci.h"
 #include "sci/event.h"
+#include "sci/engine/kernel.h"
 #include "sci/engine/state.h"
 #include "sci/engine/selector.h"
-#include "sci/graphics/gui.h"
 #include "sci/graphics/ports.h"
 #include "sci/graphics/paint16.h"
 #include "sci/graphics/animate.h"
@@ -43,8 +43,8 @@
 
 namespace Sci {
 
-GfxMenu::GfxMenu(SciEvent *event, SegManager *segMan, SciGui *gui, GfxPorts *ports, GfxPaint16 *paint16, GfxText16 *text16, GfxScreen *screen, GfxCursor *cursor)
-	: _event(event), _segMan(segMan), _gui(gui), _ports(ports), _paint16(paint16), _text16(text16), _screen(screen), _cursor(cursor) {
+GfxMenu::GfxMenu(EventManager *event, SegManager *segMan, GfxPorts *ports, GfxPaint16 *paint16, GfxText16 *text16, GfxScreen *screen, GfxCursor *cursor)
+	: _event(event), _segMan(segMan), _ports(ports), _paint16(paint16), _text16(text16), _screen(screen), _cursor(cursor) {
 
 	_menuSaveHandle = NULL_REG;
 	_barSaveHandle = NULL_REG;
@@ -63,8 +63,9 @@ void GfxMenu::reset() {
 	_itemList.clear();
 	_listCount = 0;
 
-	// We actually set active item in here and remember last selection of the user
-	//  sierra sci always defaulted to first item every time menu was called via ESC, we dont follow that logic
+	// We actually set active item in here and remember last selection of the
+	// user. Sierra SCI always defaulted to first item every time menu was
+	// called via ESC, we don't follow that logic.
 	_curMenuId = 1;
 	_curItemId = 1;
 }
@@ -92,13 +93,18 @@ void GfxMenu::kernelAddEntry(Common::String title, Common::String content, reg_t
 
 		beginPos = curPos;
 
-		// Now go through the content till we find end-marker and collect data about it
-		// ':' is an end-marker for each item
+		// Now go through the content till we find end-marker and collect data about it.
+		// ':' is an end-marker for each item.
 		tagPos = 0; rightAlignedPos = 0;
 		controlPos = 0; altPos = 0; functionPos = 0;
 		while ((curPos < contentSize) && (content[curPos] != ':')) {
 			switch (content[curPos]) {
 			case '=': // Set tag
+				// Special case for normal animation speed - they use right
+				// aligned "=" for that one, so we ignore it as being recognized
+				// as tag marker.
+				if (rightAlignedPos == curPos - 1)
+					break;
 				if (tagPos)
 					error("multiple tag markers within one menu-item");
 				tagPos = curPos;
@@ -184,13 +190,21 @@ void GfxMenu::kernelAddEntry(Common::String title, Common::String content, reg_t
 			case '-':
 			case ' ':
 				separatorCount++;
+				break;
+			case '%':
+				// Some multilingual sci01 games use e.g. '--!%G--!' (which doesn't really make sense)
+				separatorCount += 2;
+				curPos++;
 			}
 			curPos++;
 		}
 		if (separatorCount == tempPos - beginPos) {
 			itemEntry->separatorLine = true;
 		} else {
-			itemEntry->text = g_sci->strSplit(Common::String(content.c_str() + beginPos, tempPos - beginPos).c_str());
+			// We don't strSplit here, because multilingual SCI01 support
+			// language switching on the fly, so we have to do this everytime
+			// the menu is called.
+			itemEntry->text = Common::String(content.c_str() + beginPos, tempPos - beginPos);
 
 			// LSL6 uses "Ctrl-" prefix string instead of ^ like all the other games do
 			tempPtr = itemEntry->text.c_str();
@@ -211,11 +225,18 @@ void GfxMenu::kernelAddEntry(Common::String title, Common::String content, reg_t
 			if (tagPos && tagPos >= rightAlignedPos)
 				tempPos = tagPos;
 			itemEntry->textRightAligned = Common::String(content.c_str() + rightAlignedPos, tempPos - rightAlignedPos);
-			// - and + are used sometimes for volume control
+			// Remove ending space, if there is one. Strangely sometimes there
+			// are lone spaces at the end in some games
+			if (itemEntry->textRightAligned.hasSuffix(" "))
+				itemEntry->textRightAligned.deleteLastChar();
+			// - and + are used sometimes for volume control/animation speed,
+			// = sometimes for animation speed
 			if (itemEntry->textRightAligned == "-") {
 				itemEntry->keyPress = '-';
 			} else if (itemEntry->textRightAligned == "+") {
 				itemEntry->keyPress = '+';
+			} else if (itemEntry->textRightAligned == "=") {
+				itemEntry->keyPress = '=';
 			}
 		}
 
@@ -250,8 +271,15 @@ GuiMenuItemEntry *GfxMenu::findItem(uint16 menuId, uint16 itemId) {
 
 void GfxMenu::kernelSetAttribute(uint16 menuId, uint16 itemId, uint16 attributeId, reg_t value) {
 	GuiMenuItemEntry *itemEntry = findItem(menuId, itemId);
-	if (!itemEntry)
-		error("Tried to setAttribute() on non-existant menu-item %d:%d", menuId, itemId);
+
+	if (!itemEntry) {
+		// PQ2 demo calls this, for example, but has no menus (bug report #3034507). Some SCI
+		// fan games (Al Pond 2, Aquarius) call this too on non-existent menu items. The
+		// original interpreter ignored these as well.
+		debugC(2, kDebugLevelGraphics, "Tried to setAttribute() on non-existent menu-item %d:%d", menuId, itemId);
+		return;
+	}
+
 	switch (attributeId) {
 	case SCI_MENU_ATTRIBUTE_ENABLED:
 		itemEntry->enabled = value.isNull() ? false : true;
@@ -260,7 +288,7 @@ void GfxMenu::kernelSetAttribute(uint16 menuId, uint16 itemId, uint16 attributeI
 		itemEntry->saidVmPtr = value;
 		break;
 	case SCI_MENU_ATTRIBUTE_TEXT:
-		itemEntry->text = g_sci->strSplit(_segMan->getString(value).c_str());
+		itemEntry->text = _segMan->getString(value);
 		itemEntry->textVmPtr = value;
 		// We assume here that no script ever creates a separatorLine dynamically
 		break;
@@ -317,42 +345,52 @@ void GfxMenu::drawBar() {
 	listIterator = _list.begin();
 	while (listIterator != listEnd) {
 		listEntry = *listIterator;
-		_text16->Draw_String(listEntry->text.c_str());
+		_text16->Draw_String(listEntry->textSplit.c_str());
 
 		listIterator++;
 	}
 }
 
-// This helper calculates all text widths for all menus/items
-void GfxMenu::calculateTextWidth() {
+// This helper calculates all text widths for all menus (only)
+void GfxMenu::calculateMenuWidth() {
 	GuiMenuList::iterator menuIterator;
 	GuiMenuList::iterator menuEnd = _list.end();
 	GuiMenuEntry *menuEntry;
-	GuiMenuItemList::iterator itemIterator;
-	GuiMenuItemList::iterator itemEnd = _itemList.end();
-	GuiMenuItemEntry *itemEntry;
 	int16 dummyHeight;
 
 	menuIterator = _list.begin();
 	while (menuIterator != menuEnd) {
 		menuEntry = *menuIterator;
-		_text16->StringWidth(menuEntry->text.c_str(), 0, menuEntry->textWidth, dummyHeight);
+		menuEntry->textSplit = g_sci->strSplit(menuEntry->text.c_str(), NULL);
+		_text16->StringWidth(menuEntry->textSplit.c_str(), 0, menuEntry->textWidth, dummyHeight);
 
 		menuIterator++;
 	}
+}
+
+// This helper calculates all text widths for all menus/items
+void GfxMenu::calculateMenuAndItemWidth() {
+	GuiMenuItemList::iterator itemIterator;
+	GuiMenuItemList::iterator itemEnd = _itemList.end();
+	GuiMenuItemEntry *itemEntry;
+	int16 dummyHeight;
+
+	calculateMenuWidth();
 
 	itemIterator = _itemList.begin();
 	while (itemIterator != itemEnd) {
 		itemEntry = *itemIterator;
-		_text16->StringWidth(itemEntry->text.c_str(), 0, itemEntry->textWidth, dummyHeight);
+		// Split the text now for multilingual SCI01 games
+		itemEntry->textSplit = g_sci->strSplit(itemEntry->text.c_str(), NULL);
+		_text16->StringWidth(itemEntry->textSplit.c_str(), 0, itemEntry->textWidth, dummyHeight);
 		_text16->StringWidth(itemEntry->textRightAligned.c_str(), 0, itemEntry->textRightAlignedWidth, dummyHeight);
 
 		itemIterator++;
 	}
 }
 
-reg_t GfxMenu::kernelSelect(reg_t eventObject) {
-	int16 eventType = GET_SEL32V(_segMan, eventObject, SELECTOR(type));
+reg_t GfxMenu::kernelSelect(reg_t eventObject, bool pauseSound) {
+	int16 eventType = readSelectorValue(_segMan, eventObject, SELECTOR(type));
 	int16 keyPress, keyModifier;
 	Common::Point mousePosition;
 	GuiMenuItemList::iterator itemIterator = _itemList.begin();
@@ -360,19 +398,24 @@ reg_t GfxMenu::kernelSelect(reg_t eventObject) {
 	GuiMenuItemEntry *itemEntry = NULL;
 	bool forceClaimed = false;
 	EngineState *s;
-	byte saidSpec[64];
 
 	switch (eventType) {
 	case SCI_EVENT_KEYBOARD:
-		keyPress = GET_SEL32V(_segMan, eventObject, SELECTOR(message));
-		keyModifier = GET_SEL32V(_segMan, eventObject, SELECTOR(modifiers));
+		keyPress = readSelectorValue(_segMan, eventObject, SELECTOR(message));
+		keyModifier = readSelectorValue(_segMan, eventObject, SELECTOR(modifiers));
+		// If tab got pressed, handle it here as if it was Ctrl-I - at least
+		// sci0 also did it that way
+		if (keyPress == SCI_KEY_TAB) {
+			keyModifier = SCI_KEYMOD_CTRL;
+			keyPress = 'i';
+		}
 		switch (keyPress) {
 		case 0:
 			break;
 		case SCI_KEY_ESC:
-			interactiveShowMouse();
+			interactiveStart(pauseSound);
 			itemEntry = interactiveWithKeyboard();
-			interactiveRestoreMouse();
+			interactiveEnd(pauseSound);
 			forceClaimed = true;
 			break;
 		default:
@@ -394,8 +437,13 @@ reg_t GfxMenu::kernelSelect(reg_t eventObject) {
 			itemEntry = *itemIterator;
 
 			if (!itemEntry->saidVmPtr.isNull()) {
-				// TODO: get a pointer to saidVmPtr or make said() work on VmPtrs
-				_segMan->memcpy(saidSpec, itemEntry->saidVmPtr, 64);
+				byte *saidSpec = _segMan->derefBulkPtr(itemEntry->saidVmPtr, 0);
+
+				if (!saidSpec) {
+					warning("Could not dereference saidSpec");
+					continue;
+				}
+
 				if (said(s, saidSpec, 0) != SAID_NO_MATCH)
 					break;
 			}
@@ -408,9 +456,9 @@ reg_t GfxMenu::kernelSelect(reg_t eventObject) {
 	case SCI_EVENT_MOUSE_PRESS:
 		mousePosition = _cursor->getPosition();
 		if (mousePosition.y < 10) {
-			interactiveShowMouse();
+			interactiveStart(pauseSound);
 			itemEntry = interactiveWithMouse();
-			interactiveRestoreMouse();
+			interactiveEnd(pauseSound);
 			forceClaimed = true;
 		}
 		break;
@@ -430,11 +478,13 @@ reg_t GfxMenu::kernelSelect(reg_t eventObject) {
 		_paint16->bitsShow(_ports->_menuRect);
 		_barSaveHandle = NULL_REG;
 	}
-	if (_oldPort)
+	if (_oldPort) {
 		_ports->setPort(_oldPort);
+		_oldPort = NULL;
+	}
 
 	if ((itemEntry) || (forceClaimed))
-		PUT_SEL32(_segMan, eventObject, SELECTOR(claimed), make_reg(0, 1));
+		writeSelector(_segMan, eventObject, SELECTOR(claimed), make_reg(0, 1));
 	if (itemEntry)
 		return make_reg(0, (itemEntry->menuId << 8) | (itemEntry->id));
 	return NULL_REG;
@@ -531,13 +581,19 @@ void GfxMenu::drawMenu(uint16 oldMenuId, uint16 newMenuId) {
 	if (!maxTextRightAlignedWidth)
 		_menuRect.right -= 5;
 
+	// If part of menu window is outside the screen, move it into the screen
+	// (this happens in multilingual sq3 and lsl3).
+	if (_menuRect.right > _screen->getWidth()) {
+		_menuRect.translate(-(_menuRect.right - _screen->getWidth()), 0);
+	}
+
 	// Save background
-	_menuSaveHandle = _paint16->bitsSave(_menuRect, SCI_SCREEN_MASK_VISUAL);
+	_menuSaveHandle = _paint16->bitsSave(_menuRect, GFX_SCREEN_MASK_VISUAL);
 
 	// Do the drawing
-	_paint16->fillRect(_menuRect, SCI_SCREEN_MASK_VISUAL, 0);
+	_paint16->fillRect(_menuRect, GFX_SCREEN_MASK_VISUAL, 0);
 	_menuRect.left++; _menuRect.right--; _menuRect.bottom--;
-	_paint16->fillRect(_menuRect, SCI_SCREEN_MASK_VISUAL, _screen->getColorWhite());
+	_paint16->fillRect(_menuRect, GFX_SCREEN_MASK_VISUAL, _screen->getColorWhite());
 
 	_menuRect.left += 8;
 	topPos = _menuRect.top + 1;
@@ -548,7 +604,7 @@ void GfxMenu::drawMenu(uint16 oldMenuId, uint16 newMenuId) {
 			if (!listItemEntry->separatorLine) {
 				_ports->textGreyedOutput(listItemEntry->enabled ? false : true);
 				_ports->moveTo(_menuRect.left, topPos);
-				_text16->Draw_String(listItemEntry->text.c_str());
+				_text16->Draw_String(listItemEntry->textSplit.c_str());
 				_ports->moveTo(_menuRect.right - listItemEntry->textRightAlignedWidth - 5, topPos);
 				_text16->Draw_String(listItemEntry->textRightAligned.c_str());
 			} else {
@@ -557,7 +613,7 @@ void GfxMenu::drawMenu(uint16 oldMenuId, uint16 newMenuId) {
 				pixelPos.y = topPos + (_ports->_curPort->fontHeight >> 1) - 1;
 				pixelPos.x = _menuRect.left - 7;
 				while (pixelPos.x < (_menuRect.right - 1)) {
-					_screen->putPixel(pixelPos.x, pixelPos.y, SCI_SCREEN_MASK_VISUAL, 0, 0, 0);
+					_screen->putPixel(pixelPos.x, pixelPos.y, GFX_SCREEN_MASK_VISUAL, 0, 0, 0);
 					pixelPos.x += 2;
 				}
 			}
@@ -589,12 +645,16 @@ void GfxMenu::invertMenuSelection(uint16 itemId) {
 	_paint16->bitsShow(itemRect);
 }
 
-void GfxMenu::interactiveShowMouse() {
+void GfxMenu::interactiveStart(bool pauseSound) {
 	_mouseOldState = _cursor->isVisible();
 	_cursor->kernelShow();
+	if (pauseSound)
+		g_sci->_soundCmd->pauseAll(true);
 }
 
-void GfxMenu::interactiveRestoreMouse() {
+void GfxMenu::interactiveEnd(bool pauseSound) {
+	if (pauseSound)
+		g_sci->_soundCmd->pauseAll(false);
 	if (!_mouseOldState)
 		_cursor->kernelHide();
 }
@@ -646,19 +706,20 @@ uint16 GfxMenu::mouseFindMenuItemSelection(Common::Point mousePosition, uint16 m
 }
 
 GuiMenuItemEntry *GfxMenu::interactiveWithKeyboard() {
-	sciEvent curEvent;
+	SciEvent curEvent;
 	uint16 newMenuId = _curMenuId;
 	uint16 newItemId = _curItemId;
 	GuiMenuItemEntry *curItemEntry = findItem(_curMenuId, _curItemId);
 	GuiMenuItemEntry *newItemEntry = curItemEntry;
 	Common::Point mousePosition;
 
-	// We don't 100% follow sierra here: we select last item instead of selecting first item of first menu everytime
-	//  Also sierra sci didnt allow mouse interaction, when menu was activated via keyboard
+	// We don't 100% follow Sierra here: we select last item instead of
+	// selecting first item of first menu every time. Also sierra sci didn't
+	// allow mouse interaction, when menu was activated via keyboard.
 
-	calculateTextWidth();
 	_oldPort = _ports->setPort(_ports->_menuPort);
-	_barSaveHandle = _paint16->bitsSave(_ports->_menuRect, SCI_SCREEN_MASK_VISUAL);
+	calculateMenuAndItemWidth();
+	_barSaveHandle = _paint16->bitsSave(_ports->_menuRect, GFX_SCREEN_MASK_VISUAL);
 
 	_ports->penColor(0);
 	_ports->backColor(_screen->getColorWhite());
@@ -670,12 +731,13 @@ GuiMenuItemEntry *GfxMenu::interactiveWithKeyboard() {
 	_paint16->bitsShow(_menuRect);
 
 	while (true) {
-		curEvent = _event->get(SCI_EVENT_ANY);
+		curEvent = _event->getSciEvent(SCI_EVENT_ANY);
 
 		switch (curEvent.type) {
 		case SCI_EVENT_KEYBOARD:
-			// We don't 100% follow sierra here: - sierra didn't wrap around when changing item id
-			//									 - sierra allowed item id to be 0, which didnt make any sense
+			// We don't 100% follow sierra here:
+			// - sierra didn't wrap around when changing item id
+			// - sierra allowed item id to be 0, which didn't make any sense
 			do {
 				switch (curEvent.data) {
 				case SCI_KEY_ESC:
@@ -760,26 +822,27 @@ GuiMenuItemEntry *GfxMenu::interactiveWithKeyboard() {
 			}
 			break;
 		case SCI_EVENT_NONE:
-			_event->sleep(2500 / 1000);
+			g_sci->sleep(2500 / 1000);
 			break;
 		}
 	}
 }
 
-// Mouse button is currently pressed - we are now interpreting mouse coordinates till mouse button is released
-//  The menu item that is selected at that time is chosen. If no menu item is selected we cancel
-//  No keyboard interaction is allowed, cause that wouldnt make any sense at all
+// Mouse button is currently pressed - we are now interpreting mouse coordinates
+// till mouse button is released. The menu item that is selected at that time is
+// chosen. If no menu item is selected we cancel. No keyboard interaction is
+// allowed, cause that wouldnt make any sense at all.
 GuiMenuItemEntry *GfxMenu::interactiveWithMouse() {
-	sciEvent curEvent;
+	SciEvent curEvent;
 	uint16 newMenuId = 0, newItemId = 0;
 	uint16 curMenuId = 0, curItemId = 0;
 	Common::Point mousePosition = _cursor->getPosition();
 	bool firstMenuChange = true;
 	GuiMenuItemEntry *curItemEntry = NULL;
 
-	calculateTextWidth();
 	_oldPort = _ports->setPort(_ports->_menuPort);
-	_barSaveHandle = _paint16->bitsSave(_ports->_menuRect, SCI_SCREEN_MASK_VISUAL);
+	calculateMenuAndItemWidth();
+	_barSaveHandle = _paint16->bitsSave(_ports->_menuRect, GFX_SCREEN_MASK_VISUAL);
 
 	_ports->penColor(0);
 	_ports->backColor(_screen->getColorWhite());
@@ -788,7 +851,7 @@ GuiMenuItemEntry *GfxMenu::interactiveWithMouse() {
 	_paint16->bitsShow(_ports->_menuRect);
 
 	while (true) {
-		curEvent = _event->get(SCI_EVENT_ANY);
+		curEvent = _event->getSciEvent(SCI_EVENT_ANY);
 
 		switch (curEvent.type) {
 		case SCI_EVENT_MOUSE_RELEASE:
@@ -799,7 +862,7 @@ GuiMenuItemEntry *GfxMenu::interactiveWithMouse() {
 			return curItemEntry;
 
 		case SCI_EVENT_NONE:
-			_event->sleep(2500 / 1000);
+			g_sci->sleep(2500 / 1000);
 			break;
 		}
 
@@ -842,7 +905,7 @@ void GfxMenu::kernelDrawStatus(const char *text, int16 colorPen, int16 colorBack
 	_paint16->fillRect(_ports->_menuBarRect, 1, colorBack);
 	_ports->penColor(colorPen);
 	_ports->moveTo(0, 1);
-	_text16->Draw_String(text);
+	_text16->Draw_Status(text);
 	_paint16->bitsShow(_ports->_menuBarRect);
 	_ports->setPort(oldPort);
 }
@@ -850,6 +913,7 @@ void GfxMenu::kernelDrawStatus(const char *text, int16 colorPen, int16 colorBack
 void GfxMenu::kernelDrawMenuBar(bool clear) {
 	if (!clear) {
 		Port *oldPort = _ports->setPort(_ports->_menuPort);
+		calculateMenuWidth();
 		drawBar();
 		_paint16->bitsShow(_ports->_menuBarRect);
 		_ports->setPort(oldPort);
