@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sci/graphics/paint16.cpp $
- * $Id: paint16.cpp 48084 2010-02-17 23:37:32Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sci/graphics/paint16.cpp $
+ * $Id: paint16.cpp 52912 2010-09-26 15:54:04Z wjpalenstijn $
  *
  */
 
@@ -29,8 +29,10 @@
 #include "graphics/primitives.h"
 
 #include "sci/sci.h"
+#include "sci/engine/features.h"
 #include "sci/engine/state.h"
 #include "sci/engine/selector.h"
+#include "sci/engine/workarounds.h"
 #include "sci/graphics/cache.h"
 #include "sci/graphics/coordadjuster.h"
 #include "sci/graphics/ports.h"
@@ -41,13 +43,14 @@
 #include "sci/graphics/view.h"
 #include "sci/graphics/screen.h"
 #include "sci/graphics/palette.h"
+#include "sci/graphics/portrait.h"
 #include "sci/graphics/text16.h"
 #include "sci/graphics/transitions.h"
 
 namespace Sci {
 
-GfxPaint16::GfxPaint16(ResourceManager *resMan, SegManager *segMan, Kernel *kernel, SciGui *gui, GfxCache *cache, GfxPorts *ports, GfxCoordAdjuster *coordAdjuster, GfxScreen *screen, GfxPalette *palette, GfxTransitions *transitions)
-	: _resMan(resMan), _segMan(segMan), _kernel(kernel), _gui(gui), _cache(cache), _ports(ports), _coordAdjuster(coordAdjuster), _screen(screen), _palette(palette), _transitions(transitions) {
+GfxPaint16::GfxPaint16(ResourceManager *resMan, SegManager *segMan, Kernel *kernel, GfxCache *cache, GfxPorts *ports, GfxCoordAdjuster *coordAdjuster, GfxScreen *screen, GfxPalette *palette, GfxTransitions *transitions, AudioPlayer *audio)
+	: _resMan(resMan), _segMan(segMan), _kernel(kernel), _cache(cache), _ports(ports), _coordAdjuster(coordAdjuster), _screen(screen), _palette(palette), _transitions(transitions), _audio(audio) {
 }
 
 GfxPaint16::~GfxPaint16() {
@@ -60,7 +63,7 @@ void GfxPaint16::init(GfxAnimate *animate, GfxText16 *text16) {
 	_EGAdrawingVisualize = false;
 }
 
-void GfxPaint16::setEGAdrawingVisualize(bool state) {
+void GfxPaint16::debugSetEGAdrawingVisualize(bool state) {
 	_EGAdrawingVisualize = state;
 }
 
@@ -73,6 +76,11 @@ void GfxPaint16::drawPicture(GuiResourceId pictureId, int16 animationNr, bool mi
 
 	picture->draw(animationNr, mirroredFlag, addToFlag, paletteId);
 	delete picture;
+
+	// We make a call to SciPalette here, for increasing sys timestamp and also loading targetpalette, if palvary active
+	//  (SCI1.1 only)
+	if (getSciVersion() == SCI_VERSION_1_1)
+		_palette->drewPicture(pictureId);
 }
 
 // This one is the only one that updates screen!
@@ -100,12 +108,12 @@ void GfxPaint16::drawCelAndShow(GuiResourceId viewId, int16 loopNo, int16 celNo,
 }
 
 // This version of drawCel is not supposed to call BitsShow()!
-void GfxPaint16::drawCel(GuiResourceId viewId, int16 loopNo, int16 celNo, Common::Rect celRect, byte priority, uint16 paletteNo, uint16 scaleX, uint16 scaleY) {
+void GfxPaint16::drawCel(GuiResourceId viewId, int16 loopNo, int16 celNo, const Common::Rect &celRect, byte priority, uint16 paletteNo, uint16 scaleX, uint16 scaleY) {
 	drawCel(_cache->getView(viewId), loopNo, celNo, celRect, priority, paletteNo, scaleX, scaleY);
 }
 
 // This version of drawCel is not supposed to call BitsShow()!
-void GfxPaint16::drawCel(GfxView *view, int16 loopNo, int16 celNo, Common::Rect celRect, byte priority, uint16 paletteNo, uint16 scaleX, uint16 scaleY) {
+void GfxPaint16::drawCel(GfxView *view, int16 loopNo, int16 celNo, const Common::Rect &celRect, byte priority, uint16 paletteNo, uint16 scaleX, uint16 scaleY) {
 	Common::Rect clipRect = celRect;
 	clipRect.clip(_ports->_curPort->rect);
 	if (clipRect.isEmpty()) // nothing to draw
@@ -120,8 +128,8 @@ void GfxPaint16::drawCel(GfxView *view, int16 loopNo, int16 celNo, Common::Rect
 	}
 }
 
-// This is used as replacement for drawCelAndShow() when hires-cels are drawn to screen
-//  Hires-cels are available only SCI 1.1+
+// This is used as replacement for drawCelAndShow() when hires-cels are drawn to
+// screen. Hires-cels are available only SCI 1.1+.
 void GfxPaint16::drawHiresCelAndShow(GuiResourceId viewId, int16 loopNo, int16 celNo, uint16 leftPos, uint16 topPos, byte priority, uint16 paletteNo, reg_t upscaledHiresHandle, uint16 scaleX, uint16 scaleY) {
 	GfxView *view = _cache->getView(viewId);
 	Common::Rect celRect, curPortRect, clipRect, clipRectTranslated;
@@ -130,11 +138,12 @@ void GfxPaint16::drawHiresCelAndShow(GuiResourceId viewId, int16 loopNo, int16 c
 
 	if (view) {
 		if ((leftPos == 0) && (topPos == 0)) {
-			// HACK: in kq6, we get leftPos&topPos == 0 SOMETIMES, that's why we need to get coordinates from upscaledHiresHandle
-			//  I'm not sure if this is what we are supposed to do or if there is some other bug that actually makes
-			//  coordinates to be 0 in the first place
+			// HACK: in kq6, we get leftPos&topPos == 0 SOMETIMES, that's why we
+			// need to get coordinates from upscaledHiresHandle. I'm not sure if
+			// this is what we are supposed to do or if there is some other bug
+			// that actually makes coordinates to be 0 in the first place.
 			byte *memoryPtr = NULL;
-			memoryPtr = kmem(_segMan, upscaledHiresHandle);
+			memoryPtr = _segMan->getHunkPointer(upscaledHiresHandle);
 			if (memoryPtr) {
 				Common::Rect upscaledHiresRect;
 				_screen->bitsGetRect(memoryPtr, &upscaledHiresRect);
@@ -151,15 +160,18 @@ void GfxPaint16::drawHiresCelAndShow(GuiResourceId viewId, int16 loopNo, int16 c
 		// adjust curPort to upscaled hires
 		clipRect = celRect;
 		curPortRect = _ports->_curPort->rect;
-		curPortRect.top *= 2; curPortRect.bottom *= 2; curPortRect.bottom++;
-		curPortRect.left *= 2; curPortRect.right *= 2; curPortRect.right++;
+		_screen->adjustToUpscaledCoordinates(curPortRect.top, curPortRect.left);
+		_screen->adjustToUpscaledCoordinates(curPortRect.bottom, curPortRect.right);
+		curPortRect.bottom++;
+		curPortRect.right++;
 		clipRect.clip(curPortRect);
 		if (clipRect.isEmpty()) // nothing to draw
 			return;
 
 		clipRectTranslated = clipRect;
 		if (!upscaledHiresHack) {
-			curPortPos.x = _ports->_curPort->left * 2; curPortPos.y = _ports->_curPort->top * 2;
+			curPortPos.x = _ports->_curPort->left; curPortPos.y = _ports->_curPort->top;
+			_screen->adjustToUpscaledCoordinates(curPortPos.y, curPortPos.x);
 			clipRectTranslated.top += curPortPos.y; clipRectTranslated.bottom += curPortPos.y;
 			clipRectTranslated.left += curPortPos.x; clipRectTranslated.right += curPortPos.x;
 		}
@@ -172,25 +184,44 @@ void GfxPaint16::drawHiresCelAndShow(GuiResourceId viewId, int16 loopNo, int16 c
 }
 
 void GfxPaint16::clearScreen(byte color) {
-	fillRect(_ports->_curPort->rect, SCI_SCREEN_MASK_ALL, color, 0, 0);
+	fillRect(_ports->_curPort->rect, GFX_SCREEN_MASK_ALL, color, 0, 0);
 }
 
 void GfxPaint16::invertRect(const Common::Rect &rect) {
 	int16 oldpenmode = _ports->_curPort->penMode;
 	_ports->_curPort->penMode = 2;
-	fillRect(rect, 1, _ports->_curPort->penClr, _ports->_curPort->backClr);
+	fillRect(rect, GFX_SCREEN_MASK_VISUAL, _ports->_curPort->penClr, _ports->_curPort->backClr);
 	_ports->_curPort->penMode = oldpenmode;
 }
 
+// used in SCI0early exclusively
+void GfxPaint16::invertRectViaXOR(const Common::Rect &rect) {
+	Common::Rect r = rect;
+	int16 x, y;
+	byte curVisual;
+
+	r.clip(_ports->_curPort->rect);
+	if (r.isEmpty()) // nothing to invert
+		return;
+
+	_ports->offsetRect(r);
+	for (y = r.top; y < r.bottom; y++) {
+		for (x = r.left; x < r.right; x++) {
+			curVisual = _screen->getVisual(x, y);
+			_screen->putPixel(x, y, GFX_SCREEN_MASK_VISUAL, curVisual ^ 0x0f, 0, 0);
+		}
+	}
+}
+
 void GfxPaint16::eraseRect(const Common::Rect &rect) {
-	fillRect(rect, 1, _ports->_curPort->backClr);
+	fillRect(rect, GFX_SCREEN_MASK_VISUAL, _ports->_curPort->backClr);
 }
 
 void GfxPaint16::paintRect(const Common::Rect &rect) {
-	fillRect(rect, 1, _ports->_curPort->penClr);
+	fillRect(rect, GFX_SCREEN_MASK_VISUAL, _ports->_curPort->penClr);
 }
 
-void GfxPaint16::fillRect(const Common::Rect &rect, int16 drawFlags, byte clrPen, byte clrBack, byte bControl) {
+void GfxPaint16::fillRect(const Common::Rect &rect, int16 drawFlags, byte color, byte priority, byte control) {
 	Common::Rect r = rect;
 	r.clip(_ports->_curPort->rect);
 	if (r.isEmpty()) // nothing to fill
@@ -202,22 +233,22 @@ void GfxPaint16::fillRect(const Common::Rect &rect, int16 drawFlags, byte clrPen
 	byte curVisual;
 
 	// Doing visual first
-	if (drawFlags & SCI_SCREEN_MASK_VISUAL) {
+	if (drawFlags & GFX_SCREEN_MASK_VISUAL) {
 		if (oldPenMode == 2) { // invert mode
 			for (y = r.top; y < r.bottom; y++) {
 				for (x = r.left; x < r.right; x++) {
 					curVisual = _screen->getVisual(x, y);
-					if (curVisual == clrPen) {
-						_screen->putPixel(x, y, SCI_SCREEN_MASK_VISUAL, clrBack, 0, 0);
-					} else if (curVisual == clrBack) {
-						_screen->putPixel(x, y, SCI_SCREEN_MASK_VISUAL, clrPen, 0, 0);
+					if (curVisual == color) {
+						_screen->putPixel(x, y, GFX_SCREEN_MASK_VISUAL, priority, 0, 0);
+					} else if (curVisual == priority) {
+						_screen->putPixel(x, y, GFX_SCREEN_MASK_VISUAL, color, 0, 0);
 					}
 				}
 			}
-		} else { // just fill rect with ClrPen
+		} else { // just fill rect with color
 			for (y = r.top; y < r.bottom; y++) {
 				for (x = r.left; x < r.right; x++) {
-					_screen->putPixel(x, y, SCI_SCREEN_MASK_VISUAL, clrPen, 0, 0);
+					_screen->putPixel(x, y, GFX_SCREEN_MASK_VISUAL, color, 0, 0);
 				}
 			}
 		}
@@ -225,12 +256,16 @@ void GfxPaint16::fillRect(const Common::Rect &rect, int16 drawFlags, byte clrPen
 
 	if (drawFlags < 2)
 		return;
-	drawFlags &= SCI_SCREEN_MASK_PRIORITY|SCI_SCREEN_MASK_CONTROL;
+	drawFlags &= GFX_SCREEN_MASK_PRIORITY|GFX_SCREEN_MASK_CONTROL;
+
+	// we need to isolate the bits, sierra sci saved priority and control inside one byte, we don't
+	priority &= 0x0f;
+	control &= 0x0f;
 
 	if (oldPenMode != 2) {
 		for (y = r.top; y < r.bottom; y++) {
 			for (x = r.left; x < r.right; x++) {
-				_screen->putPixel(x, y, drawFlags, 0, clrBack, bControl);
+				_screen->putPixel(x, y, drawFlags, 0, priority, control);
 			}
 		}
 	} else {
@@ -286,10 +321,13 @@ reg_t GfxPaint16::bitsSave(const Common::Rect &rect, byte screenMask) {
 	if (workerRect.isEmpty()) // nothing to save
 		return NULL_REG;
 
-	if (screenMask == SCI_SCREEN_MASK_DISPLAY) {
+	if (screenMask == GFX_SCREEN_MASK_DISPLAY) {
+		// The coordinates we are given are actually up-to-including right/bottom - we extend accordingly
+		workerRect.bottom++;
+		workerRect.right++;
 		// Adjust rect to upscaled hires, but dont adjust according to port
-		workerRect.top *= 2; workerRect.bottom *= 2; workerRect.bottom++;
-		workerRect.left *= 2; workerRect.right *= 2; workerRect.right++;
+		_screen->adjustToUpscaledCoordinates(workerRect.top, workerRect.left);
+		_screen->adjustToUpscaledCoordinates(workerRect.bottom, workerRect.right);
 	} else {
 		_ports->offsetRect(workerRect);
 	}
@@ -297,9 +335,10 @@ reg_t GfxPaint16::bitsSave(const Common::Rect &rect, byte screenMask) {
 	// now actually ask _screen how much space it will need for saving
 	size = _screen->bitsGetDataSize(workerRect, screenMask);
 
-	memoryId = kalloc(_segMan, "SaveBits()", size);
-	memoryPtr = kmem(_segMan, memoryId);
-	_screen->bitsSave(workerRect, screenMask, memoryPtr);
+	memoryId = _segMan->allocateHunkEntry("SaveBits()", size);
+	memoryPtr = _segMan->getHunkPointer(memoryId);
+	if (memoryPtr)
+		_screen->bitsSave(workerRect, screenMask, memoryPtr);
 	return memoryId;
 }
 
@@ -307,7 +346,7 @@ void GfxPaint16::bitsGetRect(reg_t memoryHandle, Common::Rect *destRect) {
 	byte *memoryPtr = NULL;
 
 	if (!memoryHandle.isNull()) {
-		memoryPtr = kmem(_segMan, memoryHandle);
+		memoryPtr = _segMan->getHunkPointer(memoryHandle);
 
 		if (memoryPtr) {
 			_screen->bitsGetRect(memoryPtr, destRect);
@@ -319,19 +358,18 @@ void GfxPaint16::bitsRestore(reg_t memoryHandle) {
 	byte *memoryPtr = NULL;
 
 	if (!memoryHandle.isNull()) {
-		memoryPtr = kmem(_segMan, memoryHandle);
+		memoryPtr = _segMan->getHunkPointer(memoryHandle);
 
 		if (memoryPtr) {
 			_screen->bitsRestore(memoryPtr);
-			kfree(_segMan, memoryHandle);
+			_segMan->freeHunkEntry(memoryHandle);
 		}
 	}
 }
 
 void GfxPaint16::bitsFree(reg_t memoryHandle) {
-	if (!memoryHandle.isNull()) {
-		kfree(_segMan, memoryHandle);
-	}
+	if (!memoryHandle.isNull())	// happens in KQ5CD
+		_segMan->freeHunkEntry(memoryHandle);
 }
 
 void GfxPaint16::kernelDrawPicture(GuiResourceId pictureId, int16 animationNr, bool animationBlackoutFlag, bool mirroredFlag, bool addToFlag, int16 EGApaletteNo) {
@@ -342,6 +380,14 @@ void GfxPaint16::kernelDrawPicture(GuiResourceId pictureId, int16 animationNr, b
 		drawPicture(pictureId, animationNr, mirroredFlag, addToFlag, EGApaletteNo);
 		_transitions->setup(animationNr, animationBlackoutFlag);
 	} else {
+		// We need to set it for SCI1EARLY+ (sierra sci also did so), otherwise we get at least the following issues:
+		//  LSL5 (english) - last wakeup (taj mahal flute dream)
+		//  SQ5 (english v1.03) - during the scene following the scrubbing
+		//   in both situations a window is shown when kDrawPic is called, which would result otherwise in
+		//   no showpic getting called from kAnimate and we would get graphic corruption
+		// XMAS1990 EGA did not set it in this case, VGA did
+		if (getSciVersion() >= SCI_VERSION_1_EARLY)
+			_screen->_picNotValid = 1;
 		_ports->beginUpdate(_ports->_picWind);
 		drawPicture(pictureId, animationNr, mirroredFlag, addToFlag, EGApaletteNo);
 		_ports->endUpdate(_ports->_picWind);
@@ -349,28 +395,29 @@ void GfxPaint16::kernelDrawPicture(GuiResourceId pictureId, int16 animationNr, b
 	_ports->setPort(oldPort);
 }
 
-void GfxPaint16::kernelDrawCel(GuiResourceId viewId, int16 loopNo, int16 celNo, uint16 leftPos, uint16 topPos, int16 priority, uint16 paletteNo, bool hiresMode, reg_t upscaledHiresHandle) {
-	// some calls are hiresMode even under kq6 DOS, that's why we check for upscaled hires here
+void GfxPaint16::kernelDrawCel(GuiResourceId viewId, int16 loopNo, int16 celNo, uint16 leftPos, uint16 topPos, int16 priority, uint16 paletteNo, uint16 scaleX, uint16 scaleY, bool hiresMode, reg_t upscaledHiresHandle) {
+	// some calls are hiresMode even under kq6 DOS, that's why we check for
+	// upscaled hires here
 	if ((!hiresMode) || (!_screen->getUpscaledHires())) {
-		drawCelAndShow(viewId, loopNo, celNo, leftPos, topPos, priority, paletteNo);
+		drawCelAndShow(viewId, loopNo, celNo, leftPos, topPos, priority, paletteNo, scaleX, scaleY);
 	} else {
 		drawHiresCelAndShow(viewId, loopNo, celNo, leftPos, topPos, priority, paletteNo, upscaledHiresHandle);
 	}
 }
 
-void GfxPaint16::kernelGraphFillBoxForeground(Common::Rect rect) {
+void GfxPaint16::kernelGraphFillBoxForeground(const Common::Rect &rect) {
 	paintRect(rect);
 }
 
-void GfxPaint16::kernelGraphFillBoxBackground(Common::Rect rect) {
+void GfxPaint16::kernelGraphFillBoxBackground(const Common::Rect &rect) {
 	eraseRect(rect);
 }
 
-void GfxPaint16::kernelGraphFillBox(Common::Rect rect, uint16 colorMask, int16 color, int16 priority, int16 control) {
+void GfxPaint16::kernelGraphFillBox(const Common::Rect &rect, uint16 colorMask, int16 color, int16 priority, int16 control) {
 	fillRect(rect, colorMask, color, priority, control);
 }
 
-void GfxPaint16::kernelGraphFrameBox(Common::Rect rect, int16 color) {
+void GfxPaint16::kernelGraphFrameBox(const Common::Rect &rect, int16 color) {
 	int16 oldColor = _ports->getPort()->penClr;
 	_ports->penColor(color);
 	frameRect(rect);
@@ -378,24 +425,26 @@ void GfxPaint16::kernelGraphFrameBox(Common::Rect rect, int16 color) {
 }
 
 void GfxPaint16::kernelGraphDrawLine(Common::Point startPoint, Common::Point endPoint, int16 color, int16 priority, int16 control) {
+	_ports->clipLine(startPoint, endPoint);
 	_ports->offsetLine(startPoint, endPoint);
 	_screen->drawLine(startPoint.x, startPoint.y, endPoint.x, endPoint.y, color, priority, control);
 }
 
-reg_t GfxPaint16::kernelGraphSaveBox(Common::Rect rect, uint16 screenMask) {
+reg_t GfxPaint16::kernelGraphSaveBox(const Common::Rect &rect, uint16 screenMask) {
 	return bitsSave(rect, screenMask);
 }
 
-reg_t GfxPaint16::kernelGraphSaveUpscaledHiresBox(Common::Rect rect) {
-	return bitsSave(rect, SCI_SCREEN_MASK_DISPLAY);
+reg_t GfxPaint16::kernelGraphSaveUpscaledHiresBox(const Common::Rect &rect) {
+	return bitsSave(rect, GFX_SCREEN_MASK_DISPLAY);
 }
 
 void GfxPaint16::kernelGraphRestoreBox(reg_t handle) {
 	bitsRestore(handle);
 }
 
-void GfxPaint16::kernelGraphUpdateBox(Common::Rect rect, bool hiresMode) {
-	// some calls are hiresMode even under kq6 DOS, that's why we check for upscaled hires here
+void GfxPaint16::kernelGraphUpdateBox(const Common::Rect &rect, bool hiresMode) {
+	// some calls are hiresMode even under kq6 DOS, that's why we check for
+	// upscaled hires here
 	if ((!hiresMode) || (!_screen->getUpscaledHires()))
 		bitsShow(rect);
 	else
@@ -423,17 +472,20 @@ void GfxPaint16::kernelGraphRedrawBox(Common::Rect rect) {
 #define SCI_DISPLAY_WIDTH				106
 #define SCI_DISPLAY_SAVEUNDER			107
 #define SCI_DISPLAY_RESTOREUNDER		108
+#define SCI_DISPLAY_DUMMY1				114 // used in longbow demo/qfg1 ega demo, not supported in sierra sci - no parameters
+#define SCI_DISPLAY_DUMMY2				115 // used in longbow demo, not supported in sierra sci - has 1 parameter
 #define SCI_DISPLAY_DONTSHOWBITS		121
 
 reg_t GfxPaint16::kernelDisplay(const char *text, int argc, reg_t *argv) {
-	int displayArg;
+	reg_t displayArg;
 	TextAlignment alignment = SCI_TEXT16_ALIGNMENT_LEFT;
 	int16 colorPen = -1, colorBack = -1, width = -1, bRedraw = 1;
 	bool doSaveUnder = false;
 	Common::Rect rect;
 	reg_t result = NULL_REG;
 
-	// Make a "backup" of the port settings
+	// Make a "backup" of the port settings (required for some SCI0LATE and
+	// SCI01+ only)
 	Port oldPort = *_ports->getPort();
 
 	// setting defaults
@@ -442,9 +494,11 @@ reg_t GfxPaint16::kernelDisplay(const char *text, int argc, reg_t *argv) {
 	_ports->textGreyedOutput(false);
 	// processing codes in argv
 	while (argc > 0) {
-		displayArg = argv[0].toUint16();
+		displayArg = argv[0];
+		if (displayArg.segment)
+			displayArg.offset = 0xFFFF;
 		argc--; argv++;
-		switch (displayArg) {
+		switch (displayArg.offset) {
 		case SCI_DISPLAY_MOVEPEN:
 			_ports->moveTo(argv[0].toUint16(), argv[1].toUint16());
 			argc -= 2; argv += 2;
@@ -488,8 +542,27 @@ reg_t GfxPaint16::kernelDisplay(const char *text, int argc, reg_t *argv) {
 		case SCI_DISPLAY_DONTSHOWBITS:
 			bRedraw = 0;
 			break;
+
+		// 2 Dummy functions, longbow-demo is using those several times but sierra sci doesn't support them at all
+		// The Quest for Glory 1 EGA demo also calls kDisplay(114)
+		case SCI_DISPLAY_DUMMY1:
+		case SCI_DISPLAY_DUMMY2:
+			if (!g_sci->isDemo() || (g_sci->getGameId() != GID_LONGBOW && g_sci->getGameId() != GID_QFG1))
+				error("Unknown kDisplay argument %d", displayArg.offset);
+			if (displayArg.offset == SCI_DISPLAY_DUMMY2) {
+				if (argc) {
+					argc--; argv++;
+				} else {
+					error("No parameter left for kDisplay(115)");
+				}
+			}
+			break;
 		default:
-			warning("Unknown kDisplay argument %X", displayArg);
+			SciTrackOriginReply originReply;
+			SciWorkaroundSolution solution = trackOriginAndFindWorkaround(0, kDisplay_workarounds, &originReply);
+			if (solution.type == WORKAROUND_NONE)
+				error("Unknown kDisplay argument (%04x:%04x) from method %s::%s (script %d, localCall %x)", PRINT_REG(displayArg), originReply.objectName.c_str(), originReply.methodName.c_str(), originReply.scriptNr, originReply.localCallOffset);
+			assert(solution.type == WORKAROUND_IGNORE);
 			break;
 		}
 	}
@@ -505,9 +578,9 @@ reg_t GfxPaint16::kernelDisplay(const char *text, int argc, reg_t *argv) {
 	}
 
 	if (doSaveUnder)
-		result = bitsSave(rect, SCI_SCREEN_MASK_VISUAL);
+		result = bitsSave(rect, GFX_SCREEN_MASK_VISUAL);
 	if (colorBack != -1)
-		fillRect(rect, SCI_SCREEN_MASK_VISUAL, colorBack, 0, 0);
+		fillRect(rect, GFX_SCREEN_MASK_VISUAL, colorBack, 0, 0);
 	_text16->Box(text, 0, rect, alignment, -1);
 	if (_screen->_picNotValid == 0 && bRedraw)
 		bitsShow(rect);
@@ -515,25 +588,45 @@ reg_t GfxPaint16::kernelDisplay(const char *text, int argc, reg_t *argv) {
 	Port *currport = _ports->getPort();
 	uint16 tTop = currport->curTop;
 	uint16 tLeft = currport->curLeft;
-	*currport = oldPort;
+	if (!g_sci->_features->usesOldGfxFunctions()) {
+		// Restore port settings for some SCI0LATE and SCI01+ only.
+		//
+		// The change actually happened inbetween .530 (hoyle1) and .566 (heros
+		// quest). We don't have any detection for that currently, so we are
+		// using oldGfxFunctions (.502). The only games that could get
+		// regressions because of this are hoyle1, kq4 and funseeker. If there
+		// are regressions, we should use interpreter version (which would
+		// require exe version detection).
+		//
+		// If we restore the port for whole SCI0LATE, at least sq3old will get
+		// an issue - font 0 will get used when scanning for planets instead of
+		// font 600 - a setfont parameter is missing in one of the kDisplay
+		// calls in script 19. I assume this is a script bug, because it was
+		// added in sq3new.
+		*currport = oldPort;
+	}
 	currport->curTop = tTop;
 	currport->curLeft = tLeft;
 	return result;
 }
 
-// TODO: If this matches the sci32 implementation, we may put it into GfxScreen
-void GfxPaint16::kernelShakeScreen(uint16 shakeCount, uint16 directions) {
-	while (shakeCount--) {
-		if (directions & SCI_SHAKE_DIRECTION_VERTICAL)
-			_screen->setVerticalShakePos(10);
-		// TODO: horizontal shakes
-		g_system->updateScreen();
-		_gui->wait(3);
-		if (directions & SCI_SHAKE_DIRECTION_VERTICAL)
-			_screen->setVerticalShakePos(0);
-		g_system->updateScreen();
-		_gui->wait(3);
-	}
+reg_t GfxPaint16::kernelPortraitLoad(const Common::String &resourceName) {
+	//Portrait *myPortrait = new Portrait(g_sci->getResMan(), _screen, _palette, resourceName);
+	return NULL_REG;
+}
+
+void GfxPaint16::kernelPortraitShow(const Common::String &resourceName, Common::Point position, uint16 resourceId, uint16 noun, uint16 verb, uint16 cond, uint16 seq) {
+	Portrait *myPortrait = new Portrait(g_sci->getResMan(), g_sci->getEventManager(), _screen, _palette, _audio, resourceName);
+	// TODO: cache portraits
+	// adjust given coordinates to curPort (but dont adjust coordinates on upscaledHires_Save_Box and give us hires coordinates
+	//  on kDrawCel, yeah this whole stuff makes sense)
+	position.x += _ports->getPort()->left; position.y += _ports->getPort()->top;
+	_screen->adjustToUpscaledCoordinates(position.y, position.x);
+	myPortrait->doit(position, resourceId, noun, verb, cond, seq);
+	delete myPortrait;
+}
+
+void GfxPaint16::kernelPortraitUnload(uint16 portraitId) {
 }
 
 } // End of namespace Sci
