@@ -18,15 +18,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sci/graphics/ports.cpp $
- * $Id: ports.cpp 47916 2010-02-05 22:20:46Z m_kiewitz $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sci/graphics/ports.cpp $
+ * $Id: ports.cpp 52301 2010-08-23 16:33:19Z m_kiewitz $
  *
  */
 
 #include "common/util.h"
 
 #include "sci/sci.h"
+#include "sci/engine/features.h"
+#include "sci/engine/kernel.h"
 #include "sci/engine/state.h"
+#include "sci/engine/selector.h"
 #include "sci/graphics/screen.h"
 #include "sci/graphics/paint16.h"
 #include "sci/graphics/animate.h"
@@ -49,28 +52,26 @@ GfxPorts::GfxPorts(SegManager *segMan, GfxScreen *screen)
 }
 
 GfxPorts::~GfxPorts() {
-	// TODO: Clear _windowList and delete all stuff in it?
-	delete _mainPort;
+	// reset frees all windows but _picWind
+	reset();
+	freeWindow(_picWind);
+	delete _wmgrPort;
 	delete _menuPort;
 }
 
-void GfxPorts::init(bool usesOldGfxFunctions, SciGui *gui, GfxPaint16 *paint16, GfxText16 *text16, Common::String gameId) {
+void GfxPorts::init(bool usesOldGfxFunctions, GfxPaint16 *paint16, GfxText16 *text16) {
 	int16 offTop = 10;
 
 	_usesOldGfxFunctions = usesOldGfxFunctions;
-	_gui = gui;
 	_paint16 = paint16;
 	_text16 = text16;
 
-	// _mainPort is not known to windowmanager, that's okay according to sierra sci
-	//  its not even used currently in our engine
-	_mainPort = new Port(0);
-	setPort(_mainPort);
-	openPort(_mainPort);
+	_freeCounter = 0;
 
 	// _menuPort has actually hardcoded id 0xFFFF. Its not meant to be known to windowmanager according to sierra sci
 	_menuPort = new Port(0xFFFF);
 	openPort(_menuPort);
+	setPort(_menuPort);
 	_text16->SetFont(0);
 	_menuPort->rect = Common::Rect(0, 0, _screen->getWidth(), _screen->getHeight());
 	_menuBarRect = Common::Rect(0, 0, _screen->getWidth(), 9);
@@ -85,30 +86,101 @@ void GfxPorts::init(bool usesOldGfxFunctions, SciGui *gui, GfxPaint16 *paint16,
 	//				will work, but some scripts seem to check for 0 and initialize the variable again in that case
 	//				resulting in problems.
 
-	// Jones, Slater and Hoyle 3 were called with parameter -Nw 0 0 200 320.
-	// Mother Goose (SCI1) uses -Nw 0 0 159 262. The game will later use SetPort so we don't need to set the other fields.
+	if (getSciVersion() >= SCI_VERSION_1_LATE)
+		_styleUser = SCI_WINDOWMGR_STYLE_USER;
+	else
+		_styleUser = SCI_WINDOWMGR_STYLE_USER | SCI_WINDOWMGR_STYLE_TRANSPARENT;
+
+	// Jones, Slater, Hoyle 3&4 and Crazy Nicks Laura Bow/Kings Quest were
+	// called with parameter -Nw 0 0 200 320.
+	// Mother Goose (SCI1) uses -Nw 0 0 159 262. The game will later use
+	// SetPort so we don't need to set the other fields.
 	// This actually meant not skipping the first 10 pixellines in windowMgrPort
-	if (gameId == "jones" || gameId == "slater" || gameId == "hoyle3" || (gameId == "mothergoose" && getSciVersion() == SCI_VERSION_1_EARLY))
+	switch (g_sci->getGameId()) {
+	case GID_JONES:
+	case GID_SLATER:
+	case GID_HOYLE3:
+	case GID_HOYLE4:
+	case GID_CNICK_LAURABOW:
+	case GID_CNICK_KQ:
 		offTop = 0;
+		break;
+	case GID_MOTHERGOOSE256:
+		// only the SCI1 and SCI1.1 (VGA) versions need this
+		offTop = 0;
+		break;
+	case GID_FAIRYTALES:
+		// Mixed-Up Fairy Tales (& its demo) uses -w 26 0 200 320. If we don't
+		// also do this we will get not-fully-removed windows everywhere.
+		offTop = 26;
+		break;
+	default:
+		offTop = 10;
+		break;
+	}
 
 	openPort(_wmgrPort);
 	setPort(_wmgrPort);
-	setOrigin(0, offTop);
-	_wmgrPort->rect.bottom = _screen->getHeight() - offTop;
+	// SCI0 games till kq4 (.502 - not including) did not adjust against _wmgrPort in kNewWindow
+	//  We leave _wmgrPort top at 0, so the adjustment wont get done
+	if (!g_sci->_features->usesOldGfxFunctions()) {
+		setOrigin(0, offTop);
+		_wmgrPort->rect.bottom = _screen->getHeight() - offTop;
+	} else {
+		_wmgrPort->rect.bottom = _screen->getHeight();
+	}
 	_wmgrPort->rect.right = _screen->getWidth();
 	_wmgrPort->rect.moveTo(0, 0);
 	_wmgrPort->curTop = 0;
 	_wmgrPort->curLeft = 0;
 	_windowList.push_front(_wmgrPort);
 
-	_picWind = newWindow(Common::Rect(0, offTop, _screen->getWidth(), _screen->getHeight()), 0, 0, SCI_WINDOWMGR_STYLE_TRANSPARENT | SCI_WINDOWMGR_STYLE_NOFRAME, 0, true);
-
-	priorityBandsMemoryActive = false;
+	_picWind = addWindow(Common::Rect(0, offTop, _screen->getWidth(), _screen->getHeight()), 0, 0, SCI_WINDOWMGR_STYLE_TRANSPARENT | SCI_WINDOWMGR_STYLE_NOFRAME, 0, true);
+	// For SCI0 games till kq4 (.502 - not including) we set _picWind top to offTop instead
+	//  Because of the menu/status bar
+	if (g_sci->_features->usesOldGfxFunctions())
+		_picWind->top = offTop;
 
 	kernelInitPriorityBands();
 }
 
+// Removes any windows from windowList
+//  is used when restoring/restarting the game
+//  Sierra SCI actually saved the whole windowList, it seems we don't need to do this at all
+//  but in some games there are still windows active when restoring. Leaving those windows open
+//  would create all sorts of issues, that's why we remove them
+void GfxPorts::reset() {
+	setPort(_picWind);
+
+	// free everything after _picWind
+	for (uint id = PORTS_FIRSTSCRIPTWINDOWID; id < _windowsById.size(); id++) {
+		Window *window = (Window *)_windowsById[id];
+		if (window)
+			freeWindow(window);
+	}
+	_freeCounter = 0;
+	_windowList.clear();
+	_windowList.push_front(_wmgrPort);
+	_windowList.push_back(_picWind);
+}
+
 void GfxPorts::kernelSetActive(uint16 portId) {
+	if (_freeCounter) {
+		// Windows waiting to get freed
+		for (uint id = PORTS_FIRSTSCRIPTWINDOWID; id < _windowsById.size(); id++) {
+			Window *window = (Window *)_windowsById[id];
+			if (window) {
+				if (window->counterTillFree) {
+					window->counterTillFree--;
+					if (!window->counterTillFree) {
+						freeWindow(window);
+						_freeCounter--;
+					}
+				}
+			}
+		}
+	}
+
 	switch (portId) {
 	case 0:
 		setPort(_wmgrPort);
@@ -116,8 +188,13 @@ void GfxPorts::kernelSetActive(uint16 portId) {
 	case 0xFFFF:
 		setPort(_menuPort);
 		break;
-	default:
-		setPort(getPortById(portId));
+	default: {
+		Port *newPort = getPortById(portId);
+		if (newPort)
+			setPort(newPort);
+		else
+			error("GfxPorts::kernelSetActive was requested to set invalid port id %d", portId);
+	}
 	};
 }
 
@@ -131,9 +208,8 @@ void GfxPorts::kernelSetPicWindow(Common::Rect rect, int16 picTop, int16 picLeft
 	_picWind->rect = rect;
 	_picWind->top = picTop;
 	_picWind->left = picLeft;
-	// TODO: fixme before joining
-	//if (initPriorityBandsFlag)
-	//	initPriorityBands();
+	if (initPriorityBandsFlag)
+		kernelInitPriorityBands();
 }
 
 reg_t GfxPorts::kernelGetActive() {
@@ -143,10 +219,10 @@ reg_t GfxPorts::kernelGetActive() {
 reg_t GfxPorts::kernelNewWindow(Common::Rect dims, Common::Rect restoreRect, uint16 style, int16 priority, int16 colorPen, int16 colorBack, const char *title) {
 	Window *wnd = NULL;
 
-	if (restoreRect.top != 0 && restoreRect.left != 0 && restoreRect.height() != 0 && restoreRect.width() != 0)
-		wnd = newWindow(dims, &restoreRect, title, style, priority, false);
+	if (restoreRect.bottom != 0 && restoreRect.right != 0)
+		wnd = addWindow(dims, &restoreRect, title, style, priority, false);
 	else
-		wnd = newWindow(dims, NULL, title, style, priority, false);
+		wnd = addWindow(dims, NULL, title, style, priority, false);
 	wnd->penClr = colorPen;
 	wnd->backClr = colorBack;
 	drawWindow(wnd);
@@ -156,7 +232,15 @@ reg_t GfxPorts::kernelNewWindow(Common::Rect dims, Common::Rect restoreRect, uin
 
 void GfxPorts::kernelDisposeWindow(uint16 windowId, bool reanimate) {
 	Window *wnd = (Window *)getPortById(windowId);
-	disposeWindow(wnd, reanimate);
+	if (wnd) {
+		if (!wnd->counterTillFree) {
+			removeWindow(wnd, reanimate);
+		} else {
+			error("kDisposeWindow: used already disposed window id %d", windowId);
+		}
+	} else {
+		error("kDisposeWindow: used unknown window id %d", windowId);
+	}
 }
 
 int16 GfxPorts::isFrontWindow(Window *pWnd) {
@@ -193,10 +277,20 @@ void GfxPorts::endUpdate(Window *wnd) {
 	setPort(oldPort);
 }
 
-Window *GfxPorts::newWindow(const Common::Rect &dims, const Common::Rect *restoreRect, const char *title, uint16 style, int16 priority, bool draw) {
+Window *GfxPorts::addWindow(const Common::Rect &dims, const Common::Rect *restoreRect, const char *title, uint16 style, int16 priority, bool draw) {
 	// Find an unused window/port id
-	uint id = 1;
+	uint id = PORTS_FIRSTWINDOWID;
 	while (id < _windowsById.size() && _windowsById[id]) {
+		if (_windowsById[id]->counterTillFree) {
+			// port that is already disposed, but not freed yet
+			freeWindow((Window *)_windowsById[id]);
+			_freeCounter--;
+			break; // reuse the handle
+			// we do this especially for sq4cd. it creates and disposes the
+			//  inventory window all the time, but reuses old handles as well
+			//  this worked somewhat under the original interpreter, because
+			//  it put the new window where the old was.
+		}
 		++id;
 	}
 	if (id == _windowsById.size())
@@ -207,7 +301,7 @@ Window *GfxPorts::newWindow(const Common::Rect &dims, const Common::Rect *restor
 	Common::Rect r;
 
 	if (!pwnd) {
-		warning("Can't open window!");
+		error("Can't open window!");
 		return 0;
 	}
 
@@ -217,8 +311,19 @@ Window *GfxPorts::newWindow(const Common::Rect &dims, const Common::Rect *restor
 	else
 		_windowList.push_back(pwnd);
 	openPort(pwnd);
+
 	r = dims;
-	pwnd->rect = dims;
+	if (r.width() > _screen->getWidth()) {
+		// We get invalid dimensions at least at the end of sq3 (script bug!).
+		// Same happens very often in lsl5, sierra sci didnt fix it but it looked awful.
+		// Also happens frequently in the demo of GK1.
+		warning("Fixing too large window, left: %d, right: %d", dims.left, dims.right);
+		r.left = 0;
+		r.right = _screen->getWidth() - 1;
+		if ((style != _styleUser) && !(style & SCI_WINDOWMGR_STYLE_NOFRAME))
+			r.right--;
+	}
+	pwnd->rect = r;
 	if (restoreRect)
 		pwnd->restoreRect = *restoreRect;
 
@@ -226,14 +331,14 @@ Window *GfxPorts::newWindow(const Common::Rect &dims, const Common::Rect *restor
 	pwnd->hSaved1 = pwnd->hSaved2 = NULL_REG;
 	pwnd->bDrawn = false;
 	if ((style & SCI_WINDOWMGR_STYLE_TRANSPARENT) == 0)
-		pwnd->saveScreenMask = (priority == -1 ? SCI_SCREEN_MASK_VISUAL : SCI_SCREEN_MASK_VISUAL | SCI_SCREEN_MASK_PRIORITY);
+		pwnd->saveScreenMask = (priority == -1 ? GFX_SCREEN_MASK_VISUAL : GFX_SCREEN_MASK_VISUAL | GFX_SCREEN_MASK_PRIORITY);
 
 	if (title && (style & SCI_WINDOWMGR_STYLE_TITLE)) {
 		pwnd->title = title;
 	}
 
-	r = dims;
-	if ((style != SCI_WINDOWMGR_STYLE_USER) && !(style & SCI_WINDOWMGR_STYLE_NOFRAME)) {
+	r = pwnd->rect;
+	if ((style != _styleUser) && !(style & SCI_WINDOWMGR_STYLE_NOFRAME)) {
 		r.grow(1);
 		if (style & SCI_WINDOWMGR_STYLE_TITLE) {
 			r.top -= 10;
@@ -241,9 +346,6 @@ Window *GfxPorts::newWindow(const Common::Rect &dims, const Common::Rect *restor
 		}
 	}
 
-	// FIXME: it seems as if shadows may result in the window getting moved one upwards
-	//         so that the shadow is visible (lsl5)
-
 	pwnd->dims = r;
 	const Common::Rect *wmprect = &_wmgrPort->rect;
 	int16 oldtop = pwnd->dims.top;
@@ -264,16 +366,24 @@ Window *GfxPorts::newWindow(const Common::Rect &dims, const Common::Rect *restor
 	if (restoreRect == 0)
 		pwnd->restoreRect = pwnd->dims;
 
-	if (!(pwnd->wndStyle & (SCI_WINDOWMGR_STYLE_USER | SCI_WINDOWMGR_STYLE_NOFRAME))) {
-		// The shadow is drawn slightly outside the window.
-		// Enlarge restoreRect to cover that.
-		pwnd->restoreRect.bottom++;
-		pwnd->restoreRect.right++;
-	}
-
 	if (draw)
 		drawWindow(pwnd);
 	setPort((Port *)pwnd);
+
+	// FIXME: changing setOrigin to not clear the rightmost bit fixes the display of windows
+	// in some fanmade games (e.g. New Year's Mystery (Updated)). Since the fanmade games
+	// use an unmodified SCI interpreter, this leads me to believe that there either is some
+	// off-by-one error in the window drawing code, or there is another place where the
+	// rightmost bit should be cleeared. New Year's Mystery is a good test case for this, as
+	// it draws dialogs and then draws cels on top of them, for fancier dialog corners (like,
+	// for example, KQ5). KQ5 has a custom window style, however, whereas New Year's mystery
+	// has a "classic" style with only SCI_WINDOWMGR_STYLE_NOFRAME set. If
+	// SCI_WINDOWMGR_STYLE_NOFRAME is removed, the window is cleared correctly, because it
+	// grows slightly, covering the view pixels on the left. In any case, the views and the
+	// window have a difference of one pixel when they're drawn via kNewWindow and kDrawCel,
+	// which causes the glitch to appear when the window is closed.
+
+	// All SCI0 games till kq4 .502 (not including) did not adjust against _wmgrPort, we set _wmgrPort->top to 0 in that case
 	setOrigin(pwnd->rect.left, pwnd->rect.top + _wmgrPort->top);
 	pwnd->rect.moveTo(0, 0);
 	return pwnd;
@@ -289,38 +399,44 @@ void GfxPorts::drawWindow(Window *pWnd) {
 	Port *oldport = setPort(_wmgrPort);
 	penColor(0);
 	if ((wndStyle & SCI_WINDOWMGR_STYLE_TRANSPARENT) == 0) {
-		pWnd->hSaved1 = _paint16->bitsSave(pWnd->restoreRect, SCI_SCREEN_MASK_VISUAL);
-		if (pWnd->saveScreenMask & SCI_SCREEN_MASK_PRIORITY) {
-			pWnd->hSaved2 = _paint16->bitsSave(pWnd->restoreRect, SCI_SCREEN_MASK_PRIORITY);
+		pWnd->hSaved1 = _paint16->bitsSave(pWnd->restoreRect, GFX_SCREEN_MASK_VISUAL);
+		if (pWnd->saveScreenMask & GFX_SCREEN_MASK_PRIORITY) {
+			pWnd->hSaved2 = _paint16->bitsSave(pWnd->restoreRect, GFX_SCREEN_MASK_PRIORITY);
 			if ((wndStyle & SCI_WINDOWMGR_STYLE_USER) == 0)
-				_paint16->fillRect(pWnd->restoreRect, SCI_SCREEN_MASK_PRIORITY, 0, 15);
+				_paint16->fillRect(pWnd->restoreRect, GFX_SCREEN_MASK_PRIORITY, 0, 15);
 		}
 	}
 
 	// drawing frame,shadow and title
-	if (!(wndStyle & SCI_WINDOWMGR_STYLE_USER)) {
+	if ((getSciVersion() >= SCI_VERSION_1_LATE) ? !(wndStyle & _styleUser) : wndStyle != _styleUser) {
 		r = pWnd->dims;
 		if (!(wndStyle & SCI_WINDOWMGR_STYLE_NOFRAME)) {
-			r.translate(1, 1);
-			_paint16->frameRect(r);// shadow
+			r.top++;
+			r.left++;
+			_paint16->frameRect(r);// draw shadow
 			r.translate(-1, -1);
-			_paint16->frameRect(r);// window frame
+			_paint16->frameRect(r);// draw actual window frame
 
 			if (wndStyle & SCI_WINDOWMGR_STYLE_TITLE) {
-				_paint16->frameRect(r);
+				if (getSciVersion() <= SCI_VERSION_0_LATE) {
+					// draw a black line between titlebar and actual window content for SCI0
+					r.bottom = r.top + 10;
+					_paint16->frameRect(r);
+				}
 				r.grow(-1);
 				if (getSciVersion() <= SCI_VERSION_0_LATE)
-					_paint16->fillRect(r, SCI_SCREEN_MASK_VISUAL, 8); // grey titlebar for SCI0
+					_paint16->fillRect(r, GFX_SCREEN_MASK_VISUAL, 8); // grey titlebar for SCI0
 				else
-					_paint16->fillRect(r, SCI_SCREEN_MASK_VISUAL, 0); // black titlebar for SCI01+
+					_paint16->fillRect(r, GFX_SCREEN_MASK_VISUAL, 0); // black titlebar for SCI01+
 				if (!pWnd->title.empty()) {
 					int16 oldcolor = getPort()->penClr;
-					penColor(255);
+					penColor(_screen->getColorWhite());
 					_text16->Box(pWnd->title.c_str(), 1, r, SCI_TEXT16_ALIGNMENT_CENTER, 0);
 					penColor(oldcolor);
 				}
 
-				r = pWnd->dims;
+				r.grow(+1);
+				r.bottom = pWnd->dims.bottom - 1;
 				r.top += 9;
 			}
 
@@ -328,24 +444,43 @@ void GfxPorts::drawWindow(Window *pWnd) {
 		}
 
 		if (!(wndStyle & SCI_WINDOWMGR_STYLE_TRANSPARENT))
-			_paint16->fillRect(r, SCI_SCREEN_MASK_VISUAL, pWnd->backClr);
+			_paint16->fillRect(r, GFX_SCREEN_MASK_VISUAL, pWnd->backClr);
 
-		_paint16->bitsShow(pWnd->restoreRect);
+		_paint16->bitsShow(pWnd->dims);
 	}
 	setPort(oldport);
 }
 
-void GfxPorts::disposeWindow(Window *pWnd, bool reanimate) {
+void GfxPorts::removeWindow(Window *pWnd, bool reanimate) {
 	setPort(_wmgrPort);
 	_paint16->bitsRestore(pWnd->hSaved1);
+	pWnd->hSaved1 = NULL_REG;
 	_paint16->bitsRestore(pWnd->hSaved2);
+	pWnd->hSaved2 = NULL_REG;
 	if (!reanimate)
 		_paint16->bitsShow(pWnd->restoreRect);
 	else
 		_paint16->kernelGraphRedrawBox(pWnd->restoreRect);
 	_windowList.remove(pWnd);
 	setPort(_windowList.back());
-	_windowsById[pWnd->id] = 0;
+	// We will actually free this window after 15 kSetPort-calls
+	// Sierra sci freed the pointer immediately, but pointer to that port
+	//  still worked till the memory got overwritten. Some games depend
+	//  on this (dispose a window and then kSetPort to it again for once)
+	//  Those are actually script bugs, but patching all of those out
+	//  would be quite a hassle and this just keeps compatibility
+	//  (examples: hoyle 4 game menu and sq4cd inventory)
+	//  sq4cd gum wrapper requires more than 10
+	pWnd->counterTillFree = 15;
+	_freeCounter++;
+}
+
+void GfxPorts::freeWindow(Window *pWnd) {
+	if (!pWnd->hSaved1.isNull())
+		_segMan->freeHunkEntry(pWnd->hSaved1);
+	if (!pWnd->hSaved2.isNull())
+		_segMan->freeHunkEntry(pWnd->hSaved1);
+	_windowsById[pWnd->id] = NULL;
 	delete pWnd;
 }
 
@@ -353,11 +488,11 @@ void GfxPorts::updateWindow(Window *wnd) {
 	reg_t handle;
 
 	if (wnd->saveScreenMask && wnd->bDrawn) {
-		handle = _paint16->bitsSave(wnd->restoreRect, SCI_SCREEN_MASK_VISUAL);
+		handle = _paint16->bitsSave(wnd->restoreRect, GFX_SCREEN_MASK_VISUAL);
 		_paint16->bitsRestore(wnd->hSaved1);
 		wnd->hSaved1 = handle;
-		if (wnd->saveScreenMask & SCI_SCREEN_MASK_PRIORITY) {
-			handle = _paint16->bitsSave(wnd->restoreRect, SCI_SCREEN_MASK_PRIORITY);
+		if (wnd->saveScreenMask & GFX_SCREEN_MASK_PRIORITY) {
+			handle = _paint16->bitsSave(wnd->restoreRect, GFX_SCREEN_MASK_PRIORITY);
 			_paint16->bitsRestore(wnd->hSaved2);
 			wnd->hSaved2 = handle;
 		}
@@ -365,13 +500,9 @@ void GfxPorts::updateWindow(Window *wnd) {
 }
 
 Port *GfxPorts::getPortById(uint16 id) {
-	if (id > _windowsById.size())
-		error("getPortById() received invalid id");
-	return _windowsById[id];
+	return (id < _windowsById.size()) ? _windowsById[id] : NULL;
 }
 
-
-
 Port *GfxPorts::setPort(Port *newPort) {
 	Port *oldPort = _curPort;
 	_curPort = newPort;
@@ -383,7 +514,10 @@ Port *GfxPorts::getPort() {
 }
 
 void GfxPorts::setOrigin(int16 left, int16 top) {
-	_curPort->left = left;
+	// This looks fishy, but it's exactly what sierra did. They removed last bit of left in their interpreter
+	//  It seems sierra did it for EGA byte alignment (EGA uses 1 byte for 2 pixels) and left it in their interpreter even
+	//  when going VGA.
+	_curPort->left = left & 0x7FFE;
 	_curPort->top = top;
 }
 
@@ -410,7 +544,7 @@ void GfxPorts::openPort(Port *port) {
 	port->left = 0;
 	port->greyedOutput = false;
 	port->penClr = 0;
-	port->backClr = 255;
+	port->backClr = _screen->getColorWhite();
 	port->penMode = 0;
 	port->rect = _bounds;
 }
@@ -449,6 +583,13 @@ void GfxPorts::offsetLine(Common::Point &start, Common::Point &end) {
 	end.y += _curPort->top;
 }
 
+void GfxPorts::clipLine(Common::Point &start, Common::Point &end) {
+	start.y = CLIP<int16>(start.y, _curPort->rect.top, _curPort->rect.bottom - 1);
+	start.x = CLIP<int16>(start.x, _curPort->rect.left, _curPort->rect.right - 1);
+	end.y = CLIP<int16>(end.y, _curPort->rect.top, _curPort->rect.bottom - 1);
+	end.x = CLIP<int16>(end.x, _curPort->rect.left, _curPort->rect.right - 1);
+}
+
 void GfxPorts::priorityBandsInit(int16 bandCount, int16 top, int16 bottom) {
 	int16 y;
 	int32 bandSize;
@@ -480,6 +621,11 @@ void GfxPorts::priorityBandsInit(int16 bandCount, int16 top, int16 bottom) {
 	// We fill space that is left over with the highest band (hardcoded 200 limit, because this algo isnt meant to be used on hires)
 	for (y = _priorityBottom; y < 200; y++)
 		_priorityBands[y] = _priorityBandCount;
+
+	// adjust, if bottom is 200 (one over the actual screen range) - we could otherwise go possible out of bounds
+	//  sierra sci also adjust accordingly
+	if (_priorityBottom == 200)
+		_priorityBottom--;
 }
 
 void GfxPorts::priorityBandsInit(byte *data) {
@@ -495,22 +641,14 @@ void GfxPorts::priorityBandsInit(byte *data) {
 		_priorityBands[i++] = inx;
 }
 
-// Gets used by picture class to remember priority bands data from sci1.1 pictures that need to get applied when
-//  transitioning to that picture
-void GfxPorts::priorityBandsRemember(byte *data) {
-	int bandNo;
-	for (bandNo = 0; bandNo < 14; bandNo++) {
-		priorityBandsMemory[bandNo] = READ_LE_UINT16(data);
+// Gets used to read priority bands data from sci1.1 pictures
+void GfxPorts::priorityBandsInitSci11(byte *data) {
+	byte priorityBands[14];
+	for (int bandNo = 0; bandNo < 14; bandNo++) {
+		priorityBands[bandNo] = READ_LE_UINT16(data);
 		data += 2;
 	}
-	priorityBandsMemoryActive = true;
-}
-
-void GfxPorts::priorityBandsRecall() {
-	if (priorityBandsMemoryActive) {
-		priorityBandsInit((byte *)&priorityBandsMemory);
-		priorityBandsMemoryActive = false;
-	}
+	priorityBandsInit(priorityBands);
 }
 
 void GfxPorts::kernelInitPriorityBands() {
