@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sci/graphics/screen.cpp $
- * $Id: screen.cpp 48084 2010-02-17 23:37:32Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sci/graphics/screen.cpp $
+ * $Id: screen.cpp 52615 2010-09-07 08:12:59Z thebluegr $
  *
  */
 
@@ -27,6 +27,7 @@
 #include "common/util.h"
 #include "common/system.h"
 #include "graphics/surface.h"
+#include "engines/util.h"
 
 #include "sci/sci.h"
 #include "sci/engine/state.h"
@@ -34,23 +35,62 @@
 
 namespace Sci {
 
-GfxScreen::GfxScreen(ResourceManager *resMan, int16 width, int16 height, bool upscaledHires) :
-	_resMan(resMan), _width(width), _height(height), _upscaledHires(upscaledHires) {
+GfxScreen::GfxScreen(ResourceManager *resMan) : _resMan(resMan) {
+
+	// Scale the screen, if needed
+	_upscaledHires = GFX_SCREEN_UPSCALED_DISABLED;
+
+	// King's Quest 6 and Gabriel Knight 1 have hires content, gk1/cd was able
+	// to provide that under DOS as well, but as gk1/floppy does support
+	// upscaled hires scriptswise, but doesn't actually have the hires content
+	// we need to limit it to platform windows.
+	if (g_sci->getPlatform() == Common::kPlatformWindows) {
+		if (g_sci->getGameId() == GID_KQ6)
+			_upscaledHires = GFX_SCREEN_UPSCALED_640x440;
+#ifdef ENABLE_SCI32
+		if (g_sci->getGameId() == GID_GK1)
+			_upscaledHires = GFX_SCREEN_UPSCALED_640x480;
+#endif
+	}
+
+	if (_resMan->detectHires()) {
+		_width = 640;
+		_height = 480;
+	} else {
+		_width = 320;
+		_height = 200;
+	}
+
+	// Japanese versions of games use hi-res font on upscaled version of the game.
+	if ((g_sci->getLanguage() == Common::JA_JPN) && (getSciVersion() <= SCI_VERSION_1_1))
+		_upscaledHires = GFX_SCREEN_UPSCALED_640x400;
 
 	_pixels = _width * _height;
-	_displayWidth = _width;
-	_displayHeight = _height;
 
-	if (_upscaledHires) {
-		_displayWidth *= 2;
-		_displayHeight *= 2;
-
-#ifdef ENABLE_SCI32
-		// SCI32 also corrects the aspect ratio when upscaling the resolution.
-		// This is especially needed in GK1, as the credits video is 640x480.
-		if (getSciVersion() >= SCI_VERSION_2)
-			_displayHeight = _displayHeight * 6 / 5;
-#endif
+	switch (_upscaledHires) {
+	case GFX_SCREEN_UPSCALED_640x400:
+		_displayWidth = 640;
+		_displayHeight = 400;
+		for (int i = 0; i <= _height; i++)
+			_upscaledMapping[i] = i * 2;
+		break;
+	case GFX_SCREEN_UPSCALED_640x440:
+		_displayWidth = 640;
+		_displayHeight = 440;
+		for (int i = 0; i <= _height; i++)
+			_upscaledMapping[i] = (i * 11) / 5;
+		break;
+	case GFX_SCREEN_UPSCALED_640x480:
+		_displayWidth = 640;
+		_displayHeight = 480;
+		for (int i = 0; i <= _height; i++)
+			_upscaledMapping[i] = (i * 12) / 5;
+		break;
+	default:
+		_displayWidth = _width;
+		_displayHeight = _height;
+		memset(&_upscaledMapping, 0, sizeof(_upscaledMapping) );
+		break;
 	}
 
 	_displayPixels = _displayWidth * _displayHeight;
@@ -67,8 +107,9 @@ GfxScreen::GfxScreen(ResourceManager *resMan, int16 width, int16 height, bool up
 	_unditherState = true;
 
 	if (_resMan->isVGA() || (_resMan->isAmiga32color())) {
-		// It's not 100% accurate to set white to be 255 for amiga 32-color games
-		//  255 is defined as white in our sci at all times, so it doesnt matter
+		// It is not 100% accurate to set white to be 255 for Amiga 32-color
+		// games. But 255 is defined as white in our SCI at all times, so it
+		// doesn't matter.
 		_colorWhite = 255;
 		if (getSciVersion() >= SCI_VERSION_1_1)
 			_colorDefaultVectorData = 255;
@@ -80,7 +121,18 @@ GfxScreen::GfxScreen(ResourceManager *resMan, int16 width, int16 height, bool up
 	}
 
 	// Initialize the actual screen
-	initGraphics(_displayWidth, _displayHeight, _displayWidth > 320);
+
+	if (_resMan->isSci11Mac() && getSciVersion() == SCI_VERSION_1_1) {
+		// For SCI1.1 Mac, we need to expand the screen to accommodate for
+		// the icon bar. Of course, both KQ6 and QFG1 VGA differ in size.
+		if (g_sci->getGameId() == GID_KQ6)
+			initGraphics(_displayWidth, _displayHeight + 26, _displayWidth > 320);
+		else if (g_sci->getGameId() == GID_QFG1VGA)
+			initGraphics(_displayWidth, _displayHeight + 20, _displayWidth > 320);
+		else
+			error("Unknown SCI1.1 Mac game");
+	} else
+		initGraphics(_displayWidth, _displayHeight, _displayWidth > 320);
 }
 
 GfxScreen::~GfxScreen() {
@@ -97,7 +149,7 @@ void GfxScreen::copyToScreen() {
 void GfxScreen::copyFromScreen(byte *buffer) {
 	Graphics::Surface *screen;
 	screen = g_system->lockScreen();
-	memcpy(buffer, screen->pixels, _displayWidth * _displayHeight);
+	memcpy(buffer, screen->pixels, _displayPixels);
 	g_system->unlockScreen();
 }
 
@@ -112,11 +164,15 @@ void GfxScreen::copyRectToScreen(const Common::Rect &rect) {
 	if (!_upscaledHires)  {
 		g_system->copyRectToScreen(_activeScreen + rect.top * _displayWidth + rect.left, _displayWidth, rect.left, rect.top, rect.width(), rect.height());
 	} else {
-		g_system->copyRectToScreen(_activeScreen + rect.top * 2 * _displayWidth + rect.left * 2, _displayWidth, rect.left * 2, rect.top * 2, rect.width() * 2, rect.height() * 2);
+		int rectHeight = _upscaledMapping[rect.bottom] - _upscaledMapping[rect.top];
+		g_system->copyRectToScreen(_activeScreen + _upscaledMapping[rect.top] * _displayWidth + rect.left * 2, _displayWidth, rect.left * 2, _upscaledMapping[rect.top], rect.width() * 2, rectHeight);
 	}
 }
 
-// This copies a rect to screen w/o scaling adjustment and is only meant to be used on hires graphics used in upscaled hires mode
+/**
+ * This copies a rect to screen w/o scaling adjustment and is only meant to be
+ * used on hires graphics used in upscaled hires mode.
+ */
 void GfxScreen::copyDisplayRectToScreen(const Common::Rect &rect) {
 	if (!_upscaledHires)
 		error("copyDisplayRectToScreen: not in upscaled hires mode");
@@ -127,51 +183,82 @@ void GfxScreen::copyRectToScreen(const Common::Rect &rect, int16 x, int16 y) {
 	if (!_upscaledHires)  {
 		g_system->copyRectToScreen(_activeScreen + rect.top * _displayWidth + rect.left, _displayWidth, x, y, rect.width(), rect.height());
 	} else {
-		g_system->copyRectToScreen(_activeScreen + rect.top * 2 * _displayWidth + rect.left * 2, _displayWidth, x * 2, y * 2, rect.width() * 2, rect.height() * 2);
+		int rectHeight = _upscaledMapping[rect.bottom] - _upscaledMapping[rect.top];
+		g_system->copyRectToScreen(_activeScreen + _upscaledMapping[rect.top] * _displayWidth + rect.left * 2, _displayWidth, x * 2, _upscaledMapping[y], rect.width() * 2, rectHeight);
 	}
 }
 
 byte GfxScreen::getDrawingMask(byte color, byte prio, byte control) {
 	byte flag = 0;
 	if (color != 255)
-		flag |= SCI_SCREEN_MASK_VISUAL;
+		flag |= GFX_SCREEN_MASK_VISUAL;
 	if (prio != 255)
-		flag |= SCI_SCREEN_MASK_PRIORITY;
+		flag |= GFX_SCREEN_MASK_PRIORITY;
 	if (control != 255)
-		flag |= SCI_SCREEN_MASK_CONTROL;
+		flag |= GFX_SCREEN_MASK_CONTROL;
 	return flag;
 }
 
 void GfxScreen::putPixel(int x, int y, byte drawMask, byte color, byte priority, byte control) {
 	int offset = y * _width + x;
 
-	if (drawMask & SCI_SCREEN_MASK_VISUAL) {
+	if (drawMask & GFX_SCREEN_MASK_VISUAL) {
 		_visualScreen[offset] = color;
 		if (!_upscaledHires) {
 			_displayScreen[offset] = color;
 		} else {
-			int displayOffset = y * 2 * _displayWidth + x * 2;
-			_displayScreen[displayOffset] = color;
-			_displayScreen[displayOffset + 1] = color;
-			_displayScreen[displayOffset + _displayWidth] = color;
-			_displayScreen[displayOffset + _displayWidth + 1] = color;
+			int displayOffset = _upscaledMapping[y] * _displayWidth + x * 2;
+			int heightOffsetBreak = (_upscaledMapping[y + 1] - _upscaledMapping[y]) * _displayWidth;
+			int heightOffset = 0;
+			do {
+				_displayScreen[displayOffset + heightOffset] = color;
+				_displayScreen[displayOffset + heightOffset + 1] = color;
+				heightOffset += _displayWidth;
+			} while (heightOffset != heightOffsetBreak);
 		}
 	}
-	if (drawMask & SCI_SCREEN_MASK_PRIORITY)
+	if (drawMask & GFX_SCREEN_MASK_PRIORITY)
 		_priorityScreen[offset] = priority;
-	if (drawMask & SCI_SCREEN_MASK_CONTROL)
+	if (drawMask & GFX_SCREEN_MASK_CONTROL)
 		_controlScreen[offset] = control;
 }
 
-// This will just change a pixel directly on displayscreen. Its supposed to get only used on upscaled-Hires games where
-//  hires content needs to get drawn ONTO the upscaled display screen (like japanese fonts, hires portraits, etc.)
+/**
+ * This is used to put font pixels onto the screen - we adjust differently, so that we won't
+ *  do triple pixel lines in any case on upscaled hires. That way the font will not get distorted
+ *  Sierra SCI didn't do this
+ */
+void GfxScreen::putFontPixel(int startingY, int x, int y, byte color) {
+	int offset = (startingY + y) * _width + x;
+
+	_visualScreen[offset] = color;
+	if (!_upscaledHires) {
+		_displayScreen[offset] = color;
+	} else {
+		int displayOffset = (_upscaledMapping[startingY] + y * 2) * _displayWidth + x * 2;
+		_displayScreen[displayOffset] = color;
+		_displayScreen[displayOffset + 1] = color;
+		displayOffset += _displayWidth;
+		_displayScreen[displayOffset] = color;
+		_displayScreen[displayOffset + 1] = color;
+	}
+}
+
+/**
+ * This will just change a pixel directly on displayscreen. It is supposed to be
+ * only used on upscaled-Hires games where hires content needs to get drawn ONTO
+ * the upscaled display screen (like japanese fonts, hires portraits, etc.).
+ */
 void GfxScreen::putPixelOnDisplay(int x, int y, byte color) {
 	int offset = y * _displayWidth + x;
 	_displayScreen[offset] = color;
 }
 
-// Sierra's Bresenham line drawing
-// WARNING: Do not just blindly replace this with Graphics::drawLine(), as it seems to create issues with flood fill
+/**
+ * Sierra's Bresenham line drawing.
+ * WARNING: Do not replace this with Graphics::drawLine(), as this causes issues
+ * with flood fill, due to small difference in the Bresenham logic.
+ */
 void GfxScreen::drawLine(Common::Point startPoint, Common::Point endPoint, byte color, byte priority, byte control) {
 	int16 left = startPoint.x;
 	int16 top = startPoint.y;
@@ -234,6 +321,14 @@ void GfxScreen::drawLine(Common::Point startPoint, Common::Point endPoint, byte
 	}
 }
 
+// We put hires kanji chars onto upscaled background, so we need to adjust
+// coordinates. Caller gives use low-res ones.
+void GfxScreen::putKanjiChar(Graphics::FontSJIS *commonFont, int16 x, int16 y, uint16 chr, byte color) {
+	byte *displayPtr = _displayScreen + y * _displayWidth * 2 + x * 2;
+	// we don't use outline, so color 0 is actually not used
+	commonFont->drawChar(displayPtr, chr, _displayWidth, 1, color, 0);
+}
+
 byte GfxScreen::getVisual(int x, int y) {
 	return _visualScreen[y * _width + x];
 }
@@ -250,33 +345,34 @@ byte GfxScreen::isFillMatch(int16 x, int16 y, byte screenMask, byte t_color, byt
 	int offset = y * _width + x;
 	byte match = 0;
 
-	if (screenMask & SCI_SCREEN_MASK_VISUAL && *(_visualScreen + offset) == t_color)
-		match |= SCI_SCREEN_MASK_VISUAL;
-	if (screenMask & SCI_SCREEN_MASK_PRIORITY && *(_priorityScreen + offset) == t_pri)
-		match |= SCI_SCREEN_MASK_PRIORITY;
-	if (screenMask & SCI_SCREEN_MASK_CONTROL && *(_controlScreen + offset) == t_con)
-		match |= SCI_SCREEN_MASK_CONTROL;
+	if ((screenMask & GFX_SCREEN_MASK_VISUAL) && *(_visualScreen + offset) == t_color)
+		match |= GFX_SCREEN_MASK_VISUAL;
+	if ((screenMask & GFX_SCREEN_MASK_PRIORITY) && *(_priorityScreen + offset) == t_pri)
+		match |= GFX_SCREEN_MASK_PRIORITY;
+	if ((screenMask & GFX_SCREEN_MASK_CONTROL) && *(_controlScreen + offset) == t_con)
+		match |= GFX_SCREEN_MASK_CONTROL;
 	return match;
 }
 
 int GfxScreen::bitsGetDataSize(Common::Rect rect, byte mask) {
 	int byteCount = sizeof(rect) + sizeof(mask);
 	int pixels = rect.width() * rect.height();
-	if (mask & SCI_SCREEN_MASK_VISUAL) {
+	if (mask & GFX_SCREEN_MASK_VISUAL) {
 		byteCount += pixels; // _visualScreen
 		if (!_upscaledHires) {
 			byteCount += pixels; // _displayScreen
 		} else {
-			byteCount += pixels * 4; // _displayScreen (upscaled hires)
+			int rectHeight = _upscaledMapping[rect.bottom] - _upscaledMapping[rect.top];
+			byteCount += rectHeight * rect.width() * 2; // _displayScreen (upscaled hires)
 		}
 	}
-	if (mask & SCI_SCREEN_MASK_PRIORITY) {
+	if (mask & GFX_SCREEN_MASK_PRIORITY) {
 		byteCount += pixels; // _priorityScreen
 	}
-	if (mask & SCI_SCREEN_MASK_CONTROL) {
+	if (mask & GFX_SCREEN_MASK_CONTROL) {
 		byteCount += pixels; // _controlScreen
 	}
-	if (mask & SCI_SCREEN_MASK_DISPLAY) {
+	if (mask & GFX_SCREEN_MASK_DISPLAY) {
 		if (!_upscaledHires)
 			error("bitsGetDataSize() called w/o being in upscaled hires mode");
 		byteCount += pixels; // _displayScreen (coordinates actually are given to us for hires displayScreen)
@@ -289,17 +385,17 @@ void GfxScreen::bitsSave(Common::Rect rect, byte mask, byte *memoryPtr) {
 	memcpy(memoryPtr, (void *)&rect, sizeof(rect)); memoryPtr += sizeof(rect);
 	memcpy(memoryPtr, (void *)&mask, sizeof(mask)); memoryPtr += sizeof(mask);
 
-	if (mask & SCI_SCREEN_MASK_VISUAL) {
+	if (mask & GFX_SCREEN_MASK_VISUAL) {
 		bitsSaveScreen(rect, _visualScreen, _width, memoryPtr);
 		bitsSaveDisplayScreen(rect, memoryPtr);
 	}
-	if (mask & SCI_SCREEN_MASK_PRIORITY) {
+	if (mask & GFX_SCREEN_MASK_PRIORITY) {
 		bitsSaveScreen(rect, _priorityScreen, _width, memoryPtr);
 	}
-	if (mask & SCI_SCREEN_MASK_CONTROL) {
+	if (mask & GFX_SCREEN_MASK_CONTROL) {
 		bitsSaveScreen(rect, _controlScreen, _width, memoryPtr);
 	}
-	if (mask & SCI_SCREEN_MASK_DISPLAY) {
+	if (mask & GFX_SCREEN_MASK_DISPLAY) {
 		if (!_upscaledHires)
 			error("bitsSave() called w/o being in upscaled hires mode");
 		bitsSaveScreen(rect, _displayScreen, _displayWidth, memoryPtr);
@@ -326,9 +422,10 @@ void GfxScreen::bitsSaveDisplayScreen(Common::Rect rect, byte *&memoryPtr) {
 	if (!_upscaledHires) {
 		screen += (rect.top * _displayWidth) + rect.left;
 	} else {
-		screen += (rect.top * 2 * _displayWidth) + rect.left * 2;
+		screen += (_upscaledMapping[rect.top] * _displayWidth) + rect.left * 2;
 		width *= 2;
-		rect.top *= 2; rect.bottom *= 2;
+		rect.top = _upscaledMapping[rect.top];
+		rect.bottom = _upscaledMapping[rect.bottom];
 	}
 
 	for (y = rect.top; y < rect.bottom; y++) {
@@ -348,20 +445,25 @@ void GfxScreen::bitsRestore(byte *memoryPtr) {
 	memcpy((void *)&rect, memoryPtr, sizeof(rect)); memoryPtr += sizeof(rect);
 	memcpy((void *)&mask, memoryPtr, sizeof(mask)); memoryPtr += sizeof(mask);
 
-	if (mask & SCI_SCREEN_MASK_VISUAL) {
+	if (mask & GFX_SCREEN_MASK_VISUAL) {
 		bitsRestoreScreen(rect, memoryPtr, _visualScreen, _width);
 		bitsRestoreDisplayScreen(rect, memoryPtr);
 	}
-	if (mask & SCI_SCREEN_MASK_PRIORITY) {
+	if (mask & GFX_SCREEN_MASK_PRIORITY) {
 		bitsRestoreScreen(rect, memoryPtr, _priorityScreen, _width);
 	}
-	if (mask & SCI_SCREEN_MASK_CONTROL) {
+	if (mask & GFX_SCREEN_MASK_CONTROL) {
 		bitsRestoreScreen(rect, memoryPtr, _controlScreen, _width);
 	}
-	if (mask & SCI_SCREEN_MASK_DISPLAY) {
+	if (mask & GFX_SCREEN_MASK_DISPLAY) {
 		if (!_upscaledHires)
 			error("bitsRestore() called w/o being in upscaled hires mode");
 		bitsRestoreScreen(rect, memoryPtr, _displayScreen, _displayWidth);
+		// WORKAROUND - we are not sure what sierra is doing. If we don't do this here, portraits won't get fully removed
+		//  from screen. Some lowres showBits() call is used for that and it's not covering the whole area
+		//  We would need to find out inside the kq6 windows interpreter, but this here works already and seems not to have
+		//  any side-effects. The whole hires is hacked into the interpreter, so maybe this is even right.
+		copyDisplayRectToScreen(rect);
 	}
 }
 
@@ -385,9 +487,10 @@ void GfxScreen::bitsRestoreDisplayScreen(Common::Rect rect, byte *&memoryPtr) {
 	if (!_upscaledHires) {
 		screen += (rect.top * _displayWidth) + rect.left;
 	} else {
-		screen += (rect.top * 2 * _displayWidth) + rect.left * 2;
+		screen += (_upscaledMapping[rect.top] * _displayWidth) + rect.left * 2;
 		width *= 2;
-		rect.top *= 2; rect.bottom *= 2;
+		rect.top = _upscaledMapping[rect.top];
+		rect.bottom = _upscaledMapping[rect.bottom];
 	}
 
 	for (y = rect.top; y < rect.bottom; y++) {
@@ -396,7 +499,19 @@ void GfxScreen::bitsRestoreDisplayScreen(Common::Rect rect, byte *&memoryPtr) {
 	}
 }
 
-void GfxScreen::setPalette(Palette*pal) {
+void GfxScreen::getPalette(Palette *pal) {
+	// just copy palette to system
+	byte bpal[4 * 256];
+	// Get current palette, update it and put back
+	g_system->grabPalette(bpal, 0, 256);
+	for (int16 i = 1; i < 255; i++) {
+		pal->colors[i].r = bpal[i * 4];
+		pal->colors[i].g = bpal[i * 4 + 1];
+		pal->colors[i].b = bpal[i * 4 + 2];
+	}
+}
+
+void GfxScreen::setPalette(Palette *pal) {
 	// just copy palette to system
 	byte bpal[4 * 256];
 	// Get current palette, update it and put back
@@ -419,6 +534,22 @@ void GfxScreen::setVerticalShakePos(uint16 shakePos) {
 		g_system->setShakePos(shakePos * 2);
 }
 
+void GfxScreen::kernelShakeScreen(uint16 shakeCount, uint16 directions) {
+	while (shakeCount--) {
+		if (directions & SCI_SHAKE_DIRECTION_VERTICAL)
+			setVerticalShakePos(10);
+		// TODO: horizontal shakes
+		g_system->updateScreen();
+		g_sci->getEngineState()->wait(3);
+
+		if (directions & SCI_SHAKE_DIRECTION_VERTICAL)
+			setVerticalShakePos(0);
+
+		g_system->updateScreen();
+		g_sci->getEngineState()->wait(3);
+	}
+}
+
 void GfxScreen::dither(bool addToFlag) {
 	int y, x;
 	byte color, ditheredColor;
@@ -518,19 +649,43 @@ void GfxScreen::debugShowMap(int mapNo) {
 	copyToScreen();
 }
 
-void GfxScreen::scale2x(byte *src, byte *dst, int16 srcWidth, int16 srcHeight) {
-	int newWidth = srcWidth * 2;
-	byte *srcPtr = src;
+void GfxScreen::scale2x(const byte *src, byte *dst, int16 srcWidth, int16 srcHeight) {
+	const int newWidth = srcWidth * 2;
+	const byte *srcPtr = src;
 
 	for (int y = 0; y < srcHeight; y++) {
 		for (int x = 0; x < srcWidth; x++) {
-			int destOffset = y * 2 * newWidth + x * 2;
-			dst[destOffset] = *srcPtr;
-			dst[destOffset + 1] = *srcPtr;
-			dst[destOffset + newWidth] = *srcPtr;
-			dst[destOffset + newWidth + 1] = *srcPtr;
-			srcPtr++;
+			const byte color = *srcPtr++;
+			dst[0] = color;
+			dst[1] = color;
+			dst[newWidth] = color;
+			dst[newWidth + 1] = color;
+			dst += 2;
 		}
+		dst += newWidth;
+	}
+}
+
+void GfxScreen::adjustToUpscaledCoordinates(int16 &y, int16 &x) {
+	x *= 2;
+	y = _upscaledMapping[y];
+}
+
+void GfxScreen::adjustBackUpscaledCoordinates(int16 &y, int16 &x) {
+	switch (_upscaledHires) {
+	case GFX_SCREEN_UPSCALED_640x400:
+		x /= 2;
+		y /= 2;
+		break;
+	case GFX_SCREEN_UPSCALED_640x440:
+		x /= 2;
+		y = (y * 5) / 11;
+		break;
+	case GFX_SCREEN_UPSCALED_640x480:
+		x /= 2;
+		y = (y * 5) / 12;
+	default:
+		break;
 	}
 }
 
