@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sci/graphics/transitions.cpp $
- * $Id: transitions.cpp 48084 2010-02-17 23:37:32Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sci/graphics/transitions.cpp $
+ * $Id: transitions.cpp 52436 2010-08-29 10:39:34Z m_kiewitz $
  *
  */
 
@@ -31,15 +31,14 @@
 
 #include "sci/sci.h"
 #include "sci/engine/state.h"
-#include "sci/graphics/gui.h"
 #include "sci/graphics/screen.h"
 #include "sci/graphics/palette.h"
 #include "sci/graphics/transitions.h"
 
 namespace Sci {
 
-GfxTransitions::GfxTransitions(SciGui *gui, GfxScreen *screen, GfxPalette *palette, bool isVGA)
-	: _gui(gui), _screen(screen), _palette(palette), _isVGA(isVGA) {
+GfxTransitions::GfxTransitions(GfxScreen *screen, GfxPalette *palette, bool isVGA)
+	: _screen(screen), _palette(palette), _isVGA(isVGA) {
 	init();
 }
 
@@ -122,11 +121,24 @@ void GfxTransitions::setup(int16 number, bool blackoutFlag) {
 	}
 }
 
-void GfxTransitions::updateScreenAndWait(int msec) {
+bool GfxTransitions::doCreateFrame(uint32 shouldBeAtMsec) {
+	uint32 msecPos = g_system->getMillis() - _transitionStartTime;
+
+	if (shouldBeAtMsec > msecPos)
+		return true;
+	return false;
+}
+
+void GfxTransitions::updateScreenAndWait(uint32 shouldBeAtMsec) {
 	Common::Event ev;
-	g_system->updateScreen();
-	g_system->delayMillis(msec);
+
 	while (g_system->getEventManager()->pollEvent(ev)) {}	// discard all events
+
+	g_system->updateScreen();
+	// if we have still some time left, delay accordingly
+	uint32 msecPos = g_system->getMillis() - _transitionStartTime;
+	if (shouldBeAtMsec > msecPos)
+		g_system->delayMillis(shouldBeAtMsec - msecPos);
 }
 
 // will translate a number and return corresponding translationEntry
@@ -159,7 +171,8 @@ void GfxTransitions::doit(Common::Rect picRect) {
 	}
 
 	if (_blackoutFlag) {
-		// We need to find out what transition we are supposed to use for blackout
+		// We need to find out what transition we are supposed to use for
+		// blackout
 		translationEntry = translateNumber(_number, blackoutTransitionIDs);
 		if (translationEntry) {
 			doTransition(translationEntry->newId, true);
@@ -168,12 +181,15 @@ void GfxTransitions::doit(Common::Rect picRect) {
 		}
 	}
 
+	_palette->palVaryPrepareForTransition();
+
 	// Now we do the actual transition to the new screen
 	doTransition(_number, false);
 
 	if (picRect.bottom != _screen->getHeight()) {
 		// TODO: this is a workaround for lsl6 not showing menubar when playing
-		//  There is some new code in the sierra sci in ShowPic that seems to do something similar to this
+		//  There is some new code in the sierra sci in ShowPic that seems to do
+		//  something similar to this
 		_screen->copyToScreen();
 		g_system->updateScreen();
 	}
@@ -181,19 +197,20 @@ void GfxTransitions::doit(Common::Rect picRect) {
 	_screen->_picNotValid = 0;
 }
 
-// This may get called twice, if blackoutFlag is set. It will get once called with blackoutFlag set and another time
-//  with no blackoutFlag.
+// This may get called twice, if blackoutFlag is set. It will get once called
+// with blackoutFlag set and another time with no blackoutFlag.
 void GfxTransitions::doTransition(int16 number, bool blackoutFlag) {
 	if (number != SCI_TRANSITIONS_FADEPALETTE) {
 		setNewPalette(blackoutFlag);
 	}
 
+	_transitionStartTime = g_system->getMillis();
 	switch (number) {
 	case SCI_TRANSITIONS_VERTICALROLL_FROMCENTER:
 		verticalRollFromCenter(blackoutFlag);
 		break;
 	case SCI_TRANSITIONS_VERTICALROLL_TOCENTER:
-		verticalRollFromCenter(blackoutFlag);
+		verticalRollToCenter(blackoutFlag);
 		break;
 	case SCI_TRANSITIONS_HORIZONTALROLL_FROMCENTER:
 		horizontalRollFromCenter(blackoutFlag);
@@ -265,44 +282,60 @@ void GfxTransitions::copyRectToScreen(const Common::Rect rect, bool blackoutFlag
 		_screen->copyRectToScreen(rect);
 	} else {
 		Graphics::Surface *surface = g_system->lockScreen();
-		surface->fillRect(rect, 0);
+		if (!_screen->getUpscaledHires()) {
+			surface->fillRect(rect, 0);
+		} else {
+			Common::Rect upscaledRect = rect;
+			_screen->adjustToUpscaledCoordinates(upscaledRect.top, upscaledRect.left);
+			_screen->adjustToUpscaledCoordinates(upscaledRect.bottom, upscaledRect.right);
+			surface->fillRect(upscaledRect, 0);
+		}
 		g_system->unlockScreen();
 	}
 }
 
-// Note: dont do too many steps in here, otherwise cpu will crap out because of the load
+// Note: don't do too many steps in here, otherwise cpu will crap out because of
+// the load
 void GfxTransitions::fadeOut() {
 	byte oldPalette[4 * 256], workPalette[4 * 256];
 	int16 stepNr, colorNr;
+	// Sierra did not fade in/out color 255 for sci1.1, but they used it in
+	//  several pictures (e.g. qfg3 demo/intro), so the fading looked weird
+	int16 tillColorNr = getSciVersion() >= SCI_VERSION_1_1 ? 256 : 255;
 
 	g_system->grabPalette(oldPalette, 0, 256);
 
 	for (stepNr = 100; stepNr >= 0; stepNr -= 10) {
-		for (colorNr = 1; colorNr < 255; colorNr++){
+		for (colorNr = 1; colorNr < tillColorNr; colorNr++){
 			workPalette[colorNr * 4 + 0] = oldPalette[colorNr * 4] * stepNr / 100;
 			workPalette[colorNr * 4 + 1] = oldPalette[colorNr * 4 + 1] * stepNr / 100;
 			workPalette[colorNr * 4 + 2] = oldPalette[colorNr * 4 + 2] * stepNr / 100;
 		}
 		g_system->setPalette(workPalette + 4, 1, 254);
-		_gui->wait(2);
+		g_sci->getEngineState()->wait(2);
 	}
 }
 
-// Note: dont do too many steps in here, otherwise cpu will crap out because of the load
+// Note: don't do too many steps in here, otherwise cpu will crap out because of
+// the load
 void GfxTransitions::fadeIn() {
 	int16 stepNr;
+	// Sierra did not fade in/out color 255 for sci1.1, but they used it in
+	//  several pictures (e.g. qfg3 demo/intro), so the fading looked weird
+	int16 tillColorNr = getSciVersion() >= SCI_VERSION_1_1 ? 256 : 255;
 
 	for (stepNr = 0; stepNr <= 100; stepNr += 10) {
-		_palette->kernelSetIntensity(1, 255, stepNr, true);
-		_gui->wait(2);
+		_palette->kernelSetIntensity(1, tillColorNr, stepNr, true);
+		g_sci->getEngineState()->wait(2);
 	}
 }
 
-// pixelates the new picture over the old one - works against the whole screen
+// Pixelates the new picture over the old one - works against the whole screen.
 // TODO: it seems this needs to get applied on _picRect only if possible
-void GfxTransitions::pixelation (bool blackoutFlag) {
+void GfxTransitions::pixelation(bool blackoutFlag) {
 	uint16 mask = 0x40, stepNr = 0;
 	Common::Rect pixelRect;
+	uint32 msecCount = 0;
 
 	do {
 		mask = (mask & 1) ? (mask >> 1) ^ 0xB400 : mask >> 1;
@@ -314,17 +347,19 @@ void GfxTransitions::pixelation (bool blackoutFlag) {
 		if (!pixelRect.isEmpty())
 			copyRectToScreen(pixelRect, blackoutFlag);
 		if ((stepNr & 0x3FF) == 0) {
-			updateScreenAndWait(5);
+			msecCount += 9;
+			updateScreenAndWait(msecCount);
 		}
 		stepNr++;
 	} while (mask != 0x40);
 }
 
-// like pixelation but uses 8x8 blocks - works against the whole screen
+// Like pixelation but uses 8x8 blocks - works against the whole screen.
 // TODO: it seems this needs to get applied on _picRect only if possible
 void GfxTransitions::blocks(bool blackoutFlag) {
 	uint16 mask = 0x40, stepNr = 0;
 	Common::Rect blockRect;
+	uint32 msecCount = 0;
 
 	do {
 		mask = (mask & 1) ? (mask >> 1) ^ 0x240 : mask >> 1;
@@ -336,16 +371,19 @@ void GfxTransitions::blocks(bool blackoutFlag) {
 		if (!blockRect.isEmpty())
 			copyRectToScreen(blockRect, blackoutFlag);
 		if ((stepNr & 7) == 0) {
-			updateScreenAndWait(4);
+			msecCount += 5;
+			updateScreenAndWait(msecCount);
 		}
 		stepNr++;
 	} while (mask != 0x40);
 }
 
-// directly shows new screen starting up/down/left/right and going to the opposite direction - works on _picRect area only
+// Directly shows new screen starting up/down/left/right and going to the
+// opposite direction - works on _picRect area only
 void GfxTransitions::straight(int16 number, bool blackoutFlag) {
 	int16 stepNr = 0;
 	Common::Rect newScreenRect = _picRect;
+	uint32 msecCount = 0;
 
 	switch (number) {
 	case SCI_TRANSITIONS_STRAIGHT_FROM_RIGHT:
@@ -353,7 +391,8 @@ void GfxTransitions::straight(int16 number, bool blackoutFlag) {
 		while (newScreenRect.left >= _picRect.left) {
 			copyRectToScreen(newScreenRect, blackoutFlag);
 			if ((stepNr & 1) == 0) {
-				updateScreenAndWait(1);
+				msecCount += 2;
+				updateScreenAndWait(msecCount);
 			}
 			stepNr++;
 			newScreenRect.translate(-1, 0);
@@ -365,7 +404,8 @@ void GfxTransitions::straight(int16 number, bool blackoutFlag) {
 		while (newScreenRect.right <= _picRect.right) {
 			copyRectToScreen(newScreenRect, blackoutFlag);
 			if ((stepNr & 1) == 0) {
-				updateScreenAndWait(1);
+				msecCount += 2;
+				updateScreenAndWait(msecCount);
 			}
 			stepNr++;
 			newScreenRect.translate(1, 0);
@@ -376,7 +416,8 @@ void GfxTransitions::straight(int16 number, bool blackoutFlag) {
 		newScreenRect.top = newScreenRect.bottom - 1;
 		while (newScreenRect.top >= _picRect.top) {
 			copyRectToScreen(newScreenRect, blackoutFlag);
-			updateScreenAndWait(3);
+			msecCount += 4;
+			updateScreenAndWait(msecCount);
 			stepNr++;
 			newScreenRect.translate(0, -1);
 		}
@@ -386,7 +427,8 @@ void GfxTransitions::straight(int16 number, bool blackoutFlag) {
 		newScreenRect.bottom = newScreenRect.top + 1;
 		while (newScreenRect.bottom <= _picRect.bottom) {
 			copyRectToScreen(newScreenRect, blackoutFlag);
-			updateScreenAndWait(3);
+			msecCount += 4;
+			updateScreenAndWait(msecCount);
 			stepNr++;
 			newScreenRect.translate(0, 1);
 		}
@@ -394,87 +436,115 @@ void GfxTransitions::straight(int16 number, bool blackoutFlag) {
 	}
 }
 
-// scroll old screen (up/down/left/right) and insert new screen that way - works on _picRect area only
+void GfxTransitions::scrollCopyOldToScreen(Common::Rect screenRect, int16 x, int16 y) {
+	byte *oldScreenPtr = _oldScreen;
+	int16 screenWidth = _screen->getDisplayWidth();
+	if (_screen->getUpscaledHires()) {
+		_screen->adjustToUpscaledCoordinates(screenRect.top, screenRect.left);
+		_screen->adjustToUpscaledCoordinates(screenRect.bottom, screenRect.right);
+		_screen->adjustToUpscaledCoordinates(y, x);
+	}
+	oldScreenPtr += screenRect.left + screenRect.top * screenWidth;
+	g_system->copyRectToScreen(oldScreenPtr, screenWidth, x, y, screenRect.width(), screenRect.height());
+}
+
+// Scroll old screen (up/down/left/right) and insert new screen that way - works
+// on _picRect area only.
 void GfxTransitions::scroll(int16 number) {
 	int16 screenWidth, screenHeight;
-	byte *oldScreenPtr;
 	int16 stepNr = 0;
 	Common::Rect oldMoveRect = _picRect;
+	Common::Rect oldScreenRect = _picRect;
 	Common::Rect newMoveRect = _picRect;
 	Common::Rect newScreenRect = _picRect;
+	uint32 msecCount = 0;
 
 	_screen->copyFromScreen(_oldScreen);
 	screenWidth = _screen->getDisplayWidth(); screenHeight = _screen->getDisplayHeight();
 
-	oldScreenPtr = _oldScreen + _picRect.left + _picRect.top * screenWidth;
-
 	switch (number) {
 	case SCI_TRANSITIONS_SCROLL_LEFT:
 		newScreenRect.right = newScreenRect.left;
 		newMoveRect.left = newMoveRect.right;
 		while (oldMoveRect.left < oldMoveRect.right) {
-			oldScreenPtr++; oldMoveRect.right--;
-			if (oldMoveRect.right > oldMoveRect.left)
-				g_system->copyRectToScreen(oldScreenPtr, screenWidth, oldMoveRect.left, oldMoveRect.top, oldMoveRect.width(), oldMoveRect.height());
+			oldMoveRect.right--; oldScreenRect.left++;
 			newScreenRect.right++; newMoveRect.left--;
-			_screen->copyRectToScreen(newScreenRect, newMoveRect.left, newMoveRect.top);
 			if ((stepNr & 1) == 0) {
-				updateScreenAndWait(1);
+				msecCount += 5;
+				if (doCreateFrame(msecCount)) {
+					if (oldMoveRect.right > oldMoveRect.left)
+						scrollCopyOldToScreen(oldScreenRect, oldMoveRect.left, oldMoveRect.top);
+					_screen->copyRectToScreen(newScreenRect, newMoveRect.left, newMoveRect.top);
+					updateScreenAndWait(msecCount);
+				}
 			}
 			stepNr++;
 		}
-		if ((stepNr & 1) == 0)
-			g_system->updateScreen();
 		break;
 
 	case SCI_TRANSITIONS_SCROLL_RIGHT:
 		newScreenRect.left = newScreenRect.right;
 		while (oldMoveRect.left < oldMoveRect.right) {
-			oldMoveRect.left++;
-			if (oldMoveRect.right > oldMoveRect.left)
-				g_system->copyRectToScreen(oldScreenPtr, screenWidth, oldMoveRect.left, oldMoveRect.top, oldMoveRect.width(), oldMoveRect.height());
+			oldMoveRect.left++; oldScreenRect.right--;
 			newScreenRect.left--;
-			_screen->copyRectToScreen(newScreenRect, newMoveRect.left, newMoveRect.top);
 			if ((stepNr & 1) == 0) {
-				updateScreenAndWait(1);
+				msecCount += 5;
+				if (doCreateFrame(msecCount)) {
+					if (oldMoveRect.right > oldMoveRect.left)
+						scrollCopyOldToScreen(oldScreenRect, oldMoveRect.left, oldMoveRect.top);
+					_screen->copyRectToScreen(newScreenRect, newMoveRect.left, newMoveRect.top);
+					updateScreenAndWait(msecCount);
+				}
 			}
 			stepNr++;
 		}
-		if ((stepNr & 1) == 0)
-			g_system->updateScreen();
 		break;
 
 	case SCI_TRANSITIONS_SCROLL_UP:
 		newScreenRect.bottom = newScreenRect.top;
 		newMoveRect.top = newMoveRect.bottom;
 		while (oldMoveRect.top < oldMoveRect.bottom) {
-			oldScreenPtr += screenWidth; oldMoveRect.top++;
-			if (oldMoveRect.top < oldMoveRect.bottom)
-				g_system->copyRectToScreen(oldScreenPtr, screenWidth, _picRect.left, _picRect.top, oldMoveRect.width(), oldMoveRect.height());
+			oldMoveRect.top++; oldScreenRect.top++; 
 			newScreenRect.bottom++;	newMoveRect.top--;
-			_screen->copyRectToScreen(newScreenRect, newMoveRect.left, newMoveRect.top);
-			updateScreenAndWait(3);
+
+			msecCount += 5;
+			if (doCreateFrame(msecCount)) {
+				if (oldMoveRect.top < oldMoveRect.bottom)
+					scrollCopyOldToScreen(oldScreenRect, _picRect.left, _picRect.top);
+				_screen->copyRectToScreen(newScreenRect, newMoveRect.left, newMoveRect.top);
+				updateScreenAndWait(msecCount);
+			}
 		}
 		break;
 
 	case SCI_TRANSITIONS_SCROLL_DOWN:
 		newScreenRect.top = newScreenRect.bottom;
 		while (oldMoveRect.top < oldMoveRect.bottom) {
-			oldMoveRect.top++;
-			if (oldMoveRect.top < oldMoveRect.bottom)
-				g_system->copyRectToScreen(oldScreenPtr, screenWidth, oldMoveRect.left, oldMoveRect.top, oldMoveRect.width(), oldMoveRect.height());
+			oldMoveRect.top++; oldScreenRect.bottom--;
 			newScreenRect.top--;
-			_screen->copyRectToScreen(newScreenRect, _picRect.left, _picRect.top);
-			updateScreenAndWait(3);
+
+			msecCount += 5;
+			if (doCreateFrame(msecCount)) {
+				if (oldMoveRect.top < oldMoveRect.bottom)
+					scrollCopyOldToScreen(oldScreenRect, oldMoveRect.left, oldMoveRect.top);
+				_screen->copyRectToScreen(newScreenRect, _picRect.left, _picRect.top);
+				updateScreenAndWait(msecCount);
+			}
 		}
 		break;
 	}
+
+	// Copy over final position just in case
+	_screen->copyRectToScreen(newScreenRect);
+	g_system->updateScreen();
 }
 
-// vertically displays new screen starting from center - works on _picRect area only
+// Vertically displays new screen starting from center - works on _picRect area
+// only
 void GfxTransitions::verticalRollFromCenter(bool blackoutFlag) {
 	Common::Rect leftRect = Common::Rect(_picRect.left + (_picRect.width() / 2) -1, _picRect.top, _picRect.left + (_picRect.width() / 2), _picRect.bottom);
 	Common::Rect rightRect = Common::Rect(leftRect.right, _picRect.top, leftRect.right + 1, _picRect.bottom);
+	uint32 msecCount = 0;
 
 	while ((leftRect.left >= _picRect.left) || (rightRect.right <= _picRect.right)) {
 		if (leftRect.left < _picRect.left)
@@ -483,26 +553,32 @@ void GfxTransitions::verticalRollFromCenter(bool blackoutFlag) {
 			rightRect.translate(-1, 0);
 		copyRectToScreen(leftRect, blackoutFlag); leftRect.translate(-1, 0);
 		copyRectToScreen(rightRect, blackoutFlag); rightRect.translate(1, 0);
-		updateScreenAndWait(2);
+		msecCount += 3;
+		updateScreenAndWait(msecCount);
 	}
 }
 
-// vertically displays new screen starting from edges - works on _picRect area only
+// Vertically displays new screen starting from edges - works on _picRect area
+// only
 void GfxTransitions::verticalRollToCenter(bool blackoutFlag) {
 	Common::Rect leftRect = Common::Rect(_picRect.left, _picRect.top, _picRect.left + 1, _picRect.bottom);
-	Common::Rect rightRect = Common::Rect(leftRect.right - 1, _picRect.top, leftRect.right, _picRect.bottom);
+	Common::Rect rightRect = Common::Rect(_picRect.right - 1, _picRect.top, _picRect.right, _picRect.bottom);
+	uint32 msecCount = 0;
 
 	while (leftRect.left < rightRect.right) {
 		copyRectToScreen(leftRect, blackoutFlag); leftRect.translate(1, 0);
 		copyRectToScreen(rightRect, blackoutFlag); rightRect.translate(-1, 0);
-		updateScreenAndWait(2);
+		msecCount += 3;
+		updateScreenAndWait(msecCount);
 	}
 }
 
-// horizontally displays new screen starting from center - works on _picRect area only
+// Horizontally displays new screen starting from center - works on _picRect
+// area only
 void GfxTransitions::horizontalRollFromCenter(bool blackoutFlag) {
 	Common::Rect upperRect = Common::Rect(_picRect.left, _picRect.top + (_picRect.height() / 2) - 1, _picRect.right, _picRect.top + (_picRect.height() / 2));
 	Common::Rect lowerRect = Common::Rect(upperRect.left, upperRect.bottom, upperRect.right, upperRect.bottom + 1);
+	uint32 msecCount = 0;
 
 	while ((upperRect.top >= _picRect.top) || (lowerRect.bottom <= _picRect.bottom)) {
 		if (upperRect.top < _picRect.top)
@@ -511,30 +587,35 @@ void GfxTransitions::horizontalRollFromCenter(bool blackoutFlag) {
 			lowerRect.translate(0, -1);
 		copyRectToScreen(upperRect, blackoutFlag); upperRect.translate(0, -1);
 		copyRectToScreen(lowerRect, blackoutFlag); lowerRect.translate(0, 1);
-		updateScreenAndWait(3);
+		msecCount += 4;
+		updateScreenAndWait(msecCount);
 	}
 }
 
-// horizontally displays new screen starting from upper and lower edge - works on _picRect area only
+// Horizontally displays new screen starting from upper and lower edge - works
+// on _picRect area only
 void GfxTransitions::horizontalRollToCenter(bool blackoutFlag) {
 	Common::Rect upperRect = Common::Rect(_picRect.left, _picRect.top, _picRect.right, _picRect.top + 1);
 	Common::Rect lowerRect = Common::Rect(upperRect.left, _picRect.bottom - 1, upperRect.right, _picRect.bottom);
+	uint32 msecCount = 0;
 
 	while (upperRect.top < lowerRect.bottom) {
 		copyRectToScreen(upperRect, blackoutFlag); upperRect.translate(0, 1);
 		copyRectToScreen(lowerRect, blackoutFlag); lowerRect.translate(0, -1);
-		updateScreenAndWait(3);
+		msecCount += 4;
+		updateScreenAndWait(msecCount);
 	}
 }
 
-// diagonally displays new screen starting from center - works on _picRect area only
-//  assumes that height of rect is larger than width
+// Diagonally displays new screen starting from center - works on _picRect area
+// only. Assumes that height of rect is larger than width.
 void GfxTransitions::diagonalRollFromCenter(bool blackoutFlag) {
 	int16 halfHeight = _picRect.height() / 2;
 	Common::Rect upperRect(_picRect.left + halfHeight - 2, _picRect.top + halfHeight, _picRect.right - halfHeight + 1, _picRect.top + halfHeight + 1);
 	Common::Rect lowerRect(upperRect.left, upperRect.top, upperRect.right, upperRect.bottom);
 	Common::Rect leftRect(upperRect.left, upperRect.top, upperRect.left + 1, lowerRect.bottom);
 	Common::Rect rightRect(upperRect.right, upperRect.top, upperRect.right + 1, lowerRect.bottom);
+	uint32 msecCount = 0;
 
 	while ((upperRect.top >= _picRect.top) || (lowerRect.bottom <= _picRect.bottom)) {
 		if (upperRect.top < _picRect.top) {
@@ -553,24 +634,27 @@ void GfxTransitions::diagonalRollFromCenter(bool blackoutFlag) {
 		copyRectToScreen(lowerRect, blackoutFlag); lowerRect.translate(0, 1); lowerRect.left--; lowerRect.right++;
 		copyRectToScreen(leftRect, blackoutFlag); leftRect.translate(-1, 0);	leftRect.top--; leftRect.bottom++;
 		copyRectToScreen(rightRect, blackoutFlag); rightRect.translate(1, 0); rightRect.top--; rightRect.bottom++;
-		updateScreenAndWait(3);
+		msecCount += 4;
+		updateScreenAndWait(msecCount);
 	}
 }
 
-// diagonally displays new screen starting from edges - works on _picRect area only
-//  assumes that height of rect is larger than width
+// Diagonally displays new screen starting from edges - works on _picRect area
+// only. Assumes that height of rect is larger than width.
 void GfxTransitions::diagonalRollToCenter(bool blackoutFlag) {
 	Common::Rect upperRect(_picRect.left, _picRect.top, _picRect.right, _picRect.top + 1);
 	Common::Rect lowerRect(_picRect.left, _picRect.bottom - 1, _picRect.right, _picRect.bottom);
 	Common::Rect leftRect(_picRect.left, _picRect.top, _picRect.left + 1, _picRect.bottom);
 	Common::Rect rightRect(_picRect.right - 1, _picRect.top, _picRect.right, _picRect.bottom);
+	uint32 msecCount = 0;
 
 	while (upperRect.top < lowerRect.bottom) {
 		copyRectToScreen(upperRect, blackoutFlag); upperRect.translate(0, 1); upperRect.left++; upperRect.right--;
 		copyRectToScreen(lowerRect, blackoutFlag); lowerRect.translate(0, -1); lowerRect.left++; lowerRect.right--;
 		copyRectToScreen(leftRect, blackoutFlag); leftRect.translate(1, 0);
 		copyRectToScreen(rightRect, blackoutFlag); rightRect.translate(-1, 0);
-		updateScreenAndWait(3);
+		msecCount += 4;
+		updateScreenAndWait(msecCount);
 	}
 }
 
