@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sci/sound/drivers/adlib.cpp $
- * $Id: adlib.cpp 48086 2010-02-17 23:38:43Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sci/sound/drivers/adlib.cpp $
+ * $Id: adlib.cpp 52726 2010-09-14 21:53:22Z lordhoto $
  *
  */
 
@@ -73,6 +73,8 @@ public:
 	bool loadResource(const byte *data, uint size);
 	virtual uint32 property(int prop, uint32 param);
 
+	bool useRhythmChannel() const { return _rhythmKeyMap != NULL; }
+
 private:
 	enum ChannelID {
 		kLeftChannel = 1,
@@ -171,12 +173,14 @@ public:
 	int open(ResourceManager *resMan);
 	void close();
 
-	byte getPlayId();
+	byte getPlayId() const;
 	int getPolyphony() const { return MidiDriver_AdLib::kVoices; }
 	bool hasRhythmChannel() const { return false; }
 	void setVolume(byte volume) { static_cast<MidiDriver_AdLib *>(_driver)->setVolume(volume); }
 	void playSwitch(bool play) { static_cast<MidiDriver_AdLib *>(_driver)->playSwitch(play); }
 	void loadInstrument(int idx, byte *data);
+
+	int getLastChannel() const { return (static_cast<const MidiDriver_AdLib *>(_driver)->useRhythmChannel() ? 8 : 15); }
 };
 
 static const byte registerOffset[MidiDriver_AdLib::kVoices] = {
@@ -238,7 +242,7 @@ int MidiDriver_AdLib::open(bool isSCI0) {
 
 	MidiDriver_Emulated::open();
 
-	_mixer->playInputStream(Audio::Mixer::kPlainSoundType, &_mixerSoundHandle, this, -1, _mixer->kMaxChannelVolume, 0, DisposeAfterUse::NO);
+	_mixer->playStream(Audio::Mixer::kPlainSoundType, &_mixerSoundHandle, this, -1, _mixer->kMaxChannelVolume, 0, DisposeAfterUse::NO);
 
 	return 0;
 }
@@ -586,7 +590,7 @@ void MidiDriver_AdLib::voiceOn(int voice, int note, int velocity) {
 	}
 
 	// Set patch if different from current patch
-	if ((patch != _voices[voice].patch) && _playSwitch)
+	if (patch != _voices[voice].patch)
 		setPatch(voice, patch);
 
 	_voices[voice].velocity = velocity;
@@ -704,6 +708,7 @@ void MidiDriver_AdLib::setVelocityReg(int regOffset, int velocity, int kbScaleLe
 void MidiDriver_AdLib::setPatch(int voice, int patch) {
 	if ((patch < 0) || ((uint)patch >= _patches.size())) {
 		warning("ADLIB: Invalid patch %i requested", patch);
+		// Substitute instrument 0
 		patch = 0;
 	}
 
@@ -749,17 +754,24 @@ void MidiDriver_AdLib::playSwitch(bool play) {
 
 bool MidiDriver_AdLib::loadResource(const byte *data, uint size) {
 	if ((size != 1344) && (size != 2690) && (size != 5382)) {
-		warning("ADLIB: Unsupported patch format (%i bytes)", size);
+		error("ADLIB: Unsupported patch format (%i bytes)", size);
 		return false;
 	}
 
 	for (int i = 0; i < 48; i++)
 		loadInstrument(data + (28 * i));
 
-	if (size == 2690) {
+	if (size == 1344) {
+		byte dummy[28] = {0};
+
+		// Only 48 instruments, add dummies
+		for (int i = 0; i < 48; i++)
+			loadInstrument(dummy);
+	} else if (size == 2690) {
 		for (int i = 48; i < 96; i++)
 			loadInstrument(data + 2 + (28 * i));
-	} else if (size == 5382) {
+	} else {
+		// SCI1.1 and later
 		for (int i = 48; i < 190; i++)
 			loadInstrument(data + (28 * i));
 		_rhythmKeyMap = new byte[kRhythmKeys];
@@ -825,7 +837,7 @@ void MidiPlayer_AdLib::close() {
 	}
 }
 
-byte MidiPlayer_AdLib::getPlayId() {
+byte MidiPlayer_AdLib::getPlayId() const {
 	switch (_version) {
 	case SCI_VERSION_0_EARLY:
 		return 0x01;
