@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sci/sound/drivers/map-mt32-to-gm.h $
- * $Id: map-mt32-to-gm.h 47428 2010-01-22 01:01:08Z waltervn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sci/sound/drivers/map-mt32-to-gm.h $
+ * $Id: map-mt32-to-gm.h 52504 2010-09-03 08:46:07Z thebluegr $
  *
  */
 
@@ -421,131 +421,131 @@ static const Mt32ToGmMap Mt32MemoryTimbreMaps[] = {
 	{"Acou SD   ", MIDI_MAPPED_TO_RHYTHM, 38},   /* R (PQ2) */
 	{"AcouPnoKA ", 0, MIDI_UNMAPPED},     /* ++ (KQ1) */
 	{"BASS      ", 32, MIDI_UNMAPPED},    /* + (LSL3) */
-	{"BASSOONPCM", 70, MIDI_UNMAPPED},    /* + (CB) */
+	{"BASSOONPCM", 70, MIDI_UNMAPPED},    /* + (LB1) */
 	{"BEACH WAVE", 122, MIDI_UNMAPPED},   /* + (LSL3) */
 	{"BagPipes  ", 109, MIDI_UNMAPPED},
-	{"BassPizzMS", 45, MIDI_UNMAPPED},    /* ++ (HQ) */
+	{"BassPizzMS", 45, MIDI_UNMAPPED},    /* ++ (QFG1) */
 	{"BassoonKA ", 70, MIDI_UNMAPPED},    /* ++ (KQ1) */
-	{"Bell    MS", 112, MIDI_UNMAPPED},   /* ++ (iceMan) */
-	{"Bells   MS", 112, MIDI_UNMAPPED},   /* + (HQ) */
-	{"Big Bell  ", 14, MIDI_UNMAPPED},    /* + (CB) */
+	{"Bell    MS", 112, MIDI_UNMAPPED},   /* ++ (Iceman) */
+	{"Bells   MS", 112, MIDI_UNMAPPED},   /* + (QFG1) */
+	{"Big Bell  ", 14, MIDI_UNMAPPED},    /* + (LB1) */
 	{"Bird Tweet", 123, MIDI_UNMAPPED},
-	{"BrsSect MS", 61, MIDI_UNMAPPED},    /* +++ (iceMan) */
+	{"BrsSect MS", 61, MIDI_UNMAPPED},    /* +++ (Iceman) */
 	{"CLAPPING  ", 126, MIDI_UNMAPPED},   /* ++ (LSL3) */
-	{"Cabasa    ", MIDI_MAPPED_TO_RHYTHM, 69},   /* R (HBoG) */
-	{"Calliope  ", 82, MIDI_UNMAPPED},    /* +++ (HQ) */
-	{"CelticHarp", 46, MIDI_UNMAPPED},    /* ++ (CoC) */
-	{"Chicago MS", 1, MIDI_UNMAPPED},     /* ++ (iceMan) */
+	{"Cabasa    ", MIDI_MAPPED_TO_RHYTHM, 69},   /* R (Hoyle) */
+	{"Calliope  ", 82, MIDI_UNMAPPED},    /* +++ (QFG1) */
+	{"CelticHarp", 46, MIDI_UNMAPPED},    /* ++ (Camelot) */
+	{"Chicago MS", 1, MIDI_UNMAPPED},     /* ++ (Iceman) */
 	{"Chop      ", 117, MIDI_UNMAPPED},
-	{"Chorale MS", 52, MIDI_UNMAPPED},    /* + (CoC) */
+	{"Chorale MS", 52, MIDI_UNMAPPED},    /* + (Camelot) */
 	{"ClarinetMS", 71, MIDI_UNMAPPED},
 	{"Claves    ", MIDI_MAPPED_TO_RHYTHM, 75},   /* R (PQ2) */
-	{"Claw    MS", 118, MIDI_UNMAPPED},    /* + (HQ) */
-	{"ClockBell ", 14, MIDI_UNMAPPED},    /* + (CB) */
+	{"Claw    MS", 118, MIDI_UNMAPPED},    /* + (QFG1) */
+	{"ClockBell ", 14, MIDI_UNMAPPED},    /* + (LB1) */
 	{"ConcertCym", MIDI_MAPPED_TO_RHYTHM, 55},   /* R ? (KQ1) */
-	{"Conga   MS", MIDI_MAPPED_TO_RHYTHM, 64},   /* R (HQ) */
+	{"Conga   MS", MIDI_MAPPED_TO_RHYTHM, 64},   /* R (QFG1) */
 	{"CoolPhone ", 124, MIDI_UNMAPPED},   /* ++ (LSL3) */
-	{"CracklesMS", 115, MIDI_UNMAPPED}, /* ? (CoC, HQ) */
+	{"CracklesMS", 115, MIDI_UNMAPPED}, /* ? (Camelot, QFG1) */
 	{"CreakyD MS", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ??? (KQ1) */
-	{"Cricket   ", 120, MIDI_UNMAPPED}, /* ? (CB) */
-	{"CrshCymbMS", MIDI_MAPPED_TO_RHYTHM, 57},   /* R +++ (iceMan) */
-	{"CstlGateMS", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (HQ) */
-	{"CymSwellMS", MIDI_MAPPED_TO_RHYTHM, 55},   /* R ? (CoC, HQ) */
+	{"Cricket   ", 120, MIDI_UNMAPPED}, /* ? (LB1) */
+	{"CrshCymbMS", MIDI_MAPPED_TO_RHYTHM, 57},   /* R +++ (Iceman) */
+	{"CstlGateMS", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (QFG1) */
+	{"CymSwellMS", MIDI_MAPPED_TO_RHYTHM, 55},   /* R ? (Camelot, QFG1) */
 	{"CymbRollKA", MIDI_MAPPED_TO_RHYTHM, 57},   /* R ? (KQ1) */
 	{"Cymbal Lo ", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* R ? (LSL3) */
-	{"card      ", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (HBoG) */
-	{"DirtGtr MS", 30, MIDI_UNMAPPED},    /* + (iceMan) */
-	{"DirtGtr2MS", 29, MIDI_UNMAPPED},    /* + (iceMan) */
+	{"card      ", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (Hoyle) */
+	{"DirtGtr MS", 30, MIDI_UNMAPPED},    /* + (Iceman) */
+	{"DirtGtr2MS", 29, MIDI_UNMAPPED},    /* + (Iceman) */
 	{"E Bass  MS", 33, MIDI_UNMAPPED},    /* + (SQ3) */
 	{"ElecBassMS", 33, MIDI_UNMAPPED},
-	{"ElecGtr MS", 27, MIDI_UNMAPPED},    /* ++ (iceMan) */
+	{"ElecGtr MS", 27, MIDI_UNMAPPED},    /* ++ (Iceman) */
 	{"EnglHornMS", 69, MIDI_UNMAPPED},
 	{"FantasiaKA", 88, MIDI_UNMAPPED},
 	{"Fantasy   ", 99, MIDI_UNMAPPED},    /* + (PQ2) */
-	{"Fantasy2MS", 99, MIDI_UNMAPPED},    /* ++ (CoC, HQ) */
-	{"Filter  MS", 95, MIDI_UNMAPPED},    /* +++ (iceMan) */
-	{"Filter2 MS", 95, MIDI_UNMAPPED},    /* ++ (iceMan) */
-	{"Flame2  MS", 121, MIDI_UNMAPPED},   /* ? (HQ) */
-	{"Flames  MS", 121, MIDI_UNMAPPED},   /* ? (HQ) */
-	{"Flute   MS", 73, MIDI_UNMAPPED},    /* +++ (HQ) */
+	{"Fantasy2MS", 99, MIDI_UNMAPPED},    /* ++ (Camelot, QFG1) */
+	{"Filter  MS", 95, MIDI_UNMAPPED},    /* +++ (Iceman) */
+	{"Filter2 MS", 95, MIDI_UNMAPPED},    /* ++ (Iceman) */
+	{"Flame2  MS", 121, MIDI_UNMAPPED},   /* ? (QFG1) */
+	{"Flames  MS", 121, MIDI_UNMAPPED},   /* ? (QFG1) */
+	{"Flute   MS", 73, MIDI_UNMAPPED},    /* +++ (QFG1) */
 	{"FogHorn MS", 58, MIDI_UNMAPPED},
-	{"FrHorn1 MS", 60, MIDI_UNMAPPED},    /* +++ (HQ) */
-	{"FunnyTrmp ", 56, MIDI_UNMAPPED},    /* ++ (CB) */
+	{"FrHorn1 MS", 60, MIDI_UNMAPPED},    /* +++ (QFG1) */
+	{"FunnyTrmp ", 56, MIDI_UNMAPPED},    /* ++ (LB1) */
 	{"GameSnd MS", 80, MIDI_UNMAPPED},
-	{"Glock   MS", 9, MIDI_UNMAPPED},     /* +++ (HQ) */
-	{"Gunshot   ", 127, MIDI_UNMAPPED},   /* +++ (CB) */
-	{"Hammer  MS", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (HQ) */
-	{"Harmonica2", 22, MIDI_UNMAPPED},    /* +++ (CB) */
-	{"Harpsi 1  ", 6, MIDI_UNMAPPED},     /* + (HBoG) */
-	{"Harpsi 2  ", 6, MIDI_UNMAPPED},     /* +++ (CB) */
-	{"Heart   MS", 116, MIDI_UNMAPPED},   /* ? (iceMan) */
-	{"Horse1  MS", 115, MIDI_UNMAPPED},   /* ? (CoC, HQ) */
-	{"Horse2  MS", 115, MIDI_UNMAPPED},   /* ? (CoC, HQ) */
-	{"InHale  MS", 121, MIDI_UNMAPPED},   /* ++ (iceMan) */
+	{"Glock   MS", 9, MIDI_UNMAPPED},     /* +++ (QFG1) */
+	{"Gunshot   ", 127, MIDI_UNMAPPED},   /* +++ (LB1) */
+	{"Hammer  MS", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (QFG1) */
+	{"Harmonica2", 22, MIDI_UNMAPPED},    /* +++ (LB1) */
+	{"Harpsi 1  ", 6, MIDI_UNMAPPED},     /* + (Hoyle) */
+	{"Harpsi 2  ", 6, MIDI_UNMAPPED},     /* +++ (LB1) */
+	{"Heart   MS", 116, MIDI_UNMAPPED},   /* ? (Iceman) */
+	{"Horse1  MS", 115, MIDI_UNMAPPED},   /* ? (Camelot, QFG1) */
+	{"Horse2  MS", 115, MIDI_UNMAPPED},   /* ? (Camelot, QFG1) */
+	{"InHale  MS", 121, MIDI_UNMAPPED},   /* ++ (Iceman) */
 	{"KNIFE     ", 120, MIDI_UNMAPPED},   /* ? (LSL3) */
-	{"KenBanjo  ", 105, MIDI_UNMAPPED},   /* +++ (CB) */
-	{"Kiss    MS", 25, MIDI_UNMAPPED},    /* ++ (HQ) */
+	{"KenBanjo  ", 105, MIDI_UNMAPPED},   /* +++ (LB1) */
+	{"Kiss    MS", 25, MIDI_UNMAPPED},    /* ++ (QFG1) */
 	{"KongHit   ", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ??? (KQ1) */
 	{"Koto      ", 107, MIDI_UNMAPPED},   /* +++ (PQ2) */
-	{"Laser   MS", 81, MIDI_UNMAPPED},    /* ?? (HQ) */
-	{"Meeps   MS", 62, MIDI_UNMAPPED},    /* ? (HQ) */
-	{"MTrak   MS", 62, MIDI_UNMAPPED},    /* ?? (iceMan) */
-	{"MachGun MS", 127, MIDI_UNMAPPED},   /* ? (iceMan) */
+	{"Laser   MS", 81, MIDI_UNMAPPED},    /* ?? (QFG1) */
+	{"Meeps   MS", 62, MIDI_UNMAPPED},    /* ? (QFG1) */
+	{"MTrak   MS", 62, MIDI_UNMAPPED},    /* ?? (Iceman) */
+	{"MachGun MS", 127, MIDI_UNMAPPED},   /* ? (Iceman) */
 	{"OCEANSOUND", 122, MIDI_UNMAPPED},   /* + (LSL3) */
 	{"Oboe 2001 ", 68, MIDI_UNMAPPED},    /* + (PQ2) */
-	{"Ocean   MS", 122, MIDI_UNMAPPED},   /* + (iceMan) */
-	{"PPG 2.3 MS", 75, MIDI_UNMAPPED},    /* ? (iceMan) */
-	{"PianoCrank", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (CB) */
-	{"PicSnareMS", MIDI_MAPPED_TO_RHYTHM, 40},   /* R ? (iceMan) */
+	{"Ocean   MS", 122, MIDI_UNMAPPED},   /* + (Iceman) */
+	{"PPG 2.3 MS", 75, MIDI_UNMAPPED},    /* ? (Iceman) */
+	{"PianoCrank", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (LB1) */
+	{"PicSnareMS", MIDI_MAPPED_TO_RHYTHM, 40},   /* R ? (Iceman) */
 	{"PiccoloKA ", 72, MIDI_UNMAPPED},    /* +++ (KQ1) */
 	{"PinkBassMS", 39, MIDI_UNMAPPED},
-	{"Pizz2     ", 45, MIDI_UNMAPPED},    /* ++ (CB) */
+	{"Pizz2     ", 45, MIDI_UNMAPPED},    /* ++ (LB1) */
 	{"Portcullis", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (KQ1) */
-	{"Raspbry MS", 81, MIDI_UNMAPPED},    /* ? (HQ) */
-	{"RatSqueek ", 72, MIDI_UNMAPPED},    /* ? (CB, CoC) */
-	{"Record78  ", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* +++ (CB) */
-	{"RecorderMS", 74, MIDI_UNMAPPED},    /* +++ (CoC) */
-	{"Red Baron ", 125, MIDI_UNMAPPED},   /* ? (CB) */
-	{"ReedPipMS ", 20, MIDI_UNMAPPED},    /* +++ (Coc) */
+	{"Raspbry MS", 81, MIDI_UNMAPPED},    /* ? (QFG1) */
+	{"RatSqueek ", 72, MIDI_UNMAPPED},    /* ? (LauraBow1, Camelot) */
+	{"Record78  ", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* +++ (LB1) */
+	{"RecorderMS", 74, MIDI_UNMAPPED},    /* +++ (Camelot) */
+	{"Red Baron ", 125, MIDI_UNMAPPED},   /* ? (LB1) */
+	{"ReedPipMS ", 20, MIDI_UNMAPPED},    /* +++ (Camelot) */
 	{"RevCymb MS", 119, MIDI_UNMAPPED},
-	{"RifleShot ", 127, MIDI_UNMAPPED},   /* + (CB) */
+	{"RifleShot ", 127, MIDI_UNMAPPED},   /* + (LB1) */
 	{"RimShot MS", MIDI_MAPPED_TO_RHYTHM, 37},   /* R */
 	{"SHOWER    ", 52, MIDI_UNMAPPED},    /* ? (LSL3) */
 	{"SQ Bass MS", 32, MIDI_UNMAPPED},    /* + (SQ3) */
-	{"ShakuVibMS", 79, MIDI_UNMAPPED},    /* + (iceMan) */
-	{"SlapBassMS", 36, MIDI_UNMAPPED},    /* +++ (iceMan) */
-	{"Snare   MS", MIDI_MAPPED_TO_RHYTHM, 38},   /* R (HQ) */
-	{"Some Birds", 123, MIDI_UNMAPPED},   /* + (CB) */
-	{"Sonar   MS", 78, MIDI_UNMAPPED},    /* ? (iceMan) */
-	{"Soundtrk2 ", 97, MIDI_UNMAPPED},    /* +++ (CB) */
-	{"Soundtrack", 97, MIDI_UNMAPPED},    /* ++ (CoC) */
+	{"ShakuVibMS", 79, MIDI_UNMAPPED},    /* + (Iceman) */
+	{"SlapBassMS", 36, MIDI_UNMAPPED},    /* +++ (Iceman) */
+	{"Snare   MS", MIDI_MAPPED_TO_RHYTHM, 38},   /* R (QFG1) */
+	{"Some Birds", 123, MIDI_UNMAPPED},   /* + (LB1) */
+	{"Sonar   MS", 78, MIDI_UNMAPPED},    /* ? (Iceman) */
+	{"Soundtrk2 ", 97, MIDI_UNMAPPED},    /* +++ (LB1) */
+	{"Soundtrack", 97, MIDI_UNMAPPED},    /* ++ (Camelot) */
 	{"SqurWaveMS", 80, MIDI_UNMAPPED},
-	{"StabBassMS", 34, MIDI_UNMAPPED},    /* + (iceMan) */
-	{"SteelDrmMS", 114, MIDI_UNMAPPED},   /* +++ (iceMan) */
-	{"StrSect1MS", 48, MIDI_UNMAPPED},    /* ++ (HQ) */
-	{"String  MS", 45, MIDI_UNMAPPED},    /* + (CoC) */
+	{"StabBassMS", 34, MIDI_UNMAPPED},    /* + (Iceman) */
+	{"SteelDrmMS", 114, MIDI_UNMAPPED},   /* +++ (Iceman) */
+	{"StrSect1MS", 48, MIDI_UNMAPPED},    /* ++ (QFG1) */
+	{"String  MS", 45, MIDI_UNMAPPED},    /* + (Camelot) */
 	{"Syn-Choir ", 91, MIDI_UNMAPPED},
 	{"Syn Brass4", 63, MIDI_UNMAPPED},    /* ++ (PQ2) */
 	{"SynBass MS", 38, MIDI_UNMAPPED},
-	{"SwmpBackgr", 120, MIDI_UNMAPPED},    /* ?? (CB,HQ) */
-	{"T-Bone2 MS", 57, MIDI_UNMAPPED},    /* +++ (HQ) */
-	{"Taiko     ", 116, 35},      /* +++ (Coc) */
+	{"SwmpBackgr", 120, MIDI_UNMAPPED},    /* ?? (LB1, QFG1) */
+	{"T-Bone2 MS", 57, MIDI_UNMAPPED},    /* +++ (QFG1) */
+	{"Taiko     ", 116, 35},      /* +++ (Camelot) */
 	{"Taiko Rim ", 118, 37},      /* +++ (LSL3) */
-	{"Timpani1  ", 47, MIDI_UNMAPPED},    /* +++ (CB) */
-	{"Tom     MS", 117, 48},      /* +++ (iceMan) */
-	{"Toms    MS", 117, 48},      /* +++ (CoC, HQ) */
+	{"Timpani1  ", 47, MIDI_UNMAPPED},    /* +++ (LB1) */
+	{"Tom     MS", 117, 48},      /* +++ (Iceman) */
+	{"Toms    MS", 117, 48},      /* +++ (Camelot, QFG1) */
 	{"Tpt1prtl  ", 56, MIDI_UNMAPPED},    /* +++ (KQ1) */
-	{"TriangleMS", 112, 81},      /* R (CoC) */
-	{"Trumpet 1 ", 56, MIDI_UNMAPPED},    /* +++ (CoC) */
-	{"Type    MS", MIDI_MAPPED_TO_RHYTHM, 39},   /* + (iceMan) */
+	{"TriangleMS", 112, 81},      /* R (Camelot) */
+	{"Trumpet 1 ", 56, MIDI_UNMAPPED},    /* +++ (Camelot) */
+	{"Type    MS", MIDI_MAPPED_TO_RHYTHM, 39},   /* + (Iceman) */
 	{"WaterBells", 98, MIDI_UNMAPPED},    /* + (PQ2) */
 	{"WaterFallK", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (KQ1) */
-	{"Whiporill ", 123, MIDI_UNMAPPED},   /* + (CB) */
-	{"Wind      ", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (CB) */
-	{"Wind    MS", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (HQ, iceMan) */
-	{"Wind2   MS", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (CoC) */
-	{"Woodpecker", 115, MIDI_UNMAPPED},   /* ? (CB) */
-	{"WtrFall MS", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (CoC, HQ, iceMan) */
+	{"Whiporill ", 123, MIDI_UNMAPPED},   /* + (LB1) */
+	{"Wind      ", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (LB1) */
+	{"Wind    MS", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (QFG1, Iceman) */
+	{"Wind2   MS", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (Camelot) */
+	{"Woodpecker", 115, MIDI_UNMAPPED},   /* ? (LB1) */
+	{"WtrFall MS", MIDI_UNMAPPED, MIDI_UNMAPPED}, /* ? (Camelot, QFG1, Iceman) */
 	{0, 0, 0}
 };
 
