@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sci/sound/music.cpp $
- * $Id: music.cpp 48046 2010-02-13 17:42:49Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sci/sound/music.cpp $
+ * $Id: music.cpp 53820 2010-10-25 11:19:44Z wjpalenstijn $
  *
  */
 
@@ -30,6 +30,7 @@
 #include "sci/sci.h"
 #include "sci/console.h"
 #include "sci/resource.h"
+#include "sci/engine/features.h"
 #include "sci/engine/kernel.h"
 #include "sci/engine/state.h"
 #include "sci/sound/midiparser_sci.h"
@@ -37,15 +38,17 @@
 
 namespace Sci {
 
-// When defined, volume fading immediately sets the final sound volume
-#define DISABLE_VOLUME_FADING
-
 SciMusic::SciMusic(SciVersion soundVersion)
 	: _soundVersion(soundVersion), _soundOn(true), _masterVolume(0) {
 
 	// Reserve some space in the playlist, to avoid expensive insertion
 	// operations
 	_playList.reserve(10);
+
+	for (int i = 0; i < 16; i++)
+		_usedChannel[i] = 0;
+
+	_queuedCommands.reserve(1000);
 }
 
 SciMusic::~SciMusic() {
@@ -61,35 +64,27 @@ void SciMusic::init() {
 	// SCI sound init
 	_dwTempo = 0;
 
-	MidiDriverType midiType;
-
-	// Default to MIDI in SCI32 games, as many don't have AdLib support.
-	// WORKAROUND: Default to MIDI in Amiga SCI1_EGA+ games as we don't support those patches yet.
-	// We also don't yet support the 7.pat file of SCI1+ Mac games or SCI0 Mac patches, so we
-	// default to MIDI in those games to let them run.
+	// Default to MIDI in SCI2.1+ games, as many don't have AdLib support.
 	Common::Platform platform = g_sci->getPlatform();
+	uint32 dev = MidiDriver::detectDevice((getSciVersion() >= SCI_VERSION_2_1) ? (MDT_PCSPK | MDT_PCJR | MDT_ADLIB | MDT_MIDI | MDT_PREFER_GM) : (MDT_PCSPK | MDT_PCJR | MDT_ADLIB | MDT_MIDI));
+	_musicType = MidiDriver::getMusicType(dev);
 
-	if (getSciVersion() >= SCI_VERSION_2 || platform == Common::kPlatformMacintosh || (platform == Common::kPlatformAmiga && getSciVersion() >= SCI_VERSION_1_EGA))
-		midiType = MidiDriver::detectMusicDriver(MDT_PCSPK | MDT_ADLIB | MDT_MIDI | MDT_PREFER_MIDI);
-	else
-		midiType = MidiDriver::detectMusicDriver(MDT_PCSPK | MDT_ADLIB | MDT_MIDI);
-
-	switch (midiType) {
-	case MD_ADLIB:
+	switch (_musicType) {
+	case MT_ADLIB:
 		// FIXME: There's no Amiga sound option, so we hook it up to AdLib
-		if (g_sci->getPlatform() == Common::kPlatformAmiga)
-			_pMidiDrv = MidiPlayer_Amiga_create(_soundVersion);
+		if (g_sci->getPlatform() == Common::kPlatformAmiga || platform == Common::kPlatformMacintosh)
+			_pMidiDrv = MidiPlayer_AmigaMac_create(_soundVersion);
 		else
 			_pMidiDrv = MidiPlayer_AdLib_create(_soundVersion);
 		break;
-	case MD_PCJR:
+	case MT_PCJR:
 		_pMidiDrv = MidiPlayer_PCJr_create(_soundVersion);
 		break;
-	case MD_PCSPK:
+	case MT_PCSPK:
 		_pMidiDrv = MidiPlayer_PCSpeaker_create(_soundVersion);
 		break;
 	default:
-		if (ConfMan.getBool("enable_fb01"))
+		if (ConfMan.getBool("native_fb01"))
 			_pMidiDrv = MidiPlayer_Fb01_create(_soundVersion);
 		else
 			_pMidiDrv = MidiPlayer_Midi_create(_soundVersion);
@@ -103,11 +98,57 @@ void SciMusic::init() {
 	}
 
 	_bMultiMidi = ConfMan.getBool("multi_midi");
+
+	// Find out what the first possible channel is (used, when doing channel
+	// remapping).
+	_driverFirstChannel = _pMidiDrv->getFirstChannel();
+	_driverLastChannel = _pMidiDrv->getLastChannel();
+}
+
+void SciMusic::miditimerCallback(void *p) {
+	SciMusic *sciMusic = (SciMusic *)p;
+
+	Common::StackLock lock(sciMusic->_mutex);
+	sciMusic->onTimer();
+}
+
+void SciMusic::onTimer() {
+	const MusicList::iterator end = _playList.end();
+	// sending out queued commands that were "sent" via main thread
+	sendMidiCommandsFromQueue();
+
+	for (MusicList::iterator i = _playList.begin(); i != end; ++i)
+		(*i)->onTimer();
+}
+
+void SciMusic::putMidiCommandInQueue(byte status, byte firstOp, byte secondOp) {
+	putMidiCommandInQueue(status | ((uint32)firstOp << 8) | ((uint32)secondOp << 16));
+}
+
+void SciMusic::putMidiCommandInQueue(uint32 midi) {
+	_queuedCommands.push_back(midi);
+}
+
+// This sends the stored commands from queue to driver (is supposed to get
+// called only during onTimer()). At least mt32 emulation doesn't like getting
+// note-on commands from main thread (if we directly send, we would get a crash
+// during piano scene in lsl5).
+void SciMusic::sendMidiCommandsFromQueue() {
+	uint curCommand = 0;
+	uint commandCount = _queuedCommands.size();
+
+	while (curCommand < commandCount) {
+		_pMidiDrv->send(_queuedCommands[curCommand]);
+		curCommand++;
+	}
+	_queuedCommands.clear();
 }
 
 void SciMusic::clearPlayList() {
-	Common::StackLock lock(_mutex);
-
+	// we must NOT lock our mutex here. Playlist is modified inside soundKill() which will lock the mutex
+	//  during deletion. If we lock it here, a deadlock may occur within soundStop() because that one
+	//  calls the mixer, which will also lock the mixer mutex and if the mixer thread is active during
+	//  that time, we will get a deadlock.
 	while (!_playList.empty()) {
 		soundStop(_playList[0]);
 		soundKill(_playList[0]);
@@ -115,8 +156,6 @@ void SciMusic::clearPlayList() {
 }
 
 void SciMusic::pauseAll(bool pause) {
-	Common::StackLock lock(_mutex);
-
 	const MusicList::iterator end = _playList.end();
 	for (MusicList::iterator i = _playList.begin(); i != end; ++i) {
 		soundToggle(*i, pause);
@@ -124,22 +163,12 @@ void SciMusic::pauseAll(bool pause) {
 }
 
 void SciMusic::stopAll() {
-	Common::StackLock lock(_mutex);
-
 	const MusicList::iterator end = _playList.end();
 	for (MusicList::iterator i = _playList.begin(); i != end; ++i) {
 		soundStop(*i);
 	}
 }
 
-
-void SciMusic::miditimerCallback(void *p) {
-	SciMusic *aud = (SciMusic *)p;
-
-	Common::StackLock lock(aud->_mutex);
-	aud->onTimer();
-}
-
 void SciMusic::soundSetSoundOn(bool soundOnFlag) {
 	Common::StackLock lock(_mutex);
 
@@ -165,27 +194,46 @@ MusicEntry *SciMusic::getSlot(reg_t obj) {
 	return NULL;
 }
 
+// We return the currently active music slot for SCI0
+MusicEntry *SciMusic::getActiveSci0MusicSlot() {
+	const MusicList::iterator end = _playList.end();
+	MusicEntry *highestPrioritySlot = NULL;
+	for (MusicList::iterator i = _playList.begin(); i != end; ++i) {
+		MusicEntry *playSlot = *i;
+		if (playSlot->pMidiParser) {
+			if (playSlot->status == kSoundPlaying)
+				return playSlot;
+			if (playSlot->status == kSoundPaused) {
+				if ((!highestPrioritySlot) || (highestPrioritySlot->priority < playSlot->priority))
+					highestPrioritySlot = playSlot;
+			}
+		}
+	}
+	return highestPrioritySlot;
+}
+
 void SciMusic::setReverb(byte reverb) {
 	Common::StackLock lock(_mutex);
 	_pMidiDrv->setReverb(reverb);
 }
 
-static int f_compare(const void *arg1, const void *arg2) {
-	return ((const MusicEntry *)arg2)->prio - ((const MusicEntry *)arg1)->prio;
+static bool musicEntryCompare(const MusicEntry *l, const MusicEntry *r) {
+	return (l->priority > r->priority);
 }
 
 void SciMusic::sortPlayList() {
-	MusicEntry ** pData = _playList.begin();
-	qsort(pData, _playList.size(), sizeof(MusicEntry *), &f_compare);
+	// Sort the play list in descending priority order
+	Common::sort(_playList.begin(), _playList.end(), musicEntryCompare);
 }
+
 void SciMusic::soundInitSnd(MusicEntry *pSnd) {
 	int channelFilterMask = 0;
 	SoundResource::Track *track = pSnd->soundRes->getTrackByType(_pMidiDrv->getPlayId());
 
-	// If MIDI device is selected but there is no digital track in sound resource
-	// try to use adlib's digital sample if possible
-	// Also, if the track couldn't be found, load the digital track, as some games
-	// depend on this (e.g. the Longbow demo)
+	// If MIDI device is selected but there is no digital track in sound
+	// resource try to use adlib's digital sample if possible. Also, if the
+	// track couldn't be found, load the digital track, as some games depend on
+	// this (e.g. the Longbow demo).
 	if (!track || (_bMultiMidi && track->digitalChannelNr == -1)) {
 		SoundResource::Track *digital = pSnd->soundRes->getDigitalTrack();
 		if (digital)
@@ -214,108 +262,212 @@ void SciMusic::soundInitSnd(MusicEntry *pSnd) {
 			_mutex.lock();
 			pSnd->soundType = Audio::Mixer::kMusicSoundType;
 			if (pSnd->pMidiParser == NULL) {
-				pSnd->pMidiParser = new MidiParser_SCI(_soundVersion);
+				pSnd->pMidiParser = new MidiParser_SCI(_soundVersion, this);
 				pSnd->pMidiParser->setMidiDriver(_pMidiDrv);
 				pSnd->pMidiParser->setTimerRate(_dwTempo);
+				pSnd->pMidiParser->setMasterVolume(_masterVolume);
 			}
 
 			pSnd->pauseCounter = 0;
 
 			// Find out what channels to filter for SCI0
 			channelFilterMask = pSnd->soundRes->getChannelFilterMask(_pMidiDrv->getPlayId(), _pMidiDrv->hasRhythmChannel());
-			pSnd->pMidiParser->loadMusic(track, pSnd, channelFilterMask, _soundVersion);
 
-			// Fast forward to the last position and perform associated events when loading
-			pSnd->pMidiParser->jumpToTick(pSnd->ticker, true);
+			pSnd->pMidiParser->mainThreadBegin();
+			pSnd->pMidiParser->loadMusic(track, pSnd, channelFilterMask, _soundVersion);
+			pSnd->pMidiParser->mainThreadEnd();
 			_mutex.unlock();
 		}
 	}
 }
 
-void SciMusic::onTimer() {
-	const MusicList::iterator end = _playList.end();
-	for (MusicList::iterator i = _playList.begin(); i != end; ++i)
-		(*i)->onTimer();
+// This one checks, if requested channel is available -> in that case give
+// caller that channel. Otherwise look for an unused one
+int16 SciMusic::tryToOwnChannel(MusicEntry *caller, int16 bestChannel) {
+	// Don't even try this for SCI0
+	if (_soundVersion <= SCI_VERSION_0_LATE)
+		return bestChannel;
+	if (!_usedChannel[bestChannel]) {
+		// currently unused, so give it to caller directly
+		_usedChannel[bestChannel] = caller;
+		return bestChannel;
+	}
+	// otherwise look for unused channel
+	for (int channelNr = _driverFirstChannel; channelNr < 15; channelNr++) {
+		if (channelNr == 9) // never map to channel 9 (precussion)
+			continue;
+		if (!_usedChannel[channelNr]) {
+			_usedChannel[channelNr] = caller;
+			return channelNr;
+		}
+	}
+	// nothing found, don't map channel at all
+	//  sierra did this as well, although i'm not sure if we act exactly the same way
+	//  maybe they removed channels from previous playing music
+	return -1;
+}
+
+void SciMusic::freeChannels(MusicEntry *caller) {
+	// Remove used channels
+	for (int i = 0; i < 15; i++) {
+		if (_usedChannel[i] == caller)
+			_usedChannel[i] = 0;
+	}
+	// Also tell midiparser, that he lost ownership
+	caller->pMidiParser->lostChannels();
 }
 
 void SciMusic::soundPlay(MusicEntry *pSnd) {
 	_mutex.lock();
 
-	uint sz = _playList.size(), i;
+	uint playListCount = _playList.size();
+	uint playListNo = playListCount;
+	MusicEntry *alreadyPlaying = NULL;
+
 	// searching if sound is already in _playList
-	for (i = 0; i < sz && _playList[i] != pSnd; i++)
-		;
-	if (i == sz) {// not found
+	for (uint i = 0; i < playListCount; i++) {
+		if (_playList[i] == pSnd)
+			playListNo = i;
+		if ((_playList[i]->status == kSoundPlaying) && (_playList[i]->pMidiParser))
+			alreadyPlaying = _playList[i];
+	}
+	if (playListNo == playListCount) { // not found
 		_playList.push_back(pSnd);
 		sortPlayList();
 	}
 
 	_mutex.unlock();	// unlock to perform mixer-related calls
 
-	if (pSnd->pStreamAud && !_pMixer->isSoundHandleActive(pSnd->hCurrentAud)) {
-		if (pSnd->loop > 1) {
-			pSnd->pLoopStream = new Audio::LoopingAudioStream(pSnd->pStreamAud,
-			                                                  pSnd->loop, DisposeAfterUse::NO);
-			_pMixer->playInputStream(pSnd->soundType, &pSnd->hCurrentAud,
-			                         pSnd->pLoopStream, -1, pSnd->volume, 0,
-			                         DisposeAfterUse::NO);
-		} else {
-			_pMixer->playInputStream(pSnd->soundType, &pSnd->hCurrentAud,
-			                         pSnd->pStreamAud, -1, pSnd->volume, 0,
-			                         DisposeAfterUse::NO);
+	if (pSnd->pMidiParser) {
+		if ((_soundVersion <= SCI_VERSION_0_LATE) && (alreadyPlaying)) {
+			// Music already playing in SCI0?
+			if (pSnd->priority > alreadyPlaying->priority) {
+				// And new priority higher? pause previous music and play new one immediately.
+				// Example of such case: lsl3, when getting points (jingle is played then)
+				soundPause(alreadyPlaying);
+				alreadyPlaying->isQueued = true;
+			} else {
+				// And new priority equal or lower? queue up music and play it afterwards done by
+				//  SoundCommandParser::updateSci0Cues()
+				// Example of such case: iceman room 14
+				pSnd->isQueued = true;
+				pSnd->status = kSoundPaused;
+				return;
+			}
+		}
+	}
+
+	if (pSnd->pStreamAud) {
+		if (!_pMixer->isSoundHandleActive(pSnd->hCurrentAud)) {
+			// Sierra SCI ignores volume set when playing samples via kDoSound
+			//  At least freddy pharkas/CD has a script bug that sets volume to 0
+			//  when playing the "score" sample
+			if (pSnd->loop > 1) {
+				pSnd->pLoopStream = new Audio::LoopingAudioStream(pSnd->pStreamAud,
+																pSnd->loop, DisposeAfterUse::NO);
+				_pMixer->playStream(pSnd->soundType, &pSnd->hCurrentAud,
+										pSnd->pLoopStream, -1, _pMixer->kMaxChannelVolume, 0,
+										DisposeAfterUse::NO);
+			} else {
+				// Rewind in case we play the same sample multiple times
+				// (non-looped) like in pharkas right at the start
+				pSnd->pStreamAud->rewind();
+				_pMixer->playStream(pSnd->soundType, &pSnd->hCurrentAud,
+										pSnd->pStreamAud, -1, _pMixer->kMaxChannelVolume, 0,
+										DisposeAfterUse::NO);
+			}
 		}
 	} else {
-		_mutex.lock();
 		if (pSnd->pMidiParser) {
-			pSnd->pMidiParser->setVolume(pSnd->volume);
+			_mutex.lock();
+			pSnd->pMidiParser->mainThreadBegin();
+			pSnd->pMidiParser->tryToOwnChannels();
 			if (pSnd->status == kSoundStopped)
+				pSnd->pMidiParser->sendInitCommands();
+			pSnd->pMidiParser->setVolume(pSnd->volume);
+			if (pSnd->status == kSoundStopped) {
 				pSnd->pMidiParser->jumpToTick(0);
+			} else {
+				// Disable sound looping before fast forwarding to the last position,
+				// when loading a saved game. Fixes bug #3083151.
+				uint16 prevLoop = pSnd->loop;
+				pSnd->loop = 0;
+				// Fast forward to the last position and perform associated events when loading
+				pSnd->pMidiParser->jumpToTick(pSnd->ticker, true);
+				// Restore looping
+				pSnd->loop = prevLoop;
+			}
+			pSnd->pMidiParser->mainThreadEnd();
+			_mutex.unlock();
 		}
-		_mutex.unlock();
 	}
 
 	pSnd->status = kSoundPlaying;
 }
 
 void SciMusic::soundStop(MusicEntry *pSnd) {
+	SoundStatus previousStatus = pSnd->status;
 	pSnd->status = kSoundStopped;
+	if (_soundVersion <= SCI_VERSION_0_LATE)
+		pSnd->isQueued = false;
 	if (pSnd->pStreamAud)
 		_pMixer->stopHandle(pSnd->hCurrentAud);
 
-	_mutex.lock();
-	if (pSnd->pMidiParser)
-		pSnd->pMidiParser->stop();
-	_mutex.unlock();
+	if (pSnd->pMidiParser) {
+		_mutex.lock();
+		pSnd->pMidiParser->mainThreadBegin();
+		// We shouldn't call stop in case it's paused, otherwise we would send
+		// allNotesOff() again
+		if (previousStatus == kSoundPlaying)
+			pSnd->pMidiParser->stop();
+		freeChannels(pSnd);
+		pSnd->pMidiParser->mainThreadEnd();
+		_mutex.unlock();
+	}
+
+	pSnd->fadeStep = 0; // end fading, if fading was in progress
 }
 
 void SciMusic::soundSetVolume(MusicEntry *pSnd, byte volume) {
 	assert(volume <= MUSIC_VOLUME_MAX);
 	if (pSnd->pStreamAud) {
-		_pMixer->setChannelVolume(pSnd->hCurrentAud, volume * 2); // Mixer is 0-255, SCI is 0-127
+		// we simply ignore volume changes for samples, because sierra sci also
+		//  doesn't support volume for samples via kDoSound
 	} else if (pSnd->pMidiParser) {
 		_mutex.lock();
+		pSnd->pMidiParser->mainThreadBegin();
 		pSnd->pMidiParser->setVolume(volume);
+		pSnd->pMidiParser->mainThreadEnd();
 		_mutex.unlock();
 	}
 }
 
+// this is used to set volume of the sample, used for fading only!
+void SciMusic::soundSetSampleVolume(MusicEntry *pSnd, byte volume) {
+	assert(volume <= MUSIC_VOLUME_MAX);
+	assert(pSnd->pStreamAud);
+	_pMixer->setChannelVolume(pSnd->hCurrentAud, volume * 2); // Mixer is 0-255, SCI is 0-127
+}
+
 void SciMusic::soundSetPriority(MusicEntry *pSnd, byte prio) {
 	Common::StackLock lock(_mutex);
 
-	pSnd->prio = prio;
+	pSnd->priority = prio;
 	sortPlayList();
 }
 
 void SciMusic::soundKill(MusicEntry *pSnd) {
 	pSnd->status = kSoundStopped;
 
-	_mutex.lock();
 	if (pSnd->pMidiParser) {
+		_mutex.lock();
+		pSnd->pMidiParser->mainThreadBegin();
 		pSnd->pMidiParser->unloadMusic();
+		pSnd->pMidiParser->mainThreadEnd();
 		delete pSnd->pMidiParser;
 		pSnd->pMidiParser = NULL;
+		_mutex.unlock();
 	}
-	_mutex.unlock();
 
 	if (pSnd->pStreamAud) {
 		_pMixer->stopHandle(pSnd->hCurrentAud);
@@ -340,6 +492,17 @@ void SciMusic::soundKill(MusicEntry *pSnd) {
 }
 
 void SciMusic::soundPause(MusicEntry *pSnd) {
+	// SCI seems not to be pausing samples played back by kDoSound at all
+	//  It only stops looping samples (actually doesn't loop them again before they are unpaused)
+	//  Examples: Space Quest 1 death by acid drops (pause is called even specifically for the sample, see bug #3038048)
+	//             Eco Quest 1 during the intro when going to the abort-menu
+	//             In both cases sierra sci keeps playing
+	//            Leisure Suit Larry 1 doll scene - it seems that pausing here actually just stops
+	//             further looping from happening
+	//  This is a somewhat bigger change, I'm leaving in the old code in here just in case
+	//  I'm currently pausing looped sounds directly, non-looped sounds won't get paused
+	if ((pSnd->pStreamAud) && (!pSnd->pLoopStream))
+		return;
 	pSnd->pauseCounter++;
 	if (pSnd->status != kSoundPlaying)
 		return;
@@ -347,10 +510,14 @@ void SciMusic::soundPause(MusicEntry *pSnd) {
 	if (pSnd->pStreamAud) {
 		_pMixer->pauseHandle(pSnd->hCurrentAud, true);
 	} else {
-		_mutex.lock();
-		if (pSnd->pMidiParser)
+		if (pSnd->pMidiParser) {
+			_mutex.lock();
+			pSnd->pMidiParser->mainThreadBegin();
 			pSnd->pMidiParser->pause();
-		_mutex.unlock();
+			freeChannels(pSnd);
+			pSnd->pMidiParser->mainThreadEnd();
+			_mutex.unlock();
+		}
 	}
 }
 
@@ -361,7 +528,12 @@ void SciMusic::soundResume(MusicEntry *pSnd) {
 		return;
 	if (pSnd->status != kSoundPaused)
 		return;
-	soundPlay(pSnd);
+	if (pSnd->pStreamAud) {
+		_pMixer->pauseHandle(pSnd->hCurrentAud, false);
+		pSnd->status = kSoundPlaying;
+	} else {
+		soundPlay(pSnd);
+	}
 }
 
 void SciMusic::soundToggle(MusicEntry *pSnd, bool pause) {
@@ -380,8 +552,26 @@ void SciMusic::soundSetMasterVolume(uint16 vol) {
 
 	Common::StackLock lock(_mutex);
 
-	if (_pMidiDrv)
-		_pMidiDrv->setVolume(vol);
+	const MusicList::iterator end = _playList.end();
+	for (MusicList::iterator i = _playList.begin(); i != end; ++i) {
+		if ((*i)->pMidiParser)
+			(*i)->pMidiParser->setMasterVolume(vol);
+	}
+}
+
+void SciMusic::sendMidiCommand(uint32 cmd) {
+	Common::StackLock lock(_mutex);
+	_pMidiDrv->send(cmd);
+}
+
+void SciMusic::sendMidiCommand(MusicEntry *pSnd, uint32 cmd) {
+	Common::StackLock lock(_mutex);
+	if (!pSnd->pMidiParser)
+		error("tried to cmdSendMidi on non midi slot (%04x:%04x)", PRINT_REG(pSnd->soundObj));
+
+	pSnd->pMidiParser->mainThreadBegin();
+	pSnd->pMidiParser->sendFromScriptToDriver(cmd);
+	pSnd->pMidiParser->mainThreadEnd();
 }
 
 void SciMusic::printPlayList(Console *con) {
@@ -392,7 +582,7 @@ void SciMusic::printPlayList(Console *con) {
 	for (uint32 i = 0; i < _playList.size(); i++) {
 		MusicEntry *song = _playList[i];
 		con->DebugPrintf("%d: %04x:%04x, resource id: %d, status: %s, %s type\n", i,
-						PRINT_REG(song->soundObj), song->resnum,
+						PRINT_REG(song->soundObj), song->resourceId,
 						musicStatus[song->status], song->pMidiParser ? "MIDI" : "digital audio");
 	}
 }
@@ -406,9 +596,9 @@ void SciMusic::printSongInfo(reg_t obj, Console *con) {
 	for (MusicList::iterator i = _playList.begin(); i != end; ++i) {
 		MusicEntry *song = *i;
 		if (song->soundObj == obj) {
-			con->DebugPrintf("Resource id: %d, status: %s\n", song->resnum, musicStatus[song->status]);
+			con->DebugPrintf("Resource id: %d, status: %s\n", song->resourceId, musicStatus[song->status]);
 			con->DebugPrintf("dataInc: %d, hold: %d, loop: %d\n", song->dataInc, song->hold, song->loop);
-			con->DebugPrintf("signal: %d, priority: %d\n", song->signal, song->prio);
+			con->DebugPrintf("signal: %d, priority: %d\n", song->signal, song->priority);
 			con->DebugPrintf("ticker: %d, volume: %d\n", song->ticker, song->volume);
 
 			if (song->pMidiParser) {
@@ -442,15 +632,17 @@ MusicEntry::MusicEntry() {
 	soundObj = NULL_REG;
 
 	soundRes = 0;
-	resnum = 0;
+	resourceId = 0;
+
+	isQueued = false;
 
 	dataInc = 0;
 	ticker = 0;
 	signal = 0;
-	prio = 0;
+	priority = 0;
 	loop = 0;
 	volume = MUSIC_VOLUME_DEFAULT;
-	hold = 0;
+	hold = -1;
 
 	pauseCounter = 0;
 	sampleLoopCounter = 0;
@@ -476,6 +668,14 @@ MusicEntry::~MusicEntry() {
 }
 
 void MusicEntry::onTimer() {
+	if (!signal) {
+		if (!signalQueue.empty()) {
+			// no signal set, but signal in queue, set that one
+			signal = signalQueue[0];
+			signalQueue.remove_at(0);
+		}
+	}
+
 	if (status != kSoundPlaying)
 		return;
 
@@ -504,12 +704,6 @@ void MusicEntry::doFade() {
 
 		// Only process MIDI streams in this thread, not digital sound effects
 		if (pMidiParser) {
-#ifdef DISABLE_VOLUME_FADING
-			// Signal fading to stop...
-			volume = fadeTo;
-			fadeStep = 0;
-			fadeCompleted = true;
-#endif
 			pMidiParser->setVolume(volume);
 		}
 
@@ -517,4 +711,23 @@ void MusicEntry::doFade() {
 	}
 }
 
+void MusicEntry::setSignal(int newSignal) {
+	// For SCI0, we cache the signals to set, as some songs might
+	// update their signal faster than kGetEvent is called (which is where
+	// we manually invoke kDoSoundUpdateCues for SCI0 games). SCI01 and
+	// newer handle signalling inside kDoSoundUpdateCues. Refer to bug #3042981
+	if (g_sci->_features->detectDoSoundType() <= SCI_VERSION_0_LATE) {
+		if (!signal) {
+			signal = newSignal;
+		} else {
+			// signal already set and waiting for getting to scripts, queue new one
+			signalQueue.push_back(newSignal);
+		}
+	} else {
+		// Set the signal directly for newer games, otherwise the sound
+		// object might be deleted already later on (refer to bug #3045913)
+		signal = newSignal;
+	}
+}
+
 } // End of namespace Sci
