@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/actor.cpp $
- * $Id: actor.cpp 48500 2010-04-04 09:39:40Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/actor.cpp $
+ * $Id: actor.cpp 53813 2010-10-25 10:35:48Z Kirben $
  *
  */
 
@@ -215,6 +215,12 @@ void ScummEngine::walkActors() {
 void Actor::stopActorMoving() {
 	if (_walkScript)
 		_vm->stopScript(_walkScript);
+
+	// V0 Games will walk on the spot if the actor is stopped mid-walk
+	// So we must set the stand still frame
+	if (_vm->_game.version == 0)
+		startWalkAnim(3, -1);
+
 	_moving = 0;
 }
 
@@ -685,6 +691,7 @@ void Actor_v3::walkActor() {
 int Actor::remapDirection(int dir, bool is_walking) {
 	int specdir;
 	byte flags;
+	byte mask;
 	bool flipX;
 	bool flipY;
 
@@ -763,6 +770,14 @@ int Actor::remapDirection(int dir, bool is_walking) {
 		case 6:
 			return 180;
 		}
+
+		// MM C64 stores flags as a part of the mask
+		if (_vm->_game.version == 0) {
+			mask = _vm->getMaskFromBox(_walkbox);
+			// face the wall if climbing/descending a ladder
+			if ((mask & 0x8C) == 0x84)
+				return 0;
+		}
 	}
 	// OR 1024 in to signal direction interpolation should be done
 	return normalizeAngle(dir) | 1024;
@@ -2248,7 +2263,7 @@ void Actor::setActorCostume(int c) {
 	}
 }
 
-static const char* v0ActorNames[0x19] = {
+static const char* v0ActorNames_English[25] = {
 	"Syd",
 	"Razor",
 	"Dave",
@@ -2264,10 +2279,36 @@ static const char* v0ActorNames[0x19] = {
 	"Purple Tentacle",
 	"Green Tentacle",
 	"Meteor",
+	"",
+	"",
+	"",
 	"Plant",
 	"",
 	"",
 	"",
+	"Sandy"
+};
+
+static const char* v0ActorNames_German[25] = {
+	"Syd",
+	"Razor",
+	"Dave",
+	"Michael",
+	"Bernard",
+	"Wendy",
+	"Jeff",
+	"",
+	"Dr.Fred",
+	"Schwester Edna",
+	"Weird Ed",
+	"Ted",
+	"Lila Tentakel",
+	"Gr<nes Tentakel",
+	"Meteor",
+	"",
+	"",
+	"",
+	"Pflanze",
 	"",
 	"",
 	"",
@@ -2278,8 +2319,15 @@ const byte *Actor::getActorName() {
 	const byte *ptr = NULL;
 
 	if (_vm->_game.version == 0) {
-		if (_number)
-			ptr = (const byte *)v0ActorNames[_number - 1];
+		if (_number) {
+			switch (_vm->_language) {
+			case Common::DE_DEU:
+				ptr = (const byte *)v0ActorNames_German[_number - 1];
+				break;
+			default:
+				ptr = (const byte *)v0ActorNames_English[_number - 1];
+			}
+		}
 	} else {
 		ptr = _vm->getResourceAddress(rtActorName, _number);
 	}
