@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/akos.cpp $
- * $Id: akos.cpp 44336 2009-09-25 09:13:33Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/akos.cpp $
+ * $Id: akos.cpp 48987 2010-05-09 20:17:30Z lordhoto $
  *
  */
 
@@ -1065,11 +1065,7 @@ byte AkosRenderer::codec5(int xmoveCur, int ymoveCur) {
 	bdd.shadowMode = _shadow_mode;
 	bdd.shadowPalette = _vm->_shadowPalette;
 
-	bdd.actorPalette = 0;
-	if (_useBompPalette) {
-		for (uint i = 0; i < 256; i++)
-			bdd.actorPalette[i] = _palette[i];
- 	}
+	bdd.actorPalette = _useBompPalette ? _palette : 0;
 
 	bdd.mirror = !_mirror;
 
