@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/boxes.cpp $
- * $Id: boxes.cpp 45438 2009-10-27 12:08:19Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/boxes.cpp $
+ * $Id: boxes.cpp 53800 2010-10-25 07:07:23Z Kirben $
  *
  */
 
@@ -614,10 +614,8 @@ BoxCoords ScummEngine::getBoxCoordinates(int boxnum) {
 		box->lr.x = bp->c64.x2;
 		box->lr.y = bp->c64.y2;
 
-		if (bp->c64.mask & 0x88) {
+		if ((bp->c64.mask & 0x88) == 0x88) {
 			// walkbox for (right/left) corner
-			// TODO: ladders (incl. man-eating plant) have mask 0x8A,
-			// must those walkboxes be adjusted?
 			if (bp->c64.mask & 0x04)
 				box->ur = box->ul;
 			else
