@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/costume.cpp $
- * $Id: costume.cpp 48135 2010-02-26 16:17:12Z agent-q $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/costume.cpp $
+ * $Id: costume.cpp 48272 2010-03-16 14:34:25Z lordhoto $
  *
  */
 
@@ -75,6 +75,13 @@ static const int v1MMNESLookup[25] = {
 	0x17, 0x00, 0x01, 0x05, 0x16
 };
 
+static const byte v0ActorTalkArray[0x19] = {
+	0x00, 0x06, 0x06, 0x06, 0x06, 
+	0x06, 0x06, 0x00, 0x46, 0x06, 
+	0x06, 0x06, 0x06, 0xFF, 0xFF, 
+	0x06, 0xC0, 0x06, 0x06, 0x00, 
+	0xC0, 0xC0, 0x00, 0x06, 0x06
+};
 
 byte ClassicCostumeRenderer::mainRoutine(int xmoveCur, int ymoveCur) {
 	int i, skip = 0;
@@ -1212,37 +1219,19 @@ byte C64CostumeRenderer::drawLimb(const Actor *a, int limb) {
 	if (!width || !height)
 		return 0;
 
-	int xpos = 0;
-	int ypos = _loaded._maxHeight - offsetY;
-
-	if (flipped) {
-		if (offsetX)
-			xpos += (offsetX - 1) * 8;
-	} else {
-		xpos += offsetX * 8;
-	}
-
-	xpos += _actorX - (a->_width / 2);
-	ypos += (_actorY - _loaded._maxHeight) + 1;		// +1 as we appear to be 1 pixel away from the original interpreter
+	int xpos = _actorX + (flipped ? -1 : +1) * (offsetX * 8 - a->_width / 2);
+	// +1 as we appear to be 1 pixel away from the original interpreter
+	int ypos = _actorY - offsetY + 1;
 
 	// This code is very similar to procC64()
 	for (int y = 0; y < height; ++y) {
 		for (int x = 0; x < width; ++x) {
 			byte color = data[y * width + x];
 			byte pcolor;
-			int realX = 0;
-			if (flipped) {
-				if (offsetX == 0||offsetX == 1) {
-					realX = width-(x + 1);
-				} else if (offsetX == 2) {
-					realX = width-(x + 2);
-				}
-			} else {
-				realX = x;
-			}
 
-			int destY = y + ypos;
-			int destX = realX * 8 + xpos;
+			int destX = xpos + (flipped ? -(x + 1) : x) * 8;
+			int destY = ypos + y;
+
 			if (destY >= 0 && destY < _out.h && destX >= 0 && destX < _out.w) {
 				byte *dst = (byte *)_out.pixels + destY * _out.pitch + destX;
 				byte *mask = _vm->getMaskBuffer(0, destY, _zbuf);
@@ -1257,8 +1246,10 @@ byte C64CostumeRenderer::drawLimb(const Actor *a, int limb) {
 
 	_draw_top = MIN(_draw_top, ypos);
 	_draw_bottom = MAX(_draw_bottom, ypos + height);
-	_vm->markRectAsDirty(kMainVirtScreen, xpos, xpos + (width * 8), ypos, ypos + height, _actorID);
-
+	if (flipped)
+		_vm->markRectAsDirty(kMainVirtScreen, xpos - (width * 8), xpos, ypos, ypos + height, _actorID);
+	else
+		_vm->markRectAsDirty(kMainVirtScreen, xpos, xpos + (width * 8), ypos, ypos + height, _actorID);
 	return 0;
 }
 
@@ -1357,6 +1348,9 @@ int C64CostumeLoader::dirToDirStop(int oldDir) {
 }
 
 void C64CostumeLoader::actorSpeak(ActorC64 *a, int &cmd) {
+	if (v0ActorTalkArray[a->_number] & 0x80)
+		return;
+
 	if ((a->_speaking & 0x80))
 		cmd += 0x0C;
 	else
@@ -1372,6 +1366,9 @@ void C64CostumeLoader::costumeDecodeData(Actor *a, int frame, uint usemask) {
 
 	// Enable/Disable speaking flag
 	if (frame == a->_talkStartFrame) {
+		if (v0ActorTalkArray[a->_number] & 0x40)
+			return;
+
 		A->_speaking = 1;
 		return;
 	}
@@ -1409,15 +1406,7 @@ byte C64CostumeLoader::increaseAnims(Actor *a) {
 		int cmd = A->_costCommand;
 		A->_speakingPrev = A->_speaking;
 
-		// Update to use speak frame
-		if (A->_speaking & 0x80) {
-			actorSpeak(A, cmd);
-
-		} else {
-			// Update to use stand frame
-			if (A->_costFrame == A->_standFrame)
-				cmd = dirToDirStop(cmd);
-		}
+		actorSpeak(A, cmd);
 
 		// Update the limb frames
 		frameUpdate(A, cmd);
@@ -1426,7 +1415,11 @@ byte C64CostumeLoader::increaseAnims(Actor *a) {
 	if (A->_moving  && _vm->_currentRoom != 1 && _vm->_currentRoom != 44) {
 		if (a->_cost.soundPos == 0)
 			a->_cost.soundCounter++;
-		a->_cost.soundPos = (a->_cost.soundPos + 1) % 3;
+		
+		// Is this the correct location?
+		// 0x073C
+		if (v0ActorTalkArray[a->_number] & 0x3F)
+			a->_cost.soundPos = (a->_cost.soundPos + 1) % 3;
 	}
 
 	// increase each frame pos
