@@ -18,13 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/debugger.cpp $
- * $Id: debugger.cpp 47541 2010-01-25 01:39:44Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/debugger.cpp $
+ * $Id: debugger.cpp 50963 2010-07-17 18:38:42Z fingolfin $
  *
  */
 
-
 #include "common/config-manager.h"
+#include "common/debug-channels.h"
 #include "common/file.h"
 #include "common/str.h"
 #include "common/system.h"
@@ -47,7 +47,7 @@ void debugC(int channel, const char *s, ...) {
 
 	// FIXME: Still spew all debug at -d9, for crashes in startup etc.
 	//	  Add setting from commandline ( / abstract channel interface)
-	if (!Common::isDebugChannelEnabled(channel) && (gDebugLevel < 9))
+	if (!DebugMan.isDebugChannelEnabled(channel) && (gDebugLevel < 9))
 		return;
 
 	va_start(va, s);
@@ -62,8 +62,6 @@ ScummDebugger::ScummDebugger(ScummEngine *s)
 	_vm = s;
 
 	// Register variables
-	DVar_Register("debug_countdown", &_frame_countdown, DVAR_INT, 0);
-
 	DVar_Register("scumm_speed", &_vm->_fastMode, DVAR_BYTE, 0);
 	DVar_Register("scumm_room", &_vm->_currentRoom, DVAR_BYTE, 0);
 	DVar_Register("scumm_roomresource", &_vm->_roomResource, DVAR_INT, 0);
@@ -128,7 +126,7 @@ void ScummDebugger::postEnter() {
 bool ScummDebugger::Cmd_Restart(int argc, const char **argv) {
 	_vm->restart();
 
-	_detach_now = true;
+	detach();
 	return false;
 }
 
@@ -202,7 +200,7 @@ bool ScummDebugger::Cmd_LoadGame(int argc, const char **argv) {
 
 		_vm->requestLoad(slot);
 
-		_detach_now = true;
+		detach();
 		return false;
 	}
 
@@ -505,12 +503,12 @@ bool ScummDebugger::Cmd_Object(int argc, const char **argv) {
 }
 
 bool ScummDebugger::Cmd_Debug(int argc, const char **argv) {
-	const Common::DebugChannelList &lvls = Common::listDebugChannels();
+	const Common::DebugManager::DebugChannelList &lvls = DebugMan.listDebugChannels();
 
 	// No parameters given: Print out a list of all channels and their status
 	if (argc <= 1) {
 		DebugPrintf("Available debug channels:\n");
-		for (Common::DebugChannelList::const_iterator i = lvls.begin(); i != lvls.end(); ++i) {
+		for (Common::DebugManager::DebugChannelList::const_iterator i = lvls.begin(); i != lvls.end(); ++i) {
 			DebugPrintf("%c%s - %s (%s)\n", i->enabled ? '+' : ' ',
 					i->name.c_str(), i->description.c_str(),
 					i->enabled ? "enabled" : "disabled");
@@ -521,9 +519,9 @@ bool ScummDebugger::Cmd_Debug(int argc, const char **argv) {
 	// Enable or disable channel?
 	bool result = false;
 	if (argv[1][0] == '+') {
-		result = Common::enableDebugChannel(argv[1] + 1);
+		result = DebugMan.enableDebugChannel(argv[1] + 1);
 	} else if (argv[1][0] == '-') {
-		result = Common::disableDebugChannel(argv[1] + 1);
+		result = DebugMan.disableDebugChannel(argv[1] + 1);
 	}
 
 	if (result) {
@@ -867,10 +865,10 @@ bool ScummDebugger::Cmd_Passcode(int argc, const char **argv) {
 		}
 
 		_vm->_bootParam = 0;
-		_detach_now = true;
+		detach();
 
 	} else {
-		DebugPrintf("Use 'passcode <SEGA CD Passcode>'\n");
+		DebugPrintf("Current Passcode is %d \nUse 'passcode <SEGA CD Passcode>'\n",_vm->_scummVars[411]);
 		return true;
 	}
 	return false;
@@ -878,9 +876,7 @@ bool ScummDebugger::Cmd_Passcode(int argc, const char **argv) {
 
 bool ScummDebugger::Cmd_ResetCursors(int argc, const char **argv) {
 	_vm->resetCursors();
-
-	_detach_now = true;
-
+	detach();
 	return false;
 }
 
