@@ -18,14 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/dialogs.cpp $
- * $Id: dialogs.cpp 48142 2010-02-27 02:49:27Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/dialogs.cpp $
+ * $Id: dialogs.cpp 52715 2010-09-13 23:35:55Z lordhoto $
  */
 
 #include "common/config-manager.h"
 #include "common/savefile.h"
 #include "common/system.h"
 #include "common/events.h"
+#include "common/translation.h"
 
 #include "graphics/scaler.h"
 
@@ -59,14 +60,10 @@
 
 using GUI::CommandSender;
 using GUI::StaticTextWidget;
-using GUI::kCloseCmd;
 using Graphics::kTextAlignCenter;
 using Graphics::kTextAlignLeft;
 using GUI::WIDGET_ENABLED;
 
-typedef GUI::OptionsDialog GUI_OptionsDialog;
-typedef GUI::Dialog GUI_Dialog;
-
 namespace Scumm {
 
 struct ResString {
@@ -207,6 +204,38 @@ static const ResString string_map_table_v345[] = {
 
 #pragma mark -
 
+#ifndef DISABLE_HELP
+
+class HelpDialog : public ScummDialog {
+public:
+	HelpDialog(const GameSettings &game);
+	virtual void handleCommand(GUI::CommandSender *sender, uint32 cmd, uint32 data);
+
+	virtual void reflowLayout();
+
+protected:
+	typedef Common::String String;
+
+	GUI::ButtonWidget *_nextButton;
+	GUI::ButtonWidget *_prevButton;
+
+	GUI::StaticTextWidget *_title;
+	GUI::StaticTextWidget *_key[HELP_NUM_LINES];
+	GUI::StaticTextWidget *_dsc[HELP_NUM_LINES];
+
+	int _page;
+	int _numPages;
+	int _numLines;
+
+	const GameSettings _game;
+
+	void displayKeyBindings();
+};
+
+#endif
+
+#pragma mark -
+
 ScummDialog::ScummDialog(int x, int y, int w, int h) : GUI::Dialog(x, y, w, h) {
 	_backgroundType = GUI::ThemeEngine::kDialogBackgroundSpecial;
 }
@@ -217,224 +246,28 @@ ScummDialog::ScummDialog(String name) : GUI::Dialog(name) {
 
 #pragma mark -
 
-enum {
-	kSaveCmd = 'SAVE',
-	kLoadCmd = 'LOAD',
-	kPlayCmd = 'PLAY',
-	kOptionsCmd = 'OPTN',
-	kHelpCmd = 'HELP',
-	kAboutCmd = 'ABOU',
-	kQuitCmd = 'QUIT',
-	kChooseCmd = 'CHOS'
-};
+#ifndef DISABLE_HELP
 
 ScummMenuDialog::ScummMenuDialog(ScummEngine *scumm)
-	: ScummDialog("ScummMain"), _vm(scumm) {
-
-	new GUI::ButtonWidget(this, "ScummMain.Resume", "Resume", kPlayCmd, 'P');
-
-	_loadButton = new GUI::ButtonWidget(this, "ScummMain.Load", "Load", kLoadCmd, 'L');
-	_saveButton = new GUI::ButtonWidget(this, "ScummMain.Save", "Save", kSaveCmd, 'S');
-
-	new GUI::ButtonWidget(this, "ScummMain.Options", "Options", kOptionsCmd, 'O');
-#ifndef DISABLE_HELP
-	new GUI::ButtonWidget(this, "ScummMain.Help", "Help", kHelpCmd, 'H');
-#endif
-	new GUI::ButtonWidget(this, "ScummMain.About", "About", kAboutCmd, 'A');
-
-	new GUI::ButtonWidget(this, "ScummMain.Quit", "Quit", kQuitCmd, 'Q');
-
-	//
-	// Create the sub dialog(s)
-	//
-	_aboutDialog = new GUI::AboutDialog();
-	_optionsDialog = new ConfigDialog();
-#ifndef DISABLE_HELP
+	: MainMenuDialog(scumm) {
 	_helpDialog = new HelpDialog(scumm->_game);
-#endif
-	_saveDialog = new GUI::SaveLoadChooser("Save game:", "Save");
-	_saveDialog->setSaveMode(true);
-	_loadDialog = new GUI::SaveLoadChooser("Load game:", "Load");
-	_loadDialog->setSaveMode(false);
+	_helpButton->setEnabled(true);
 }
 
 ScummMenuDialog::~ScummMenuDialog() {
-	delete _aboutDialog;
-	delete _optionsDialog;
-#ifndef DISABLE_HELP
 	delete _helpDialog;
-#endif
-	delete _saveDialog;
-	delete _loadDialog;
-}
-
-int ScummMenuDialog::runModal() {
-	_loadButton->setEnabled(_vm->canLoadGameStateCurrently());
-	_saveButton->setEnabled(_vm->canSaveGameStateCurrently());
-	return ScummDialog::runModal();
-}
-
-void ScummMenuDialog::reflowLayout() {
-	_loadButton->setEnabled(_vm->canLoadGameStateCurrently());
-	_saveButton->setEnabled(_vm->canSaveGameStateCurrently());
-	Dialog::reflowLayout();
 }
 
 void ScummMenuDialog::handleCommand(CommandSender *sender, uint32 cmd, uint32 data) {
 	switch (cmd) {
-	case kSaveCmd:
-		save();
-		break;
-	case kLoadCmd:
-		load();
-		break;
-	case kPlayCmd:
-		close();
-		break;
-	case kOptionsCmd:
-		_optionsDialog->runModal();
-		break;
-	case kAboutCmd:
-		_aboutDialog->runModal();
-		break;
-#ifndef DISABLE_HELP
 	case kHelpCmd:
 		_helpDialog->runModal();
 		break;
-#endif
-	case kQuitCmd:
-		_vm->quitGame();
-		close();
-		break;
 	default:
-		ScummDialog::handleCommand(sender, cmd, data);
+		MainMenuDialog::handleCommand(sender, cmd, data);
 	}
 }
 
-void ScummMenuDialog::save() {
-	Common::String gameId = ConfMan.get("gameid");
-
-	const EnginePlugin *plugin = 0;
-	EngineMan.findGame(gameId, &plugin);
-
-	int idx = _saveDialog->runModal(plugin, ConfMan.getActiveDomainName());
-	if (idx >= 0) {
-		String result(_saveDialog->getResultString());
-		char buffer[20];
-		const char *str;
-		if (result.empty()) {
-			// If the user was lazy and entered no save name, come up with a default name.
-			sprintf(buffer, "Save %d", idx);
-			str = buffer;
-		} else
-			str = result.c_str();
-		_vm->requestSave(idx, str);
-		close();
-	}
-}
-
-void ScummMenuDialog::load() {
-	Common::String gameId = ConfMan.get("gameid");
-
-	const EnginePlugin *plugin = 0;
-	EngineMan.findGame(gameId, &plugin);
-
-	int idx = _loadDialog->runModal(plugin, ConfMan.getActiveDomainName());
-	if (idx >= 0) {
-		_vm->requestLoad(idx);
-		close();
-	}
-}
-
-#pragma mark -
-
-enum {
-	kOKCmd = 'ok  '
-};
-
-enum {
-	kKeysCmd = 'KEYS'
-};
-
-// FIXME: We use the empty string as domain name here. This tells the
-// ConfigManager to use the 'default' domain for all its actions. We do that
-// to get as close as possible to editing the 'active' settings.
-//
-// However, that requires bad & evil hacks in the ConfigManager code,
-// and even then still doesn't work quite correctly.
-// For example, if the transient domain contains 'false' for the 'fullscreen'
-// flag, but the user used a hotkey to switch to windowed mode, then the dialog
-// will display the wrong value anyway.
-//
-// Proposed solution consisting of multiple steps:
-// 1) Add special code to the open() code that reads out everything stored
-//    in the transient domain that is controlled by this dialog, and updates
-//    the dialog accordingly.
-// 2) Even more code is added to query the backend for current settings, like
-//    the fullscreen mode flag etc., and also updates the dialog accordingly.
-// 3) The domain being edited is set to the active game domain.
-// 4) If the dialog is closed with the "OK" button, then we remove everything
-//    stored in the transient domain (or at least everything corresponding to
-//    switches in this dialog.
-//    If OTOH the dialog is closed with "Cancel" we do no such thing.
-//
-// These changes will achieve two things at once: Allow us to get rid of using
-//  "" as value for the domain, and in fact provide a somewhat better user
-// experience at the same time.
-ConfigDialog::ConfigDialog()
-	: GUI::OptionsDialog("", "ScummConfig") {
-
-	//
-	// Sound controllers
-	//
-
-	addVolumeControls(this, "ScummConfig.");
-
-	//
-	// Some misc options
-	//
-
-	// SCUMM has a talkspeed range of 0-9
-	addSubtitleControls(this, "ScummConfig.", 9);
-
-	//
-	// Add the buttons
-	//
-
-	new GUI::ButtonWidget(this, "ScummConfig.Ok", "OK", GUI::OptionsDialog::kOKCmd, 'O');
-	new GUI::ButtonWidget(this, "ScummConfig.Cancel", "Cancel", kCloseCmd, 'C');
-#ifdef SMALL_SCREEN_DEVICE
-	new GUI::ButtonWidget(this, "ScummConfig.Keys", "Keys", kKeysCmd, 'K');
-	_keysDialog = NULL;
-#endif
-}
-
-ConfigDialog::~ConfigDialog() {
-#ifdef SMALL_SCREEN_DEVICE
-	delete _keysDialog;
-#endif
-}
-
-void ConfigDialog::handleCommand(CommandSender *sender, uint32 cmd, uint32 data) {
-	switch (cmd) {
-	case kKeysCmd:
-#ifdef SMALL_SCREEN_DEVICE
-		//
-		// Create the sub dialog(s)
-		//
-		_keysDialog = new GUI::KeysDialog();
-		_keysDialog->runModal();
-		delete _keysDialog;
-		_keysDialog = NULL;
-#endif
-		break;
-	default:
-		GUI_OptionsDialog::handleCommand (sender, cmd, data);
-	}
-}
-
-#ifndef DISABLE_HELP
-
 #pragma mark -
 
 enum {
@@ -451,9 +284,9 @@ HelpDialog::HelpDialog(const GameSettings &game)
 
 	_numPages = ScummHelp::numPages(_game.id);
 
-	_prevButton = new GUI::ButtonWidget(this, "ScummHelp.Prev", "Previous", kPrevCmd, 'P');
-	_nextButton = new GUI::ButtonWidget(this, "ScummHelp.Next", "Next", kNextCmd, 'N');
-	new GUI::ButtonWidget(this, "ScummHelp.Close", "Close", kCloseCmd, 'C');
+	_prevButton = new GUI::ButtonWidget(this, "ScummHelp.Prev", _("~P~revious"), 0, kPrevCmd);
+	_nextButton = new GUI::ButtonWidget(this, "ScummHelp.Next", _("~N~ext"), 0, kNextCmd);
+	new GUI::ButtonWidget(this, "ScummHelp.Close", _("~C~lose"), 0, GUI::kCloseCmd);
 	_prevButton->clearFlags(WIDGET_ENABLED);
 
 	_numLines = HELP_NUM_LINES;
@@ -640,14 +473,25 @@ void PauseDialog::handleKeyDown(Common::KeyState state) {
 }
 
 ConfirmDialog::ConfirmDialog(ScummEngine *scumm, int res)
-	: InfoDialog(scumm, res) {
+	: InfoDialog(scumm, res), _yesKey('y'), _noKey('n') {
+
+	if (_message.lastChar() != ')') {
+		_yesKey = _message.lastChar();
+		_message.deleteLastChar();
+
+		if (_yesKey >= 'A' && _yesKey <= 'Z')
+			_yesKey += 'a' - 'A';
+
+		_text->setLabel(_message);
+		reflowLayout();
+	}
 }
 
 void ConfirmDialog::handleKeyDown(Common::KeyState state) {
-	if (state.keycode == Common::KEYCODE_n) {
+	if (state.keycode == Common::KEYCODE_n || state.ascii == _noKey) {
 		setResult(0);
 		close();
-	} else if (state.keycode == Common::KEYCODE_y) {
+	} else if (state.keycode == Common::KEYCODE_y || state.ascii == _yesKey) {
 		setResult(1);
 		close();
 	} else
@@ -674,7 +518,7 @@ void ValueDisplayDialog::drawDialog() {
 }
 
 void ValueDisplayDialog::handleTickle() {
-	if (getMillis() > _timer) {
+	if (g_system->getMillis() > _timer) {
 		close();
 	}
 }
@@ -702,7 +546,7 @@ void ValueDisplayDialog::handleKeyDown(Common::KeyState state) {
 			_value--;
 
 		setResult(_value);
-		_timer = getMillis() + kDisplayDelay;
+		_timer = g_system->getMillis() + kDisplayDelay;
 		draw();
 	} else {
 		close();
@@ -710,9 +554,9 @@ void ValueDisplayDialog::handleKeyDown(Common::KeyState state) {
 }
 
 void ValueDisplayDialog::open() {
-	GUI_Dialog::open();
+	GUI::Dialog::open();
 	setResult(_value);
-	_timer = getMillis() + kDisplayDelay;
+	_timer = g_system->getMillis() + kDisplayDelay;
 }
 
 SubtitleSettingsDialog::SubtitleSettingsDialog(ScummEngine *scumm, int value)
@@ -722,7 +566,7 @@ SubtitleSettingsDialog::SubtitleSettingsDialog(ScummEngine *scumm, int value)
 
 void SubtitleSettingsDialog::handleTickle() {
 	InfoDialog::handleTickle();
-	if (getMillis() > _timer)
+	if (g_system->getMillis() > _timer)
 		close();
 }
 
@@ -759,7 +603,7 @@ void SubtitleSettingsDialog::cycleValue() {
 	else
 		setInfoText(subtitleDesc[_value]);
 
-	_timer = getMillis() + 1500;
+	_timer = g_system->getMillis() + 1500;
 }
 
 Indy3IQPointsDialog::Indy3IQPointsDialog(ScummEngine *scumm, char* text)
