@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/he/script_v90he.cpp $
- * $Id: script_v90he.cpp 45302 2009-10-21 09:59:18Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/he/script_v90he.cpp $
+ * $Id: script_v90he.cpp 53665 2010-10-21 05:33:13Z Kirben $
  *
  */
 
@@ -1437,7 +1437,7 @@ void ScummEngine_v90he::o90_videoOps() {
 			}
 		} else if (_videoParams.status == 165) {
 			// Stop video
-			_moviePlay->closeFile();
+			_moviePlay->close();
 		}
 		break;
 	default:
@@ -1464,7 +1464,7 @@ void ScummEngine_v90he::o90_getVideoData() {
 		break;
 	case 52:	// Get current frame
 		pop();
-		push(_moviePlay->getCurFrame());
+		push(_moviePlay->endOfVideo() ? -1 : (_moviePlay->getCurFrame() + 1));
 		break;
 	case 63:	// Get image number
 		pop();
@@ -1948,39 +1948,41 @@ void ScummEngine_v90he::getArrayDim(int array, int *dim2start, int *dim2end, int
 	}
 }
 
+static int sortArrayOffset;
+
 static int compareByteArray(const void *a, const void *b) {
-	int va = *((const uint8 *)a);
-	int vb = *((const uint8 *)a);
+	int va = *((const uint8 *)a + sortArrayOffset);
+	int vb = *((const uint8 *)b + sortArrayOffset);
 	return va - vb;
 }
 
 static int compareByteArrayReverse(const void *a, const void *b) {
-	int va = *((const uint8 *)a);
-	int vb = *((const uint8 *)a);
+	int va = *((const uint8 *)a + sortArrayOffset);
+	int vb = *((const uint8 *)b + sortArrayOffset);
 	return vb - va;
 }
 
 static int compareIntArray(const void *a, const void *b) {
-	int va = (int16)READ_LE_UINT16((const uint8 *)a);
-	int vb = (int16)READ_LE_UINT16((const uint8 *)b);
+	int va = (int16)READ_LE_UINT16((const uint8 *)a + sortArrayOffset * 2);
+	int vb = (int16)READ_LE_UINT16((const uint8 *)b + sortArrayOffset * 2);
 	return va - vb;
 }
 
 static int compareIntArrayReverse(const void *a, const void *b) {
-	int va = (int16)READ_LE_UINT16((const uint8 *)a);
-	int vb = (int16)READ_LE_UINT16((const uint8 *)b);
+	int va = (int16)READ_LE_UINT16((const uint8 *)a + sortArrayOffset * 2);
+	int vb = (int16)READ_LE_UINT16((const uint8 *)b + sortArrayOffset * 2);
 	return vb - va;
 }
 
 static int compareDwordArray(const void *a, const void *b) {
-	int va = (int32)READ_LE_UINT32((const uint8 *)a);
-	int vb = (int32)READ_LE_UINT32((const uint8 *)b);
+	int va = (int32)READ_LE_UINT32((const uint8 *)a + sortArrayOffset * 4);
+	int vb = (int32)READ_LE_UINT32((const uint8 *)b + sortArrayOffset * 4);
 	return va - vb;
 }
 
 static int compareDwordArrayReverse(const void *a, const void *b) {
-	int va = (int32)READ_LE_UINT32((const uint8 *)a);
-	int vb = (int32)READ_LE_UINT32((const uint8 *)b);
+	int va = (int32)READ_LE_UINT32((const uint8 *)a + sortArrayOffset * 4);
+	int vb = (int32)READ_LE_UINT32((const uint8 *)b + sortArrayOffset * 4);
 	return vb - va;
 }
 
@@ -1994,8 +1996,8 @@ void ScummEngine_v90he::sortArray(int array, int dim2start, int dim2end, int dim
 
 	const int num = dim2end - dim2start + 1;
 	const int pitch = FROM_LE_32(ah->dim1end) - FROM_LE_32(ah->dim1start) + 1;
-	const int offset = pitch * (dim2start - FROM_LE_32(ah->dim2start))
-						+ dim1start - FROM_LE_32(ah->dim1start);
+	const int offset = pitch * (dim2start - FROM_LE_32(ah->dim2start));
+	sortArrayOffset = dim1start - FROM_LE_32(ah->dim1start);
 
 	switch (FROM_LE_32(ah->type)) {
 	case kByteArray:
