@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/he/wiz_he.cpp $
- * $Id: wiz_he.cpp 47541 2010-01-25 01:39:44Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/he/wiz_he.cpp $
+ * $Id: wiz_he.cpp 53553 2010-10-17 12:46:15Z Kirben $
  *
  */
 
@@ -358,6 +358,7 @@ static bool calcClipRects(int dst_w, int dst_h, int src_x, int src_y, int src_w,
 
 void Wiz::writeColor(uint8 *dstPtr, int dstType, uint16 color) {
 	switch (dstType) {
+	case kDstCursor:
 	case kDstScreen:
 		WRITE_UINT16(dstPtr, color);
 		break;
@@ -1519,7 +1520,7 @@ uint8 *Wiz::drawWizImage(int resNum, int state, int maskNum, int maskState, int
 		cw = width;
 		ch = height;
 		dstPitch = cw * _vm->_bytesPerPixel;
-		dstType = kDstMemory;
+		dstType = (_cursorImage) ? kDstCursor : kDstMemory;
 	} else {
 		if (dstResNum) {
 			uint8 *dstPtr = _vm->getResourceAddress(rtImage, dstResNum);
