@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/imuse/imuse_player.cpp $
- * $Id: imuse_player.cpp 47279 2010-01-12 21:07:56Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/imuse/imuse_player.cpp $
+ * $Id: imuse_player.cpp 52198 2010-08-18 21:38:43Z athrxx $
  */
 
 
@@ -47,7 +47,6 @@ namespace Scumm {
 #define PERCUSSION_CHANNEL 9
 
 extern MidiParser *MidiParser_createRO();
-extern MidiParser *MidiParser_createEUP();
 
 uint16 Player::_active_notes[128];
 
@@ -195,7 +194,11 @@ int Player::start_seq_sound(int sound, bool reset_vars) {
 		_parser = MidiParser_createRO();
 	} else if (!memcmp(ptr, "SO", 2)) {
 		// Euphony (FM-TOWNS) resource
-		_parser = MidiParser_createEUP();
+		
+		//////////// REMOVE
+		//_parser = MidiParser_createEUP();
+		///////////
+
 	} else if (!memcmp(ptr, "FORM", 4)) {
 		// Humongous Games XMIDI resource
 		_parser = MidiParser::createParser_XMIDI();
