@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/player_v2.cpp $
- * $Id: player_v2.cpp 47182 2010-01-08 22:09:43Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/player_v2.cpp $
+ * $Id: player_v2.cpp 48637 2010-04-12 09:14:17Z fingolfin $
  *
  */
 
@@ -339,7 +339,7 @@ Player_V2::Player_V2(ScummEngine *scumm, Audio::Mixer *mixer, bool pcjr) {
 	_isV3Game = (scumm->_game.version >= 3);
 	_vm = scumm;
 	_mixer = mixer;
-	_sample_rate = _mixer->getOutputRate();
+	_sampleRate = _mixer->getOutputRate();
 
 	_header_len = (scumm->_game.features & GF_OLD_BUNDLE) ? 4 : 6;
 
@@ -352,7 +352,7 @@ Player_V2::Player_V2(ScummEngine *scumm, Audio::Mixer *mixer, bool pcjr) {
 		clear_channel(i);
 
 	_next_tick = 0;
-	_tick_len = (_sample_rate << FIXP_SHIFT) / FREQ_HZ;
+	_tick_len = (_sampleRate << FIXP_SHIFT) / FREQ_HZ;
 
 	// Initialize V3 music timer
 	_music_timer_ctr = _music_timer = 0;
@@ -366,26 +366,26 @@ Player_V2::Player_V2(ScummEngine *scumm, Audio::Mixer *mixer, bool pcjr) {
 	set_pcjr(pcjr);
 	setMusicVolume(255);
 
-	_mixer->playInputStream(Audio::Mixer::kPlainSoundType, &_soundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, DisposeAfterUse::NO, true);
+	_mixer->playStream(Audio::Mixer::kPlainSoundType, &_soundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, DisposeAfterUse::NO, true);
 }
 
 Player_V2::~Player_V2() {
-	mutex_up();
+	Common::StackLock lock(_mutex);
 	_mixer->stopHandle(_soundHandle);
-	mutex_down();
 }
 
 void Player_V2::set_pcjr(bool pcjr) {
-	mutex_up();
+	Common::StackLock lock(_mutex);
+
 	_pcjr = pcjr;
 
 	if (_pcjr) {
 		_decay = PCJR_DECAY;
-		_update_step = (_sample_rate << FIXP_SHIFT) / (111860 * 2);
+		_update_step = (_sampleRate << FIXP_SHIFT) / (111860 * 2);
 		_freqs_table = pcjr_freq_table;
 	} else {
 		_decay = SPK_DECAY;
-		_update_step = (_sample_rate << FIXP_SHIFT) / (1193000 * 2);
+		_update_step = (_sampleRate << FIXP_SHIFT) / (1193000 * 2);
 		_freqs_table = spk_freq_table;
 	}
 
@@ -393,15 +393,12 @@ void Player_V2::set_pcjr(bool pcjr) {
 	 * sample rate doubles.
 	 */
 	int i;
-	for (i = 0; (_sample_rate << i) < 30000; i++)
+	for (i = 0; (_sampleRate << i) < 30000; i++)
 		_decay = _decay * _decay / 65536;
 
-
 	_timer_output = 0;
 	for (i = 0; i < 4; i++)
 		_timer_count[i] = 0;
-
-	mutex_down();
 }
 
 void Player_V2::setMusicVolume (int vol) {
@@ -452,17 +449,18 @@ void Player_V2::chainNextSound() {
 }
 
 void Player_V2::stopAllSounds() {
-	mutex_up();
+	Common::StackLock lock(_mutex);
+
 	for (int i = 0; i < 4; i++) {
 		clear_channel(i);
 	}
 	_next_nr = _current_nr = 0;
 	_next_data = _current_data = 0;
-	mutex_down();
 }
 
 void Player_V2::stopSound(int nr) {
-	mutex_up();
+	Common::StackLock lock(_mutex);
+
 	if (_next_nr == nr) {
 		_next_nr = 0;
 		_next_data = 0;
@@ -475,14 +473,13 @@ void Player_V2::stopSound(int nr) {
 		_current_data = 0;
 		chainNextSound();
 	}
-	mutex_down();
 }
 
 void Player_V2::startSound(int nr) {
 	byte *data = _vm->getResourceAddress(rtSound, nr);
 	assert(data);
 
-	mutex_up();
+	Common::StackLock lock(_mutex);
 
 	int cprio = _current_data ? *(_current_data + _header_len) : 0;
 	int prio  = *(data + _header_len);
@@ -516,8 +513,6 @@ void Player_V2::startSound(int nr) {
 		_next_nr = nr;
 		_next_data = data;
 	}
-
-	mutex_down();
 }
 
 int Player_V2::getSoundStatus(int nr) const {
@@ -787,7 +782,8 @@ void Player_V2::next_freqs(ChannelInfo *channel) {
 }
 
 void Player_V2::do_mix(int16 *data, uint len) {
-	mutex_up();
+	Common::StackLock lock(_mutex);
+
 	uint step;
 
 	do {
@@ -806,8 +802,6 @@ void Player_V2::do_mix(int16 *data, uint len) {
 		data += 2 * step;
 		_next_tick -= step << FIXP_SHIFT;
 	} while (len -= step);
-
-	mutex_down();
 }
 
 void Player_V2::nextTick() {
@@ -957,12 +951,4 @@ void Player_V2::generatePCjrSamples(int16 *data, uint len) {
 		lowPassFilter(data, len);
 }
 
-void Player_V2::mutex_up() {
-	_mutex.lock();
-}
-
-void Player_V2::mutex_down() {
-	_mutex.unlock();
-}
-
 } // End of namespace Scumm
