@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/player_v2cms.cpp $
- * $Id: player_v2cms.cpp 47541 2010-01-25 01:39:44Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/player_v2cms.cpp $
+ * $Id: player_v2cms.cpp 48637 2010-04-12 09:14:17Z fingolfin $
  *
  */
 
@@ -30,6 +30,7 @@
 #include "sound/mixer.h"
 
 namespace Scumm {
+
 #define FREQ_HZ 236 // Don't change!
 
 #define FIXP_SHIFT  16
@@ -55,8 +56,7 @@ namespace Scumm {
 #define PROCESS_VIBRATO 5
 
 /* this structure defines a channel */
-struct saa1099_channel
-{
+struct saa1099_channel {
 	int frequency;				/* frequency (0x00..0xff) */
 	int freq_enable;			/* frequency enable */
 	int noise_enable;			/* noise enable */
@@ -71,8 +71,7 @@ struct saa1099_channel
 };
 
 /* this structure defines a noise channel */
-struct saa1099_noise
-{
+struct saa1099_noise {
 	/* vars to simulate the noise generator output */
 	double counter;
 	double freq;
@@ -80,8 +79,7 @@ struct saa1099_noise
 };
 
 /* this structure defines a SAA1099 chip */
-struct SAA1099
-{
+struct SAA1099 {
 	int stream;				/* our stream */
 	int noise_params[2];			/* noise generators parameters */
 	int env_enable[2];			/* envelope generators enable */
@@ -818,7 +816,7 @@ Player_V2CMS::Player_V2CMS(ScummEngine *scumm, Audio::Mixer *mixer) {
 	_vm = scumm;
 	_mixer = mixer;
 //	debug("mixer rate: %d", _mixer->getOutputRate());
-	_sample_rate = CMS_RATE;
+	_sampleRate = CMS_RATE;
 
 	_header_len = (scumm->_game.features & GF_OLD_BUNDLE) ? 4 : 6;
 
@@ -831,7 +829,7 @@ Player_V2CMS::Player_V2CMS(ScummEngine *scumm, Audio::Mixer *mixer) {
 		clear_channel(i);
 
 	_next_tick = 0;
-	_tick_len = (_sample_rate << FIXP_SHIFT) / FREQ_HZ;
+	_tick_len = (_sampleRate << FIXP_SHIFT) / FREQ_HZ;
 
 	// Initialize V3 music timer
 	_music_timer_ctr = _music_timer = 0;
@@ -878,7 +876,7 @@ Player_V2CMS::Player_V2CMS(ScummEngine *scumm, Audio::Mixer *mixer) {
 	_cmsVoices[7].octaveOutput = &(_cmsChips[1].octave[1]);
 
 	// inits the CMS Emulator like in the original
-	g_cmsEmu = new CMSEmulator(_sample_rate);
+	g_cmsEmu = new CMSEmulator(_sampleRate);
 	static const byte cmsInitData[13*2] = {
 		0x1C, 0x02,
 		0x00, 0x00, 0x01, 0x00, 0x02, 0x00, 0x03, 0x00, 0x04, 0x00, 0x05, 0x00,
@@ -893,14 +891,14 @@ Player_V2CMS::Player_V2CMS(ScummEngine *scumm, Audio::Mixer *mixer) {
 		}
 	}
 
-	_mixer->playInputStream(Audio::Mixer::kPlainSoundType, &_soundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, DisposeAfterUse::NO, true);
+	_mixer->playStream(Audio::Mixer::kPlainSoundType, &_soundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, DisposeAfterUse::NO, true);
 }
 
 Player_V2CMS::~Player_V2CMS() {
-	mutex_up();
+	Common::StackLock lock(_mutex);
+
 	_mixer->stopHandle(_soundHandle);
 	delete g_cmsEmu;
-	mutex_down();
 }
 
 void Player_V2CMS::setMusicVolume(int vol) {
@@ -935,7 +933,8 @@ void Player_V2CMS::chainNextSound() {
 }
 
 void Player_V2CMS::stopAllSounds() {
-	mutex_up();
+	Common::StackLock lock(_mutex);
+
 	for (int i = 0; i < 4; i++) {
 		clear_channel(i);
 	}
@@ -945,11 +944,11 @@ void Player_V2CMS::stopAllSounds() {
 	_midiSongBegin = 0;
 	_midiDelay = 0;
 	offAllChannels();
-	mutex_down();
 }
 
 void Player_V2CMS::stopSound(int nr) {
-	mutex_up();
+	Common::StackLock lock(_mutex);
+
 	if (_next_nr == nr) {
 		_next_nr = 0;
 		_next_data = 0;
@@ -968,20 +967,17 @@ void Player_V2CMS::stopSound(int nr) {
 		_midiDelay = 0;
 		offAllChannels();
 	}
-	mutex_down();
 }
 
 void Player_V2CMS::startSound(int nr) {
+	Common::StackLock lock(_mutex);
+
 	byte *data = _vm->getResourceAddress(rtSound, nr);
 	assert(data);
 
 	if (data[6] == 0x80) {
-		mutex_up();
 		loadMidiData(data, nr);
-		mutex_down();
 	} else {
-		mutex_up();
-
 		int cprio = _current_data ? *(_current_data + _header_len) : 0;
 		int prio  = *(data + _header_len);
 		int nprio = _next_data ? *(_next_data + _header_len) : 0;
@@ -1014,8 +1010,6 @@ void Player_V2CMS::startSound(int nr) {
 			_next_nr = nr;
 			_next_data = data;
 		}
-
-		mutex_down();
 	}
 }
 
@@ -1415,7 +1409,8 @@ void Player_V2CMS::processMidiData(uint ticks) {
 }
 
 int Player_V2CMS::readBuffer(int16 *buffer, const int numSamples) {
-	mutex_up();
+	Common::StackLock lock(_mutex);
+
 	uint step = 1;
 	int len = numSamples/2;
 
@@ -1452,7 +1447,6 @@ int Player_V2CMS::readBuffer(int16 *buffer, const int numSamples) {
 		_next_tick -= step << FIXP_SHIFT;
 	} while (len -= step);
 
-	mutex_down();
 	return numSamples;
 }
 
@@ -1845,11 +1839,4 @@ void Player_V2CMS::playMusicChips(const MusicChip *table) {
 	} while ((cmsPort & 2) == 0);
 }
 
-void Player_V2CMS::mutex_up() {
-	_mutex.lock();
-}
-
-void Player_V2CMS::mutex_down() {
-	_mutex.unlock();
-}
 } // End of namespace Scumm
