@@ -18,12 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/resource.cpp $
- * $Id: resource.cpp 45387 2009-10-26 02:45:26Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/resource.cpp $
+ * $Id: resource.cpp 48938 2010-05-04 12:02:33Z fingolfin $
  *
  */
 
 #include "common/str.h"
+#ifndef MACOSX
+#include "common/config-manager.h"
+#endif
 
 #include "scumm/charset.h"
 #include "scumm/dialogs.h"
@@ -226,7 +229,7 @@ void ScummEngine::askForDisk(const char *filename, int disknum) {
 #ifdef MACOSX
 		sprintf(buf, "Cannot find file: '%s'\nPlease insert disc %d.\nPress OK to retry, Quit to exit", filename, disknum);
 #else
-		sprintf(buf, "Cannot find file: '%s'\nInsert disc %d into drive %s\nPress OK to retry, Quit to exit", filename, disknum, _gameDataDir.getPath().c_str());
+		sprintf(buf, "Cannot find file: '%s'\nInsert disc %d into drive %s\nPress OK to retry, Quit to exit", filename, disknum, ConfMan.get("path").c_str());
 #endif
 
 		result = displayMessage("Quit", "%s", buf);
