@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/saveload.cpp $
- * $Id: saveload.cpp 48500 2010-04-04 09:39:40Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/saveload.cpp $
+ * $Id: saveload.cpp 52198 2010-08-18 21:38:43Z athrxx $
  *
  */
 
@@ -32,6 +32,7 @@
 #include "scumm/charset.h"
 #include "scumm/imuse_digi/dimuse.h"
 #include "scumm/imuse/imuse.h"
+#include "player_towns.h"
 #include "scumm/he/intern_he.h"
 #include "scumm/object.h"
 #include "scumm/resource.h"
@@ -184,6 +185,8 @@ bool ScummEngine::saveState(int slot, bool compat) {
 	Common::String filename;
 	Common::OutSaveFile *out;
 
+	pauseEngine(true);
+
 	if (_saveLoadSlot == 255) {
 		// Allow custom filenames for save game system in HE Games
 		filename = _saveLoadFileName;
@@ -207,6 +210,9 @@ bool ScummEngine::saveState(int slot, bool compat) {
 		return false;
 	}
 	debug(1, "State saved as '%s'", filename.c_str());
+
+	pauseEngine(false);
+
 	return true;
 }
 
@@ -359,7 +365,7 @@ bool ScummEngine::loadState(int slot, bool compat) {
 			}
 		}
 
-		Graphics::skipThumbnailHeader(*in);
+		Graphics::skipThumbnail(*in);
 	}
 
 	// Since version 56 we save additional information about the creation of
@@ -442,6 +448,9 @@ bool ScummEngine::loadState(int slot, bool compat) {
 	// Update volume settings
 	syncSoundSettings();
 
+	if (_townsPlayer && (hdr.ver >= VER(81)))
+		_townsPlayer->restoreAfterLoad();
+
 	// Init NES costume data
 	if (_game.platform == Common::kPlatformNES) {
 		if (hdr.ver < VER(47))
@@ -572,6 +581,10 @@ bool ScummEngine::loadState(int slot, bool compat) {
 	// Fixes bug #1766072: MANIACNES: Music Doesn't Start On Load Game
 	if (_game.platform == Common::kPlatformNES) {
 		runScript(5, 0, 0, 0);
+
+		if (VAR(224)) {
+			_sound->addSoundToQueue(VAR(224));
+		}
 	}
 
 	return true;
@@ -588,7 +601,7 @@ void ScummEngine::listSavegames(bool *marks, int num) {
 
 	char slot[3];
 	int slotNum;
-	Common::StringList files;
+	Common::StringArray files;
 
 	Common::String prefix = makeSavegameName(99, false);
 	prefix.setChar('*', prefix.size()-2);
@@ -596,7 +609,7 @@ void ScummEngine::listSavegames(bool *marks, int num) {
 	memset(marks, false, num * sizeof(bool));	//assume no savegames for this title
 	files = _saveFileMan->listSavefiles(prefix);
 
-	for (Common::StringList::const_iterator file = files.begin(); file != files.end(); ++file) {
+	for (Common::StringArray::const_iterator file = files.begin(); file != files.end(); ++file) {
 		//Obtain the last 2 digits of the filename, since they correspond to the save slot
 		slot[0] = file->c_str()[file->size()-2];
 		slot[1] = file->c_str()[file->size()-1];
@@ -712,7 +725,7 @@ bool ScummEngine::loadInfosFromSlot(const char *target, int slot, InfoStuff *stu
 		return false;
 	}
 
-	if (!Graphics::skipThumbnailHeader(*in)) {
+	if (!Graphics::skipThumbnail(*in)) {
 		delete in;
 		return false;
 	}
@@ -1385,6 +1398,11 @@ void ScummEngine::saveOrLoad(Serializer *s) {
 		_imuse->save_or_load(s, this);
 	}
 
+
+	// Save/load FM-Towns audio status
+	if (_townsPlayer)
+		_townsPlayer->saveLoadWithSerializer(s);
+
 	//
 	// Save/load the charset renderer state
 	//
