@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/script.cpp $
- * $Id: script.cpp 47622 2010-01-28 00:28:08Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/script.cpp $
+ * $Id: script.cpp 52129 2010-08-16 19:57:35Z fingolfin $
  *
  */
 
@@ -708,25 +708,6 @@ void ScummEngine::writeVar(uint var, int value) {
 	error("Illegal varbits (w)");
 }
 
-void ScummEngine::getResultPos() {
-	int a;
-
-	_resultVarNumber = fetchScriptWord();
-	if (_resultVarNumber & 0x2000) {
-		a = fetchScriptWord();
-		if (a & 0x2000) {
-			_resultVarNumber += readVar(a & ~0x2000);
-		} else {
-			_resultVarNumber += a & 0xFFF;
-		}
-		_resultVarNumber &= ~0x2000;
-	}
-}
-
-void ScummEngine::setResult(int value) {
-	writeVar(_resultVarNumber, value);
-}
-
 void ScummEngine::push(int a) {
 	assert(_scummStackPos >= 0 && _scummStackPos < ARRAYSIZE(_vmStack));
 	_vmStack[_scummStackPos++] = a;
