@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/script_v6.cpp $
- * $Id: script_v6.cpp 48500 2010-04-04 09:39:40Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/script_v6.cpp $
+ * $Id: script_v6.cpp 48499 2010-04-04 09:36:10Z Kirben $
  *
  */
 
@@ -745,6 +745,27 @@ void ScummEngine_v6::o6_startScript() {
 		return;
 	}
 
+	// WORKAROUND bug #1878514: When turning pages in the recipe book
+	// (found on Blood Island), there is a brief moment where it displays
+	// text from two different pages at the same time.
+	//
+	// The content of the books is drawing (in an endless loop) by local
+	// script 2007. Changing the page is handled by script 2006, which
+	// first stops script 2007; then switches the page; then restarts
+	// script 2007. But it fails to clear the blast texts beforehand.
+	// Hence, the next time blast text is drawn, both the old one (from
+	// the old instance of script 2007) and the new text (from the new
+	// instance) are briefly drawn simultaneously.
+	//
+	// This looks like a script bug to me (a missing call to clearTextQueue).
+	// But this could also hint at a subtle bug in ScummVM; we should check
+	// whether this bug occurs with the original engine or not.
+	if (_game.id == GID_CMI && script == 2007 &&
+		_currentRoom == 62 && vm.slot[_currentScript].number == 2006) {
+
+		removeBlastTexts();
+	}
+
 	runScript(script, (flags & 1) != 0, (flags & 2) != 0, args);
 }
 
