@@ -18,16 +18,20 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/scumm.cpp $
- * $Id: scumm.cpp 48500 2010-04-04 09:39:40Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/scumm.cpp $
+ * $Id: scumm.cpp 53732 2010-10-23 13:44:30Z Kirben $
  *
  */
 
 #include "common/config-manager.h"
+#include "common/debug-channels.h"
 #include "common/md5.h"
 #include "common/events.h"
 #include "common/EventRecorder.h"
 #include "common/system.h"
+#include "common/translation.h"
+
+#include "engines/util.h"
 
 #include "gui/message.h"
 #include "gui/GuiManager.h"
@@ -45,6 +49,7 @@
 #include "scumm/imuse_digi/dimuse.h"
 #include "scumm/smush/smush_mixer.h"
 #include "scumm/smush/smush_player.h"
+#include "scumm/player_towns.h"
 #include "scumm/insane/insane.h"
 #include "scumm/he/animation_he.h"
 #include "scumm/he/intern_he.h"
@@ -105,7 +110,7 @@ ScummEngine::ScummEngine(OSystem *syst, const DetectorResult &dr)
 	  _language(dr.language),
 	  _debugger(0),
 	  _currentScript(0xFF), // Let debug() work on init stage
-	  _messageDialog(0), _pauseDialog(0), _scummMenuDialog(0), _versionDialog(0) {
+	  _messageDialog(0), _pauseDialog(0), _versionDialog(0) {
 
 	if (_game.platform == Common::kPlatformNES) {
 		_gdi = new GdiNES(this);
@@ -137,19 +142,18 @@ ScummEngine::ScummEngine(OSystem *syst, const DetectorResult &dr)
 
 	_fileHandle = 0;
 
-
 	// Init all vars
 	_v0ObjectIndex = false;
 	_v0ObjectInInventory = false;
 	_imuse = NULL;
 	_imuseDigital = NULL;
 	_musicEngine = NULL;
+	_townsPlayer = NULL;
 	_verbs = NULL;
 	_objs = NULL;
 	_sound = NULL;
 	memset(&vm, 0, sizeof(vm));
 	_pauseDialog = NULL;
-	_scummMenuDialog = NULL;
 	_versionDialog = NULL;
 	_fastMode = 0;
 	_actors = NULL;
@@ -205,7 +209,6 @@ ScummEngine::ScummEngine(OSystem *syst, const DetectorResult &dr)
 	_saveLoadSlot = 0;
 	_lastSaveTime = 0;
 	_saveTemporaryState = false;
-	memset(_saveLoadFileName, 0, sizeof(_saveLoadFileName));
 	memset(_saveLoadName, 0, sizeof(_saveLoadName));
 	memset(_localScriptOffsets, 0, sizeof(_localScriptOffsets));
 	_scriptPointer = NULL;
@@ -213,7 +216,6 @@ ScummEngine::ScummEngine(OSystem *syst, const DetectorResult &dr)
 	_opcode = 0;
 	vm.numNestedScripts = 0;
 	_lastCodePtr = NULL;
-	_resultVarNumber = 0;
 	_scummStackPos = 0;
 	memset(_vmStack, 0, sizeof(_vmStack));
 	_fileOffset = 0;
@@ -547,14 +549,20 @@ ScummEngine::ScummEngine(OSystem *syst, const DetectorResult &dr)
 
 	// Add debug levels
 	for (int i = 0; i < ARRAYSIZE(debugChannels); ++i)
-		Common::addDebugChannel(debugChannels[i].flag,  debugChannels[i].channel, debugChannels[i].desc);
+		DebugMan.addDebugChannel(debugChannels[i].flag,  debugChannels[i].channel, debugChannels[i].desc);
+
+#ifndef DISABLE_HELP
+	// Create custom GMM dialog providing a help subdialog
+	assert(!_mainMenuDialog);
+	_mainMenuDialog = new ScummMenuDialog(this);
+#endif
 
 	g_eventRec.registerRandomSource(_rnd, "scumm");
 }
 
 
 ScummEngine::~ScummEngine() {
-	Common::clearAllDebugChannels();
+	DebugMan.clearAllDebugChannels();
 
 	delete _musicEngine;
 
@@ -569,7 +577,6 @@ ScummEngine::~ScummEngine() {
 	delete _charset;
 	delete _messageDialog;
 	delete _pauseDialog;
-	delete _scummMenuDialog;
 	delete _versionDialog;
 	delete _fileHandle;
 
@@ -625,6 +632,10 @@ ScummEngine_v5::ScummEngine_v5(OSystem *syst, const DetectorResult &dr)
 	_flashlight.xStrips = 7;
 	_flashlight.yStrips = 7;
 	_flashlight.buffer = NULL;
+
+	memset(_saveLoadVarsFilename, 0, sizeof(_saveLoadVarsFilename));
+
+	_resultVarNumber = 0;
 }
 
 ScummEngine_v4::ScummEngine_v4(OSystem *syst, const DetectorResult &dr)
@@ -686,6 +697,10 @@ ScummEngine_v0::ScummEngine_v0(OSystem *syst, const DetectorResult &dr)
 	_activeObject2Inv = false;
 	_activeObjectObtained = false;
 	_activeObject2Obtained = false;
+
+	VAR_ACTIVE_ACTOR = 0xFF;
+	VAR_IS_SOUND_RUNNING = 0xFF;
+	VAR_ACTIVE_VERB = 0xFF;
 }
 
 ScummEngine_v6::ScummEngine_v6(OSystem *syst, const DetectorResult &dr)
@@ -790,6 +805,7 @@ ScummEngine_v72he::ScummEngine_v72he(OSystem *syst, const DetectorResult &dr)
 	VAR_NUM_COSTUMES = 0xFF;
 	VAR_NUM_IMAGES = 0xFF;
 	VAR_NUM_CHARSETS = 0xFF;
+	VAR_SOUND_ENABLED = 0xFF;
 	VAR_POLYGONS_ONLY = 0xFF;
 	VAR_MOUSE_STATE = 0xFF;
 	VAR_PLATFORM = 0xFF;
@@ -933,22 +949,24 @@ ScummEngine_v8::~ScummEngine_v8() {
 
 Common::Error ScummEngine::init() {
 
+	const Common::FSNode gameDataDir(ConfMan.get("path"));
+
 	// Add default file directories.
 	if (((_game.platform == Common::kPlatformAmiga) || (_game.platform == Common::kPlatformAtariST)) && (_game.version <= 4)) {
 		// This is for the Amiga version of Indy3/Loom/Maniac/Zak
-		SearchMan.addSubDirectoryMatching(_gameDataDir, "rooms");
+		SearchMan.addSubDirectoryMatching(gameDataDir, "rooms");
 	}
 
 	if ((_game.platform == Common::kPlatformMacintosh) && (_game.version == 3)) {
 		// This is for the Mac version of Indy3/Loom
-		SearchMan.addSubDirectoryMatching(_gameDataDir, "rooms 1");
-		SearchMan.addSubDirectoryMatching(_gameDataDir, "rooms 2");
-		SearchMan.addSubDirectoryMatching(_gameDataDir, "rooms 3");
+		SearchMan.addSubDirectoryMatching(gameDataDir, "rooms 1");
+		SearchMan.addSubDirectoryMatching(gameDataDir, "rooms 2");
+		SearchMan.addSubDirectoryMatching(gameDataDir, "rooms 3");
 	}
 
 #ifdef ENABLE_SCUMM_7_8
 #ifdef MACOSX
-	if (_game.version == 8 && !memcmp(_gameDataDir.getPath().c_str(), "/Volumes/MONKEY3_", 17)) {
+	if (_game.version == 8 && !memcmp(gameDataDir.getPath().c_str(), "/Volumes/MONKEY3_", 17)) {
 		// Special case for COMI on Mac OS X. The mount points on OS X depend
 		// on the volume name. Hence if playing from CD, we'd get a problem.
 		// So if loading of a resource file fails, we fall back to the (fixed)
@@ -963,12 +981,12 @@ Common::Error ScummEngine::init() {
 #endif
 	if (_game.version == 8)
 		// This is for COMI
-		SearchMan.addSubDirectoryMatching(_gameDataDir, "resource");
+		SearchMan.addSubDirectoryMatching(gameDataDir, "resource");
 
 	if (_game.version == 7) {
 		// This is for Full Throttle & The Dig
-		SearchMan.addSubDirectoryMatching(_gameDataDir, "video");
-		SearchMan.addSubDirectoryMatching(_gameDataDir, "data");
+		SearchMan.addSubDirectoryMatching(gameDataDir, "video");
+		SearchMan.addSubDirectoryMatching(gameDataDir, "data");
 	}
 #endif
 
@@ -1577,6 +1595,10 @@ void ScummEngine_v90he::resetScumm() {
 			_logicHE = new LogicHEsoccer(this);
 			break;
 
+		case GID_BASEBALL2001:
+			_logicHE = new LogicHEbaseball2001(this);
+			break;
+
 		case GID_BASKETBALL:
 			_logicHE = new LogicHEbasketball(this);
 			break;
@@ -1625,30 +1647,38 @@ void ScummEngine_v100he::resetScumm() {
 #endif
 
 void ScummEngine::setupMusic(int midi) {
-	MidiDriverType midiDriver = MidiDriver::detectMusicDriver(midi);
-	_native_mt32 = ((midiDriver == MD_MT32) || ConfMan.getBool("native_mt32"));
+	MidiDriver::DeviceHandle dev = MidiDriver::detectDevice(midi);
+	_native_mt32 = ((MidiDriver::getMusicType(dev) == MT_MT32) || ConfMan.getBool("native_mt32"));
 
-	switch (midiDriver) {
-	case MD_NULL:
+	switch (MidiDriver::getMusicType(dev)) {
+	case MT_NULL:
 		_musicType = MDT_NONE;
 		break;
-	case MD_PCSPK:
-	case MD_PCJR:
+	case MT_PCSPK:
 		_musicType = MDT_PCSPK;
 		break;
-	case MD_CMS:
+	case MT_PCJR:
+		_musicType = MDT_PCJR;
+		break;
+	//case MT_CMS:
 #if 1
 		_musicType = MDT_ADLIB;
 #else
 		_musicType = MDT_CMS; // Still has number of bugs, disable by default
 #endif
 		break;
-	case MD_TOWNS:
+	case MT_TOWNS:
 		_musicType = MDT_TOWNS;
 		break;
-	case MD_ADLIB:
+	case MT_ADLIB:
 		_musicType = MDT_ADLIB;
 		break;
+	case MT_C64:
+		_musicType = MDT_C64;
+		break;
+	case MT_APPLEIIGS:
+		_musicType = MDT_APPLEIIGS;
+		break;
 	default:
 		_musicType = MDT_MIDI;
 		break;
@@ -1697,8 +1727,8 @@ void ScummEngine::setupMusic(int midi) {
 	 * automatically when samples need to be generated */
 	if (!_mixer->isReady()) {
 		warning("Sound mixer initialization failed");
-		if (_musicType == MDT_ADLIB || _musicType == MDT_PCSPK || _musicType == MDT_CMS)	{
-			midiDriver = MD_NULL;
+		if (_musicType == MDT_ADLIB || _musicType == MDT_PCSPK || _musicType == MDT_PCJR || _musicType == MDT_CMS) {
+			dev = 0;
 			_musicType = MDT_NONE;
 			warning("MIDI driver depends on sound mixer, switching to null MIDI driver");
 		}
@@ -1714,7 +1744,9 @@ void ScummEngine::setupMusic(int midi) {
 		_musicEngine = new Player_SID(this, _mixer);
 #endif
 	} else if (_game.platform == Common::kPlatformNES && _game.version == 1) {
+#ifndef DISABLE_NES_APU
 		_musicEngine = new Player_NES(this, _mixer);
+#endif
 	} else if (_game.platform == Common::kPlatformAmiga && _game.version == 2) {
 		_musicEngine = new Player_V2A(this, _mixer);
 	} else if (_game.platform == Common::kPlatformAmiga && _game.version == 3) {
@@ -1726,26 +1758,30 @@ void ScummEngine::setupMusic(int midi) {
 	} else if (_game.platform == Common::kPlatformAmiga && _game.version <= 4) {
 		_musicEngine = new Player_V4A(this, _mixer);
 	} else if (_game.id == GID_MANIAC && _game.version == 1) {
-		_musicEngine = new Player_V1(this, _mixer, midiDriver != MD_PCSPK);
+		_musicEngine = new Player_V1(this, _mixer, MidiDriver::getMusicType(dev) != MT_PCSPK);
 	} else if (_game.version <= 2) {
-		_musicEngine = new Player_V2(this, _mixer, midiDriver != MD_PCSPK);
-	} else if ((_musicType == MDT_PCSPK) && (_game.version > 2 && _game.version <= 4)) {
-		_musicEngine = new Player_V2(this, _mixer, midiDriver != MD_PCSPK);
+		_musicEngine = new Player_V2(this, _mixer, MidiDriver::getMusicType(dev) != MT_PCSPK);
+	} else if ((_musicType == MDT_PCSPK || _musicType == MDT_PCJR) && (_game.version > 2 && _game.version <= 4)) {
+		_musicEngine = new Player_V2(this, _mixer, MidiDriver::getMusicType(dev) != MT_PCSPK);
 	} else if (_musicType == MDT_CMS) {
 		_musicEngine = new Player_V2CMS(this, _mixer);
 	} else if (_game.platform == Common::kPlatform3DO && _game.heversion <= 62) {
 		// 3DO versions use digital music and sound samples.
+	} else if (_game.platform == Common::kPlatformFMTowns && (_game.version == 3 || _game.id == GID_MONKEY)) {
+		_musicEngine = _townsPlayer = new Player_Towns(this, _mixer);
+		if (!_townsPlayer->init())
+			error("Failed to initialize FM-Towns audio driver.");
 	} else if (_game.version >= 3 && _game.heversion <= 62) {
 		MidiDriver *nativeMidiDriver = 0;
 		MidiDriver *adlibMidiDriver = 0;
 
 		if (_musicType != MDT_ADLIB)
-			nativeMidiDriver = MidiDriver::createMidi(midiDriver);
+			nativeMidiDriver = MidiDriver::createMidi(dev);
 		if (nativeMidiDriver != NULL && _native_mt32)
 			nativeMidiDriver->property(MidiDriver::PROP_CHANNEL_MASK, 0x03FE);
 		bool multi_midi = ConfMan.getBool("multi_midi") && _musicType != MDT_NONE && (midi & MDT_ADLIB);
 		if (_musicType == MDT_ADLIB || multi_midi) {
-			adlibMidiDriver = MidiDriver_ADLIB_create();
+			adlibMidiDriver = MidiDriver::createMidi(MidiDriver::detectDevice(MDT_ADLIB));
 			adlibMidiDriver->property(MidiDriver::PROP_OLD_ADLIB, (_game.features & GF_SMALL_HEADER) ? 1 : 0);
 		}
 
@@ -1760,7 +1796,7 @@ void ScummEngine::setupMusic(int midi) {
 			// YM2162 driver can't handle midi->getPercussionChannel(), NULL shouldn't init MT-32/GM/GS
 			if ((midi != MDT_TOWNS) && (midi != MDT_NONE)) {
 				_imuse->property(IMuse::PROP_NATIVE_MT32, _native_mt32);
-				if (midiDriver != MD_MT32) // MT-32 Emulation shouldn't be GM/GS initialized
+				if (MidiDriver::getMusicType(dev) != MT_MT32) // MT-32 Emulation shouldn't be GM/GS initialized
 					_imuse->property(IMuse::PROP_GS, _enable_gs);
 			}
 			if (_game.heversion >= 60 || midi == MDT_TOWNS) {
@@ -1780,10 +1816,23 @@ void ScummEngine::syncSoundSettings() {
 	int soundVolumeSfx = ConfMan.getInt("sfx_volume");
 	int soundVolumeSpeech = ConfMan.getInt("speech_volume");
 
+	bool mute = false;
+
+	if (ConfMan.hasKey("mute")) {
+		mute = ConfMan.getBool("mute");
+
+		if (mute)
+			soundVolumeMusic = soundVolumeSfx = soundVolumeSpeech = 0;
+	}
+
 	if (_musicEngine) {
 		_musicEngine->setMusicVolume(soundVolumeMusic);
 	}
 
+	if (_townsPlayer) {
+		_townsPlayer->setSfxVolume(soundVolumeSfx);
+	}
+
 	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, soundVolumeSfx);
 	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, soundVolumeMusic);
 	_mixer->setVolumeForSoundType(Audio::Mixer::kSpeechSoundType, soundVolumeSpeech);
@@ -1831,8 +1880,7 @@ Common::Error ScummEngine::go() {
 
 	while (!shouldQuit()) {
 
-		if (_debugger->isAttached())
-			_debugger->onFrame();
+		_debugger->onFrame();
 
 		// Randomize the PRNG by calling it at regular intervals. This ensures
 		// that it will be in a different state each time you run the program.
@@ -1890,6 +1938,12 @@ void ScummEngine::waitForTimer(int msec_delay) {
 	}
 }
 
+void ScummEngine_v0::scummLoop(int delta) {
+	VAR(VAR_IS_SOUND_RUNNING) = (_sound->_lastSound && _sound->isSoundRunning(_sound->_lastSound) != 0);
+
+	ScummEngine::scummLoop(delta);
+}
+
 void ScummEngine::scummLoop(int delta) {
 	if (_game.version >= 3) {
 		VAR(VAR_TMR_1) += delta;
@@ -2072,6 +2126,19 @@ void ScummEngine::scummLoop_updateScummVars() {
 	if (_game.version >= 7) {
 		VAR(VAR_CAMERA_POS_X) = camera._cur.x;
 		VAR(VAR_CAMERA_POS_Y) = camera._cur.y;
+	} else if (_game.platform == Common::kPlatformNES) {
+#if 0
+		// WORKAROUND:
+		// Since there are 2 2-stripes wide borders in MM NES screen,
+		// we have to compensate for it here. This fixes paning effects.
+		// Fixes bug #1328120: "MANIACNES: Screen width incorrect, camera halts sometimes"
+		// But do not do it when only scrolling right to left, since otherwise Ed will not show
+		// up on the doorbell (Bug #3039004)
+		if (VAR(VAR_CAMERA_POS_X) < (camera._cur.x >> V12_X_SHIFT) + 2)
+			VAR(VAR_CAMERA_POS_X) = (camera._cur.x >> V12_X_SHIFT) + 2;
+		else
+#endif
+			VAR(VAR_CAMERA_POS_X) = (camera._cur.x >> V12_X_SHIFT);
 	} else if (_game.version <= 2) {
 		VAR(VAR_CAMERA_POS_X) = camera._cur.x >> V12_X_SHIFT;
 	} else {
@@ -2116,14 +2183,14 @@ void ScummEngine::scummLoop_handleSaveLoad() {
 		if (_saveLoadFlag == 1) {
 			success = saveState(_saveLoadSlot, _saveTemporaryState);
 			if (!success)
-				errMsg = "Failed to save game state to file:\n\n%s";
+				errMsg = _("Failed to save game state to file:\n\n%s");
 
 			if (success && _saveTemporaryState && VAR_GAME_LOADED != 0xFF && _game.version <= 7)
 				VAR(VAR_GAME_LOADED) = 201;
 		} else {
 			success = loadState(_saveLoadSlot, _saveTemporaryState);
 			if (!success)
-				errMsg = "Failed to load game state from file:\n\n%s";
+				errMsg = _("Failed to load game state from file:\n\n%s");
 
 			if (success && _saveTemporaryState && VAR_GAME_LOADED != 0xFF)
 				VAR(VAR_GAME_LOADED) = (_game.version == 8) ? 1 : 203;
@@ -2135,7 +2202,7 @@ void ScummEngine::scummLoop_handleSaveLoad() {
 		} else if (_saveLoadFlag == 1 && _saveLoadSlot != 0 && !_saveTemporaryState) {
 			// Display "Save successful" message, except for auto saves
 			char buf[256];
-			snprintf(buf, sizeof(buf), "Successfully saved game state in file:\n\n%s", filename.c_str());
+			snprintf(buf, sizeof(buf), _("Successfully saved game state in file:\n\n%s"), filename.c_str());
 
 			GUI::TimedMessageDialog dialog(buf, 1500);
 			runDialog(dialog);
@@ -2431,13 +2498,6 @@ void ScummEngine::versionDialog() {
 	runDialog(*_versionDialog);
 }
 
-void ScummEngine::scummMenuDialog() {
-	if (!_scummMenuDialog)
-		_scummMenuDialog = new ScummMenuDialog(this);
-	runDialog(*_scummMenuDialog);
-	syncSoundSettings();
-}
-
 void ScummEngine::confirmExitDialog() {
 	ConfirmDialog d(this, 6);
 
