@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/string.cpp $
- * $Id: string.cpp 46238 2009-12-02 03:55:06Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/string.cpp $
+ * $Id: string.cpp 54666 2010-11-30 07:57:59Z Kirben $
  *
  */
 
@@ -224,10 +224,7 @@ void ScummEngine_v6::removeBlastTexts() {
 void ScummEngine_v7::processSubtitleQueue() {
 	for (int i = 0; i < _subtitleQueuePos; ++i) {
 		SubtitleText *st = &_subtitleQueue[i];
-		if (!ConfMan.getBool("subtitles") || VAR(VAR_VOICE_MODE) == 0)
-			// subtitles are disabled, don't display the text
-			continue;
-		if (!ConfMan.getBool("subtitles") && (!st->actorSpeechMsg || _mixer->isSoundHandleActive(_sound->_talkChannelHandle)))
+		if ((!ConfMan.getBool("subtitles") || VAR(VAR_VOICE_MODE) == 0) && (!st->actorSpeechMsg || _mixer->isSoundHandleActive(_sound->_talkChannelHandle)))
 			// no subtitles and there's a speech variant of the message, don't display the text
 			continue;
 		enqueueText(st->text, st->xpos, st->ypos, st->color, st->charset, false);
@@ -1110,6 +1107,8 @@ int ScummEngine::convertMessageToString(const byte *msg, byte *dst, int dstSize)
 				}
 				num += (_game.version == 8) ? 4 : 2;
 			}
+		} else if (_game.id == GID_DIG && (chr == 1 || chr == 2 || chr == 3 || chr == 8)) {
+			// Skip these characters
 		} else {
 			if (!(chr == '@') || (_game.id == GID_CMI && _language == Common::ZH_TWN) ||
 				(_game.id == GID_LOOM && _game.platform == Common::kPlatformPCEngine && _language == Common::JA_JPN))
@@ -1351,6 +1350,8 @@ void ScummEngine_v7::loadLanguageBundle() {
 				// File contains Korean text (Hangul). just ignore it
 			} else if (*ptr == 'j') {
 				// File contains Japanese text. just ignore it
+			} else if (*ptr == 'c') {
+				// File contains Chinese text. just ignore it
 			} else if (*ptr == 'e') {
 				// File is encoded!
 				enc = 0x13;
