@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/scumm/vars.cpp $
- * $Id: vars.cpp 47621 2010-01-28 00:24:02Z Kirben $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/scumm/vars.cpp $
+ * $Id: vars.cpp 52797 2010-09-18 14:22:22Z Kirben $
  *
  */
 
@@ -116,10 +116,10 @@ void ScummEngine_v0::setupScummVars() {
 	VAR_CAMERA_POS_X = 2;
 	VAR_HAVE_MSG = 3;
 	VAR_ROOM = 4;
-	//VAR_ACTIVE_ACTOR = 5;
+	VAR_ACTIVE_ACTOR = 5;
 	VAR_OVERRIDE = 6;
-	//VAR_IS_SOUND_RUNNING = 8;
-	//VAR_ACTIVE_VERB = 9;
+	VAR_IS_SOUND_RUNNING = 8;
+	VAR_ACTIVE_VERB = 9;
 	VAR_CHARCOUNT = 10;
 }
 
@@ -296,6 +296,7 @@ void ScummEngine_v72he::setupScummVars() {
 	VAR_POLYGONS_ONLY = 76;
 
 	if (_game.heversion <= 74) {
+		VAR_SOUND_ENABLED = 54;
 		VAR_NUM_SOUND_CHANNELS = 56;
 	}
 
@@ -654,6 +655,11 @@ void ScummEngine_v72he::resetScummVars() {
 	VAR(VAR_NUM_CHARSETS) = _numCharsets - 1;
 	VAR(VAR_NUM_GLOBAL_OBJS) = _numGlobalObjects - 1;
 
+	if (_game.heversion <= 74) {
+		// Songs are disabled, if sound is disabled.
+		VAR(VAR_SOUND_ENABLED) = 1;
+	}
+
 	if (_game.heversion == 74) {
 		// Uses different values, compared to later HE80+ games.
 		if (_game.platform == Common::kPlatformMacintosh) {
@@ -721,6 +727,9 @@ void ScummEngine::resetScummVars() {
 		case MDT_PCSPK:
 			VAR(VAR_SOUNDCARD) = 0;
 			break;
+		case MDT_PCJR:
+			VAR(VAR_SOUNDCARD) = 1;
+			break;
 		case MDT_CMS:
 			VAR(VAR_SOUNDCARD) = 2;
 			break;
