@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sword1/sword1.cpp $
- * $Id: sword1.cpp 47541 2010-01-25 01:39:44Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sword1/sword1.cpp $
+ * $Id: sword1.cpp 52828 2010-09-20 20:38:34Z sev $
  *
  */
 
@@ -38,6 +38,8 @@
 
 #include "common/config-manager.h"
 
+#include "engines/util.h"
+
 #include "gui/message.h"
 #include "gui/GuiManager.h"
 
@@ -56,13 +58,14 @@ SwordEngine::SwordEngine(OSystem *syst)
 		_features = 0;
 
 	// Add default file directories
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "clusters");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "music");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "speech");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "video");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "smackshi");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "english");//PSX Demo
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "italian");//PSX Demo
+	const Common::FSNode gameDataDir(ConfMan.get("path"));
+	SearchMan.addSubDirectoryMatching(gameDataDir, "clusters");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "music");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "speech");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "video");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "smackshi");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "english");//PSX Demo
+	SearchMan.addSubDirectoryMatching(gameDataDir, "italian");//PSX Demo
 }
 
 SwordEngine::~SwordEngine() {
@@ -211,12 +214,20 @@ void SwordEngine::syncSoundSettings() {
 		sfxVolL = 255;
 	}
 
-	_music->setVolume(musicVolL, musicVolR);
-	_sound->setSpeechVol(speechVolL, speechVolR);
-	_sound->setSfxVol(sfxVolL, sfxVolR);
+	bool mute = ConfMan.getBool("mute");
 
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, ConfMan.getInt("sfx_volume"));
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSpeechSoundType, ConfMan.getInt("speech_volume"));
+	if (mute) {
+		_music->setVolume(0, 0);
+		_sound->setSpeechVol(0, 0);
+		_sound->setSfxVol(0, 0);
+	} else {
+		_music->setVolume(musicVolL, musicVolR);
+		_sound->setSpeechVol(speechVolL, speechVolR);
+		_sound->setSfxVol(sfxVolL, sfxVolR);
+	}
+
+	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, mute ? 0 : ConfMan.getInt("sfx_volume"));
+	_mixer->setVolumeForSoundType(Audio::Mixer::kSpeechSoundType, mute ? 0 : ConfMan.getInt("speech_volume"));
 }
 
 void SwordEngine::flagsToBool(bool *dest, uint8 flags) {
