@@ -21,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sword2/console.cpp $
- * $Id: console.cpp 39896 2009-04-07 19:52:46Z Hkz $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sword2/console.cpp $
+ * $Id: console.cpp 48458 2010-04-01 16:11:29Z eriktorbjorn $
  */
 
 
@@ -122,6 +122,7 @@ Debugger::Debugger(Sword2Engine *vm)
 	DCmd_Register("english",  WRAP_METHOD(Debugger, Cmd_English));
 	DCmd_Register("finnish",  WRAP_METHOD(Debugger, Cmd_Finnish));
 	DCmd_Register("polish",   WRAP_METHOD(Debugger, Cmd_Polish));
+	DCmd_Register("fxq",      WRAP_METHOD(Debugger, Cmd_FxQueue));
 }
 
 void Debugger::varGet(int var) {
@@ -795,4 +796,9 @@ bool Debugger::Cmd_Polish(int argc, const char **argv) {
 	return true;
 }
 
+bool Debugger::Cmd_FxQueue(int argc, const char **argv) {
+	_vm->_sound->printFxQueue();
+	return true;
+}
+
 } // End of namespace Sword2
