@@ -21,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sword2/music.cpp $
- * $Id: music.cpp 47847 2010-02-03 09:42:40Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sword2/music.cpp $
+ * $Id: music.cpp 50923 2010-07-16 03:07:30Z eriktorbjorn $
  */
 
 // One feature still missing is the original's DipMusic() function which, as
@@ -136,10 +136,8 @@ static Audio::AudioStream *getAudioStream(SoundFileHandle *fh, const char *base,
 			return NULL;
 		}
 		if (fh->fileSize != fh->file.size()) {
-			if (fh->idxTab) {
-				free(fh->idxTab);
-				fh->idxTab = NULL;
-			}
+			free(fh->idxTab);
+			fh->idxTab = NULL;
 		}
 	} else
 		alreadyOpen = true;
@@ -188,8 +186,6 @@ static Audio::AudioStream *getAudioStream(SoundFileHandle *fh, const char *base,
 
 	fh->file.seek(pos, SEEK_SET);
 
-	SafeSubReadStream *tmp = 0;
-
 	switch (fh->fileType) {
 	case kCLUMode:
 		if (Sword2Engine::isPsx())
@@ -197,19 +193,22 @@ static Audio::AudioStream *getAudioStream(SoundFileHandle *fh, const char *base,
 		else
 			return makeCLUStream(&fh->file, enc_len);
 #ifdef USE_MAD
-	case kMP3Mode:
-		tmp = new SafeSubReadStream(&fh->file, pos, pos + enc_len);
+	case kMP3Mode: {
+		SafeSubReadStream *tmp = new SafeSubReadStream(&fh->file, pos, pos + enc_len);
 		return Audio::makeMP3Stream(tmp, DisposeAfterUse::YES);
+		}
 #endif
 #ifdef USE_VORBIS
-	case kVorbisMode:
-		tmp = new SafeSubReadStream(&fh->file, pos, pos + enc_len);
+	case kVorbisMode: {
+		SafeSubReadStream *tmp = new SafeSubReadStream(&fh->file, pos, pos + enc_len);
 		return Audio::makeVorbisStream(tmp, DisposeAfterUse::YES);
+		}
 #endif
 #ifdef USE_FLAC
-	case kFLACMode:
-		tmp = new SafeSubReadStream(&fh->file, pos, pos + enc_len);
+	case kFLACMode: {
+		SafeSubReadStream *tmp = new SafeSubReadStream(&fh->file, pos, pos + enc_len);
 		return Audio::makeFLACStream(tmp, DisposeAfterUse::YES);
+		}
 #endif
 	default:
 		return NULL;
@@ -799,7 +798,7 @@ int32 Sound::playCompSpeech(uint32 speechId, uint8 vol, int8 pan) {
 		p = -p;
 
 	// Start the speech playing
-	_vm->_mixer->playInputStream(Audio::Mixer::kSpeechSoundType, &_soundHandleSpeech, input, -1, volume, p);
+	_vm->_mixer->playStream(Audio::Mixer::kSpeechSoundType, &_soundHandleSpeech, input, -1, volume, p);
 	return RD_OK;
 }
 
