@@ -21,8 +21,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sword2/sprite.cpp $
- * $Id: sprite.cpp 47427 2010-01-21 23:05:33Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sword2/sprite.cpp $
+ * $Id: sprite.cpp 52812 2010-09-19 21:08:12Z sev $
  */
 
 #include "common/endian.h"
@@ -528,8 +528,11 @@ int32 Screen::drawSprite(SpriteInfo *s) {
 				decompData = decompressHIF(s->data, tempBuf);
 
 				// Check that we correctly decompressed data
-				if (!decompData)
+				if (!decompData) {
+					free(tempBuf);
+
 					return RDERR_DECOMPRESSION;
+				}
 
 				s->w = (decompData / (s->h / 2)) * 2;
 				byte *tempBuf2 = (byte *)malloc(s->w * s->h * 10);
@@ -571,8 +574,11 @@ int32 Screen::drawSprite(SpriteInfo *s) {
 					uint32 decompData = decompressHIF(s->data, tempBuf);
 
 					// Check that we correctly decompressed data
-					if (!decompData)
+					if (!decompData) {
+						free(tempBuf);
+
 						return RDERR_DECOMPRESSION;
+					}
 
 					s->w = (decompData / (s->h / 2));
 					sprite = (byte *)malloc(s->w * s->h);
@@ -701,7 +707,7 @@ int32 Screen::drawSprite(SpriteInfo *s) {
 		// We cannot use good scaling for PSX version, as we are missing
 		// some required data.
 		if (_renderCaps & RDBLTFX_EDGEBLEND && !Sword2Engine::isPsx())
-			scaleImageGood(newSprite, s->scaledWidth, s->scaledWidth, s->scaledHeight, sprite, s->w, s->w, s->h, _buffer + _screenWide * rd.top + rd.left);
+			scaleImageGood(newSprite, s->scaledWidth, s->scaledWidth, s->scaledHeight, sprite, s->w, s->w, s->h, _buffer, rd.left, rd.top);
 		else
 			scaleImageFast(newSprite, s->scaledWidth, s->scaledWidth, s->scaledHeight, sprite, s->w, s->w, s->h);
 
