@@ -21,12 +21,10 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/sword2/sword2.cpp $
- * $Id: sword2.cpp 48101 2010-02-21 04:04:13Z bluddy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/sword2/sword2.cpp $
+ * $Id: sword2.cpp 52826 2010-09-20 20:32:50Z sev $
  */
 
-
-
 #include "base/plugins.h"
 
 #include "common/config-manager.h"
@@ -38,6 +36,7 @@
 #include "common/system.h"
 
 #include "engines/metaengine.h"
+#include "engines/util.h"
 
 #include "sword2/sword2.h"
 #include "sword2/defs.h"
@@ -184,7 +183,7 @@ GameList Sword2MetaEngine::detectGames(const Common::FSList &fslist) const {
 
 SaveStateList Sword2MetaEngine::listSaves(const char *target) const {
 	Common::SaveFileManager *saveFileMan = g_system->getSavefileManager();
-	Common::StringList filenames;
+	Common::StringArray filenames;
 	char saveDesc[SAVE_DESCRIPTION_LEN];
 	Common::String pattern = target;
 	pattern += ".???";
@@ -193,7 +192,7 @@ SaveStateList Sword2MetaEngine::listSaves(const char *target) const {
 	sort(filenames.begin(), filenames.end());	// Sort (hopefully ensuring we are sorted numerically..)
 
 	SaveStateList saveList;
-	for (Common::StringList::const_iterator file = filenames.begin(); file != filenames.end(); ++file) {
+	for (Common::StringArray::const_iterator file = filenames.begin(); file != filenames.end(); ++file) {
 		// Obtain the last 3 digits of the filename, since they correspond to the save slot
 		int slotNum = atoi(file->c_str() + file->size() - 3);
 
@@ -257,10 +256,11 @@ namespace Sword2 {
 
 Sword2Engine::Sword2Engine(OSystem *syst) : Engine(syst) {
 	// Add default file directories
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "clusters");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "sword2");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "video");
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "smacks");
+	const Common::FSNode gameDataDir(ConfMan.get("path"));
+	SearchMan.addSubDirectoryMatching(gameDataDir, "clusters");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "sword2");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "video");
+	SearchMan.addSubDirectoryMatching(gameDataDir, "smacks");
 
 	if (!scumm_stricmp(ConfMan.get("gameid").c_str(), "sword2demo") || !scumm_stricmp(ConfMan.get("gameid").c_str(), "sword2psxdemo"))
 		_features = GF_DEMO;
@@ -326,12 +326,26 @@ void Sword2Engine::registerDefaultSettings() {
 }
 
 void Sword2Engine::syncSoundSettings() {
-	// Sound settings. At the time of writing, not all of these can be set
-	// by the global options dialog, but it seems silly to split them up.
-	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, ConfMan.getInt("music_volume"));
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSpeechSoundType, ConfMan.getInt("speech_volume"));
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, ConfMan.getInt("sfx_volume"));
+	bool mute = ConfMan.getBool("mute");
+
+	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, mute ? 0 : ConfMan.getInt("music_volume"));
+	_mixer->setVolumeForSoundType(Audio::Mixer::kSpeechSoundType, mute ? 0 : ConfMan.getInt("speech_volume"));
+	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, mute ? 0 : ConfMan.getInt("sfx_volume"));
 	setSubtitles(ConfMan.getBool("subtitles"));
+
+	// Our own settings dialog can mute the music, speech and sound effects
+	// individually. ScummVM's settings dialog has one master mute setting.
+
+	if (ConfMan.hasKey("mute")) {
+		ConfMan.setBool("music_mute", ConfMan.getBool("mute"));
+		ConfMan.setBool("speech_mute", ConfMan.getBool("mute"));
+		ConfMan.setBool("sfx_mute", ConfMan.getBool("mute"));
+
+		if (!mute) // it is false
+			// So remove it in order to let individual volumes work
+			ConfMan.removeKey("mute", ConfMan.getActiveDomainName());
+	}
+
 	_sound->muteMusic(ConfMan.getBool("music_mute"));
 	_sound->muteSpeech(ConfMan.getBool("speech_mute"));
 	_sound->muteFx(ConfMan.getBool("sfx_mute"));
@@ -356,6 +370,13 @@ void Sword2Engine::writeSettings() {
 	ConfMan.setBool("object_labels", _mouse->getObjectLabels());
 	ConfMan.setInt("reverse_stereo", _sound->isReverseStereo());
 
+	// If even one sound type is unmuted, we can't say that all sound is
+	// muted.
+
+	if (!_sound->isMusicMute() || !_sound->isSpeechMute() || !_sound->isFxMute()) {
+		ConfMan.setBool("mute", false);
+	}
+
 	ConfMan.flushToDisk();
 }
 
@@ -458,8 +479,7 @@ Common::Error Sword2Engine::run() {
 	_screen->initialiseRenderCycle();
 
 	while (1) {
-		if (_debugger->isAttached())
-			_debugger->onFrame();
+		_debugger->onFrame();
 
 #ifdef SWORD2_DEBUG
 		if (_stepOneCycle) {
