@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/teenagent/callbacks.cpp $
- * $Id: callbacks.cpp 48372 2010-03-23 21:23:56Z megath $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/teenagent/callbacks.cpp $
+ * $Id: callbacks.cpp 48382 2010-03-24 19:34:08Z megath $
  */
 
 #include "teenagent/scene.h"
@@ -4010,13 +4010,16 @@ bool TeenAgentEngine::processCallback(uint16 addr) {
 			playActorAnimation(794);
 			wait(100);
 			setLan(1, 0);
-			//scene->getWalkbox(0)->rect.left = 0;
-			//scene->getWalkbox(0)->rect.top = 0;
 			moveTo(151, 197, 2);
 			playActorAnimation(795);
 			moveTo(186, 198, 2, true);
 			moveTo(220, 198, 4);
-			//scene->getWalkbox(0)->rect.top = 200;
+			{
+				Walkbox * w = scene->getWalkbox(0);
+				w->rect.left = 0;
+				w->rect.bottom = 199;
+				w->save();
+			}
 			setLan(1, 0xff);
 
 			Dialog::showMark(scene, 0x58a9);
