@@ -18,13 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/teenagent/scene.cpp $
- * $Id: scene.cpp 48400 2010-03-25 10:11:26Z megath $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/teenagent/scene.cpp $
+ * $Id: scene.cpp 48933 2010-05-04 11:56:52Z fingolfin $
  */
 
 #include "common/config-manager.h"
 #include "common/debug.h"
 #include "common/algorithm.h"
+#include "common/ptr.h"
+
 #include "teenagent/scene.h"
 #include "teenagent/resources.h"
 #include "teenagent/surface.h"
@@ -211,7 +213,7 @@ void Scene::init(TeenAgentEngine *engine, OSystem *system) {
 	FilePack varia;
 	varia.open("varia.res");
 
-	ScopedPtr<Common::SeekableReadStream> s(varia.getStream(1));
+	Common::ScopedPtr<Common::SeekableReadStream> s(varia.getStream(1));
 	if (!s)
 		error("invalid resource data");
 
@@ -335,7 +337,7 @@ void Scene::loadOns() {
 	if (ons_count > 0) {
 		ons = new Surface[ons_count];
 		for (uint32 i = 0; i < ons_count; ++i) {
-			ScopedPtr<Common::SeekableReadStream> s(res->ons.getStream(on_id[i]));
+			Common::ScopedPtr<Common::SeekableReadStream> s(res->ons.getStream(on_id[i]));
 			if (s) {
 				ons[i].load(s, Surface::kTypeOns);
 			}
@@ -358,7 +360,7 @@ void Scene::loadLans() {
 		if (bxv == 0)
 			continue;
 
-		ScopedPtr<Common::SeekableReadStream> s(res->loadLan000(res_id));
+		Common::ScopedPtr<Common::SeekableReadStream> s(res->loadLan000(res_id));
 		if (s) {
 			animation[i].load(s, Animation::kTypeLan);
 			if (bxv != 0 && bxv != 0xff)
@@ -396,7 +398,7 @@ void Scene::init(int id, const Common::Point &pos) {
 		}
 	}
 
-	ScopedPtr<Common::SeekableReadStream> stream(res->on.getStream(id));
+	Common::ScopedPtr<Common::SeekableReadStream> stream(res->on.getStream(id));
 	int sub_hack = 0;
 	if (id == 7) { //something patched in the captains room
 		switch(res->dseg.get_byte(0xdbe6)) {
@@ -427,7 +429,7 @@ void Scene::init(int id, const Common::Point &pos) {
 void Scene::playAnimation(byte idx, uint id, bool loop, bool paused, bool ignore) {
 	debug(0, "playAnimation(%u, %u, loop:%s, paused:%s, ignore:%s)", idx, id, loop?"true":"false", paused?"true":"false", ignore?"true":"false");
 	assert(idx < 4);
-	ScopedPtr<Common::SeekableReadStream> s(Resources::instance()->loadLan(id + 1));
+	Common::ScopedPtr<Common::SeekableReadStream> s(Resources::instance()->loadLan(id + 1));
 	if (!s)
 		error("playing animation %u failed", id);
 
@@ -439,7 +441,7 @@ void Scene::playAnimation(byte idx, uint id, bool loop, bool paused, bool ignore
 
 void Scene::playActorAnimation(uint id, bool loop, bool ignore) {
 	debug(0, "playActorAnimation(%u, loop:%s, ignore:%s)", id, loop?"true":"false", ignore?"true":"false");
-	ScopedPtr<Common::SeekableReadStream> s(Resources::instance()->loadLan(id + 1));
+	Common::ScopedPtr<Common::SeekableReadStream> s(Resources::instance()->loadLan(id + 1));
 	if (!s)
 		error("playing animation %u failed", id);
 
