@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/teenagent/teenagent.cpp $
- * $Id: teenagent.cpp 48400 2010-03-25 10:11:26Z megath $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/teenagent/teenagent.cpp $
+ * $Id: teenagent.cpp 50963 2010-07-17 18:38:42Z fingolfin $
  */
 
 #include "common/config-manager.h"
@@ -27,11 +27,17 @@
 #include "common/events.h"
 #include "common/savefile.h"
 #include "common/system.h"
+
 #include "engines/advancedDetector.h"
+#include "engines/util.h"
+
+#include "sound/audiocd.h"
 #include "sound/mixer.h"
 #include "sound/decoders/raw.h"
+
 #include "graphics/cursorman.h"
 #include "graphics/thumbnail.h"
+
 #include "teenagent/console.h"
 #include "teenagent/music.h"
 #include "teenagent/objects.h"
@@ -190,7 +196,7 @@ void TeenAgentEngine::deinit() {
 
 Common::Error TeenAgentEngine::loadGameState(int slot) {
 	debug(0, "loading from slot %d", slot);
-	ScopedPtr<Common::InSaveFile> 
+	Common::ScopedPtr<Common::InSaveFile> 
 		in(_saveFileMan->openForLoading(Common::String::printf("teenagent.%02d", slot)));
 	if (!in)
 		in.reset(_saveFileMan->openForLoading(Common::String::printf("teenagent.%d", slot)));
@@ -226,7 +232,7 @@ Common::Error TeenAgentEngine::loadGameState(int slot) {
 
 Common::Error TeenAgentEngine::saveGameState(int slot, const char *desc) {
 	debug(0, "saving to slot %d", slot);
-	ScopedPtr<Common::OutSaveFile> out(_saveFileMan->openForSaving(Common::String::printf("teenagent.%02d", slot)));
+	Common::ScopedPtr<Common::OutSaveFile> out(_saveFileMan->openForSaving(Common::String::printf("teenagent.%02d", slot)));
 	if (!out)
 		return Common::kWritingFailed;
 
@@ -268,6 +274,39 @@ int TeenAgentEngine::skipEvents() const {
 	return 0;
 }
 
+bool TeenAgentEngine::showCDLogo() {
+	Common::File cdlogo;
+	if (!cdlogo.exists("cdlogo.res") || !cdlogo.open("cdlogo.res"))
+		return true;
+
+	byte bg[0xfa00];
+	byte palette[0x400];
+
+	cdlogo.read(bg, sizeof(bg));
+	memset(palette, 0, sizeof(palette));
+
+	for(uint c = 0; c < 0x100; ++c) {
+		uint idx = c * 4;
+		cdlogo.read(palette + idx, 3);
+		palette[idx] *= 4;
+		palette[idx + 1] *= 4;
+		palette[idx + 2] *= 4;
+	}
+	_system->setPalette(palette, 0, 0x100);
+	_system->copyRectToScreen(bg, 320, 0, 0, 320, 200);
+	_system->updateScreen();
+
+	for(uint i = 0; i < 20; ++i) {
+		int r = skipEvents();
+		if (r != 0)
+			return r > 0? true: false;
+		_system->delayMillis(100);
+	}
+	cdlogo.close();
+
+	return true;
+}
+
 bool TeenAgentEngine::showLogo() {
 	FilePack logo;
 	if (!logo.open("unlogic.res"))
@@ -276,7 +315,7 @@ bool TeenAgentEngine::showLogo() {
 	byte bg[0xfa00];
 	byte palette[0x400];
 
-	ScopedPtr<Common::SeekableReadStream> frame(logo.getStream(1));
+	Common::ScopedPtr<Common::SeekableReadStream> frame(logo.getStream(1));
 	if (!frame)
 		return true;
 
@@ -329,7 +368,7 @@ bool TeenAgentEngine::showMetropolis() {
 	byte palette[0x400];
 	memset(palette, 0, sizeof(palette));
 	{
-		ScopedPtr<Common::SeekableReadStream> s(varia.getStream(5));
+		Common::ScopedPtr<Common::SeekableReadStream> s(varia.getStream(5));
 		for(uint c = 0; c < 0x400; c += 4) {
 			s->read(palette + c, 3);
 			palette[c] *= 4;
@@ -432,14 +471,16 @@ Common::Error TeenAgentEngine::run() {
 
 	syncSoundSettings();
 
-	music->load(1);
-	_mixer->playInputStream(Audio::Mixer::kMusicSoundType, &_musicHandle, music, -1, Audio::Mixer::kMaxChannelVolume, 0, DisposeAfterUse::NO, false);
+	_mixer->playStream(Audio::Mixer::kMusicSoundType, &_musicHandle, music, -1, Audio::Mixer::kMaxChannelVolume, 0, DisposeAfterUse::NO, false);
+	setMusic(1);
 	music->start();
 
 	int load_slot = Common::ConfigManager::instance().getInt("save_slot");
 	if (load_slot >= 0) {
 		loadGameState(load_slot);
 	} else {
+		if (!showCDLogo())
+			return Common::kNoError;
 		if (!showLogo())
 			return Common::kNoError;
 		if (!showMetropolis())
@@ -493,6 +534,10 @@ Common::Error TeenAgentEngine::run() {
 				//	debug(0, "%d, %s", current_object->id, current_object->name.c_str());
 				if (scene->getId() < 0)
 					break;
+
+				if (current_object == NULL)
+					break;
+
 				if (res->dseg.get_byte(0) == 3 && current_object->id == 1) {
 					processCallback(0x5189); //boo!
 					break;
@@ -573,9 +618,7 @@ Common::Error TeenAgentEngine::run() {
 
 		_system->updateScreen();
 
-		if (console->isAttached()) {
-			console->onFrame();
-		}
+		console->onFrame();
 
 		uint32 next_tick = MIN(game_timer, mark_timer);
 		if (next_tick > 0) {
@@ -921,16 +964,28 @@ void TeenAgentEngine::playSoundNow(byte id) {
 	//debug(0, "playing %u samples...", size);
 
 	Audio::AudioStream *stream = Audio::makeRawStream(data, size, 11025, 0);
-	_mixer->playInputStream(Audio::Mixer::kSFXSoundType, &_soundHandle, stream); //dispose is YES by default
+	_mixer->playStream(Audio::Mixer::kSFXSoundType, &_soundHandle, stream); //dispose is YES by default
 }
 
 
 void TeenAgentEngine::setMusic(byte id) {
 	debug(0, "starting music %u", id);
-	if (!music->load(id))
-		return;
-	*Resources::instance()->dseg.ptr(0xDB90) = id;
-	music->start();
+	Resources *res = Resources::instance();
+	
+	if (id != 1) //intro music
+		*res->dseg.ptr(0xDB90) = id;
+	
+	if (_gameDescription->flags & ADGF_CD) {
+		byte track2cd[] = {7, 2, 0, 9, 3, 6, 8, 10, 4, 5, 11};
+		if (id == 0 || id > 11 || track2cd[id - 1] == 0) {
+			debug(0, "no cd music for id %u", id);
+			return;
+		}
+		byte track = track2cd[id - 1];
+		debug(0, "playing cd track %u", track);
+		Audio::AudioCDManager::instance().play(track, -1, 0, 0);
+	} else if (music->load(id))
+		music->start();
 }
 
 bool TeenAgentEngine::hasFeature(EngineFeature f) const {
