@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/tinsel/bmv.cpp $
- * $Id: bmv.cpp 48666 2010-04-16 11:00:34Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/tinsel/bmv.cpp $
+ * $Id: bmv.cpp 50924 2010-07-16 03:14:03Z eriktorbjorn $
  *
  * The movie player.
  */
@@ -427,7 +427,7 @@ void BMVPlayer::MovieAudio(int audioOffset, int blobs) {
 
 	if (currentSoundFrame == ADVANCE_SOUND) {
 		if (!audioStarted) {
-			_vm->_mixer->playInputStream(Audio::Mixer::kSFXSoundType,
+			_vm->_mixer->playStream(Audio::Mixer::kSFXSoundType,
 					&_audioHandle, _audioStream, -1, Audio::Mixer::kMaxChannelVolume, 0, DisposeAfterUse::NO);
 			audioStarted = true;
 		}
@@ -730,16 +730,12 @@ void BMVPlayer::FinishBMV() {
 		stream.close();
 
 	// Release the data buffer
-	if (bigBuffer != NULL) {
-		free(bigBuffer);
-		bigBuffer = NULL;
-	}
+	free(bigBuffer);
+	bigBuffer = NULL;
 
 	// Release the screen buffer
-	if (screenBuffer != NULL) {
-		free(screenBuffer);
-		screenBuffer = NULL;
-	}
+	free(screenBuffer);
+	screenBuffer = NULL;
 
 	// Ditch any text objects
 	for (i = 0; i < 2; i++) {
