@@ -18,22 +18,26 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/tinsel/tinsel.cpp $
- * $Id: tinsel.cpp 48101 2010-02-21 04:04:13Z bluddy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/tinsel/tinsel.cpp $
+ * $Id: tinsel.cpp 52736 2010-09-15 22:00:20Z lordhoto $
  *
  */
 
+#include "common/debug-channels.h"
 #include "common/endian.h"
 #include "common/error.h"
 #include "common/events.h"
 #include "common/EventRecorder.h"
 #include "common/keyboard.h"
 #include "common/file.h"
+#include "common/fs.h"
 #include "common/savefile.h"
 #include "common/config-manager.h"
 #include "common/serializer.h"
 #include "common/stream.h"
 
+#include "engines/util.h"
+
 #include "graphics/cursorman.h"
 
 #include "base/plugins.h"
@@ -100,8 +104,6 @@ static bool bCuttingScene = false;
 
 static bool bChangingForRestore = false;
 
-static Common::Point clickPos;
-
 #ifdef DEBUG
 bool bFast;		// set to make it go ludicrously fast
 #endif
@@ -144,7 +146,6 @@ void KeyboardProcess(CORO_PARAM, const void *) {
 		// Get the next keyboard event off the stack
 		Common::Event evt = _vm->_keypresses.front();
 		_vm->_keypresses.pop_front();
-		const Common::Point mousePos = _vm->getMousePosition();
 
 		// Switch for special keys
 		switch (evt.kbd.keycode) {
@@ -265,7 +266,7 @@ void KeyboardProcess(CORO_PARAM, const void *) {
  * Handles launching a single click action result if the timeout for a double-click
  * expires
  */
-static void SingleLeftProcess(CORO_PARAM, const void *) {
+static void SingleLeftProcess(CORO_PARAM, const void *param) {
 	CORO_BEGIN_CONTEXT;
 		uint32 endTicks;
 	CORO_END_CONTEXT(_ctx);
@@ -280,8 +281,10 @@ static void SingleLeftProcess(CORO_PARAM, const void *) {
 		CORO_SLEEP(1);
 	} while (DwGetCurrentTime() < _ctx->endTicks);
 
-	if (GetProvNotProcessed())
+	if (GetProvNotProcessed()) {
+		const Common::Point clickPos = *(const Common::Point *)param;
 		PlayerEvent(PLR_WALKTO, clickPos);
+	}
 
 	CORO_KILL_SELF();
 	CORO_END_CODE;
@@ -296,6 +299,7 @@ static void MouseProcess(CORO_PARAM, const void *) {
 		bool lastLWasDouble;
 		bool lastRWasDouble;
 		uint32 lastLeftClick, lastRightClick;
+		Common::Point clickPos;
 	CORO_END_CONTEXT(_ctx);
 
 	CORO_BEGIN_CODE(_ctx);
@@ -329,7 +333,7 @@ static void MouseProcess(CORO_PARAM, const void *) {
 				if (TinselV2) {
 					// Kill off the button process and fire off the action command
 					g_scheduler->killMatchingProcess(PID_BTN_CLICK, -1);
-					PlayerEvent(PLR_ACTION, clickPos);
+					PlayerEvent(PLR_ACTION, _ctx->clickPos);
 				} else {
 					// signal left drag start
 					ProcessButEvent(PLR_DRAG1_START);
@@ -371,8 +375,8 @@ static void MouseProcess(CORO_PARAM, const void *) {
 				// If player control is enabled, start a process which, if it times out,
 				// will activate a single button click
 				if (TinselV2 && ControlIsOn()) {
-					clickPos = mousePos;
-					g_scheduler->createProcess(PID_BTN_CLICK, SingleLeftProcess, NULL, 0);
+					_ctx->clickPos = mousePos;
+					g_scheduler->createProcess(PID_BTN_CLICK, SingleLeftProcess, &_ctx->clickPos, sizeof(Common::Point));
 				}
 			} else
 				_ctx->lastLeftClick -= _vm->_config->_dclickSpeed;
@@ -391,7 +395,7 @@ static void MouseProcess(CORO_PARAM, const void *) {
 			if (DwGetCurrentTime() - _ctx->lastRightClick < (uint32)_vm->_config->_dclickSpeed) {
 				// Right button double-click
 				if (TinselV2) {
-					PlayerEvent(PLR_NOEVENT, clickPos);
+					PlayerEvent(PLR_NOEVENT, _ctx->clickPos);
 				} else {
 					// signal right drag start
 					ProcessButEvent(PLR_DRAG2_START);
@@ -823,21 +827,21 @@ TinselEngine::TinselEngine(OSystem *syst, const TinselGameDescription *gameDesc)
 	_config = new Config(this);
 
 	// Register debug flags
-	Common::addDebugChannel(kTinselDebugAnimations, "animations", "Animations debugging");
-	Common::addDebugChannel(kTinselDebugActions, "actions", "Actions debugging");
-	Common::addDebugChannel(kTinselDebugSound, "sound", "Sound debugging");
-	Common::addDebugChannel(kTinselDebugMusic, "music", "Music debugging");
+	DebugMan.addDebugChannel(kTinselDebugAnimations, "animations", "Animations debugging");
+	DebugMan.addDebugChannel(kTinselDebugActions, "actions", "Actions debugging");
+	DebugMan.addDebugChannel(kTinselDebugSound, "sound", "Sound debugging");
+	DebugMan.addDebugChannel(kTinselDebugMusic, "music", "Music debugging");
 
 	// Setup mixer
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, ConfMan.getInt("sfx_volume"));
-	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, ConfMan.getInt("music_volume"));
+	syncSoundSettings();
 
 	// Add DW2 subfolder to search path in case user is running directly from the CDs
-	SearchMan.addSubDirectoryMatching(_gameDataDir, "dw2");
+	const Common::FSNode gameDataDir(ConfMan.get("path"));
+	SearchMan.addSubDirectoryMatching(gameDataDir, "dw2");
 
 	// Add subfolders needed for psx versions of Discworld 1
 	if (TinselV1PSX)
-		SearchMan.addDirectory(_gameDataDir.getPath(), _gameDataDir, 0, 3, true);
+		SearchMan.addDirectory(gameDataDir.getPath(), gameDataDir, 0, 3, true);
 
 	const GameSettings *g;
 
@@ -850,11 +854,11 @@ TinselEngine::TinselEngine(OSystem *syst, const TinselGameDescription *gameDesc)
 	if (cd_num >= 0)
 		_system->openCD(cd_num);
 
-	MidiDriverType midiDriver = MidiDriver::detectMusicDriver(MDT_MIDI | MDT_ADLIB | MDT_PREFER_MIDI);
-	bool native_mt32 = ((midiDriver == MD_MT32) || ConfMan.getBool("native_mt32"));
-	//bool adlib = (midiDriver == MD_ADLIB);
+	MidiDriver::DeviceHandle dev = MidiDriver::detectDevice(MDT_MIDI | MDT_ADLIB | MDT_PREFER_GM);
+	bool native_mt32 = ((MidiDriver::getMusicType(dev) == MT_MT32) || ConfMan.getBool("native_mt32"));
+	//bool adlib = (MidiDriver::getMusicType(dev) == MT_ADLIB);
 
-	_driver = MidiDriver::createMidi(midiDriver);
+	_driver = MidiDriver::createMidi(dev);
 	if (native_mt32)
 		_driver->property(MidiDriver::PROP_CHANNEL_MASK, 0x03FE);
 
@@ -863,6 +867,11 @@ TinselEngine::TinselEngine(OSystem *syst, const TinselGameDescription *gameDesc)
 	//_midiMusic->setNativeMT32(native_mt32);
 	//_midiMusic->setAdLib(adlib);
 
+	if (native_mt32)
+		_driver->sendMT32Reset();
+	else
+		_driver->sendGMReset();
+
 	_musicVolume = ConfMan.getInt("music_volume");
 
 	_sound = new SoundManager(this);
@@ -901,17 +910,6 @@ TinselEngine::~TinselEngine() {
 	MemoryDeinit();
 }
 
-void TinselEngine::syncSoundSettings() {
-	// Sync the engine with the config manager
-	int soundVolumeMusic = ConfMan.getInt("music_volume");
-	int soundVolumeSFX = ConfMan.getInt("sfx_volume");
-	int soundVolumeSpeech = ConfMan.getInt("speech_volume");
-
-	_mixer->setVolumeForSoundType(Audio::Mixer::kMusicSoundType, soundVolumeMusic);
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSFXSoundType, soundVolumeSFX);
-	_mixer->setVolumeForSoundType(Audio::Mixer::kSpeechSoundType, soundVolumeSpeech);
-}
-
 Common::String TinselEngine::getSavegameFilename(int16 saveNum) const {
 	char filename[256];
 	snprintf(filename, 256, "%s.%03d", getTargetName().c_str(), saveNum);
@@ -995,8 +993,7 @@ Common::Error TinselEngine::run() {
 	uint32 timerVal = 0;
 	while (!shouldQuit()) {
 		assert(_console);
-		if (_console->isAttached())
-			_console->onFrame();
+		_console->onFrame();
 
 		// Check for time to do next game cycle
 		if ((g_system->getMillis() > timerVal + GAME_FRAME_DELAY)) {
@@ -1176,7 +1173,11 @@ void TinselEngine::RestartDrivers() {
 	}
 
 	// Set midi volume
-	SetMidiVolume(_vm->_config->_musicVolume);
+	bool mute = false;
+	if (ConfMan.hasKey("mute"))
+		mute = ConfMan.getBool("mute");
+
+	SetMidiVolume(mute ? 0 : _vm->_config->_musicVolume);
 }
 
 /**
