@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/touche/detection.cpp $
- * $Id: detection.cpp 41272 2009-06-06 17:56:41Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/touche/detection.cpp $
+ * $Id: detection.cpp 49929 2010-06-17 13:42:02Z Kirben $
  *
  */
 
@@ -124,6 +124,11 @@ static const ADFileBasedFallback fileBasedFallback[] = {
 
 } // End of namespace Touche
 
+static const char *directoryGlobs[] = {
+	"database",
+	0
+};
+
 static const ADParams detectionParams = {
 	(const byte *)Touche::gameDescriptions,
 	sizeof(ADGameDescription),
@@ -134,7 +139,11 @@ static const ADParams detectionParams = {
 	Touche::fileBasedFallback, // file-based detection data to enable not yet known versions to start
 	kADFlagPrintWarningOnFileBasedFallback,
 	// Additional GUI options (for every game}
-	Common::GUIO_NONE
+	Common::GUIO_NONE,
+	// Maximum directory depth
+	2,
+	// List of directory globs
+	directoryGlobs
 };
 
 class ToucheMetaEngine : public AdvancedMetaEngine {
@@ -180,11 +189,11 @@ bool ToucheMetaEngine::createInstance(OSystem *syst, Engine **engine, const ADGa
 
 SaveStateList ToucheMetaEngine::listSaves(const char *target) const {
 	Common::String pattern = Touche::generateGameStateFileName(target, 0, true);
-	Common::StringList filenames = g_system->getSavefileManager()->listSavefiles(pattern);
+	Common::StringArray filenames = g_system->getSavefileManager()->listSavefiles(pattern);
 	bool slotsTable[Touche::kMaxSaveStates];
 	memset(slotsTable, 0, sizeof(slotsTable));
 	SaveStateList saveList;
-	for (Common::StringList::const_iterator file = filenames.begin(); file != filenames.end(); ++file) {
+	for (Common::StringArray::const_iterator file = filenames.begin(); file != filenames.end(); ++file) {
 		int slot = Touche::getGameStateFileSlot(file->c_str());
 		if (slot >= 0 && slot < Touche::kMaxSaveStates) {
 			slotsTable[slot] = true;
