@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/touche/menu.cpp $
- * $Id: menu.cpp 41000 2009-05-29 14:38:22Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/touche/menu.cpp $
+ * $Id: menu.cpp 48287 2010-03-18 15:54:40Z fingolfin $
  *
  */
 
@@ -377,8 +377,8 @@ void ToucheEngine::handleOptions(int forceDisplay) {
 						menuData.saveLoadDescriptionsTable[i][0] = 0;
 					}
 					Common::String gameStateFileName = generateGameStateFileName(_targetName.c_str(), 0, true);
-					Common::StringList filenames = _saveFileMan->listSavefiles(gameStateFileName);
-					for (Common::StringList::const_iterator it = filenames.begin(); it != filenames.end(); ++it) {
+					Common::StringArray filenames = _saveFileMan->listSavefiles(gameStateFileName);
+					for (Common::StringArray::const_iterator it = filenames.begin(); it != filenames.end(); ++it) {
 						int i = getGameStateFileSlot(it->c_str());
 						if (i >= 0 && i < kMaxSaveStates) {
 							Common::InSaveFile *f = _saveFileMan->openForLoading(*it);
