@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/touche/resource.cpp $
- * $Id: resource.cpp 47846 2010-02-03 09:42:11Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/touche/resource.cpp $
+ * $Id: resource.cpp 48637 2010-04-12 09:14:17Z fingolfin $
  *
  */
 
@@ -588,9 +588,9 @@ void ToucheEngine::res_loadSound(int priority, int num) {
 		uint32 size;
 		const uint32 offs = res_getDataOffset(kResourceTypeSound, num, &size);
 		_fData.seek(offs);
-		Audio::AudioStream *stream = Audio::makeVOCStream(_fData, Audio::FLAG_UNSIGNED);
+		Audio::AudioStream *stream = Audio::makeVOCStream(&_fData, Audio::FLAG_UNSIGNED);
 		if (stream) {
-			_mixer->playInputStream(Audio::Mixer::kSFXSoundType, &_sfxHandle, stream);
+			_mixer->playStream(Audio::Mixer::kSFXSoundType, &_sfxHandle, stream);
 		}
 	}
 }
@@ -646,7 +646,7 @@ void ToucheEngine::res_loadSpeechSegment(int num) {
 				return;
 			}
 			_fSpeech[i].seek(offs);
-			stream = Audio::makeVOCStream(_fSpeech[i], Audio::FLAG_UNSIGNED);
+			stream = Audio::makeVOCStream(&_fSpeech[i], Audio::FLAG_UNSIGNED);
 		} else {
 			if (num >= 750) {
 				num -= 750;
@@ -672,7 +672,7 @@ void ToucheEngine::res_loadSpeechSegment(int num) {
 		}
 		if (stream) {
 			_speechPlaying = true;
-			_mixer->playInputStream(Audio::Mixer::kSpeechSoundType, &_speechHandle, stream);
+			_mixer->playStream(Audio::Mixer::kSpeechSoundType, &_speechHandle, stream);
 		}
 	}
 }
