@@ -18,17 +18,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/touche/touche.cpp $
- * $Id: touche.cpp 48101 2010-02-21 04:04:13Z bluddy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/touche/touche.cpp $
+ * $Id: touche.cpp 49929 2010-06-17 13:42:02Z Kirben $
  *
  */
 
 
 #include "common/config-manager.h"
+#include "common/debug-channels.h"
 #include "common/events.h"
 #include "common/EventRecorder.h"
+#include "common/file.h"
+#include "common/fs.h"
 #include "common/system.h"
 
+#include "engines/util.h"
 #include "graphics/cursorman.h"
 #include "sound/mididrv.h"
 
@@ -68,17 +72,21 @@ ToucheEngine::ToucheEngine(OSystem *system, Common::Language language)
 	_menuRedrawCounter = 0;
 	memset(_paletteBuffer, 0, sizeof(_paletteBuffer));
 
-	Common::addDebugChannel(kDebugEngine,   "Engine",   "Engine debug level");
-	Common::addDebugChannel(kDebugGraphics, "Graphics", "Graphics debug level");
-	Common::addDebugChannel(kDebugResource, "Resource", "Resource debug level");
-	Common::addDebugChannel(kDebugOpcodes,  "Opcodes",  "Opcodes debug level");
-	Common::addDebugChannel(kDebugMenu,     "Menu",     "Menu debug level");
+	const Common::FSNode gameDataDir(ConfMan.get("path"));
+
+	SearchMan.addSubDirectoryMatching(gameDataDir, "database");
+
+	DebugMan.addDebugChannel(kDebugEngine,   "Engine",   "Engine debug level");
+	DebugMan.addDebugChannel(kDebugGraphics, "Graphics", "Graphics debug level");
+	DebugMan.addDebugChannel(kDebugResource, "Resource", "Resource debug level");
+	DebugMan.addDebugChannel(kDebugOpcodes,  "Opcodes",  "Opcodes debug level");
+	DebugMan.addDebugChannel(kDebugMenu,     "Menu",     "Menu debug level");
 
 	g_eventRec.registerRandomSource(_rnd, "touche");
 }
 
 ToucheEngine::~ToucheEngine() {
-	Common::clearAllDebugChannels();
+	DebugMan.clearAllDebugChannels();
 	delete _midiPlayer;
 }
 
