@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/engines/tucker/resource.cpp $
- * $Id: resource.cpp 47846 2010-02-03 09:42:11Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/engines/tucker/resource.cpp $
+ * $Id: resource.cpp 48637 2010-04-12 09:14:17Z fingolfin $
  *
  */
 
@@ -954,7 +954,7 @@ void TuckerEngine::loadSound(Audio::Mixer::SoundType type, int num, int volume,
 
 	if (stream) {
 		_mixer->stopHandle(*handle);
-		_mixer->playInputStream(type, handle, Audio::makeLoopingAudioStream(stream, loop ? 0 : 1),
+		_mixer->playStream(type, handle, Audio::makeLoopingAudioStream(stream, loop ? 0 : 1),
 		                        -1, scaleMixerVolume(volume, kMaxSoundVolume));
 	}
 }
