@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/graphics/font.cpp $
- * $Id: font.cpp 46779 2009-12-30 23:00:55Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/graphics/font.cpp $
+ * $Id: font.cpp 51586 2010-08-01 19:31:37Z lordhoto $
  */
 
 #include "common/stream.h"
@@ -609,8 +609,8 @@ bool NewFont::cacheFontData(const NewFont &font, const Common::String &filename)
 	cacheFile.writeUint16BE(font.desc.height);
 	cacheFile.writeUint16BE(font.desc.fbbw);
 	cacheFile.writeUint16BE(font.desc.fbbh);
-	cacheFile.writeUint16BE(font.desc.fbbx);
-	cacheFile.writeUint16BE(font.desc.fbby);
+	cacheFile.writeSint16BE(font.desc.fbbx);
+	cacheFile.writeSint16BE(font.desc.fbby);
 	cacheFile.writeUint16BE(font.desc.ascent);
 	cacheFile.writeUint16BE(font.desc.firstchar);
 	cacheFile.writeUint16BE(font.desc.size);
@@ -667,8 +667,8 @@ NewFont *NewFont::loadFromCache(Common::SeekableReadStream &stream) {
 	data->height = stream.readUint16BE();
 	data->fbbw = stream.readUint16BE();
 	data->fbbh = stream.readUint16BE();
-	data->fbbx = stream.readUint16BE();
-	data->fbby = stream.readUint16BE();
+	data->fbbx = stream.readSint16BE();
+	data->fbby = stream.readSint16BE();
 	data->ascent = stream.readUint16BE();
 	data->firstchar = stream.readUint16BE();
 	data->size = stream.readUint16BE();
@@ -864,10 +864,10 @@ void Font::drawString(Surface *dst, const Common::String &sOld, int x, int y, in
 
 
 struct WordWrapper {
-	Common::StringList &lines;
+	Common::Array<Common::String> &lines;
 	int actualMaxLineWidth;
 
-	WordWrapper(Common::StringList &l) : lines(l), actualMaxLineWidth(0) {
+	WordWrapper(Common::Array<Common::String> &l) : lines(l), actualMaxLineWidth(0) {
 	}
 
 	void add(Common::String &line, int &w) {
@@ -881,7 +881,7 @@ struct WordWrapper {
 	}
 };
 
-int Font::wordWrapText(const Common::String &str, int maxWidth, Common::StringList &lines) const {
+int Font::wordWrapText(const Common::String &str, int maxWidth, Common::Array<Common::String> &lines) const {
 	WordWrapper wrapper(lines);
 	Common::String line;
 	Common::String tmpStr;
