@@ -18,14 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/graphics/font.h $
- * $Id: font.h 46329 2009-12-10 21:40:38Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/graphics/font.h $
+ * $Id: font.h 49488 2010-06-07 17:17:41Z jvprat $
  */
 
 #ifndef GRAPHICS_FONT_H
 #define GRAPHICS_FONT_H
 
 #include "common/str.h"
+#include "common/array.h"
 #include "graphics/surface.h"
 
 namespace Common {
@@ -66,10 +67,10 @@ public:
 	int getStringWidth(const Common::String &str) const;
 
 	/**
-	 * Take a text (which may contain newlines characters) and word wrap it so thata
+	 * Take a text (which may contain newline characters) and word wrap it so that
 	 * no text line is wider than maxWidth pixels. If necessary, additional line breaks
-	 * are generated, preferably between words (i.e. were whitespaces are).
-	 * The resulting lines are appended to the string list lines.
+	 * are generated, preferably between words (i.e. where whitespaces are).
+	 * The resulting lines are appended to the lines string list.
 	 * It returns the maximal width of any of the new lines (i.e. a value which is less
 	 * or equal to maxWidth).
 	 *
@@ -78,7 +79,7 @@ public:
 	 * @param lines	the string list to which the text lines from str are appended
 	 * @return the maximal width of any of the lines added to lines
 	 */
-	int wordWrapText(const Common::String &str, int maxWidth, Common::StringList &lines) const;
+	int wordWrapText(const Common::String &str, int maxWidth, Common::Array<Common::String> &lines) const;
 };
 
 
