@@ -18,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/graphics/fontman.cpp $
- * $Id: fontman.cpp 46329 2009-12-10 21:40:38Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/graphics/fontman.cpp $
+ * $Id: fontman.cpp 50448 2010-06-28 15:17:10Z sev $
  */
 
 #include "graphics/fontman.h"
 //#include "gui/consolefont.h"
 
-DECLARE_SINGLETON(Graphics::FontManager);
+DECLARE_SINGLETON(Graphics::FontManager)
 
 namespace Graphics {
 
@@ -57,7 +57,29 @@ FontManager::~FontManager() {
 	g_consolefont = 0;
 }
 
+const struct {
+	const char *name;
+	FontManager::FontUsage id;
+} builtinFontNames[] = {
+	{ "builtinOSD", FontManager::kOSDFont },
+	{ "builtinConsole", FontManager::kConsoleFont },
+	{ "fixed5x8.bdf", FontManager::kConsoleFont },
+	{ "fixed5x8-iso-8859-1.bdf", FontManager::kConsoleFont },
+	{ "fixed5x8-ascii.bdf", FontManager::kConsoleFont },
+	{ "clR6x12.bdf", FontManager::kGUIFont },
+	{ "clR6x12-iso-8859-1.bdf", FontManager::kGUIFont },
+	{ "clR6x12-ascii.bdf", FontManager::kGUIFont },
+	{ "helvB12.bdf", FontManager::kBigGUIFont },
+	{ "helvB12-iso-8859-1.bdf", FontManager::kBigGUIFont },
+	{ "helvB12-ascii.bdf", FontManager::kBigGUIFont },
+	{ 0, FontManager::kOSDFont }
+};
+
 const Font *FontManager::getFontByName(const Common::String &name) const {
+	for (int i = 0; builtinFontNames[i].name; i++)
+		if (!scumm_stricmp(name.c_str(), builtinFontNames[i].name))
+			return getFontByUsage(builtinFontNames[i].id);
+
 	if (!_fontMap.contains(name))
 		return 0;
 	return _fontMap[name];
