@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/graphics/surface.cpp $
- * $Id: surface.cpp 35965 2009-01-20 22:52:29Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/graphics/surface.cpp $
+ * $Id: surface.cpp 52268 2010-08-21 20:14:46Z sev $
  */
 
 #include "common/algorithm.h"
@@ -135,23 +135,40 @@ void Surface::fillRect(Common::Rect r, uint32 color) {
 		return;
 
 	int width = r.width();
+	int lineLen = width;
 	int height = r.height();
-//	int i;
+	bool useMemset = true;
 
-	if (bytesPerPixel == 1) {
+	if (bytesPerPixel == 2) {
+		lineLen *= 2;
+		if ((uint16)color != ((color & 0xff) | (color & 0xff) << 8))
+			useMemset = false;
+	} else if (bytesPerPixel == 4) {
+		useMemset = false;
+	} else if (bytesPerPixel != 1) {
+		error("Surface::fillRect: bytesPerPixel must be 1, 2 or 4");
+	}
+
+	if (useMemset) {
 		byte *ptr = (byte *)getBasePtr(r.left, r.top);
 		while (height--) {
-			memset(ptr, (byte)color, width);
+			memset(ptr, (byte)color, lineLen);
 			ptr += pitch;
 		}
-	} else if (bytesPerPixel == 2) {
-		uint16 *ptr = (uint16 *)getBasePtr(r.left, r.top);
-		while (height--) {
-			Common::set_to(ptr, ptr + width, (uint16)color);
-			ptr += pitch/2;
-		}
 	} else {
-		error("Surface::fillRect: bytesPerPixel must be 1 or 2");
+		if (bytesPerPixel == 2) {
+			uint16 *ptr = (uint16 *)getBasePtr(r.left, r.top);
+			while (height--) {
+				Common::set_to(ptr, ptr + width, (uint16)color);
+				ptr += pitch/2;
+			}
+		} else {
+			uint32 *ptr = (uint32 *)getBasePtr(r.left, r.top);
+			while (height--) {
+				Common::set_to(ptr, ptr + width, color);
+				ptr += pitch / 4;
+			}
+		}
 	}
 }
 
