@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/graphics/video/codecs/codec.h $
- * $Id: codec.h 47541 2010-01-25 01:39:44Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/graphics/video/codecs/codec.h $
+ * $Id: codec.h 49079 2010-05-18 14:17:24Z mthreepwood $
  *
  */
 
@@ -28,6 +28,7 @@
 
 #include "common/stream.h"
 #include "graphics/surface.h"
+#include "graphics/pixelformat.h"
 
 namespace Graphics {
 
@@ -35,7 +36,9 @@ class Codec {
 public:
 	Codec() {}
 	virtual ~Codec() {}
+
 	virtual Surface *decodeImage(Common::SeekableReadStream *stream) = 0;
+	virtual PixelFormat getPixelFormat() const = 0;
 };
 
 } // End of namespace Graphics
