@@ -18,14 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/graphics/video/smk_decoder.cpp $
- * $Id: smk_decoder.cpp 47579 2010-01-26 22:48:45Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/graphics/video/smk_decoder.cpp $
+ * $Id: smk_decoder.cpp 53896 2010-10-27 23:11:15Z athrxx $
  *
  */
 
 // Based on http://wiki.multimedia.cx/index.php?title=Smacker
 // and the FFmpeg Smacker decoder (libavcodec/smacker.c), revision 16143
-// http://svn.ffmpeg.org/ffmpeg/trunk/libavcodec/smacker.c?revision=16143&view=markup
+// http://git.ffmpeg.org/?p=ffmpeg;a=blob;f=libavcodec/smacker.c;hb=b8437a00a2f14d4a437346455d624241d726128e
 
 #include "graphics/video/smk_decoder.h"
 
@@ -252,9 +252,12 @@ BigHuffmanTree::BigHuffmanTree(BitStream &bs, int allocSize)
 	_loBytes = new SmallHuffmanTree(_bs);
 	_hiBytes = new SmallHuffmanTree(_bs);
 
-	_markers[0] = _bs.getBits8() | (_bs.getBits8() << 8);
-	_markers[1] = _bs.getBits8() | (_bs.getBits8() << 8);
-	_markers[2] = _bs.getBits8() | (_bs.getBits8() << 8);
+	_markers[0] = _bs.getBits8();
+	_markers[0] |= (_bs.getBits8() << 8);
+	_markers[1] = _bs.getBits8();
+	_markers[1] |= (_bs.getBits8() << 8);
+	_markers[2] = _bs.getBits8();
+	_markers[2] |= (_bs.getBits8() << 8);
 
 	_last[0] = _last[1] = _last[2] = 0xffffffff;
 
@@ -351,52 +354,28 @@ uint32 BigHuffmanTree::getCode(BitStream &bs) {
 
 SmackerDecoder::SmackerDecoder(Audio::Mixer *mixer, Audio::Mixer::SoundType soundType)
 	: _audioStarted(false), _audioStream(0), _mixer(mixer), _soundType(soundType) {
+	_surface = 0;
+	_fileStream = 0;
+	_dirtyPalette = false;
 }
 
 SmackerDecoder::~SmackerDecoder() {
-	closeFile();
+	close();
 }
 
-int SmackerDecoder::getHeight() {
-	if (!_fileStream)
-		return 0;
-	return (_header.flags ? 2 : 1) * _videoInfo.height;
+uint32 SmackerDecoder::getElapsedTime() const {
+	if (_audioStream && _audioStarted)
+		return _mixer->getSoundElapsedTime(_audioHandle);
+
+	return VideoDecoder::getElapsedTime();
 }
 
-int32 SmackerDecoder::getAudioLag() {
-	if (!_fileStream)
-		return 0;
+bool SmackerDecoder::load(Common::SeekableReadStream *stream) {
+	close();
 
-	int32 frameDelay = getFrameDelay();
-	int32 videoTime = _videoInfo.currentFrame * frameDelay;
-	int32 audioTime;
-
-	if (!_audioStream) {
-		/* No audio.
-		   Calculate the lag by how much time has gone by since the first frame
-		   and how much time *should* have passed.
-		*/
-
-		audioTime = (g_system->getMillis() - _videoInfo.startTime) * 100;
-	} else {
-		const Audio::Timestamp ts = _mixer->getElapsedTime(_audioHandle);
-		audioTime = ts.convertToFramerate(100000).totalNumberOfFrames();
-	}
-
-	return videoTime - audioTime;
-}
-
-bool SmackerDecoder::loadFile(const char *fileName) {
-	int32 frameRate;
-
-	closeFile();
-
-	_fileStream = SearchMan.createReadStreamForMember(fileName);
-	if (!_fileStream)
-		return false;
+	_fileStream = stream;
 
 	// Seek to the first frame
-	_videoInfo.currentFrame = 0;
 	_header.signature = _fileStream->readUint32BE();
 
 	// No BINK support available
@@ -408,21 +387,17 @@ bool SmackerDecoder::loadFile(const char *fileName) {
 
 	assert(_header.signature == MKID_BE('SMK2') || _header.signature == MKID_BE('SMK4'));
 
-	_videoInfo.width = _fileStream->readUint32LE();
-	_videoInfo.height = _fileStream->readUint32LE();
-	_videoInfo.frameCount = _fileStream->readUint32LE();
-	frameRate = _fileStream->readSint32LE();
+	uint32 width = _fileStream->readUint32LE();
+	uint32 height = _fileStream->readUint32LE();
+	_frameCount = _fileStream->readUint32LE();
+	int32 frameRate = _fileStream->readSint32LE();
 
-	if (frameRate > 0) {
-		_videoInfo.frameRate = 1000 / frameRate;
-		_videoInfo.frameDelay = frameRate * 100;
-	} else if (frameRate < 0) {
-		_videoInfo.frameRate = 100000 / (-frameRate);
-		_videoInfo.frameDelay = -frameRate;
-	} else {
-		_videoInfo.frameRate = 10;
-		_videoInfo.frameDelay = 10000;
-	}
+	if (frameRate > 0)
+		_frameRate = Common::Rational(1000, frameRate);
+	else if (frameRate < 0)
+		_frameRate = Common::Rational(100000, -frameRate);
+	else
+		_frameRate = 10;
 
 	// Flags are determined by which bit is set, which can be one of the following:
 	// 0 - set to 1 if file contains a ring frame.
@@ -447,39 +422,49 @@ bool SmackerDecoder::loadFile(const char *fileName) {
 	_header.fullSize = _fileStream->readUint32LE();
 	_header.typeSize = _fileStream->readUint32LE();
 
-	uint32 audioInfo;
 	for (i = 0; i < 7; ++i) {
 		// AudioRate - Frequency and format information for each sound track, up to 7 audio tracks.
 		// The 32 constituent bits have the following meaning:
-		// * bit 31 - data is compressed
+		// * bit 31 - indicates Huffman + DPCM compression
 		// * bit 30 - indicates that audio data is present for this track
 		// * bit 29 - 1 = 16-bit audio; 0 = 8-bit audio
 		// * bit 28 - 1 = stereo audio; 0 = mono audio
-		// * bits 27-26 - if both set to zero - use v2 sound decompression
+		// * bit 27 - indicates Bink RDFT compression
+		// * bit 26 - indicates Bink DCT compression
 		// * bits 25-24 - unused
 		// * bits 23-0 - audio sample rate
-		audioInfo = _fileStream->readUint32LE();
-		_header.audioInfo[i].isCompressed = audioInfo & 0x80000000;
+		uint32 audioInfo = _fileStream->readUint32LE();
 		_header.audioInfo[i].hasAudio = audioInfo & 0x40000000;
 		_header.audioInfo[i].is16Bits = audioInfo & 0x20000000;
 		_header.audioInfo[i].isStereo = audioInfo & 0x10000000;
-		_header.audioInfo[i].hasV2Compression = !(audioInfo & 0x8000000) &&
-												!(audioInfo & 0x4000000);
 		_header.audioInfo[i].sampleRate = audioInfo & 0xFFFFFF;
 
-		if (_header.audioInfo[i].hasAudio && i == 0) {
-			_audioStream = Audio::makeQueuingAudioStream(_header.audioInfo[0].sampleRate, _header.audioInfo[0].isStereo);
+		if (audioInfo & 0x8000000)
+			_header.audioInfo[i].compression = kCompressionRDFT;
+		else if (audioInfo & 0x4000000)
+			_header.audioInfo[i].compression = kCompressionDCT;
+		else if (audioInfo & 0x80000000)
+			_header.audioInfo[i].compression = kCompressionDPCM;
+		else
+			_header.audioInfo[i].compression = kCompressionNone;
+
+		if (_header.audioInfo[i].hasAudio) {
+			if (_header.audioInfo[i].compression == kCompressionRDFT || _header.audioInfo[i].compression == kCompressionDCT)
+				warning("Unhandled Smacker v2 audio compression");
+
+			if (i == 0)
+				_audioStream = Audio::makeQueuingAudioStream(_header.audioInfo[0].sampleRate, _header.audioInfo[0].isStereo);
 		}
 	}
 
 	_header.dummy = _fileStream->readUint32LE();
 
-	_frameSizes = (uint32 *)malloc(_videoInfo.frameCount * sizeof(uint32));
-	for (i = 0; i < _videoInfo.frameCount; ++i)
+	_frameSizes = new uint32[_frameCount];
+	for (i = 0; i < _frameCount; ++i)
 		_frameSizes[i] = _fileStream->readUint32LE();
 
-	_frameTypes = (byte *)malloc(_videoInfo.frameCount);
-	for (i = 0; i < _videoInfo.frameCount; ++i)
+	_frameTypes = new byte[_frameCount];
+	for (i = 0; i < _frameCount; ++i)
 		_frameTypes[i] = _fileStream->readByte();
 
 	byte *huffmanTrees = new byte[_header.treesSize];
@@ -494,17 +479,17 @@ bool SmackerDecoder::loadFile(const char *fileName) {
 
 	delete[] huffmanTrees;
 
-	_videoFrameBuffer = (byte *)malloc(2 * _videoInfo.width * _videoInfo.height);
-	memset(_videoFrameBuffer, 0, 2 * _videoInfo.width * _videoInfo.height);
+	_surface = new Graphics::Surface();
+
+	// Height needs to be doubled if we have flags (Y-interlaced or Y-doubled)
+	_surface->create(width, height * (_header.flags ? 2 : 1), 1);
+
 	_palette = (byte *)malloc(3 * 256);
 	memset(_palette, 0, 3 * 256);
-
-	_videoInfo.firstframeOffset = _fileStream->pos();
-
 	return true;
 }
 
-void SmackerDecoder::closeFile() {
+void SmackerDecoder::close() {
 	if (!_fileStream)
 		return;
 
@@ -517,44 +502,48 @@ void SmackerDecoder::closeFile() {
 	delete _fileStream;
 	_fileStream = 0;
 
+	_surface->free();
+	delete _surface;
+	_surface = 0;
+
 	delete _MMapTree;
 	delete _MClrTree;
 	delete _FullTree;
 	delete _TypeTree;
 
-	free(_frameSizes);
-	free(_frameTypes);
-	free(_videoFrameBuffer);
+	delete[] _frameSizes;
+	delete[] _frameTypes;
 	free(_palette);
+
+	reset();
 }
 
-bool SmackerDecoder::decodeNextFrame() {
+Surface *SmackerDecoder::decodeNextFrame() {
 	uint i;
 	uint32 chunkSize = 0;
 	uint32 dataSizeUnpacked = 0;
 
 	uint32 startPos = _fileStream->pos();
 
-	if (_videoInfo.currentFrame == 0)
-		_videoInfo.startTime = g_system->getMillis();
+	_curFrame++;
 
 	// Check if we got a frame with palette data, and
 	// call back the virtual setPalette function to set
 	// the current palette
-	if (_frameTypes[_videoInfo.currentFrame] & 1) {
+	if (_frameTypes[_curFrame] & 1) {
 		unpackPalette();
-		setPalette(_palette);
+		_dirtyPalette = true;
 	}
 
 	// Load audio tracks
 	for (i = 0; i < 7; ++i) {
-		if (!(_frameTypes[_videoInfo.currentFrame] & (2 << i)))
+		if (!(_frameTypes[_curFrame] & (2 << i)))
 			continue;
 
 		chunkSize = _fileStream->readUint32LE();
 		chunkSize -= 4;    // subtract the first 4 bytes (chunk size)
 
-		if (_header.audioInfo[i].isCompressed) {
+		if (_header.audioInfo[i].compression != kCompressionNone) {
 			dataSizeUnpacked = _fileStream->readUint32LE();
 			chunkSize -= 4;    // subtract the next 4 bytes (unpacked data size)
 		} else {
@@ -567,7 +556,11 @@ bool SmackerDecoder::decodeNextFrame() {
 
 			_fileStream->read(soundBuffer, chunkSize);
 
-			if (_header.audioInfo[i].isCompressed) {
+			if (_header.audioInfo[i].compression == kCompressionRDFT || _header.audioInfo[i].compression == kCompressionDCT) {
+				// TODO: Compressed audio (Bink RDFT/DCT encoded)
+				free(soundBuffer);
+				continue;
+			} else if (_header.audioInfo[i].compression == kCompressionDPCM) {
 				// Compressed audio (Huffman DPCM encoded)
 				queueCompressedBuffer(soundBuffer, chunkSize, dataSizeUnpacked, i);
 				free(soundBuffer);
@@ -584,7 +577,7 @@ bool SmackerDecoder::decodeNextFrame() {
 			}
 
 			if (!_audioStarted) {
-				_mixer->playInputStream(_soundType, &_audioHandle, _audioStream, -1, 255);
+				_mixer->playStream(_soundType, &_audioHandle, _audioStream, -1, 255);
 				_audioStarted = true;
 			}
 		} else {
@@ -596,10 +589,10 @@ bool SmackerDecoder::decodeNextFrame() {
 		}
 	}
 
-	uint32 frameSize = _frameSizes[_videoInfo.currentFrame] & ~3;
+	uint32 frameSize = _frameSizes[_curFrame] & ~3;
 
 	if (_fileStream->pos() - startPos > frameSize)
-		exit(1);
+		error("Smacker actual frame size exceeds recorded frame size");
 
 	uint32 frameDataSize = frameSize - (_fileStream->pos() - startPos);
 
@@ -613,13 +606,14 @@ bool SmackerDecoder::decodeNextFrame() {
 	_FullTree->reset();
 	_TypeTree->reset();
 
-	uint bw = _videoInfo.width / 4;
-	uint bh = _videoInfo.height / 4;
-	uint stride = _videoInfo.width;
-	uint block = 0, blocks = bw*bh;
-
+	// Height needs to be doubled if we have flags (Y-interlaced or Y-doubled)
 	uint doubleY = _header.flags ? 2 : 1;
 
+	uint bw = getWidth() / 4;
+	uint bh = getHeight() / doubleY / 4;
+	uint stride = getWidth();
+	uint block = 0, blocks = bw*bh;
+
 	byte *out;
 	uint type, run, j, mode;
 	uint32 p1, p2, clr, map;
@@ -634,7 +628,7 @@ bool SmackerDecoder::decodeNextFrame() {
 			while (run-- && block < blocks) {
 				clr = _MClrTree->getCode(bs);
 				map = _MMapTree->getCode(bs);
-				out = _videoFrameBuffer + (block / bw) * (stride * 4 * doubleY) + (block % bw) * 4;
+				out = (byte *)_surface->pixels + (block / bw) * (stride * 4 * doubleY) + (block % bw) * 4;
 				hi = clr >> 8;
 				lo = clr & 0xff;
 				for (i = 0; i < 4; i++) {
@@ -667,7 +661,7 @@ bool SmackerDecoder::decodeNextFrame() {
 			}
 
 			while (run-- && block < blocks) {
-				out = _videoFrameBuffer + (block / bw) * (stride * 4 * doubleY) + (block % bw) * 4;
+				out = (byte *)_surface->pixels + (block / bw) * (stride * 4 * doubleY) + (block % bw) * 4;
 				switch (mode) {
 					case 0:
 						for (i = 0; i < 4; ++i) {
@@ -733,7 +727,7 @@ bool SmackerDecoder::decodeNextFrame() {
 			uint32 col;
 			mode = type >> 8;
 			while (run-- && block < blocks) {
-				out = _videoFrameBuffer + (block / bw) * (stride * 4 * doubleY) + (block % bw) * 4;
+				out = (byte *)_surface->pixels + (block / bw) * (stride * 4 * doubleY) + (block % bw) * 4;
 				col = mode * 0x01010101;
 				for (i = 0; i < 4 * doubleY; ++i) {
 					out[0] = out[1] = out[2] = out[3] = col;
@@ -749,7 +743,10 @@ bool SmackerDecoder::decodeNextFrame() {
 
 	free(_frameData);
 
-	return ++_videoInfo.currentFrame < _videoInfo.frameCount;
+	if (_curFrame == 0)
+		_startTime = g_system->getMillis();
+
+	return _surface;
 }
 
 void SmackerDecoder::queueCompressedBuffer(byte *buffer, uint32 bufferSize,
@@ -780,13 +777,23 @@ void SmackerDecoder::queueCompressedBuffer(byte *buffer, uint32 bufferSize,
 
 	int32 bases[2];
 
-	if (isStereo)
-		bases[1] = (!is16Bits) ?   audioBS.getBits8() :
-		               ((int16) (((audioBS.getBits8() << 8) | audioBS.getBits8())));
-
-	bases[0] = (!is16Bits) ?   audioBS.getBits8() :
-	               ((int16) (((audioBS.getBits8() << 8) | audioBS.getBits8())));
+	if (isStereo) {
+		if (is16Bits) {
+			byte hi = audioBS.getBits8();
+			byte lo = audioBS.getBits8();
+			bases[1] = (int16) ((hi << 8) | lo);
+		} else {
+			bases[1] = audioBS.getBits8();
+		}
+	}
 
+	if (is16Bits) {
+		byte hi = audioBS.getBits8();
+		byte lo = audioBS.getBits8();
+		bases[0] = (int16) ((hi << 8) | lo);
+	} else {
+		bases[0] = audioBS.getBits8();
+	}
 
 	// The bases are the first samples, too
 	for (int i = 0; i < (isStereo ? 2 : 1); i++, curPointer += (is16Bits ? 2 : 1), curPos += (is16Bits ? 2 : 1)) {
@@ -804,18 +811,16 @@ void SmackerDecoder::queueCompressedBuffer(byte *buffer, uint32 bufferSize,
 		// If the sample is stereo, the data is stored for the left and right channel, respectively
 		// (the exact opposite to the base values)
 		if (!is16Bits) {
-
 			for (int k = 0; k < (isStereo ? 2 : 1); k++) {
 				bases[k] += (int8) ((int16) audioTrees[k]->getCode(audioBS));
 				*curPointer++ = CLIP<int>(bases[k], 0, 255) ^ 0x80;
 				curPos++;
 			}
-
 		} else {
-
 			for (int k = 0; k < (isStereo ? 2 : 1); k++) {
-				bases[k] += (int16) (audioTrees[k * 2]->getCode(audioBS) |
-				                    (audioTrees[k * 2 + 1]->getCode(audioBS) << 8));
+				byte lo = audioTrees[k * 2]->getCode(audioBS);
+				byte hi = audioTrees[k * 2 + 1]->getCode(audioBS);
+				bases[k] += (int16) (lo | (hi << 8));
 
 				WRITE_BE_UINT16(curPointer, CLIP<int32>(bases[k], -32768, 32767));
 				curPointer += 2;
@@ -885,7 +890,6 @@ void SmackerDecoder::unpackPalette() {
 	}
 
 	_fileStream->seek(startPos + len);
-
 	free(chunk);
 }
 
