@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/gui/EditTextWidget.h $
- * $Id: EditTextWidget.h 41267 2009-06-06 17:53:25Z sev $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/gui/EditTextWidget.h $
+ * $Id: EditTextWidget.h 49779 2010-06-15 10:54:22Z sev $
  */
 
 #ifndef GUI_EDITTEXTWIDGET_H
@@ -41,8 +41,8 @@ protected:
 	int				_rightPadding;
 
 public:
-	EditTextWidget(GuiObject *boss, int x, int y, int w, int h, const String &text, uint32 cmd = 0);
-	EditTextWidget(GuiObject *boss, const String &name, const String &text, uint32 cmd = 0);
+	EditTextWidget(GuiObject *boss, int x, int y, int w, int h, const String &text, const char *tooltip = 0, uint32 cmd = 0);
+	EditTextWidget(GuiObject *boss, const String &name, const String &text, const char *tooltp = 0, uint32 cmd = 0);
 
 	void setEditString(const String &str);
 
