@@ -18,13 +18,14 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/gui/ListWidget.cpp $
- * $Id: ListWidget.cpp 48101 2010-02-21 04:04:13Z bluddy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/gui/ListWidget.cpp $
+ * $Id: ListWidget.cpp 52716 2010-09-14 00:15:20Z lordhoto $
  */
 
 #include "common/system.h"
 #include "common/events.h"
 #include "common/frac.h"
+#include "common/tokenizer.h"
 
 #include "gui/ListWidget.h"
 #include "gui/ScrollBarWidget.h"
@@ -35,8 +36,8 @@
 
 namespace GUI {
 
-ListWidget::ListWidget(GuiObject *boss, const String &name, uint32 cmd)
-	: EditableWidget(boss, name), _cmd(cmd) {
+ListWidget::ListWidget(Dialog *boss, const String &name, const char *tooltip, uint32 cmd)
+	: EditableWidget(boss, name, tooltip), _cmd(cmd) {
 
 	_scrollBar = NULL;
 	_textWidth = NULL;
@@ -67,8 +68,8 @@ ListWidget::ListWidget(GuiObject *boss, const String &name, uint32 cmd)
 	_editColor = ThemeEngine::kFontColorNormal;
 }
 
-ListWidget::ListWidget(GuiObject *boss, int x, int y, int w, int h, uint32 cmd)
-	: EditableWidget(boss, x, y, w, h), _cmd(cmd) {
+ListWidget::ListWidget(Dialog *boss, int x, int y, int w, int h, const char *tooltip, uint32 cmd)
+	: EditableWidget(boss, x, y, w, h, tooltip), _cmd(cmd) {
 
 	_scrollBar = NULL;
 	_textWidth = NULL;
@@ -152,7 +153,7 @@ ThemeEngine::FontColor ListWidget::getSelectionColor() const {
 		return _listColors[_listIndex[_selectedItem]];
 }
 
-void ListWidget::setList(const StringList &list, const ColorList *colors) {
+void ListWidget::setList(const StringArray &list, const ColorList *colors) {
 	if (_editMode && _caretVisible)
 		drawCaret(true);
 
@@ -290,7 +291,7 @@ bool ListWidget::handleKeyDown(Common::KeyState state) {
 		// Quick selection mode: Go to first list item starting with this key
 		// (or a substring accumulated from the last couple key presses).
 		// Only works in a useful fashion if the list entries are sorted.
-		uint32 time = getMillis();
+		uint32 time = g_system->getMillis();
 		if (_quickSelectTime < time) {
 			_quickSelectStr = (char)state.ascii;
 		} else {
@@ -306,7 +307,7 @@ bool ListWidget::handleKeyDown(Common::KeyState state) {
 			int newSelectedItem = 0;
 			int bestMatch = 0;
 			bool stop;
-			for (StringList::const_iterator i = _list.begin(); i != _list.end(); ++i) {
+			for (StringArray::const_iterator i = _list.begin(); i != _list.end(); ++i) {
 				const int match = matchingCharsIgnoringCase(i->c_str(), _quickSelectStr.c_str(), stop);
 				if (match > bestMatch || stop) {
 					_selectedItem = newSelectedItem;
@@ -438,11 +439,6 @@ bool ListWidget::handleKeyDown(Common::KeyState state) {
 		_scrollBar->draw();
 	}
 
-#if !defined(PALMOS_MODE)
-	// not done on PalmOS because keyboard is emulated and keyup is not generated
-	_currentKeyDown = state.keycode;
-#endif
-
 	return handled;
 }
 
@@ -453,11 +449,15 @@ bool ListWidget::handleKeyUp(Common::KeyState state) {
 }
 
 void ListWidget::receivedFocusWidget() {
+	_inversion = ThemeEngine::kTextInversionFocus;
+
 	// Redraw the widget so the selection color will change
 	draw();
 }
 
 void ListWidget::lostFocusWidget() {
+	_inversion = ThemeEngine::kTextInversion;
+
 	// If we lose focus, we simply forget the user changes
 	_editMode = false;
 	g_system->setFeatureState(OSystem::kFeatureVirtualKeyboard, false);
@@ -471,6 +471,10 @@ void ListWidget::handleCommand(CommandSender *sender, uint32 cmd, uint32 data) {
 		if (_currentPos != (int)data) {
 			_currentPos = data;
 			draw();
+
+			// Scrollbar actions cause list focus (which triggers a redraw)
+			// NOTE: ListWidget's boss is always GUI::Dialog
+			((GUI::Dialog *)_boss)->setFocusWidget(this);
 		}
 		break;
 	}
@@ -491,12 +495,8 @@ void ListWidget::drawWidget() {
 		ThemeEngine::TextInversionState inverted = ThemeEngine::kTextInversionNone;
 
 		// Draw the selected item inverted, on a highlighted background.
-		if (_selectedItem == pos) {
-			if (_hasFocus)
-				inverted = ThemeEngine::kTextInversionFocus;
-			else
-				inverted = ThemeEngine::kTextInversion;
-		}
+		if (_selectedItem == pos)
+			inverted = _inversion;
 
 		Common::Rect r(getEditRect());
 		int pad = _leftPadding;
@@ -692,7 +692,7 @@ void ListWidget::setFilter(const String &filter, bool redraw) {
 		_list.clear();
 		_listIndex.clear();
 
-		for (StringList::iterator i = _dataList.begin(); i != _dataList.end(); ++i, ++n) {
+		for (StringArray::iterator i = _dataList.begin(); i != _dataList.end(); ++i, ++n) {
 			tmp = *i;
 			tmp.toLowercase();
 			bool matches = true;
