@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/gui/ListWidget.h $
- * $Id: ListWidget.h 44802 2009-10-08 21:28:57Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/gui/ListWidget.h $
+ * $Id: ListWidget.h 49779 2010-06-15 10:54:22Z sev $
  */
 
 #ifndef GUI_LISTWIDGET_H
@@ -52,11 +52,11 @@ enum {
 class ListWidget : public EditableWidget {
 public:
 	typedef Common::String String;
-	typedef Common::StringList StringList;
+	typedef Common::Array<Common::String> StringArray;
 	typedef Common::Array<ThemeEngine::FontColor> ColorList;
 protected:
-	StringList		_list;
-	StringList		_dataList;
+	StringArray		_list;
+	StringArray		_dataList;
 	ColorList		_listColors;
 	Common::Array<int>		_listIndex;
 	bool			_editable;
@@ -87,14 +87,14 @@ protected:
 	ThemeEngine::FontColor _editColor;
 
 public:
-	ListWidget(GuiObject *boss, const String &name, uint32 cmd = 0);
-	ListWidget(GuiObject *boss, int x, int y, int w, int h, uint32 cmd = 0);
+	ListWidget(Dialog *boss, const String &name, const char *tooltip = 0, uint32 cmd = 0);
+	ListWidget(Dialog *boss, int x, int y, int w, int h, const char *tooltip = 0, uint32 cmd = 0);
 	virtual ~ListWidget();
 
 	virtual Widget *findWidget(int x, int y);
 
-	void setList(const StringList &list, const ColorList *colors = 0);
-	const StringList &getList()	const			{ return _dataList; }
+	void setList(const StringArray &list, const ColorList *colors = 0);
+	const StringArray &getList()	const			{ return _dataList; }
 
 	void append(const String &s, ThemeEngine::FontColor color = ThemeEngine::kFontColorNormal);
 
