@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/gui/PopUpWidget.cpp $
- * $Id: PopUpWidget.cpp 48101 2010-02-21 04:04:13Z bluddy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/gui/PopUpWidget.cpp $
+ * $Id: PopUpWidget.cpp 49774 2010-06-15 10:52:35Z sev $
  */
 
 #include "common/system.h"
@@ -170,7 +170,7 @@ void PopUpDialog::drawDialog() {
 
 	if (_openTime == 0) {
 		// Time the popup was opened
-		_openTime = getMillis();
+		_openTime = g_system->getMillis();
 	}
 }
 
@@ -178,7 +178,7 @@ void PopUpDialog::handleMouseUp(int x, int y, int button, int clickCount) {
 	// Mouse was released. If it wasn't moved much since the original mouse down,
 	// let the popup stay open. If it did move, assume the user made his selection.
 	int dist = (_clickX - x) * (_clickX - x) + (_clickY - y) * (_clickY - y);
-	if (dist > 3 * 3 || getMillis() - _openTime > 300) {
+	if (dist > 3 * 3 || g_system->getMillis() - _openTime > 300) {
 		setResult(_selection);
 		close();
 	}
@@ -378,8 +378,8 @@ void PopUpDialog::drawMenuEntry(int entry, bool hilite) {
 // PopUpWidget
 //
 
-PopUpWidget::PopUpWidget(GuiObject *boss, const String &name)
-	: Widget(boss, name), CommandSender(boss) {
+PopUpWidget::PopUpWidget(GuiObject *boss, const String &name, const char *tooltip)
+	: Widget(boss, name, tooltip), CommandSender(boss) {
 	setFlags(WIDGET_ENABLED | WIDGET_CLEARBG | WIDGET_RETAIN_FOCUS | WIDGET_IGNORE_DRAG);
 	_type = kPopUpWidget;
 
