@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/gui/ThemeParser.cpp $
- * $Id: ThemeParser.cpp 44495 2009-09-30 16:16:53Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/gui/ThemeParser.cpp $
+ * $Id: ThemeParser.cpp 50925 2010-07-16 03:14:58Z eriktorbjorn $
  *
  */
 
@@ -30,6 +30,8 @@
 
 #include "graphics/VectorRenderer.h"
 
+#include "common/tokenizer.h"
+
 namespace GUI {
 
 struct TextDataInfo {
@@ -40,7 +42,8 @@ struct TextDataInfo {
 static const TextDataInfo kTextDataDefaults[] = {
 	{ kTextDataDefault,			"text_default" },
 	{ kTextDataButton,			"text_button" },
-	{ kTextDataNormalFont,		"text_normal" }
+	{ kTextDataNormalFont,		"text_normal" },
+	{ kTextDataTooltip,			"tooltip_normal" }
 };
 
 
@@ -362,10 +365,8 @@ bool ThemeParser::parserCallback_drawdata(ParserNode *node) {
 	if (_theme->addDrawData(node->values["id"], cached) == false)
 		return parserError("Error adding Draw Data set: Invalid DrawData name.");
 
-	if (_defaultStepLocal) {
-		delete _defaultStepLocal;
-		_defaultStepLocal = 0;
-	}
+	delete _defaultStepLocal;
+	_defaultStepLocal = 0;
 
 	return true;
 }
