@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/gui/about.h $
- * $Id: about.h 27786 2007-06-30 12:26:59Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/gui/about.h $
+ * $Id: about.h 48282 2010-03-18 15:09:24Z fingolfin $
  */
 
 #ifndef ABOUT_DIALOG_H
@@ -32,11 +32,11 @@
 namespace GUI {
 
 class AboutDialog : public Dialog {
-	typedef Common::StringList StringList;
+	typedef Common::Array<Common::String> StringArray;
 protected:
 	int			_scrollPos;
 	uint32		_scrollTime;
-	StringList	_lines;
+	StringArray	_lines;
 	uint32		_lineHeight;
 	bool		_willClose;
 
