@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/gui/console.cpp $
- * $Id: console.cpp 48101 2010-02-21 04:04:13Z bluddy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/gui/console.cpp $
+ * $Id: console.cpp 50964 2010-07-17 18:41:38Z fingolfin $
  */
 
 #include "gui/console.h"
@@ -156,7 +156,9 @@ void ConsoleDialog::open() {
 	_slideMode = kDownSlideMode;
 
 	Dialog::open();
-	if (_promptStartPos == -1) {
+	if ((_promptStartPos == -1) || (_currentPos > _promptEndPos)) {
+		// we print a prompt, if this is the first time we are called or if the
+		//  engine wrote onto us since the last call
 		print(PROMPT);
 		_promptStartPos = _promptEndPos = _currentPos;
 	}
@@ -667,11 +669,7 @@ int ConsoleDialog::printf(const char *format, ...) {
 }
 
 int ConsoleDialog::vprintf(const char *format, va_list argptr) {
-#ifdef PALMOS_MODE
-	char	buf[256];
-#else
 	char	buf[2048];
-#endif
 
 #if defined(WIN32)
 	int count = _vsnprintf(buf, sizeof(buf), format, argptr);
