@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/gui/editable.h $
- * $Id: editable.h 48101 2010-02-21 04:04:13Z bluddy $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/gui/editable.h $
+ * $Id: editable.h 52716 2010-09-14 00:15:20Z lordhoto $
  */
 
 #ifndef GUI_EDITABLE_H
@@ -54,9 +54,11 @@ protected:
 
 	ThemeEngine::FontStyle  _font;
 
+	ThemeEngine::TextInversionState  _inversion;
+
 public:
-	EditableWidget(GuiObject *boss, int x, int y, int w, int h, uint32 cmd = 0);
-	EditableWidget(GuiObject *boss, const String &name, uint32 cmd = 0);
+	EditableWidget(GuiObject *boss, int x, int y, int w, int h, const char *tooltip = 0, uint32 cmd = 0);
+	EditableWidget(GuiObject *boss, const String &name, const char *tooltip = 0, uint32 cmd = 0);
 	virtual ~EditableWidget();
 
 	void init();
