@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/gui/launcher.h $
- * $Id: launcher.h 46380 2009-12-15 08:19:34Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/gui/launcher.h $
+ * $Id: launcher.h 48282 2010-03-18 15:09:24Z fingolfin $
  */
 
 #ifndef GUI_LAUNCHER_DIALOG_H
@@ -40,7 +40,7 @@ Common::String addGameToConf(const GameDescriptor &result);
 
 class LauncherDialog : public Dialog {
 	typedef Common::String String;
-	typedef Common::StringList StringList;
+	typedef Common::Array<Common::String> StringArray;
 public:
 	LauncherDialog();
 	~LauncherDialog();
@@ -64,7 +64,7 @@ protected:
 #endif
 	StaticTextWidget	*_searchDesc;
 	ButtonWidget	*_searchClearButton;
-	StringList		_domains;
+	StringArray		_domains;
 	BrowserDialog	*_browser;
 	SaveLoadChooser	*_loadDialog;
 
