@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/sound/audiocd.cpp $
- * $Id: audiocd.cpp 47579 2010-01-26 22:48:45Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/sound/audiocd.cpp $
+ * $Id: audiocd.cpp 51741 2010-08-04 16:53:09Z athrxx $
  *
  */
 
@@ -32,7 +32,7 @@
 #include "common/util.h"
 #include "common/system.h"
 
-DECLARE_SINGLETON(Audio::AudioCDManager);
+DECLARE_SINGLETON(Audio::AudioCDManager)
 
 namespace Audio {
 
@@ -42,6 +42,8 @@ AudioCDManager::AudioCDManager() {
 	_cd.start = 0;
 	_cd.duration = 0;
 	_cd.numLoops = 0;
+	_cd.volume = Mixer::kMaxChannelVolume;
+	_cd.balance = 0;
 	_mixer = g_system->getMixer();
 	_emulating = false;
 	assert(_mixer);
@@ -78,8 +80,9 @@ void AudioCDManager::play(int track, int numLoops, int startFrame, int duration,
 			repetitions. Finally, -1 means infinitely many
 			*/
 			_emulating = true;
-			_mixer->playInputStream(Mixer::kMusicSoundType, &_handle,
-			                        makeLoopingAudioStream(stream, start, end, (numLoops < 1) ? numLoops + 1 : numLoops));
+			_mixer->playStream(Mixer::kMusicSoundType, &_handle,
+			                        makeLoopingAudioStream(stream, start, end, (numLoops < 1) ? numLoops + 1 : numLoops), -1, _cd.volume, _cd.balance);
+
 		} else {
 			_emulating = false;
 			if (!only_emulate)
@@ -109,6 +112,38 @@ bool AudioCDManager::isPlaying() const {
 	}
 }
 
+void AudioCDManager::setVolume(byte volume) {
+	_cd.volume = volume;
+	if (_emulating) {
+		// Audio CD emulation
+		if (_mixer->isSoundHandleActive(_handle))
+			_mixer->setChannelVolume(_handle, _cd.volume);
+	} else {
+		// Real Audio CD
+		
+		// Unfortunately I can't implement this atm
+		// since SDL doesn't seem to offer an interface method for this.
+
+		// g_system->setVolumeCD(_cd.volume);
+	}
+}
+
+void AudioCDManager::setBalance(int8 balance) {
+	_cd.balance = balance;
+	if (_emulating) {
+		// Audio CD emulation
+		if (isPlaying())
+			_mixer->setChannelBalance(_handle, _cd.balance);
+	} else {
+		// Real Audio CD
+		
+		// Unfortunately I can't implement this atm
+		// since SDL doesn't seem to offer an interface method for this.
+
+		// g_system->setBalanceCD(_cd.balance);
+	}
+}
+
 void AudioCDManager::updateCD() {
 	if (_emulating) {
 		// Check whether the audio track stopped playback
