@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/sound/decoders/flac.cpp $
- * $Id: flac.cpp 47847 2010-02-03 09:42:40Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/sound/decoders/flac.cpp $
+ * $Id: flac.cpp 49843 2010-06-15 12:33:20Z fingolfin $
  *
  */
 
@@ -230,7 +230,7 @@ FLACStream::FLACStream(Common::SeekableReadStream *inStream, bool dispose)
 		if (processUntilEndOfMetadata() && _streaminfo.channels > 0) {
 			_lastSample = _streaminfo.total_samples + 1;
 			_length = Timestamp(0, _lastSample - 1, getRate());
-			return; // no error occured
+			return; // no error occurred
 		}
 	}
 
@@ -355,7 +355,7 @@ int FLACStream::readBuffer(int16 *buffer, const int numSamples) {
 		break;
 	default:
 		decoderOk = false;
-		warning("FLACStream: An error occured while decoding. DecoderState is: %s",
+		warning("FLACStream: An error occurred while decoding. DecoderState is: %s",
 			FLAC__StreamDecoderStateString[getStreamDecoderState()]);
 	}
 
@@ -668,7 +668,7 @@ inline void FLACStream::callbackMetadata(const ::FLAC__StreamMetadata *metadata)
 }
 inline void FLACStream::callbackError(::FLAC__StreamDecoderErrorStatus status) {
 	// some of these are non-critical-Errors
-	debug(1, "FLACStream: An error occured while decoding. DecoderState is: %s",
+	debug(1, "FLACStream: An error occurred while decoding. DecoderState is: %s",
 			FLAC__StreamDecoderErrorStatusString[status]);
 }
 
