@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/sound/mididrv.h $
- * $Id: mididrv.h 47530 2010-01-25 00:13:32Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/sound/mididrv.h $
+ * $Id: mididrv.h 52736 2010-09-15 22:00:20Z lordhoto $
  *
  */
 
@@ -30,6 +30,8 @@
 #include "common/timer.h"
 
 class MidiChannel;
+class MusicDevice;
+
 namespace Audio {
 	class Mixer;
 }
@@ -43,67 +45,52 @@ namespace Common { class String; }
  *
  * @todo Rename MidiDriverType to MusicDriverType
  */
-enum MidiDriverType {
-	// Pseudo drivers
-	MD_AUTO,
-	MD_NULL,
 
-	// Windows
-	MD_WINDOWS,
-
-	// Atari ST
-	MD_STMIDI,
-
-	// Linux
-	MD_ALSA,
-	MD_SEQ,
-
-	// Mac OS X
-	MD_QTMUSIC,
-	MD_COREAUDIO,
-	MD_COREMIDI,
-
-	// PalmOS
-	MD_YPA1,
-	MD_ZODIAC,
-
-	// IRIX
-	MD_DMEDIA,
-
-	// AMIGAOS4
-	MD_CAMD,
-
-	// MIDI softsynths
-	MD_FLUIDSYNTH,
-	MD_MT32,
-
-	// "Fake" MIDI devices
-	MD_ADLIB,
-	MD_PCSPK,
-	MD_CMS,
-	MD_PCJR,
-	MD_TOWNS,
-	MD_TIMIDITY
+/**
+ * Music types that music drivers can implement and engines can rely on.
+ */
+enum MusicType {
+	MT_INVALID = -1,	// Invalid output
+	MT_AUTO = 0,		// Auto
+	MT_NULL,			// Null
+	MT_PCSPK,			// PC Speaker
+	MT_PCJR,			// PCjr
+	MT_CMS,				// CMS
+	MT_ADLIB,			// AdLib
+	MT_C64,				// C64
+	MT_AMIGA,			// Amiga
+	MT_APPLEIIGS,		// Apple IIGS
+	MT_TOWNS,			// FM-TOWNS
+	MT_PC98,			// PC98
+	MT_GM,				// General MIDI
+	MT_MT32,			// MT-32
+	MT_GS				// Roland GS
 };
 
 /**
- * A set of flags to be passed to detectMusicDriver() which can be used to
+ * A set of flags to be passed to detectDevice() which can be used to
  * specify what kind of music driver is preferred / accepted.
  *
- * The flags (except for MDT_PREFER_MIDI) indicate whether a given driver
+ * The flags (except for MDT_PREFER_MT32 and MDT_PREFER_GM) indicate whether a given driver
  * type is acceptable. E.g. the TOWNS music driver could be returned by
- * detectMusicDriver if and only if MDT_TOWNS is specified.
+ * detectDevice if and only if MDT_TOWNS is specified.
  *
  * @todo Rename MidiDriverFlags to MusicDriverFlags
  */
 enum MidiDriverFlags {
-	MDT_NONE   = 0,
-	MDT_PCSPK  = 1 << 0,      // PC Speaker: Maps to MD_PCSPK and MD_PCJR
-	MDT_CMS    = 1 << 1,      // Creative Music System / Gameblaster: Maps to MD_CMS
-	MDT_ADLIB  = 1 << 2,      // AdLib: Maps to MD_ADLIB
-	MDT_TOWNS  = 1 << 3,      // FM-TOWNS: Maps to MD_TOWNS
-	MDT_MIDI   = 1 << 4,      // Real MIDI
-	MDT_PREFER_MIDI = 1 << 5  // Real MIDI output is preferred
+	MDT_NONE        = 0,
+	MDT_PCSPK       = 1 << 0,		// PC Speaker: Maps to MD_PCSPK and MD_PCJR
+	MDT_CMS         = 1 << 1,		// Creative Music System / Gameblaster: Maps to MD_CMS
+	MDT_PCJR        = 1 << 2,		// Tandy/PC Junior driver
+	MDT_ADLIB       = 1 << 3,		// AdLib: Maps to MT_ADLIB
+	MDT_C64         = 1 << 4,
+	MDT_AMIGA       = 1 << 5,
+	MDT_APPLEIIGS   = 1 << 6,
+	MDT_TOWNS       = 1 << 7,		// FM-TOWNS: Maps to MT_TOWNS
+	MDT_PC98        = 1 << 8,		// FM-TOWNS: Maps to MT_PC98
+	MDT_MIDI        = 1 << 9,		// Real MIDI
+	MDT_PREFER_MT32 = 1 << 10,		// MT-32 output is preferred
+	MDT_PREFER_GM   = 1 << 11		// GM output is preferred
 };
 
 /**
@@ -113,12 +100,6 @@ enum MidiDriverFlags {
  *
  * @todo Rename MidiDriverType to MusicDriverType
  */
-struct MidiDriverDescription {
-	const char *name;
-	const char *description;
-	MidiDriverType id;		// A unique ID for each driver
-	int flags;				// Capabilities of this driver
-};
 
 /**
  * Abstract MIDI Driver Class
@@ -127,22 +108,45 @@ struct MidiDriverDescription {
  */
 class MidiDriver {
 public:
-	/** Find the music driver matching the given driver name/description. */
-	static const MidiDriverDescription *findMusicDriver(const Common::String &str);
-
-	/** Get the id of the music driver matching the given driver name, or MD_AUTO if there is no match. */
-	static MidiDriverType parseMusicDriver(const Common::String &str);
-
 	/**
-	 * Get a list of all available MidiDriver types.
-	 * @return list of all available midi drivers, terminated by  a zero entry
+	 * The device handle.
+	 *
+	 * The value 0 is reserved for an invalid device for now.
+	 * TODO: Maybe we should use -1 (i.e. 0xFFFFFFFF) as
+	 * invalid device?
 	 */
-	static const MidiDriverDescription *getAvailableMidiDrivers();
+	typedef uint32 DeviceHandle;
 
-	static MidiDriver *createMidi(MidiDriverType midiDriver);
+	enum DeviceStringType {
+		kDriverName,
+		kDriverId,
+		kDeviceId
+	};
 
-	static MidiDriverType detectMusicDriver(int flags);
+	static uint32 musicType2GUIO(uint32 musicType);
 
+	/** Create music driver matching the given device handle, or NULL if there is no match. */
+	static MidiDriver *createMidi(DeviceHandle handle);
+
+	/** Returns device handle based on the present devices and the flags parameter. */
+	static DeviceHandle detectDevice(int flags);
+
+	/** Find the music driver matching the given driver name/description. */
+	static DeviceHandle getDeviceHandle(const Common::String &identifier);
+
+	/** Get the music type matching the given device handle, or MT_AUTO if there is no match. */
+	static MusicType getMusicType(DeviceHandle handle);
+
+	/** Get the device description string matching the given device handle and the given type. */
+	static Common::String getDeviceString(DeviceHandle handle, DeviceStringType type);
+
+private:
+	// If detectDevice() detects MT32 and we have a preferred MT32 device
+	// we use this to force getMusicType() to return MT_MT32 so that we don't
+	// have to rely on the 'True Roland MT-32' config manager setting (since nobody
+	// would possibly think about activating 'True Roland MT-32' when he has set
+	// 'Music Driver' to '<default>')
+	static bool _forceTypeMT32;
 
 public:
 	virtual ~MidiDriver() { }
@@ -211,6 +215,16 @@ public:
 		send(0xB0 | channel, 100, 127);
 	}
 
+	/**
+	 * Send a Roland MT-32 reset sysEx to the midi device.
+	 */
+	void sendMT32Reset();
+
+	/**
+	 * Send a General MIDI reset sysEx to the midi device.
+	 */
+	void sendGMReset();
+
 	/**
 	 * Transmit a sysEx to the midi device.
 	 *
@@ -271,28 +285,4 @@ public:
 	virtual void sysEx_customInstrument(uint32 type, const byte *instr) = 0;
 };
 
-
-// Factory functions, for faster compile
-extern MidiDriver *MidiDriver_NULL_create();
-extern MidiDriver *MidiDriver_ADLIB_create();
-extern MidiDriver *MidiDriver_WIN_create();
-extern MidiDriver *MidiDriver_STMIDI_create();
-extern MidiDriver *MidiDriver_SEQ_create();
-extern MidiDriver *MidiDriver_TIMIDITY_create();
-extern MidiDriver *MidiDriver_QT_create();
-extern MidiDriver *MidiDriver_CORE_create();
-extern MidiDriver *MidiDriver_CoreMIDI_create();
-extern MidiDriver *MidiDriver_ALSA_create();
-extern MidiDriver *MidiDriver_DMEDIA_create();
-extern MidiDriver *MidiDriver_CAMD_create();
-extern MidiDriver *MidiDriver_YM2612_create();
-#ifdef USE_FLUIDSYNTH
-extern MidiDriver *MidiDriver_FluidSynth_create();
-#endif
-#ifdef USE_MT32EMU
-extern MidiDriver *MidiDriver_MT32_create();
-#endif
-extern MidiDriver *MidiDriver_YamahaPa1_create();
-extern MidiDriver *MidiDriver_Zodiac_create();
-
 #endif
