@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/sound/midiparser.cpp $
- * $Id: midiparser.cpp 46897 2010-01-02 20:20:36Z wjpalenstijn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/sound/midiparser.cpp $
+ * $Id: midiparser.cpp 50926 2010-07-16 03:31:45Z eriktorbjorn $
  *
  */
 
@@ -47,6 +47,10 @@ _num_tracks(0),
 _active_track(255),
 _abort_parse(0) {
 	memset(_active_notes, 0, sizeof(_active_notes));
+	_next_event.start = NULL;
+	_next_event.delta = 0;
+	_next_event.event = 0;
+	_next_event.length = 0;
 }
 
 void MidiParser::property(int prop, int value) {
@@ -63,6 +67,10 @@ void MidiParser::property(int prop, int value) {
 	}
 }
 
+void MidiParser::sendToDriver(uint32 b) {
+	_driver->send(b);
+}
+
 void MidiParser::setTempo(uint32 tempo) {
 	_tempo = tempo;
 	if (_ppqn)
@@ -123,7 +131,7 @@ void MidiParser::hangingNote(byte channel, byte note, uint32 time_left, bool rec
 			best = ptr;
 			if (ptr->time_left) {
 				if (recycle)
-					_driver->send(0x80 | channel, note, 0);
+					sendToDriver(0x80 | channel, note, 0);
 				--_hanging_notes_count;
 			}
 			break;
@@ -168,7 +176,7 @@ void MidiParser::onTimer() {
 		for (i = ARRAYSIZE(_hanging_notes); i; --i, ++ptr) {
 			if (ptr->time_left) {
 				if (ptr->time_left <= _timer_rate) {
-					_driver->send(0x80 | ptr->channel, ptr->note, 0);
+					sendToDriver(0x80 | ptr->channel, ptr->note, 0);
 					ptr->time_left = 0;
 					--_hanging_notes_count;
 				} else {
@@ -228,7 +236,7 @@ void MidiParser::onTimer() {
 				else
 					activeNote(info.channel(), info.basic.param1, true);
 			}
-			_driver->send(info.event, info.basic.param1, info.basic.param2);
+			sendToDriver(info.event, info.basic.param1, info.basic.param2);
 		}
 
 
@@ -254,7 +262,7 @@ void MidiParser::allNotesOff() {
 	for (i = 0; i < 128; ++i) {
 		for (j = 0; j < 16; ++j) {
 			if (_active_notes[i] & (1 << j)) {
-				_driver->send(0x80 | j, i, 0);
+				sendToDriver(0x80 | j, i, 0);
 			}
 		}
 	}
@@ -262,7 +270,7 @@ void MidiParser::allNotesOff() {
 	// Turn off all hanging notes
 	for (i = 0; i < ARRAYSIZE(_hanging_notes); i++) {
 		if (_hanging_notes[i].time_left) {
-			_driver->send(0x80 | _hanging_notes[i].channel, _hanging_notes[i].note, 0);
+			sendToDriver(0x80 | _hanging_notes[i].channel, _hanging_notes[i].note, 0);
 			_hanging_notes[i].time_left = 0;
 		}
 	}
@@ -272,7 +280,7 @@ void MidiParser::allNotesOff() {
 	// support this...).
 
 	for (i = 0; i < 16; ++i) {
-		_driver->send(0xB0 | i, 0x7b, 0); // All notes off
+		sendToDriver(0xB0 | i, 0x7b, 0); // All notes off
 	}
 
 	memset(_active_notes, 0, sizeof(_active_notes));
@@ -325,7 +333,7 @@ void MidiParser::hangAllActiveNotes() {
 
 	uint32 advance_tick = _position._last_event_tick;
 	while (true) {
-		int i, j;
+		int i;
 		for (i = 0; i < 128; ++i)
 			if (temp_active[i] != 0)
 				break;
@@ -341,10 +349,10 @@ void MidiParser::hangAllActiveNotes() {
 		} else if (_next_event.event == 0xFF && _next_event.ext.type == 0x2F) {
 			// warning("MidiParser::hangAllActiveNotes(): Hit End of Track with active notes left");
 			for (i = 0; i < 128; ++i) {
-				for (j = 0; j < 16; ++j) {
+				for (int j = 0; j < 16; ++j) {
 					if (temp_active[i] & (1 << j)) {
 						activeNote(j, i, false);
-						_driver->send(0x80 | j, i, 0);
+						sendToDriver(0x80 | j, i, 0);
 					}
 				}
 			}
@@ -395,7 +403,7 @@ bool MidiParser::jumpToTick(uint32 tick, bool fireEvents, bool stopNotes) {
 					else
 						_driver->sysEx(info.ext.data, (uint16)info.length);
 				} else
-					_driver->send(info.event, info.basic.param1, info.basic.param2);
+					sendToDriver(info.event, info.basic.param1, info.basic.param2);
 			}
 
 			parseNextEvent(_next_event);
@@ -437,7 +445,7 @@ void MidiParser::unloadMusic() {
 
 		if (_driver) {
 			for (int i = 0; i < 16; ++i) {
-				_driver->send(0xE0 | i, 0, 0x40);
+				sendToDriver(0xE0 | i, 0, 0x40);
 			}
 		}
 	}
