@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/sound/midiparser_smf.cpp $
- * $Id: midiparser_smf.cpp 46941 2010-01-03 19:37:43Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/sound/midiparser_smf.cpp $
+ * $Id: midiparser_smf.cpp 50926 2010-07-16 03:31:45Z eriktorbjorn $
  *
  */
 
@@ -230,10 +230,8 @@ bool MidiParser_SMF::loadMusic(byte *data, uint32 size) {
 
 	// If this is a Type 1 MIDI, we need to now compress
 	// our tracks down into a single Type 0 track.
-	if (_buffer) {
-		free(_buffer);
-		_buffer = 0;
-	}
+	free(_buffer);
+	_buffer = 0;
 
 	if (midi_type == 1) {
 		// FIXME: Doubled the buffer size to prevent crashes with the
