@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/sound/mixer.cpp $
- * $Id: mixer.cpp 48204 2010-03-08 21:54:54Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/sound/mixer.cpp $
+ * $Id: mixer.cpp 52357 2010-08-24 18:44:14Z dhewg $
  *
  */
 
@@ -44,7 +44,7 @@ namespace Audio {
  */
 class Channel {
 public:
-	Channel(Mixer *mixer, Mixer::SoundType type, AudioStream *input, DisposeAfterUse::Flag autofreeStream, bool reverseStereo, int id, bool permanent);
+	Channel(Mixer *mixer, Mixer::SoundType type, AudioStream *stream, DisposeAfterUse::Flag autofreeStream, bool reverseStereo, int id, bool permanent);
 	~Channel();
 
 	/**
@@ -60,7 +60,7 @@ public:
 	/**
 	 * Queries whether the channel is still playing or not.
 	 */
-	bool isFinished() const { return _input->endOfStream(); }
+	bool isFinished() const { return _stream->endOfStream(); }
 
 	/**
 	 * Queries whether the channel is a permanent channel.
@@ -152,7 +152,7 @@ private:
 
 	DisposeAfterUse::Flag _autofreeStream;
 	RateConverter *_converter;
-	AudioStream *_input;
+	AudioStream *_stream;
 };
 
 #pragma mark -
@@ -160,8 +160,10 @@ private:
 #pragma mark -
 
 
-MixerImpl::MixerImpl(OSystem *system)
-	: _syst(system), _sampleRate(0), _mixerReady(false), _handleSeed(0) {
+MixerImpl::MixerImpl(OSystem *system, uint sampleRate)
+	: _syst(system), _sampleRate(sampleRate), _mixerReady(false), _handleSeed(0) {
+
+	assert(sampleRate > 0);
 
 	int i;
 
@@ -179,21 +181,12 @@ MixerImpl::~MixerImpl() {
 
 void MixerImpl::setReady(bool ready) {
 	_mixerReady = ready;
-
-	// If the mixer is set to ready, then we better have a positive sample rate!
-	assert(!_mixerReady || _sampleRate > 0);
 }
 
 uint MixerImpl::getOutputRate() const {
 	return _sampleRate;
 }
 
-void MixerImpl::setOutputRate(uint sampleRate) {
-	if (_sampleRate != 0 && _sampleRate != sampleRate)
-		error("Changing the Audio::Mixer output sample rate is not supported");
-	_sampleRate = sampleRate;
-}
-
 void MixerImpl::insertChannel(SoundHandle *handle, Channel *chan) {
 	int index = -1;
 	for (int i = 0; i != NUM_CHANNELS; i++) {
@@ -219,71 +212,46 @@ void MixerImpl::insertChannel(SoundHandle *handle, Channel *chan) {
 		*handle = chanHandle;
 }
 
-void MixerImpl::playInputStream(
+void MixerImpl::playStream(
 			SoundType type,
 			SoundHandle *handle,
-			AudioStream *input,
+			AudioStream *stream,
 			int id, byte volume, int8 balance,
 			DisposeAfterUse::Flag autofreeStream,
 			bool permanent,
 			bool reverseStereo) {
 	Common::StackLock lock(_mutex);
 
-	if (input == 0) {
-		warning("input stream is 0");
+	if (stream == 0) {
+		warning("stream is 0");
 		return;
 	}
 
+
+	assert(_mixerReady);
+
 	// Prevent duplicate sounds
 	if (id != -1) {
 		for (int i = 0; i != NUM_CHANNELS; i++)
 			if (_channels[i] != 0 && _channels[i]->getId() == id) {
+				// Delete the stream if were asked to auto-dispose it.
+				// Note: This could cause trouble if the client code does not
+				// yet expect the stream to be gone. The primary example to
+				// keep in mind here is QueuingAudioStream.
+				// Thus, as a quick rule of thumb, you should never, ever,
+				// try to play QueuingAudioStreams with a sound id.
 				if (autofreeStream == DisposeAfterUse::YES)
-					delete input;
+					delete stream;
 				return;
 			}
 	}
 
-	// Check if the mixer is not ready. This most probably means that
-	// no Audio output is possible according to the backend (we should
-	// clarify this in the Mixer creation API, though).
-	//
-	// For now we deal with this by simply ignore the sound here, never
-	// scheduling it for playback, never giving it a valid sound
-	// handle. For a game engine, this is indistinguishable from a
-	// sound which finishes playback before playInputStream returns.
-	//
-	// This is certainly not ideal for many engine; e.g. if a game has
-	// scripts which sync by waiting for certain sounds to play, then
-	// this syncing is broken if we just remove the sound.
-	//
-	// We could try to be more graceful, by using TimerManager and
-	// emulating (or rather: faking) actual audio playback; essentially
-	// we run the mixer callback from a timer instead of an audio
-	// callback.
-	// However, this may very well lead to new problems of its own,
-	// plus it would complicate the Mixer code. It seems that a better
-	// solution would be to adapt backends to setup a fake mixer thread
-	// which calls the mixer callback. We'd then still need a way to
-	// signal the mixer / the client code that no actual audio playback
-	// takes places... Anyway, either way, we first would have to
-	// investigate ramifications of any such approach.
-	//
-	// And also, by far the best solution is to adapt engines to be
-	// properly aware of the possibility of missing audio, and how to
-	// deal with it; be it by refusing to launch (e.g. when audio is an
-	// integral part of a game), by switching to alternate script
-	// syncing means, etc. It also seems important to test every game
-	// individually in a system without audio, if we really want
-	// to support such systems.
-	if (!_mixerReady) {
-		if (autofreeStream == DisposeAfterUse::YES)
-			delete input;
-		return;
-	}
+#ifdef AUDIO_REVERSE_STEREO
+	reverseStereo = !reverseStereo;
+#endif
 
 	// Create the channel
-	Channel *chan = new Channel(this, type, input, autofreeStream, reverseStereo, id, permanent);
+	Channel *chan = new Channel(this, type, stream, autofreeStream, reverseStereo, id, permanent);
 	chan->setVolume(volume);
 	chan->setBalance(balance);
 	insertChannel(handle, chan);
@@ -299,7 +267,6 @@ void MixerImpl::mixCallback(byte *samples, uint len) {
 
 	// Since the mixer callback has been called, the mixer must be ready...
 	_mixerReady = true;
-	assert(_sampleRate > 0);
 
 	//  zero the buf
 	memset(buf, 0, 2 * len * sizeof(int16));
@@ -376,7 +343,7 @@ Timestamp MixerImpl::getElapsedTime(SoundHandle handle) {
 
 	const int index = handle._val % NUM_CHANNELS;
 	if (!_channels[index] || _channels[index]->getHandle()._val != handle._val)
-		return Timestamp(0, _sampleRate ? _sampleRate : 1);
+		return Timestamp(0, _sampleRate);
 
 	return _channels[index]->getElapsedTime();
 }
@@ -473,23 +440,23 @@ int MixerImpl::getVolumeForSoundType(SoundType type) const {
 #pragma mark --- Channel implementations ---
 #pragma mark -
 
-Channel::Channel(Mixer *mixer, Mixer::SoundType type, AudioStream *input,
+Channel::Channel(Mixer *mixer, Mixer::SoundType type, AudioStream *stream,
                  DisposeAfterUse::Flag autofreeStream, bool reverseStereo, int id, bool permanent)
     : _type(type), _mixer(mixer), _id(id), _permanent(permanent), _volume(Mixer::kMaxChannelVolume),
       _balance(0), _pauseLevel(0), _samplesConsumed(0), _samplesDecoded(0), _mixerTimeStamp(0),
       _pauseStartTime(0), _pauseTime(0), _autofreeStream(autofreeStream), _converter(0),
-      _input(input) {
+      _stream(stream) {
 	assert(mixer);
-	assert(input);
+	assert(stream);
 
 	// Get a rate converter instance
-	_converter = makeRateConverter(_input->getRate(), mixer->getOutputRate(), _input->isStereo(), reverseStereo);
+	_converter = makeRateConverter(_stream->getRate(), mixer->getOutputRate(), _stream->isStereo(), reverseStereo);
 }
 
 Channel::~Channel() {
 	delete _converter;
 	if (_autofreeStream == DisposeAfterUse::YES)
-		delete _input;
+		delete _stream;
 }
 
 void Channel::setVolume(const byte volume) {
@@ -572,9 +539,9 @@ Timestamp Channel::getElapsedTime() {
 }
 
 void Channel::mix(int16 *data, uint len) {
-	assert(_input);
+	assert(_stream);
 
-	if (_input->endOfData()) {
+	if (_stream->endOfData()) {
 		// TODO: call drain method
 	} else {
 		assert(_converter);
@@ -582,7 +549,7 @@ void Channel::mix(int16 *data, uint len) {
 		_samplesConsumed = _samplesDecoded;
 		_mixerTimeStamp = g_system->getMillis();
 		_pauseTime = 0;
-		_samplesDecoded += _converter->flow(*_input, data, len, _volL, _volR);
+		_samplesDecoded += _converter->flow(*_stream, data, len, _volL, _volR);
 	}
 }
 
