@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/sound/softsynth/sid.cpp $
- * $Id: sid.cpp 48137 2010-02-26 17:42:40Z agent-q $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/sound/softsynth/sid.cpp $
+ * $Id: sid.cpp 51995 2010-08-11 18:54:56Z athrxx $
  *
  */
 
@@ -31,6 +31,7 @@
 #ifndef DISABLE_SID
 
 #include "sid.h"
+#include "sound/null.h"
 #include <math.h>
 
 namespace Resid {
@@ -46,11 +47,6 @@ const int SID::FIXP_MASK = 0xffff;
 WaveformGenerator::WaveformGenerator() {
 	sync_source = this;
 
-	wave__ST = wave6581__ST;
-	wave_P_T = wave6581_P_T;
-	wave_PS_ = wave6581_PS_;
-	wave_PST = wave6581_PST;
-
 	reset();
 }
 
@@ -226,19 +222,19 @@ RESID_INLINE reg12 WaveformGenerator::outputN___() {
 // Combined waveforms:
 
 RESID_INLINE reg12 WaveformGenerator::output__ST() {
-	return wave__ST[output__S_()] << 4;
+	return wave6581__ST[output__S_()] << 4;
 }
 
 RESID_INLINE reg12 WaveformGenerator::output_P_T() {
-	return (wave_P_T[output___T() >> 1] << 4) & output_P__();
+	return (wave6581_P_T[output___T() >> 1] << 4) & output_P__();
 }
 
 RESID_INLINE reg12 WaveformGenerator::output_PS_() {
-	return (wave_PS_[output__S_()] << 4) & output_P__();
+	return (wave6581_PS_[output__S_()] << 4) & output_P__();
 }
 
 RESID_INLINE reg12 WaveformGenerator::output_PST() {
-	return (wave_PST[output__S_()] << 4) & output_P__();
+	return (wave6581_PST[output__S_()] << 4) & output_P__();
 }
 
 // Combined waveforms including noise:
@@ -506,7 +502,7 @@ Filter::Filter() {
 		+ sizeof(f0_points_6581)/sizeof(*f0_points_6581) - 1,
 		PointPlotter<sound_sample>(f0_6581), 1.0);
 
-	mixer_DC = -0xfff*0xff/18 >> 7;
+	mixer_DC = (-0xfff*0xff/18) >> 7;
 
 	f0 = f0_6581;
 	f0_points = f0_points_6581;
@@ -1427,4 +1423,34 @@ int SID::clock(cycle_count& delta_t, short* buf, int n, int interleave) {
 
 }
 
+//	Plugin interface
+//	(This can only create a null driver since C64 audio support is not part of the
+//	midi driver architecture. But we need the plugin for the options menu in the launcher
+//	and for MidiDriver::detectDevice() which is more or less used by all engines.)
+
+class C64MusicPlugin : public NullMusicPlugin {
+public:
+	const char *getName() const {
+		return _s("C64 Audio Emulator");
+	}
+
+	const char *getId() const {
+		return "C64";
+	}
+
+	MusicDevices getDevices() const;
+};
+
+MusicDevices C64MusicPlugin::getDevices() const {
+	MusicDevices devices;
+	devices.push_back(MusicDevice(this, "", MT_C64));
+	return devices;
+}
+
+//#if PLUGIN_ENABLED_DYNAMIC(C64)
+	//REGISTER_PLUGIN_DYNAMIC(C64, PLUGIN_TYPE_MUSIC, C64MusicPlugin);
+//#else
+	REGISTER_PLUGIN_STATIC(C64, PLUGIN_TYPE_MUSIC, C64MusicPlugin);
+//#endif
+
 #endif
