@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/sound/softsynth/ym2612.cpp $
- * $Id: ym2612.cpp 47182 2010-01-08 22:09:43Z fingolfin $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/sound/softsynth/ym2612.cpp $
+ * $Id: ym2612.cpp 50128 2010-06-21 21:36:36Z fingolfin $
  */
 
 #include <math.h>
@@ -27,6 +27,7 @@
 #include "sound/softsynth/ym2612.h"
 #include "common/util.h"
 #include "sound/musicplugin.h"
+#include "common/translation.h"
 
 ////////////////////////////////////////
 //
@@ -587,7 +588,7 @@ int MidiDriver_YM2612::open() {
 
 	MidiDriver_Emulated::open();
 
-	_mixer->playInputStream(Audio::Mixer::kPlainSoundType, &_mixerSoundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, DisposeAfterUse::NO, true);
+	_mixer->playStream(Audio::Mixer::kPlainSoundType, &_mixerSoundHandle, this, -1, Audio::Mixer::kMaxChannelVolume, 0, DisposeAfterUse::NO, true);
 	return 0;
 }
 
@@ -758,7 +759,7 @@ void MidiDriver_YM2612::removeLookupTables() {
 class TownsEmuMusicPlugin : public MusicPluginObject {
 public:
 	const char *getName() const {
-		return "FM Towns Emulator";
+		return _s("FM Towns Emulator");
 	}
 
 	const char *getId() const {
@@ -766,7 +767,7 @@ public:
 	}
 
 	MusicDevices getDevices() const;
-	Common::Error createInstance(MidiDriver **mididriver) const;
+	Common::Error createInstance(MidiDriver **mididriver, MidiDriver::DeviceHandle = 0) const;
 };
 
 MusicDevices TownsEmuMusicPlugin::getDevices() const {
@@ -775,21 +776,12 @@ MusicDevices TownsEmuMusicPlugin::getDevices() const {
 	return devices;
 }
 
-Common::Error TownsEmuMusicPlugin::createInstance(MidiDriver **mididriver) const {
+Common::Error TownsEmuMusicPlugin::createInstance(MidiDriver **mididriver, MidiDriver::DeviceHandle) const {
 	*mididriver = new MidiDriver_YM2612(g_system->getMixer());
 
 	return Common::kNoError;
 }
 
-MidiDriver *MidiDriver_YM2612_create() {
-	MidiDriver *mididriver;
-
-	TownsEmuMusicPlugin p;
-	p.createInstance(&mididriver);
-
-	return mididriver;
-}
-
 //#if PLUGIN_ENABLED_DYNAMIC(TOWNS)
 	//REGISTER_PLUGIN_DYNAMIC(TOWNS, PLUGIN_TYPE_MUSIC, TownsEmuMusicPlugin);
 //#else
