@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/tools/create_kyradat/extract.cpp $
- * $Id: extract.cpp 47541 2010-01-25 01:39:44Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/tools/create_kyradat/extract.cpp $
+ * $Id: extract.cpp 48972 2010-05-08 21:25:18Z lordhoto $
  *
  */
 
@@ -108,7 +108,7 @@ const TypeTable typeTable[] = {
 	{ kLolTypeSpellData, 9 },
 	{ kLolTypeCompassData, 10 },
 	{ kLolTypeFlightShpData, 11 },
-	{ -1, 0 }
+	{ -1, 1 }
 };
 
 } // end of anonymous namespace
@@ -124,7 +124,7 @@ const ExtractType *findExtractType(const int type) {
 }
 
 byte getTypeID(int type) {
-	return std::find(typeTable, typeTable + ARRAYSIZE(typeTable), type)->value;
+	return std::find(typeTable, typeTable + ARRAYSIZE(typeTable) - 1, type)->value;
 }
 // Extractor implementation
 
