@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/tools/create_kyradat/games.cpp $
- * $Id: games.cpp 47476 2010-01-23 15:00:11Z strangerke $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/tools/create_kyradat/games.cpp $
+ * $Id: games.cpp 51834 2010-08-07 14:38:07Z athrxx $
  *
  */
 
@@ -99,6 +99,7 @@ const Game lolGames[] = {
 	{ kLol, { EN_ANY, -1, -1 }, kPlatformPC, kDemoVersion, { "30bb5af87d38adb47d3e6ce06b1cb042", 0 } },
 
 	// DOS floppy (no language specifc strings)
+	{ kLol, { EN_ANY, -1, -1 }, kPlatformPC, kNoSpecial, { "0cc764a204f7ba8cefe1a5f14c479619", 0 } },
 	{ kLol, { DE_DEU, -1, -1 }, kPlatformPC, kNoSpecial, { "6b843869772c1b779e1386be868c15dd", 0 } },
 
 	// PC98 (no language specifc strings)
@@ -411,6 +412,8 @@ const int kyra1TownsNeed[] = {
 	k1NewGameString,
 	k1ConfigStrings,
 
+	k1TownsMusicFadeTable,
+	k1TownsMusicFadeTable,
 	k1TownsSFXwdTable,
 	k1TownsSFXbtTable,
 	k1TownsCDATable,
@@ -638,12 +641,9 @@ const int kyra2PC98Need[] = {
 	k2SeqplayStrings,
 	k2SeqplaySfxFiles,
 	k2SeqplaySeqData,
-	//k2SeqplayIntroCDA,
-	//k2SeqplayFinaleCDA,
 	k2IngamePakFiles,
 	k2IngameSfxFiles,
 	k2IngameSfxIndex,
-	//k2IngameCDA,
 	k2IngameTalkObjIndex,
 	k2IngameTimJpStrings,
 	k2IngameShapeAnimData,
