@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/tools/create_msvc/create_msvc.cpp $
- * $Id: create_msvc.cpp 48212 2010-03-09 00:52:25Z lordhoto $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/tools/create_msvc/create_msvc.cpp $
+ * $Id: create_msvc.cpp 53066 2010-10-08 08:15:17Z lordhoto $
  *
  */
 
@@ -210,8 +210,9 @@ public:
 	 * @param bits Number of bits the platform supports.
 	 * @param defines Defines the platform needs to have set.
 	 * @param prefix File prefix, used to add additional include paths.
+	 * @param isWin32 Bitness of property file
 	 */
-	virtual void outputGlobalPropFile(std::ofstream &properties, int bits, const std::string &defines, const std::string &prefix) = 0;
+	virtual void outputGlobalPropFile(std::ofstream &properties, int bits, const std::string &defines, const std::string &prefix, bool isWin32) = 0;
 
 	/**
 	 * Generates the project properties for debug and release settings.
@@ -282,7 +283,7 @@ public:
 
 	void writeReferences(std::ofstream &output);
 
-	void outputGlobalPropFile(std::ofstream &properties, int bits, const std::string &defines, const std::string &prefix);
+	void outputGlobalPropFile(std::ofstream &properties, int bits, const std::string &defines, const std::string &prefix, bool isWin32);
 
 	void createBuildProp(const BuildSetup &setup, bool isRelease, bool isWin32);
 
@@ -305,7 +306,7 @@ public:
 
 	void writeReferences(std::ofstream &output);
 
-	void outputGlobalPropFile(std::ofstream &properties, int bits, const std::string &defines, const std::string &prefix);
+	void outputGlobalPropFile(std::ofstream &properties, int bits, const std::string &defines, const std::string &prefix, bool isWin32);
 
 	void createBuildProp(const BuildSetup &setup, bool isRelease, bool isWin32);
 
@@ -550,6 +551,8 @@ int main(int argc, char *argv[]) {
 	//
 	// 4510 ('class' : default constructor could not be generated)
 	//
+	// 4511 ('class' : copy constructor could not be generated)
+	//
 	// 4610 (object 'class' can never be instantiated - user-defined constructor required)
 	//   "correct" but harmless (as is 4510)
 	//
@@ -558,6 +561,7 @@ int main(int argc, char *argv[]) {
 	globalWarnings = "4068;4100;4103;4127;4244;4250;4310;4351;4512;4702;4706;4800;4996";
 
 	projectWarnings["agi"] = "4510;4610";
+	projectWarnings["agos"] = "4511";
 	projectWarnings["lure"] = "4189;4355";
 	projectWarnings["kyra"] = "4355";
 	projectWarnings["m4"] = "4355";
@@ -825,16 +829,21 @@ TokenList tokenize(const std::string &input) {
 namespace {
 const Feature s_features[] = {
 	// Libraries
-	{    "libz",      "USE_ZLIB", "zlib.lib", true, "zlib (compression) support" },
-	{     "mad",       "USE_MAD", "libmad.lib", true, "libmad (MP3) support" },
-	{  "vorbis",    "USE_VORBIS", "libvorbisfile_static.lib libvorbis_static.lib libogg_static.lib", true, "Ogg Vorbis support" },
-	{    "flac",      "USE_FLAC", "libFLAC_static.lib", true, "FLAC support" },
-	{   "mpeg2",     "USE_MPEG2", "libmpeg2.lib", false, "mpeg2 codec for cutscenes" },
+	{      "libz",       "USE_ZLIB", "zlib.lib", true, "zlib (compression) support" },
+	{       "mad",        "USE_MAD", "libmad.lib", true, "libmad (MP3) support" },
+	{    "vorbis",     "USE_VORBIS", "libvorbisfile_static.lib libvorbis_static.lib libogg_static.lib", true, "Ogg Vorbis support" },
+	{      "flac",       "USE_FLAC", "libFLAC_static.lib", true, "FLAC support" },
+	{     "mpeg2",      "USE_MPEG2", "libmpeg2.lib", false, "mpeg2 codec for cutscenes" },
 
 	// ScummVM feature flags
-	{   "16bit", "USE_RGB_COLOR", "", true, "16bit color support" },
-	{ "mt32emu",   "USE_MT32EMU", "", true, "integrated MT-32 emulator" },
-	{    "nasm",      "USE_NASM", "", true, "IA-32 assembly support" }, // This feature is special in the regard, that it needs additional handling.
+	{     "scalers",     "USE_SCALERS", "", true, "Scalers" },
+	{   "hqscalers",  "USE_HQ_SCALERS", "", true, "HQ scalers" },
+	{       "16bit",   "USE_RGB_COLOR", "", true, "16bit color support" },
+	{     "mt32emu",     "USE_MT32EMU", "", true, "integrated MT-32 emulator" },
+	{        "nasm",        "USE_NASM", "", true, "IA-32 assembly support" }, // This feature is special in the regard, that it needs additional handling.
+	{ "translation", "USE_TRANSLATION", "", true, "Translation support" },
+	{  "langdetect",  "USE_DETECTLANG", "", true, "System language detection support" } // This feature actually depends on "translation", there
+	                                                                                    // is just no current way of properly detecting this...
 };
 } // End of anonymous namespace
 
@@ -1318,7 +1327,7 @@ void ProjectProvider::createGlobalProp(const BuildSetup &setup) {
 		defines += *i;
 	}
 
-	outputGlobalPropFile(properties, 32, defines, convertPathToWin(setup.filePrefix));
+	outputGlobalPropFile(properties, 32, defines, convertPathToWin(setup.filePrefix), true);
 	properties.close();
 
 	properties.open((setup.outputDir + '/' + "ScummVM_Global64" + getPropertiesExtension()).c_str());
@@ -1340,7 +1349,7 @@ void ProjectProvider::createGlobalProp(const BuildSetup &setup) {
 		defines += *i;
 	}
 
-	outputGlobalPropFile(properties, 64, defines, convertPathToWin(setup.filePrefix));
+	outputGlobalPropFile(properties, 64, defines, convertPathToWin(setup.filePrefix), false);
 }
 
 void ProjectProvider::addFilesToProject(const std::string &dir, std::ofstream &projectFile,
@@ -1643,7 +1652,7 @@ void VisualStudioProvider::writeReferences(std::ofstream &output) {
 	output << "\tEndProjectSection\n";
 }
 
-void VisualStudioProvider::outputGlobalPropFile(std::ofstream &properties, int bits, const std::string &defines, const std::string &prefix) {
+void VisualStudioProvider::outputGlobalPropFile(std::ofstream &properties, int bits, const std::string &defines, const std::string &prefix, bool isWin32) {
 	properties << "<?xml version=\"1.0\" encoding=\"Windows-1252\"?>\n"
 	              "<VisualStudioPropertySheet\n"
 	              "\tProjectType=\"Visual C++\"\n"
@@ -1656,7 +1665,7 @@ void VisualStudioProvider::outputGlobalPropFile(std::ofstream &properties, int b
 	              "\t\tName=\"VCCLCompilerTool\"\n"
 	              "\t\tDisableLanguageExtensions=\"true\"\n"
 	              "\t\tDisableSpecificWarnings=\"" << _globalWarnings << "\"\n"
-	              "\t\tAdditionalIncludeDirectories=\"" << prefix << ";" << prefix << "\\engines\"\n"
+	              "\t\tAdditionalIncludeDirectories=\"" << prefix << ";" << prefix << "\\engines;$(SCUMMVM_LIBS)\\include\"\n"
 	              "\t\tPreprocessorDefinitions=\"" << defines << "\"\n"
 	              "\t\tExceptionHandling=\"0\"\n"
 	              "\t\tRuntimeTypeInfo=\"false\"\n"
@@ -1673,6 +1682,7 @@ void VisualStudioProvider::outputGlobalPropFile(std::ofstream &properties, int b
 	              "\t\tIgnoreDefaultLibraryNames=\"\"\n"
 	              "\t\tSubSystem=\"1\"\n"
 	              "\t\tEntryPointSymbol=\"WinMainCRTStartup\"\n"
+	              "\t\tAdditionalLibraryDirectories=\"$(SCUMMVM_LIBS)\\libs\\" << (isWin32 ? "x86" : "x64") << "\"\n"
 	              "\t/>\n"
 	              "\t<Tool\n"
 	              "\t\tName=\"VCResourceCompilerTool\"\n"
@@ -1705,7 +1715,7 @@ void VisualStudioProvider::createBuildProp(const BuildSetup &setup, bool isRelea
 	if (isRelease) {
 		properties << "\t\tEnableIntrinsicFunctions=\"true\"\n"
 		              "\t\tWholeProgramOptimization=\"true\"\n"
-		              "\t\tPreprocessorDefinitions=\"WIN32\"\n"
+		              "\t\tPreprocessorDefinitions=\"WIN32;RELEASE_BUILD\"\n"
 		              "\t\tStringPooling=\"true\"\n"
 		              "\t\tBufferSecurityCheck=\"false\"\n"
 		              "\t\tDebugInformationFormat=\"0\"\n"
@@ -1840,7 +1850,7 @@ int MSBuildProvider::getVisualStudioVersion() {
 
 #define OUTPUT_PROPERTIES_MSBUILD(config, properties) \
 	project << "\t<ImportGroup Condition=\"'$(Configuration)|$(Platform)'=='" << config << "'\" Label=\"PropertySheets\">\n" \
-	           "\t\t<Import Project=\"$(UserRootDir)\\Microsoft.Cpp.$(Platform).user.props\" Condition=\"exists('$(UserRootDir)\\Microsoft.Cpp.$(Platform).user.props')\" />\n" \
+	           "\t\t<Import Project=\"$(UserRootDir)\\Microsoft.Cpp.$(Platform).user.props\" Condition=\"exists('$(UserRootDir)\\Microsoft.Cpp.$(Platform).user.props')\" Label=\"LocalAppDataPlatform\" />\n" \
 	           "\t\t<Import Project=\"" << properties << "\" />\n" \
 	           "\t</ImportGroup>\n"
 
@@ -1890,13 +1900,7 @@ void MSBuildProvider::createProjectFile(const std::string &name, const std::stri
 
 	// Project version number
 	project << "\t<PropertyGroup>\n"
-	           "\t\t<_ProjectFileVersion>10.0.21006.1</_ProjectFileVersion>\n"; // FIXME: update temporary entry _ProjectFileVersion
-
-	if (name == "scummvm")
-		project << "<ExecutablePath>$(SCUMMVM_LIBS)\\bin;$(VCInstallDir)bin;$(WindowsSdkDir)bin\\NETFX 4.0 Tools;$(WindowsSdkDir)bin;$(VSInstallDir)Common7\\Tools\\bin;$(VSInstallDir)Common7\\tools;$(VSInstallDir)Common7\\ide;$(ProgramFiles)\\HTML Help Workshop;$(FrameworkSDKDir)\\bin;$(MSBuildToolsPath32);$(VSInstallDir);$(SystemRoot)\\SysWow64;$(FxCopDir);$(PATH)</ExecutablePath>\n"
-		           "<IncludePath>$(SCUMMVM_LIBS)\\include;$(VCInstallDir)include;$(VCInstallDir)atlmfc\\include;$(WindowsSdkDir)include;$(FrameworkSDKDir)\\include;</IncludePath>\n"
-		           "<LibraryPath>$(SCUMMVM_LIBS)\\lib;$(VCInstallDir)lib;$(VCInstallDir)atlmfc\\lib;$(WindowsSdkDir)lib;$(FrameworkSDKDir)\\lib</LibraryPath>\n";
-
+	           "\t\t<_ProjectFileVersion>10.0.30319.1</_ProjectFileVersion>\n";
 	project << "\t</PropertyGroup>\n";
 
 	// Project-specific settings
@@ -2041,39 +2045,41 @@ void MSBuildProvider::outputProjectSettings(std::ofstream &project, const std::s
 	project << "\t</ItemDefinitionGroup>\n";
 }
 
-void MSBuildProvider::outputGlobalPropFile(std::ofstream &properties, int bits, const std::string &defines, const std::string &prefix) {
-	// FIXME: update temporary entries _ProjectFileVersion & _PropertySheetDisplayName
+void MSBuildProvider::outputGlobalPropFile(std::ofstream &properties, int bits, const std::string &defines, const std::string &prefix, bool isWin32) {	
 	properties << "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
 	              "<Project DefaultTargets=\"Build\" ToolsVersion=\"4.0\" xmlns=\"http://schemas.microsoft.com/developer/msbuild/2003\">\n"
-	              "<PropertyGroup>\n"
-	              "<_ProjectFileVersion>10.0.21006.1</_ProjectFileVersion>\n"
-	              "<_PropertySheetDisplayName>ScummVM_Global</_PropertySheetDisplayName>\n"
-	              "<OutDir>$(Configuration)" << bits << "\\</OutDir>\n"
-	              "<IntDir>$(Configuration)" << bits << "/$(ProjectName)\\</IntDir>\n"
-	              "</PropertyGroup>\n"
-	              "<ItemDefinitionGroup>\n"
-	              "<ClCompile>\n"
-	              "<DisableLanguageExtensions>true</DisableLanguageExtensions>\n"
-	              "<DisableSpecificWarnings>" << _globalWarnings << ";%(DisableSpecificWarnings)</DisableSpecificWarnings>\n"
-	              "<AdditionalIncludeDirectories>" << prefix << ";" << prefix << "\\engines;%(AdditionalIncludeDirectories)</AdditionalIncludeDirectories>\n"
-	              "<PreprocessorDefinitions>" << defines << ";%(PreprocessorDefinitions)</PreprocessorDefinitions>\n"
-	              "<ExceptionHandling>\n"
-	              "</ExceptionHandling>\n"
-	              "<RuntimeTypeInfo>false</RuntimeTypeInfo>\n"
-	              "<WarningLevel>Level4</WarningLevel>\n"
-	              "<TreatWarningAsError>false</TreatWarningAsError>\n"
-	              "<CompileAs>Default</CompileAs>\n"
-	              "</ClCompile>\n"
-	              "<Link>\n"
-	              "<IgnoreSpecificDefaultLibraries>%(IgnoreSpecificDefaultLibraries)</IgnoreSpecificDefaultLibraries>\n"
-	              "<SubSystem>Console</SubSystem>\n"
-	              "<EntryPointSymbol>WinMainCRTStartup</EntryPointSymbol>\n"
-	              "</Link>\n"
-	              "<ResourceCompile>\n"
-	              "<PreprocessorDefinitions>HAS_INCLUDE_SET;%(PreprocessorDefinitions)</PreprocessorDefinitions>\n"
-	              "<AdditionalIncludeDirectories>" << prefix << ";%(AdditionalIncludeDirectories)</AdditionalIncludeDirectories>\n"
-	              "</ResourceCompile>\n"
-	              "</ItemDefinitionGroup>\n"
+	              "\t<PropertyGroup>\n"
+	              "\t\t<_ProjectFileVersion>10.0.30319.1</_ProjectFileVersion>\n"
+	              "\t\t<_PropertySheetDisplayName>ScummVM_Global</_PropertySheetDisplayName>\n"
+	              "\t\t<ExecutablePath>$(SCUMMVM_LIBS)\\bin;$(ExecutablePath)</ExecutablePath>\n"
+	              "\t\t<LibraryPath>$(SCUMMVM_LIBS)\\libs\\" << (isWin32 ? "x86" : "x64") << ";$(LibraryPath)</LibraryPath>\n"
+	              "\t\t<IncludePath>$(SCUMMVM_LIBS)\\include;$(IncludePath)</IncludePath>\n"
+	              "\t\t<OutDir>$(Configuration)" << bits << "\\</OutDir>\n"
+	              "\t\t<IntDir>$(Configuration)" << bits << "/$(ProjectName)\\</IntDir>\n"
+	              "\t</PropertyGroup>\n"
+	              "\t<ItemDefinitionGroup>\n"
+	              "\t\t<ClCompile>\n"
+	              "\t\t\t<DisableLanguageExtensions>true</DisableLanguageExtensions>\n"
+	              "\t\t\t<DisableSpecificWarnings>" << _globalWarnings << ";%(DisableSpecificWarnings)</DisableSpecificWarnings>\n"
+	              "\t\t\t<AdditionalIncludeDirectories>" << prefix << ";" << prefix << "\\engines;$(AdditionalIncludeDirectories)</AdditionalIncludeDirectories>\n"
+	              "\t\t\t<PreprocessorDefinitions>" << defines << ";%(PreprocessorDefinitions)</PreprocessorDefinitions>\n"
+	              "\t\t\t<ExceptionHandling>\n"
+	              "\t\t\t</ExceptionHandling>\n"
+	              "\t\t\t<RuntimeTypeInfo>false</RuntimeTypeInfo>\n"
+	              "\t\t\t<WarningLevel>Level4</WarningLevel>\n"
+	              "\t\t\t<TreatWarningAsError>false</TreatWarningAsError>\n"
+	              "\t\t\t<CompileAs>Default</CompileAs>\n"
+	              "\t\t</ClCompile>\n"
+	              "\t\t<Link>\n"
+	              "\t\t\t<IgnoreSpecificDefaultLibraries>%(IgnoreSpecificDefaultLibraries)</IgnoreSpecificDefaultLibraries>\n"
+	              "\t\t\t<SubSystem>Console</SubSystem>\n"
+	              "\t\t\t<EntryPointSymbol>WinMainCRTStartup</EntryPointSymbol>\n"
+	              "\t\t</Link>\n"
+	              "\t\t<ResourceCompile>\n"
+	              "\t\t\t<PreprocessorDefinitions>HAS_INCLUDE_SET;%(PreprocessorDefinitions)</PreprocessorDefinitions>\n"
+	              "\t\t\t<AdditionalIncludeDirectories>" << prefix << ";%(AdditionalIncludeDirectories)</AdditionalIncludeDirectories>\n"
+	              "\t\t</ResourceCompile>\n"
+	              "\t</ItemDefinitionGroup>\n"
 	              "</Project>\n";
 
 	properties.flush();
@@ -2087,24 +2093,25 @@ void MSBuildProvider::createBuildProp(const BuildSetup &setup, bool isRelease, b
 	if (!properties)
 		error("Could not open \"" + setup.outputDir + '/' + "ScummVM_" + outputType + (isWin32 ? "" : "64") + getPropertiesExtension() + "\" for writing");
 
-	// FIXME: update temporary entries _ProjectFileVersion & _PropertySheetDisplayName
 	properties << "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
 	              "<Project DefaultTargets=\"Build\" ToolsVersion=\"4.0\" xmlns=\"http://schemas.microsoft.com/developer/msbuild/2003\">\n"
 	              "\t<ImportGroup Label=\"PropertySheets\">\n"
 	              "\t\t<Import Project=\"ScummVM_Global" << (isWin32 ? "" : "64") << ".props\" />\n"
 	              "\t</ImportGroup>\n"
 	              "\t<PropertyGroup>\n"
-	              "\t\t<_ProjectFileVersion>10.0.21006.1</_ProjectFileVersion>\n"
+	              "\t\t<_ProjectFileVersion>10.0.30319.1</_ProjectFileVersion>\n"
 	              "\t\t<_PropertySheetDisplayName>ScummVM_" << outputType << outputBitness << "</_PropertySheetDisplayName>\n"
 	              "\t\t<LinkIncremental>" << (isRelease ? "false" : "true") << "</LinkIncremental>\n"
 	              "\t</PropertyGroup>\n"
 	              "\t<ItemDefinitionGroup>\n"
 	              "\t\t<ClCompile>\n";
 
+
+
 	if (isRelease) {
 		properties << "\t\t\t<IntrinsicFunctions>true</IntrinsicFunctions>\n"
 		              "\t\t\t<WholeProgramOptimization>true</WholeProgramOptimization>\n"
-		              "\t\t\t<PreprocessorDefinitions>WIN32;%(PreprocessorDefinitions)</PreprocessorDefinitions>\n"
+		              "\t\t\t<PreprocessorDefinitions>WIN32;RELEASE_BUILD;%(PreprocessorDefinitions)</PreprocessorDefinitions>\n"
 		              "\t\t\t<StringPooling>true</StringPooling>\n"
 		              "\t\t\t<BufferSecurityCheck>false</BufferSecurityCheck>\n"
 		              "\t\t\t<DebugInformationFormat></DebugInformationFormat>\n"
@@ -2136,12 +2143,12 @@ void MSBuildProvider::createBuildProp(const BuildSetup &setup, bool isRelease, b
 }
 
 #define OUTPUT_NASM_COMMAND_MSBUILD(config) \
-	projectFile << "\t\t\t<Command Condition=\"'$(Configuration)|$(Platform)'=='" << config << "|Win32'\">nasm.exe -f win32 -g -o \"$(IntDir)" << (isDuplicate ? (*entry).prefix : "") << "%(FileName).obj\" \"%(FullPath)\"</Command>\n" \
-	               "\t\t\t<Outputs Condition=\"'$(Configuration)|$(Platform)'=='" << config << "|Win32'\">$(IntDir)" << (isDuplicate ? (*entry).prefix : "") << "%(FileName).obj;%(Outputs)</Outputs>\n";
+	projectFile << "\t\t\t<Command Condition=\"'$(Configuration)|$(Platform)'=='" << config << "|Win32'\">nasm.exe -f win32 -g -o \"$(IntDir)" << (isDuplicate ? (*entry).prefix : "") << "%(Filename).obj\" \"%(FullPath)\"</Command>\n" \
+	               "\t\t\t<Outputs Condition=\"'$(Configuration)|$(Platform)'=='" << config << "|Win32'\">$(IntDir)" << (isDuplicate ? (*entry).prefix : "") << "%(Filename).obj;%(Outputs)</Outputs>\n";
 
 #define OUPUT_OBJECT_FILENAME_MSBUILD(config, platform, prefix) \
-	projectFile << "\t\t<ObjectFileName Condition=\"'$(Configuration)|$(Platform)'=='" << config << "|" << platform << "'\">$(IntDir)" << prefix << "%(FileName).obj</ObjectFileName>\n" \
-	               "\t\t<XMLDocumentationFileName Condition=\"'$(Configuration)|$(Platform)'=='" << config << "|" << platform << "'\">$(IntDir)" << prefix << "%(FileName).xdc</XMLDocumentationFileName>\n";
+	projectFile << "\t\t<ObjectFileName Condition=\"'$(Configuration)|$(Platform)'=='" << config << "|" << platform << "'\">$(IntDir)" << prefix << "%(Filename).obj</ObjectFileName>\n" \
+	               "\t\t<XMLDocumentationFileName Condition=\"'$(Configuration)|$(Platform)'=='" << config << "|" << platform << "'\">$(IntDir)" << prefix << "%(Filename).xdc</XMLDocumentationFileName>\n";
 
 #define OUPUT_FILES_MSBUILD(files, action) \
 	if (!files.empty()) { \
