@@ -366,12 +366,13 @@ sub add_person {
 	my $tab;
 
 	if ($mode eq "TEXT") {
+		my $min_name_width = length $desc > 0 ? $max_name_width : 0;
 		$name = $nick if $name eq "";
 		$name = html_entities_to_ascii($name);
 		$desc = html_entities_to_ascii($desc);
 
 		$tab = " " x ($section_level * 2 + 1);
-		printf $tab."%-".$max_name_width.".".$max_name_width."s", $name;
+		printf $tab."%-".$min_name_width.".".$max_name_width."s", $name;
 
 		# Print desc wrapped
 		if (length $desc > 0) {
@@ -531,6 +532,7 @@ begin_credits("Credits");
 
 	  begin_section("Broken Sword");
 		  add_person("Fabio Battaglia", "Hkz", "PSX version support");
+		  add_person("Thierry Crozat", "criezy", "Mac version support");
 		  add_person("Robert G&ouml;ffringmann", "lavosspawn", "(retired)");
 	  end_section();
 
@@ -664,6 +666,10 @@ begin_credits("Credits");
 
 
     begin_section("Backend Teams");
+	  begin_section("Android");
+		  add_person("Angus Lees", "Gus", "");
+	  end_section();
+
 	  begin_section("Dreamcast");
 		  add_person("Marcus Comstedt", "", "");
 	  end_section();
@@ -692,10 +698,6 @@ begin_credits("Credits");
 		  add_person("Neil Millstone", "agent-q", "");
 	  end_section();
 
-	  begin_section("PalmOS");
-		  add_person("Chris Apers", "chrilith ", "");
-	  end_section();
-
 	  begin_section("PocketPC / WinCE");
 		add_person("Nicolas Bacca", "arisme", "(retired)");
 		add_person("Kostas Nakos", "Jubanka", "");
@@ -708,6 +710,7 @@ begin_credits("Credits");
 
 	  begin_section("PSP (PlayStation Portable)");
 		  add_person("Joost Peters", "joostp", "");
+		  add_person("Yotam Barnoy", "bluddy", "");
 	  end_section();
 
 	  begin_section("SDL (Win/Linux/OS X/etc.)");
@@ -750,7 +753,7 @@ begin_credits("Credits");
 
     begin_section("Website (code)");
 	  begin_persons();
-		add_person("Fredrik Wendel", "", "");
+		add_person("Fredrik Wendel", "", "(retired)");
 	  end_persons();
     end_section();
 
@@ -760,6 +763,7 @@ begin_credits("Credits");
 
     begin_section("Documentation");
 	  begin_persons();
+		add_person("Thierry Crozat", "criezy", "Numerous contributions to documentation");
 		add_person("Joachim Eberhard", "joachimeberhard", "Numerous contributions to documentation");
 		add_person("Matthew Hoops", "clone2727", "Wiki editor");
 	  end_persons();
@@ -767,6 +771,7 @@ begin_credits("Credits");
 
     begin_section("Retired Team Members");
 	  begin_persons();
+		  add_person("Chris Apers", "chrilith ", "Former PalmOS porter");
 		  add_person("Ralph Brorsen", "painelf", "Help with GUI implementation");
 		  add_person("Jamieson Christian", "jamieson630", "iMUSE, MIDI, all things musical");
 		  add_person("Felix Jakschitsch", "yot", "Zak256 reverse engineering");
@@ -813,7 +818,7 @@ begin_credits("Credits");
 	  end_section();
 
 	  begin_section("Mandriva");
-		  add_person("Dominik Scherer", "", "");
+		  add_person("Dominik Scherer", "", "(retired)");
 	  end_section();
 
 	  begin_section("MorphOS");
@@ -846,6 +851,41 @@ begin_credits("Credits");
 		  add_person("Johannes Schickel", "LordHoto", "");
 	  end_section();
 	end_section();
+	
+	begin_section("Translations");
+		begin_persons();
+			add_person("Thierry Crozat", "criezy", "Translation Lead");
+		end_persons();
+		begin_section("Catalan");
+			add_person("Jordi Vilalta Prat", "jvprat", "");
+		end_section();
+		begin_section("French");
+			add_person("Thierry Crozat", "criezy", "");
+		end_section();
+		begin_section("German");
+			add_person("Simon Sawatzki", "SimSaw", "");
+			add_person("Lothar Serra Mari", "Lothar93", "");
+		end_section();
+		begin_section("Hungarian");
+			add_person("Alex Bevilacqua", "", "");
+			add_person("George Kormendi", "GoodOldGeorg", "");
+		end_section();
+		begin_section("Italian");
+			add_person("Matteo Angelino", "Maff", "");
+		end_section();
+		begin_section("Brazilian Portuguese");
+			add_person("ScummBR Team", "", "");
+		end_section();
+		begin_section("Russian");
+			add_person("Eugene Sandulenko", "sev", "");
+		end_section();
+		begin_section("Spanish");
+			add_person("Tom&aacute;s Maidagan", "Truido", "");
+		end_section();
+		begin_section("Ukrainian");
+			add_person("Lubomyr Lisen", "", "");
+		end_section();
+	end_section();
 
 	begin_section("Websites (design)");
 	  begin_persons();
@@ -862,7 +902,6 @@ begin_credits("Credits");
 		  add_person("Ori Avtalion", "salty-horse", "Subtitle control options in the GUI; BASS GUI fixes");
 		  add_person("Stuart Caie", "", "Decoders for Amiga and AtariST data files (AGOS engine)");
 		  add_person("Paolo Costabel", "", "PSP port contributions");
-		  add_person("Thierry Crozat", "criezy", "Support for Broken Sword Macintosh version");
 		  add_person("Martin Doucha", "next_ghost", "CinE engine objectification");
 		  add_person("Thomas Fach-Pedersen", "madmoose", "ProTracker module player");
 		  add_person("Tobias Gunkel", "hennymcc", "Sound support for C64 version of MM/Zak, Loom PCE support");
@@ -872,7 +911,6 @@ begin_credits("Credits");
 		  add_person("Andreas Karlsson", "Sprawl", "Initial port for SymbianOS");
 		  add_person("Claudio Matsuoka", "", "Daily Linux builds");
 		  add_person("Thomas Mayer", "", "PSP port contributions");
-		  add_person("Yotam Barnoy", "bluddy", "PSP port suspend/resume support");
 		  add_person("Sean Murray", "lightcast", "ScummVM tools GUI application (GSoC 2007 task)");
 		  add_person("", "n0p", "Windows CE port aspect ratio correction scaler and right click input method");
 		  add_person("Mikesch Nepomuk", "mnepomuk", "MI1 VGA floppy patches");
@@ -916,7 +954,7 @@ begin_credits("Credits");
 		add_person("Ravi I.", "", "SCI0 sound resource specification");
 		add_person("Ruediger Hanke", "", "Port to the MorphOS platform");
 		add_person("Rune Orsval", "", "Configuration file editor");
-		add_person("Rickard Lind", "", "MT32->GM MIDI mapping magic, sound research");
+		add_person("Rickard Lind", "", "MT-32->GM MIDI mapping magic, sound research");
 		add_person("Rink Springer", "", "Port to the DOS platform, several bug fixes");
 		add_person("Robey Pointer", "", "Bug tracking system hosting");
 		add_person("Sergey Lapin", "", "Port of Carl's type 2 decompression code");
@@ -927,8 +965,8 @@ begin_credits("Credits");
 		add_person("Sean Terrell", "", "");
 	  end_persons();
 	  add_paragraph("Special thanks to Prof. Dr. Gary Nutt ".
-					  "for allowing the FreeSCI VM extension as a ".
-					  "course project in his Advanced OS course.");
+	                "for allowing the FreeSCI VM extension as a ".
+	                "course project in his Advanced OS course.");
 	  add_paragraph("Special thanks to Bob Heitman and Corey Cole for their support of FreeSCI.");
 	end_section();
 
@@ -989,6 +1027,15 @@ begin_credits("Credits");
   "John Young, Colin Smythe and especially Terry Pratchett himself for ".
   "sharing the source code of Discworld I &amp; II with us.");
 
+  add_paragraph(
+  "Emilio de Paz Arag&oacute;n from Alcachofa Soft for sharing the source code ".
+  "of Drascula: The Vampire Strikes Back with us and his generosity with ".
+  "freewaring the game.");
+
+  add_paragraph(
+  "David P. Gray from Gray Design Associate for sharing the source code ".
+  "of the Hugo trilogy.");
+
   end_section();
 
 end_credits();
