@@ -18,8 +18,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-1-1/tools/make-scumm-fontdata.c $
- * $Id: make-scumm-fontdata.c 35056 2008-11-14 14:03:08Z eriktorbjorn $
+ * $URL: https://scummvm.svn.sourceforge.net/svnroot/scummvm/scummvm/tags/release-1-2-1/tools/make-scumm-fontdata.c $
+ * $Id: make-scumm-fontdata.c 49888 2010-06-15 19:26:14Z lordhoto $
  */
 
 #include <stdio.h>
@@ -821,7 +821,7 @@ static const unsigned char spanishCharsetDataV2[] = {
 unsigned char *specialCharsetData = NULL;
 int numSpecialChars = 0;
 
-void compressCharset(const unsigned char *data, char *var, char *name) {
+void compressCharset(const unsigned char *data, const char *var, const char *name) {
 	int i;
 
 	printf("// %s\n", name);
